program ANDRep;

uses
  classes,
  Forms,
  Dialogs,
  Registry,
  Printers,
  SysUtils,
  AREditorMain,
  ARUserRep,
  ARPreviewForm,
  ARResStrings,
  ARExcelExport,
  ARJPEGExport,
  ARHTMLExport,
  RepData in 'RepData.pas';

{$R *.RES}


procedure ExecReport;
var
  F: TForm;
  UserRep: TARUserRep;
  rd: TRepData;
begin
  f := TForm.Create(Application);
  UserRep:= TARUserRep.Create(Application);
  UserRep.Parent := f;
  UserRep.DeleteBands;
  UserRep.Load(ParamStr(1));
  rd := TRepData.Create(UserRep, ParamStr(2));
  rd.Load;
  rd.Free;
  UserRep.Preview;
  UserRep.Free;
  f.Free;
end;


procedure ExecEditor;
var
  AREditor: TAREditorDlg;
  reg: TRegistry;
begin
  AREditor := TAREditorDlg.Create(Application);
  if AREditor.PC.PageCount > 1 then
    AREditor.FileClosePageExecute(nil);
  try
    AREditor.ShowModal;
  finally
    AREditor.Free;
  end;

  reg:=TRegistry.Create;
  try
    reg.DeleteKey('\Software\ANDReport\Editor\');
  finally
    reg.Free;
  end;
end;

function GetSaveFile(FileName,FileExt: String): String;
var
  sd: TSaveDialog;
begin
  sd := TSaveDialog.Create(nil);
  sd.FileName:= FileName+FileExt;
  if sd.Execute then
    GetSaveFile := sd.FileName
  else
    GetSaveFile := '';
  ChDir(ExtractFilePath(ParamStr(0)));
  sd.free;
end;


procedure doMultiRep(sl: TStringList);
var
  F: TForm;
  UserRep: TARUserRep;
  rd: TRepData;
  dlg: TARPreviewDialog;
  i,j: Integer;
  Copies: Integer;
  typeOut,zoom,PrnName,RepName: string;
  ExpDir,curDir: String;
  FirstTime: boolean;
  xe : TARExcelExport;
  xj : TARJPEGExport;
  xh : TARHTMLExport;
begin
  FirstTime := true;
  CurDir := GetCurrentDir;
  Copies := 1;
  f := TForm.Create(Application);
  UserRep:= TARUserRep.Create(Application);
  UserRep.Parent := f;
  UserRep.DeleteBands;

  for i := 0 to sl.Count -1 do begin
    if pos('TYPE:',UpperCase(sl[i])) = 1      then typeOut := UpperCase(trim(copy(sl[i],6,MaxInt)));
    if pos('ZOOM:',UpperCase(sl[i])) = 1      then zoom    := UpperCase(trim(copy(sl[i],6,MaxInt)));
    if pos('COPIES:',UpperCase(sl[i])) = 1    then Copies  := StrToIntDef(trim(copy(sl[i],8,MaxInt)),1);
    if pos('PRINTER:',UpperCase(sl[i])) = 1   then PrnName := UpperCase(trim(copy(sl[i],9,MaxInt)));
    if pos('NAME:',UpperCase(sl[i])) = 1      then RepName := UpperCase(trim(copy(sl[i],6,MaxInt)));
    if pos('DIR:',UpperCase(sl[i])) = 1       then ExpDir  := UpperCase(trim(copy(sl[i],5,MaxInt)));
    if pos('REPORT:',UpperCase(sl[i])) = 1    then UserRep.Load(trim(copy(sl[i],8,MaxInt)));
    if pos('DATAFILE:',UpperCase(sl[i])) = 1 then begin
      UserRep.DeleteBands;
      rd := TRepData.Create(UserRep,trim(copy(sl[i],10,MaxInt)));
      rd.Load;

      if RepName = '' then RepName := 'AndRep';
      if ExpDir <> '' then begin
        j := LastDelimiter('\:',ExpDir);
        if j = 0 then ExpDir := ExpDir + '\';
      end;

      userRep.prn.Copies := copies;
      if FirstTime then
        for j := 0 to Printer.Printers.Count-1 do
          if pos(PrnName, UpperCase(Printer.Printers[j])) = 1 then
            Printer.PrinterIndex := j;

      if          typeOut = 'PRINT'    then begin
        if FirstTime then
          UserRep.PrintDialog
        else
          UserRep.Print;
      end else if typeOut = 'PREVIEW'  then begin
        UserRep.ShowPrintProgress := false;
        if UserRep.CreateCanvasImage(rsARReportPreview_JobBreakPages) then begin
          dlg:=TARPreviewDialog.Create(Application.MainForm);
          try
            dlg.ARPrinter:=UserRep.prn;
            if zoom = 'TOWIDTH' then
              dlg.ZoomToWidthBtnClick( nil);
            if zoom = 'TO100' then
              dlg.ZoomTo100BtnClick(nil);
            if zoom = 'TOFIT' then
              dlg.ZoomToFitBtnClick(nil);
            dlg.ShowModal;
          finally
            dlg.Free;
          end;
        end;
      end else if typeOut = 'TOHTML'   then begin
        xh := TARHTMLExport.Create(nil);
        xh.ARRep := UserRep;
        xh.FileName := GetSaveFile(ExpDir+RepName,'.htm');
        if xh.FileName <> '' then xh.Execute;
        xh.Free;
      end else if typeOut = 'TOEXCEL'  then begin
        xe := TARExcelExport.Create(nil);
        xe.ARRep := UserRep;
        xe.FileName := GetSaveFile(ExpDir+RepName,'.xls');
        if xe.FileName <> '' then xe.Execute;
        xe.Free;
      end else if typeOut = 'TOJPG'    then begin
        xj := TARJPEGExport.Create(nil);
        xj.ARRep := UserRep;
        xj.CompressionQuality := 50;
        xj.FileName := ExpDir+RepName+'.jpg';
        xj.ShowSetupDialog := true;
        xj.Execute;
        xj.Free;
      end;

      rd.Free;
      SetCurrentDir(curDir);
      FirstTime := false;
    end;
  end;

  UserRep.Free;
  f.Free;
end;

function multiReport: Boolean;
var sl : TStringList;
begin
  sl := TStringList.Create;
  try
    sl.LoadFromFile(ParamStr(1));
    result := pos('*** ANDREP CONFIGURATION FILE ***',sl[0]) <> 0;
    if result then
      doMultiRep(sl);
  finally
    sl.Free;
  end
end;

var i: Integer;

begin
  Application.Initialize;
  Application.Run;
  i := ParamCount;
  if i > 1 then
    execReport
  else begin
    if (i = 0) or (not multiReport) then
      execEditor;
  end;
end.
