unit ARActions;
//************************************************************//
//                         AND Report                        //
//  Eugene Hrulev (mrHru) Laxian Key, Omsk, Russia 1999,2001 //
//  Oleg Melnikov (muler) Laxian Key, Omsk, Russia 1999,2001 //
//************************************************************//

{$INCLUDE DEFINES.INC}

interface

uses
  Classes, SysUtils,
  {$IFDEF CLX}
    QControls, QGraphics, QMenus,
  {$ELSE}
    Graphics, Menus, Controls, ComCtrls,
  {$ENDIF}

  ARRepCommon, ARUtils, ARButtons;

type
  TARAction = class;

  TCreateControlEvent = procedure (Sender:TARAction; AOwner:TComponent; Parent:TParControl; var Control:TControl) of object;

// TARAction
  TARAction = class(TComponent)
  private
    FCategory   :string;
    FHint       :string;
    FCaption    :string;
    FImage      :TBitmap;
    FShortCut   :TShortCut;
    FAction     :TNotifyEvent;
    FEnabled    :boolean;
    FChecked    :boolean;
    FColor      :TColor;
    FIntValue   :integer;
    FGridKind   :TFrameStyle;
    FValue      :string;
    FStrList    :TAIStringList;
    FBorderStyle: TLineStyle;

    procedure   SetCategory(ACategory:string);
    procedure   SetHint(AHint:string);
    procedure   SetCaption(ACaption:string);
    procedure   SetShortCut(AShortCut:TShortCut);
    procedure   SetAction(AAction:TNotifyEvent);
    procedure   SetEnabled(AEnabled:boolean);
    procedure   SetChecked(AChecked:boolean);
    procedure   SetColor(AColor:TColor);
    procedure   SetIntValue(AIntValue:integer);
    procedure   SetGridKind(AGridKind:TFrameStyle);
    procedure   SetValue(AValue:string);
    procedure   SetStrList(AStrList:TAIStringList);
    procedure   SetImage(AImage:TBitmap);
    procedure   SetBorderStyle(const Value: TLineStyle);

  protected
    procedure   UpdateCategory;
    procedure   UpdateHint;
    procedure   UpdateCaption;
    procedure   UpdateShortCut;
    procedure   UpdateActions;
    procedure   UpdateEnabled;
    procedure   UpdateChecked;
    procedure   UpdateImage;
    procedure   UpdateColor;
    procedure   UpdateIntValue;
    procedure   UpdateBorderStyle;
    procedure   UpdateGridKind;
    procedure   UpdateValue;
    procedure   UpdateStrList;
  public
    OnCreateControl:TCreateControlEvent;

    property    Category:string         read  FCategory     write SetCategory;
    property    Hint:string             read  FHint         write SetHint;
    property    Caption:string          read  FCaption      write SetCaption;

    property    Image:TBitmap           read  FImage        write SetImage;
    property    ShortCut:TShortCut      read  FShortCut     write SetShortCut;
    property    Action:TNotifyEvent     read  FAction       write SetAction;
    property    Enabled:boolean         read  FEnabled      write SetEnabled;
    property    Checked:boolean         read  FChecked      write SetChecked;
    property    Color:TColor            read  FColor        write SetColor;
    property    IntValue:integer        read  FIntValue     write SetIntValue;
    property    BorderStyle:TLineStyle  read  FBorderStyle  write SetBorderStyle;
    property    GridKind:TFrameStyle    read  FGridKind     write SetGridKind;
    property    Value:string            read  FValue        write SetValue;
    property    StrList:TAIStringList   read  FStrList      write SetStrList;

    constructor Create(AOwner:TComponent);override;
    destructor  Destroy;override;
    procedure   AddComponent(AComponent:TComponent);
    function    CreateControl(AOwner:TComponent; Parent:TParControl):TControl;
  end;

// TARActionList
  TARActionList = class
  private
    ItemList    :TAIStringList;
    function    GetItems(ACategory:string; ACaption:string):TARAction;
    function    GetItem(index:integer):TARAction;
  public
    CategoryList:TAIStringList;

    constructor Create;
    destructor  Destroy;override;

    procedure   Clear;
    procedure   Add(AAction:TARAction);
    function    Action(ACaption:string):TARAction;
    property    Items[ACategory:string; AName:string]:TARAction read GetItems;default;
    property    Item[index:integer]:TARAction read GetItem;
    function    Count:integer;
    procedure   RemoveControl(AControl:TControl);
    function    FindComponent(AComponentName:string):TComponent;
  end;

type
  TDummyControl = class(TControl) // access to protected properties
  public
    property  DragMode;
    {$IFDEF CLX}
    {$ELSE}
      {$IFNDEF D3}
        property  DragKind;
        property  OnStartDock;
        property  OnEndDock;
      {$ENDIF}
    {$ENDIF}
  end;

function  Flat(s:string):string;

implementation

// TARAction
constructor TARAction.Create(AOwner:TComponent);
begin
  inherited;

  FEnabled:=true;

  FImage:=TBitmap.Create;
  {$IFNDEF D3}
    FImage.TransparentMode:=tmAuto;
  {$ENDIF}
  FImage.Transparent:=true;

  FStrList:=TAIStringList.Create('TARAction.FStrList');
end;

destructor  TARAction.Destroy;
begin
  FStrList.Free;
  FImage.Free;

  inherited;
end;

procedure   TARAction.AddComponent(AComponent:TComponent);
begin
  AComponent.Owner.RemoveComponent(AComponent);
  InsertComponent(AComponent);

  UpdateCategory;
  UpdateHint;
  UpdateCaption;
  UpdateShortCut;
  UpdateActions;
  UpdateEnabled;
  UpdateChecked;
  UpdateImage;
end;

///////////
procedure   TARAction.UpdateImage;
var i:integer;
begin
  for i:=0 to ComponentCount - 1 do begin
    if TComponent(Components[i]) is TMenuItem then begin
      {$IFNDEF D3}
        TMenuItem(Components[i]).Bitmap:=FImage;
      {$ENDIF}
    end else

    if TComponent(Components[i]) is TARComboButton then begin
    end else

    if TComponent(Components[i]) is TARGlyphButton then begin
      if not FImage.Empty then begin
        TARGlyphButton(Components[i]).Image :=FImage;
//        TARGlyphButton(Components[i]).Caption :='    ';
      end else begin
        TARGlyphButton(Components[i]).Caption :=Flat(FCaption);
        TARGlyphButton(Components[i]).Hint    :=FHint;
      end;
    end;
  end;
end;

procedure   TARAction.UpdateCategory;
begin
end;

procedure   TARAction.UpdateHint;
var i:integer;
begin
  for i:=0 to ComponentCount - 1 do begin
    if TComponent(Components[i]) is TMenuItem then begin
      TMenuItem(Components[i]).Hint:=FHint;
    end else

    if TComponent(Components[i]) is TControl then begin
      TControl(Components[i]).Hint:=Flat(FCaption) + '|' + FHint;
    end;
  end;
end;

procedure   TARAction.UpdateCaption;
var i:integer;
begin
  for i:=0 to ComponentCount - 1 do begin
    if TComponent(Components[i]) is TMenuItem then begin
      TMenuItem(Components[i]).Caption:=FCaption;
    end else
    if TComponent(Components[i]) is TARGlyphButton then begin
      TARGlyphButton(Components[i]).Hint:=Format('%s (%s)', [FHint, ShortCutToText(FShortCut)]);
    end;
  end;
end;

procedure   TARAction.UpdateShortCut;
var i:integer;
begin
  for i:=0 to ComponentCount - 1 do begin
    if TComponent(Components[i]) is TMenuItem then begin
      TMenuItem(Components[i]).ShortCut:=FShortCut;
    end else
    if TComponent(Components[i]) is TARGlyphButton then begin
      if FImage.Empty then begin
        TARGlyphButton(Components[i]).Caption:=Flat(FCaption);
      end else begin
        TARGlyphButton(Components[i]).Hint:=Format('%s (%s)', [FHint, ShortCutToText(FShortCut)]);
      end;
    end;
  end;
end;

procedure   TARAction.UpdateActions;
var i:integer;
begin
  for i:=0 to ComponentCount - 1 do begin
    if TComponent(Components[i]) is TMenuItem then begin
      TMenuItem(Components[i]).OnClick:=FAction;
    end else
    if TComponent(Components[i]) is TARComboButton then begin
      TARComboButton(Components[i]).OnChange:=FAction;
    end else
    if TComponent(Components[i]) is TARGridButton then begin
      TARColorButton(Components[i]).OnChange:=FAction;
    end else
    if TComponent(Components[i]) is TARColorButton then begin
      TARColorButton(Components[i]).OnChange:=FAction;
      TARColorButton(Components[i]).OnClick :=FAction;
    end else
    if TComponent(Components[i]) is TARGlyphButton then begin
      TARGlyphButton(Components[i]).OnClick  :=FAction;
    end;
  end;
end;

procedure   TARAction.UpdateEnabled;
var i:integer;
begin
  for i:=0 to ComponentCount - 1 do begin
    if TComponent(Components[i]) is TMenuItem then begin
      TMenuItem(Components[i]).Enabled:=FEnabled;
    end else
    if TComponent(Components[i]) is TControl then begin
      TControl(Components[i]).Enabled:=FEnabled;
    end;
  end;
end;

var
  fGroupIndex:integer = 0;

procedure   TARAction.UpdateChecked;
var i:integer;
begin
  for i:=0 to ComponentCount - 1 do begin
    if TComponent(Components[i]) is TMenuItem then begin
      TMenuItem(Components[i]).Checked:=FChecked;
    end else
    if TComponent(Components[i]) is TARGridButton then begin
      TARGridButton(Components[i]).Down :=FChecked;
    end else
    if TComponent(Components[i]) is TARGlyphButton then begin
      if FChecked then begin
        if TARGlyphButton(Components[i]).GroupIndex = 0 then begin
          inc(fGroupIndex);
          TARGlyphButton(Components[i]).GroupIndex:=fGroupIndex;
          TARGlyphButton(Components[i]).AllowAllUp:=true;
        end;
        TARGlyphButton(Components[i]).Down:=true;
      end else begin
        TARGlyphButton(Components[i]).Down:=false;
      end;
    end;
  end;
end;

procedure   TARAction.UpdateColor;
var i:integer;
begin
  for i:=0 to ComponentCount - 1 do begin
    if TComponent(Components[i]) is TARColorButton then begin
      TARColorButton(Components[i]).Color:=FColor;
    end;
  end;
end;

procedure   TARAction.UpdateGridKind;
var i:integer;
begin
  for i:=0 to ComponentCount - 1 do begin
    if TComponent(Components[i]) is TARGridButton then begin
      TARGridButton(Components[i]).GridKind:=FGridKind;
    end;
  end;
end;

procedure   TARAction.UpdateValue;
var i:integer;
begin
  for i:=0 to ComponentCount - 1 do begin
    if TComponent(Components[i]) is TARComboButton then begin
      TARComboButton(Components[i]).Value:=FValue;
    end;
  end;
end;

procedure   TARAction.UpdateStrList;
var i:integer;
begin
  for i:=0 to ComponentCount - 1 do begin
    if TComponent(Components[i]) is TARComboButton then begin
      TARComboButton(Components[i]).Items.Assign(FStrList);
    end;
  end;
end;

///////////
procedure   TARAction.SetCategory(ACategory:string);
begin
  if FCategory <> ACategory then begin
    FCategory:=ACategory;
    UpdateCategory;
  end;
end;

procedure   TARAction.SetHint(AHint:string);
begin
  if FHint <> AHint then begin
    FHint:=AHint;
    UpdateHint;
  end;
end;

procedure   TARAction.SetCaption(ACaption:string);
begin
  if FCaption <> ACaption then begin
    FCaption:=ACaption;
    UpdateCaption;
  end;
end;

procedure   TARAction.SetShortCut(AShortCut:TShortCut);
begin
  if FShortCut <> AShortCut then begin
    FShortCut:=AShortCut;
    UpdateShortCut;
  end;
end;

procedure   TARAction.SetAction(AAction:TNotifyEvent);
begin
  FAction:=AAction;
  UpdateActions;
end;

procedure   TARAction.SetEnabled(AEnabled:boolean);
begin
  if FEnabled <> AEnabled then begin
    FEnabled:=AEnabled;
    UpdateEnabled;
  end;
end;

procedure   TARAction.SetChecked(AChecked:boolean);
begin
  if FChecked <> AChecked then begin
    FChecked:=AChecked;
    UpdateChecked;
  end;
end;

procedure   TARAction.SetIntValue(AIntValue:integer);
begin
  if FIntValue <> AIntValue then begin
    FIntValue:=AIntValue;
    UpdateIntValue;
  end;
end;

procedure   TARAction.SetGridKind(AGridKind:TFrameStyle);
begin
  if FGridKind <> AGridKind then begin
    FGridKind:=AGridKind;
    UpdateGridKind;
  end;
end;

procedure   TARAction.SetValue(AValue:string);
begin
  if FValue <> AValue then begin
    FValue:=AValue;
    UpdateValue;
  end;
end;

procedure   TARAction.SetStrList(AStrList:TAIStringList);
begin
  FStrList.Assign(AStrList);
  UpdateStrList;
end;

procedure   TARAction.SetImage(AImage:TBitmap);
begin
  FImage.Assign(AImage);
end;

function    TARAction.CreateControl(AOwner:TComponent; Parent:TParControl):TControl;
begin
  Result:=nil;
  if Assigned(OnCreateControl) then begin
    OnCreateControl(Self, AOwner, Parent, Result);
  end;

  if Assigned(Result) then begin
    AddComponent(Result);
  end;
end;

// TARActionList
constructor TARActionList.Create;
begin
  inherited;

  ItemList:=TAIStringList.Create('TARActionList.ItemList');
  ItemList.Sorted:=true;

  CategoryList:=TAIStringList.Create('TARActionList.CategoryList');
  CategoryList.Sorted:=true;
  CategoryList.Duplicates:=dupIgnore;
end;

destructor  TARActionList.Destroy;
begin
  ItemList.Free;
  CategoryList.Free;
  inherited;
end;

procedure   TARActionList.Clear;
var i:integer;
begin
  for i:=0 to ItemList.Count - 1 do begin
    TARAction(ItemList[0]).Free;
  end;
  ItemList.Clear();
end;

function  Flat(s:string):string;
var idx:integer;
begin
  Result:=s;
  idx:=System.Pos('&', Result);
  while idx > 0 do begin
    System.Delete(Result, idx, 1);
    idx:=System.Pos('&', Result);
  end;

  idx:=System.Pos(' ', Result);
  while idx > 0 do begin
    System.Delete(Result, idx, 1);
    if idx <= Length(Result) then Result[idx]:=ANSIUpperCase(Result[idx])[1];
    idx:=System.Pos(' ', Result);
  end;
end;

procedure   TARActionList.Add(AAction:TARAction);
begin
  ItemList.AddObject(Flat(AAction.Category + AAction.Caption), AAction);
  CategoryList.Add(Flat(AAction.Category)); 
end;

function    TARActionList.GetItems(ACategory:string; ACaption:string):TARAction;
var idx:integer;
begin
  idx:=ItemList.IndexOf(Flat(ACategory + ACaption));
  if idx = -1 then begin
    raise Exception.CreateFmt('Unknown action Category: <%s>, Caption: <%s>', [ACategory, ACaption]);
  end else begin
    Result:=TARAction(ItemList.Objects[idx]);
  end;
end;

function    TARActionList.Action(ACaption:string):TARAction;
var i:integer;
begin
  for i:=0 to ItemList.Count - 1 do begin
    if Flat(TARAction(ItemList.Objects[i]).Caption) = Flat(ACaption) then begin
      Result:=TARAction(ItemList.Objects[i]);
      exit;
    end;
  end;

  raise Exception.CreateFmt('Unknown action Caption: <%s>', [ACaption]);
end;

function    TARActionList.GetItem(index:integer):TARAction;
begin
  Result:=TARAction(ItemList.Objects[index]);
end;

function    TARActionList.Count:integer;
begin
  Result:=ItemList.Count;
end;

function    TARActionList.FindComponent(AComponentName:string):TComponent;
var i:integer;
begin
  for i:=0 to ItemList.Count - 1 do begin
    Result:=TARAction(ItemList.Objects[i]).FindComponent(AComponentName);
    if Assigned(Result) then begin
      exit;
    end;
  end;

  raise Exception.CreateFmt('Can''t find component <%s>', [AComponentName]);
end;

procedure   TARActionList.RemoveControl(AControl:TControl);
begin
end;

procedure   TARAction.SetColor(AColor:TColor);
begin
  if FColor <> AColor then begin
    FColor:=AColor;
    UpdateColor;
  end;
end;

procedure   TARAction.UpdateIntValue;
var i:integer;
begin
  for i:=0 to ComponentCount - 1 do begin
    if TComponent(Components[i]) is TARGridButton then begin
      TARGridButton(Components[i]).GridWidth:=FIntValue;
    end;
  end;
end;

procedure   TARAction.UpdateBorderStyle;
var i:integer;
begin
  for i:=0 to ComponentCount - 1 do begin
    if TComponent(Components[i]) is TARGridButton then begin
      TARGridButton(Components[i]).GridStyle:=FBorderStyle;
    end;
  end;
end;

procedure TARAction.SetBorderStyle(const Value: TLineStyle);
begin
//  if FBorderStyle <> Value then begin
    FBorderStyle:=Value;
    UpdateBorderStyle;
//  end;
end;

end.
