unit ARBarcode;
//************************************************************//
//                         AND Report                        //
//  Eugene Hrulev (mrHru) Laxian Key, Osmk, Russia 1999,2000 //
//  Oleg Melnikov (muler) Laxian Key, Osmk, Russia 1999,2000 //
//************************************************************//

{$INCLUDE DEFINES.INC}

interface

uses
  SysUtils, Classes, Math,
  {$IFDEF CLX}
    QGraphics, Types,
  {$ELSE}
    Windows, Messages, Graphics,
  {$ENDIF}

  ARUtils, ARUserRep, Barcode;

type
  TARBarcode = class(TARUserRepPlugIn)
  private
    FDefCheckSum      :boolean;
    FDefRatio         :double;
    FDefAngle         :double;
    FDefWidth         :integer;
    FDefHeight        :integer;
    FDefModul         :integer;
    FDefShowText      :TBarcodeOption;
    FDefTyp           :TBarcodeType;
    FDefCheckSumMethod:TCheckSumMethod;
    FDefColorBar      :TColor;
    FDefColor         :TColor;

    function    GetBarCodesByName(const VarName: string): TAsBarcode;
    function    GetBarCodesByIndex(VarIndex:integer):TAsBarcode;
  protected
    function    CreateVar:TObject;override;
    procedure   ApplyValues;override;
  public
    constructor Create(AOwner:TComponent);override;
    destructor  Destroy;override;

    property    BarCodes[const VarName:string]:TAsBarcode     read GetBarCodesByName;default;
    property    BarCodesByIndex[VarIndex:integer]:TAsBarcode  read GetBarCodesByIndex;
  published
   //default properties
   { Height of Barcode (Pixel)}
    property    DefHeight         :integer          read FDefHeight          write FDefHeight;
    property    DefWidth          :integer          read FDefWidth           write FDefWidth;
   { Width of the smallest line in a Barcode }
    property    DefModul          :integer          read FDefModul           write FDefModul;
    property    DefRatio          :double           read FDefRatio           write FDefRatio;
    property    DefTyp            :TBarcodeType     read FDefTyp             write FDefTyp            default bcCode_2_5_interleaved;
   { build CheckSum ? }
    property    DefChecksum       :boolean          read FDefCheckSum        write FDefCheckSum       default false;
    property    DefCheckSumMethod :TCheckSumMethod  read FDefCheckSumMethod  write FDefCheckSumMethod default csmModulo10;
   { 0 - 360 degree }
    property    DefAngle          :double           read FDefAngle           write FDefAngle;

    property    DefShowText       :TBarcodeOption   read FDefShowText        write FDefShowText       default bcoNone;
    property    DefColor          :TColor           read FDefColor           write FDefColor          default clWhite;
    property    DefColorBar       :TColor           read FDefColorBar        write FDefColorBar       default clBlack;
  end;

procedure Register;

implementation

{$R *.res}

procedure Register;
begin
  RegisterComponents('ARComponents', [TARBarcode]);
end;

{ TARBarcode }

constructor TARBarcode.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FDefCheckSum      :=false;
  FDefRatio         :=2.0;
  FDefAngle         :=0.0;
  FDefModul         :=1;
  FDefShowText      :=bcoBoth;
  FDefTyp           :=bcCodeEAN13;
  FDefCheckSumMethod:=csmModulo10;
  FDefColorBar      :=clBlack;
  FDefColor         :=clWhite;

//  FDefWidth         :=100;
//  FDefHeight        :=145;
end;

function    TARBarcode.CreateVar: TObject;
var tmp:TAsBarcode;
begin
  tmp:=TAsBarcode.Create(nil);

  tmp.Checksum      :=FDefCheckSum;
  tmp.Ratio         :=FDefRatio;
  tmp.Angle         :=FDefAngle;
  tmp.Width         :=FDefWidth;
  tmp.Height        :=FDefHeight;
  tmp.Modul         :=FDefModul;
  tmp.ShowText      :=FDefShowText;
  tmp.Typ           :=FDefTyp;
  tmp.CheckSumMethod:=FDefCheckSumMethod;
  tmp.ColorBar      :=FDefColorBar;
  tmp.Color         :=FDefColor;

  Result:=tmp;
end;

destructor  TARBarcode.Destroy;
begin
  inherited;
end;

function    TARBarcode.GetBarCodesByName(const VarName: string): TAsBarcode;
begin
  Result:=TAsBarcode(VarByName[VarName]);
end;

function TARBarcode.GetBarCodesByIndex(VarIndex: integer): TAsBarcode;
begin
  Result:=TAsBarcode(VarByIndex[VarIndex]);
end;

// calculate width and height of a rect rotated by alpha angle

function  CalcWH(w:integer; h:integer; alpha:double):TPoint;
var sinus, cosinus : extended;
    minx, miny, maxx, maxy:integer;
    p:TPoint;
  function Rotate2D(x:integer; y:integer): TPoint;
  begin
    result.x:=Round( x * cosinus + y * sinus);
    result.y:=Round(-x * sinus   + y * cosinus);
  end;
begin
  sinus   := sin(alpha/180.0*pi);
  cosinus := cos(alpha/180.0*pi);

  minx:=0; miny:=0; maxx:=0; maxy:=0;

  p:=Rotate2D(w, 0);
  minx:=Min(minx, p.X); miny:=Min(miny, p.Y); maxx:=Max(maxx, p.X); maxy:=Max(maxy, p.Y);

  p:=Rotate2D(0, h);
  minx:=Min(minx, p.X); miny:=Min(miny, p.Y); maxx:=Max(maxx, p.X); maxy:=Max(maxy, p.Y);

  p:=Rotate2D(w, h);
  minx:=Min(minx, p.X); miny:=Min(miny, p.Y); maxx:=Max(maxx, p.X); maxy:=Max(maxy, p.Y);

  Result.X:=maxx - minx + 1; Result.Y:=maxy - miny + 1;
end;

procedure   TARBarcode.ApplyValues;
var i:integer;
    {$IFDEF CLX}
//      img:TDrawing;
      img:TBitmap;
    {$ELSE}
      img:TMetafile;
      canvas:TMetafileCanvas;
    {$ENDIF}
    p:TPoint;
begin
  for i:=0 to VarCount - 1 do begin
    {$IFDEF CLX}
//      img:=TDrawing.Create;
      img:=TBitmap.Create;
    {$ELSE}
      img:=TMetafile.Create;
    {$ENDIF}
    try
      BarCodesByIndex[i].Top  :=0;
      BarCodesByIndex[i].Left :=0;

      p:=CalcWH(BarCodesByIndex[i].Width, BarCodesByIndex[i].Height, BarCodesByIndex[i].Angle);
      img.Width :=p.X;
      img.Height:=p.Y;

      {$IFDEF CLX}
        img.Canvas.Start(true);
        try
          BarCodesByIndex[i].DrawBarcode(img.Canvas);
        finally
          img.Canvas.Stop;
        end;
      {$ELSE}
        canvas:=TMetafileCanvas.Create(img, 0);
        try
          BarCodesByIndex[i].DrawBarcode(canvas);
        finally
          canvas.Free;
        end;
      {$ENDIF}

      ARUserRep.G[VarNames[i]]:=img;
    finally
      img.Free;
    end;
  end;
end;

(*
procedure   TARBarcode.ApplyValues;
var i:integer;
    img:TBitmap;
    p:TPoint;
begin
  for i:=0 to VarCount - 1 do begin
    img:=TBitmap.Create;
    try
      BarCodesByIndex[i].Top  :=0;
      BarCodesByIndex[i].Left :=0;

      p:=CalcWH(BarCodesByIndex[i].Width, BarCodesByIndex[i].Height, BarCodesByIndex[i].Angle);
      img.Width :=p.X;
      img.Height:=p.Y;

      BarCodesByIndex[i].DrawBarcode(img.Canvas);

      ARUserRep.G[VarNames[i]]:=img;
    finally
      img.Free;
    end;
  end;
end;
*)

end.


