unit ARButtons;
//************************************************************//
//                         AND Report                        //
//  Eugene Hrulev (mrHru) Laxian Key, Omsk, Russia 1999,2001 //
//  Oleg Melnikov (muler) Laxian Key, Omsk, Russia 1999,2001 //
//************************************************************//

// changes
// sa: Sertac Akyuz, mailto:prosoft@turk.net // 11.06.2001

{$INCLUDE DEFINES.INC}

interface

uses
  SysUtils, Math, Classes,
  {$IFDEF CLX}
    QButtons, QGraphics, QForms, QControls, QExtCtrls, QStdCtrls, Types,
    QComCtrls, QDialogs,
  {$ELSE}
    Buttons, Graphics, Forms, Controls, ExtCtrls, StdCtrls, ComCtrls, Dialogs,

    Windows, Menus,
  {$ENDIF}

  ARUtils, ARRepCommon;

const
  DropDownWidth = 14;

type
  {$IFDEF CLX}
    TARBitmap = QGraphics.TBitmap;
  {$ELSE}
    TARBitmap = Graphics.TBitmap;
  {$ENDIF}

  TARGlyphButton = class(TSpeedButton)
  private
    FCaption      :string;
    FImage        :TARBitmap;
    procedure     SetCaption(ACaption:string);
  protected
    function      GetImage:TARBitmap;
    procedure     SetImage(AImage:TARBitmap);

    procedure     Paint; override;
    procedure     UpdateWidth;virtual;
  public
    property      Image:TARBitmap read  GetImage  write SetImage;

    constructor   Create(AOwner: TComponent); override;
//sa: 11.06.2001
    destructor    Destroy;override;
//--
  published
    property      Caption:string  read  FCaption  write SetCaption;
  end;

//  TARButton
  TARButton = class(TARGlyphButton)
  private
		FPopupForm      :TForm;
		FOnChange       :TNotifyEvent;
		FChecked        :boolean;
    FLoaded         :boolean;
		FPopupFormHiding:boolean;

		procedure       PopupDeactivate(Sender: TObject);
  	procedure       PopupKeyPress(Sender:TObject; var Key: Char);
		procedure       SetChecked(AChecked:boolean);
    procedure       OnShowPopupForm(Sender:TObject);
    procedure       OnClosePopupForm(Sender:TObject; var Action:TCloseAction);
  protected
    MouseInControl  :boolean;
    {$IFDEF CLX}
      procedure     MouseEnter(AControl: TControl);override;
      procedure     MouseLeave(AControl: TControl);override;
    {$ENDIF}
    procedure       CloseUp(Action:boolean);
    procedure       RecreatePopupForm;virtual;
    procedure       DoClick;virtual;
    procedure       DoChange;virtual;
    procedure       Loaded;override;
    procedure       MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);override;
    procedure       Paint;override;
    procedure       UpdateWidth;override;
  public
    constructor     Create(AOwner: TComponent); override;
    destructor      Destroy; override;
    procedure       Popup(Sender:TObject);virtual;
  published
		property        OnChange:TNotifyEvent read FOnChange    write FOnChange;
		property        Checked:boolean       read FChecked     write SetChecked;
  end;

const
  MorePanelHeight = 36;

type
// TARColorButton
  TARColorButton = class(TARButton)
  private
    FColor      :TColor;
    FDocColors  :TAIList;
    FDocBtns    :TAIList;

    FStdBtns    :TAIList;
    FStdPanel   :TPanel;
    FDocPanel   :TPanel;
    FMorePanel  :TPanel;
		FOnGetDocColors:TNotifyEvent;

    {$IFDEF CLX}
      FMoreBtn    :TButton;
    {$ELSE}
      FMoreBtn    :TSpeedButton;
    {$ENDIF}

    procedure   SetColor(AColor:TColor);
  protected
    procedure   Paint; override;
    procedure   RecreatePopupForm;override;
		procedure   ColorBtnClick(Sender: TObject);
		procedure   MoreBtnClick(Sender: TObject);
    procedure   DoClick;override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;

    procedure   ClearDocColor;
    procedure   AddDocColor(AColor:TColor);
  published
    property    Color:TColor                read FColor           write SetColor;
		property    OnGetDocColors:TNotifyEvent read FOnGetDocColors  write FOnGetDocColors;
  end;

// TARGridButton
  TARGridButton = class(TARColorButton)
  private
    FGridPanel  :TPanel;
    FStylePanel :TPanel;
    FGridKind   :TFrameStyle;
    FGridStyle  :TLineStyle;
    FGridWidth  :integer;
    FShowColors :boolean;

    FGridBtns   :TAIList;
    FStyleBtns  :TAIList;

    procedure   SetGridWidth(AGridWidth:integer);
    procedure   SetGridKind(AGridKind:TFrameStyle);
    procedure   SetShowColors(AShowColors:boolean);
    procedure   SetGridStyle(const Value: TLineStyle);
  protected
    procedure   Paint; override;
    procedure   RecreatePopupForm;override;
		procedure   GridBtnClick(Sender: TObject);
		procedure   GridStyleBtnClick(Sender: TObject);
  public
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;
  published
    property    GridWidth:integer     read  FGridWidth  write SetGridWidth;
    property    GridKind:TFrameStyle  read  FGridKind   write SetGridKind;
    property    GridStyle:TLineStyle  read  FGridStyle  write SetGridStyle;
    property    ShowColors:boolean    read  FShowColors write SetShowColors;
  end;

// TARComboButton
  TARComboButton = class(TARButton)
  private
    FListBox    :TListBox;
    FMainPanel  :TPanel;

    function    GetItems:TStrings;
    procedure   SetItems(AItems:TStrings);
    function    GetItemIndex:integer;
    procedure   SetItemIndex(AItemIndex:integer);
    function    GetPopupWidth:integer;
    procedure   SetPopupWidth(APopupWidth:integer);
  protected
    function    GetValue:string;virtual;
    procedure   SetValue(AValue:string);virtual;
    procedure   Paint; override;
    procedure   RecreatePopupForm;override;
    procedure   LBClick(Sender:TObject);virtual;
    procedure   SelfClick(Sender:TObject);virtual;
    procedure   UpdateWidth;override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;

  published
    property    Items:TStrings      read  GetItems      write SetItems;
    property    Value:string        read  GetValue      write SetValue;
    property    ItemIndex:integer   read  GetItemIndex  write SetItemIndex;
    property    PopupWidth:integer  read  GetPopupWidth write SetPopupWidth default 100;
  end;

// TARFontButton
  TARFontButton = class(TARComboButton)
  private
  protected
    procedure   Paint; override;
    {$IFDEF CLX}
      procedure   LBDrawItem(Sender:TObject; Index:integer; Rect:TRect; State:TOwnerDrawState; var Handled:boolean);
    {$ELSE}
      procedure   LBDrawItem(Control:TWinControl; Index:integer; Rect:TRect; State:TOwnerDrawState);
    {$ENDIF}
    procedure   UpdateWidth;override;
  public
    constructor Create(AOwner: TComponent); override;
  published
  end;

// TARComboMoreButton
  TARComboMoreButton = class(TARComboButton)
  private
    FOnMoreBtnClick:TNotifyEvent;
    FMoreBtn    :TSpeedButton;
  protected
		procedure   MoreBtnClick(Sender: TObject);
    procedure   RecreatePopupForm;override;
    procedure   UpdateWidth;override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property    OnMoreBtnClick:TNotifyEvent  read FOnMoreBtnClick write FOnMoreBtnClick;
  end;

// TARComboEditButton
  TARComboEditButton = class(TARComboButton)
  private
		FEditForm   :TForm;
    FEdit       :TEdit;
    FValue      :string;

		FEditFormHiding:boolean;
  protected
    function    GetValue:string;override;
    procedure   SetValue(AValue:string);override;

    procedure   CloseUpEdit(Action:boolean);
    procedure   RecreateEditForm;
    procedure   SelfClick(Sender:TObject);override;
  	procedure   EditFormDeactivate(Sender:TObject);
  	procedure   EditFormKeyPress(Sender:TObject; var Key: Char);
    procedure   PopupEditForm(Sender:TObject);
    procedure   UpdateWidth;override;
  public
    constructor Create(AOwner: TComponent); override;
  published
  end;

procedure   Register;
procedure   CloseupAllPopupForms;

implementation

procedure   Register;
begin
{
  RegisterComponents('ARComponents', [TARGlyphButton]);
  RegisterComponents('ARComponents', [TARColorButton]);
  RegisterComponents('ARComponents', [TARGridButton]);
  RegisterComponents('ARComponents', [TARComboButton]);
  RegisterComponents('ARComponents', [TARFontButton]);
  RegisterComponents('ARComponents', [TARComboMoreButton]);
  RegisterComponents('ARComponents', [TARComboEditButton]);
}
end;

procedure RegisterPopup(AForm:TForm);forward;
procedure RegisterCloseup(AForm:TForm);forward;

var
  {$IFDEF CLX}
    BtnGlyph:QGraphics.TBitmap;
  {$ELSE}
    BtnGlyph:Graphics.TBitmap;
  {$ENDIF}

constructor TARGlyphButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  Width:=23;
  Height:=22;
  {$IFNDEF D3}
    Constraints.MinWidth  :=23;
    Constraints.MinHeight :=22;
    Transparent:=false;
  {$ENDIF}
  Flat:=true;
end;

function    TARGlyphButton.GetImage:TARBitmap;
begin
  if not Assigned(FImage) then begin
    FImage:=TARBitmap.Create;
  end;
  Result:=FImage;
end;

procedure   TARGlyphButton.SetImage(AImage:TARBitmap);
begin
  if not Assigned(FImage) then begin
    FImage:=TARBitmap.Create;
  end;

  FImage.Assign(AImage);
end;

procedure   TARGlyphButton.UpdateWidth;
var w:integer;
begin
  if Assigned(FImage) and (not FImage.Empty) then begin
    w:=Max(23, FImage.Width + 0);
  end else begin
    w:=Max(23, Canvas.TextWidth(FCaption) + 7);
  end;
  if w <> Width then Width:=w;
end;

procedure   TARGlyphButton.SetCaption(ACaption:string);
begin
  FCaption:=ACaption;
  UpdateWidth;
end;

procedure   TARGlyphButton.Paint;
var w, h:integer;
begin
  inherited;

  if Assigned(FImage) and (not FImage.Empty) then begin
    w:=(ClientHeight - FImage.Height) div 2 + 1;
    Canvas.Draw(w, w, FImage);
  end else begin
    if Length(Caption) > 0 then begin
      h:=Canvas.TextHeight(Caption);
      Canvas.TextOut((ClientHeight - h) div 2 - 1, (ClientHeight - h) div 2 - 1, Caption);
    end;
  end;
end;


//////////////////////////////////////////////////////////////////////
//  TARButton
constructor TARButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

	FPopupForm             	  :=TForm.Create(Self);
	FPopupForm.FormStyle   	  :=fsStayOnTop;
	FPopupForm.BorderIcons 	  :=[];
	FPopupForm.Parent         :=Application.MainForm;
  {$IFDEF CLX}
	  FPopupForm.BorderStyle  :=fbsNone;
  	FPopupForm.Color       	:=clButton;
  {$ELSE}
	  FPopupForm.BorderStyle  :=bsNone;
  	FPopupForm.Color       	:=clRed;
  {$ENDIF}
	FPopupForm.OnDeactivate	  :=PopupDeactivate;
	FPopupForm.KeyPreview  	  :=true;
	FPopupForm.OnKeyPress     :=PopupKeyPress;
	FPopupForm.OnShow         :=OnShowPopupForm;
	FPopupForm.OnClose        :=OnClosePopupForm;
end;

destructor  TARButton.Destroy;
begin
  inherited Destroy;
end;

procedure   TARButton.OnShowPopupForm(Sender:TObject);
begin
  RegisterPopup(TForm(Sender));
end;

procedure   TARButton.OnClosePopupForm(Sender:TObject; var Action:TCloseAction);
begin
  RegisterCloseup(TForm(Sender));
  Action:=caHide;
end;

procedure   TARButton.UpdateWidth;
begin
  inherited;
  Width:=Width + DropDownWidth;
end;

{$IFDEF CLX}
procedure   TARButton.MouseEnter(AControl: TControl);
begin
  inherited MouseEnter(AControl);

  if Flat and not MouseInControl and Enabled and (GetCaptureControl = nil) then begin
    MouseInControl:=true;
    Repaint;
  end;
end;

procedure   TARButton.MouseLeave(AControl: TControl);
begin
  inherited MouseLeave(AControl);

  if Flat and MouseInControl and Enabled and not Dragging then begin
    MouseInControl:=false;
    Invalidate;
  end;
end;
{$ENDIF}

procedure   TARButton.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if X < (ClientWidth - DropDownWidth) then begin
    inherited;
  end else begin
    Popup(Self);
  end;
end;

procedure   TARButton.SetChecked(AChecked:boolean);
begin
  FChecked:=AChecked;
  Invalidate;
end;

procedure   TARButton.Loaded;
begin
  inherited;
  FLoaded:=true;
end;

procedure   TARButton.DoClick;
begin
  if Assigned(OnClick) then OnClick(Self);
end;

procedure   TARButton.DoChange;
begin
  if ComponentState * [csLoading, csUpdating] = [] then begin
    if Assigned(OnChange) then OnChange(Self);
  end;
end;

procedure   TARButton.PopupDeactivate(Sender: TObject);
begin
	if not FPopupFormHiding then begin
		CloseUp(false);
	end;
end;

procedure   TARButton.CloseUp(Action:boolean);
begin
	FPopupFormHiding:=true;
  	FPopupForm.Close;
	FPopupFormHiding:=false;

  Down:=false;

	if Action then begin
		DoClick;
	end;
end;

procedure   TARButton.PopupKeyPress(Sender:TObject; var Key: Char);
begin
  if Key = #$1b then begin
    CloseUp(false);
    Key:=#$0;
  end;
  inherited;
end;

procedure   TARButton.RecreatePopupForm;
begin
end;

procedure   TARButton.Popup(Sender:Tobject);
var P:TPoint;
		Y:integer;
begin
	RecreatePopupForm;

	P := Parent.ClientToScreen(Point(0, 0));
	P.X:=P.X + Left;
	Y := P.Y + Top + Height;
	if Y + FPopupForm.Height > Screen.Height then Y := P.Y - FPopupForm.Height;
	FPopupForm.SetBounds(P.X, Y, FPopupForm.Width, FPopupForm.Height);

	FPopupForm.Show;
	FPopupForm.SetFocus;
end;

procedure   TARButton.Paint;
var r:TRect;
begin
  inherited;
  if not BtnGlyph.Empty then begin
    Canvas.Draw(ClientWidth - DropDownWidth + 1, (ClientHeight - BtnGlyph.Height) div 2 - 1, BtnGlyph);
  end;

  if MouseInControl then begin
    r:=Rect(0, 0, ClientWidth - DropDownWidth - 0, ClientHeight);
    {$IFDEF CLX}
      DrawEdge(Canvas, r, esRaised, esNone, [ebRight]);
    {$ELSE}
      DrawEdge(Canvas.Handle, r, BDR_RAISEDINNER, BF_RIGHT);
    {$ENDIF}

    r:=Rect(ClientWidth - DropDownWidth, 0, ClientWidth, ClientHeight);
    {$IFDEF CLX}
      DrawEdge(Canvas, r, esRaised, esNone, [ebLeft, ebTop, ebRight, ebBottom]);
    {$ELSE}
      DrawEdge(Canvas.Handle, r, BDR_RAISEDINNER, BF_RECT);
    {$ENDIF}
  end;
end;
//////////////////////////////////////////////////////////////////////
type
  TColorBtn = class(TSpeedButton)
  protected
    procedure Paint;override;
  public
    Color       :TColor;
  end;

procedure     TColorBtn.Paint;
begin
  inherited;
  Canvas.Pen.Color   :=Color and TColor($ff7f7f7f);
  Canvas.Brush.Color :=Color;
  {$IFDEF D3}
    Canvas.Rectangle((Width - 13) div 2, (Height - 13) div 2, (Width - 13) div 2 + 13, (Height - 13) div 2 + 13);
  {$ELSE}
    Canvas.Rectangle(Rect((Width - 13) div 2, (Height - 13) div 2, (Width - 13) div 2 + 13, (Height - 13) div 2 + 13));
  {$ENDIF}
end;

//////////////////////////////////////////////////////////////////////
var
  StdColors:array[0..23] of TColor = (
    clBlack,    clWhite,    clMaroon,   clGreen,    clOlive,    clNavy,     clPurple,   clTeal,
    clGray,     clSilver,   clRed,      clLime,     clYellow,   clBlue,     clFuchsia,  clAqua,
    $00404040,  $00E0E0E0,  $00C0C0FF,  $00C0FFC0,  $00C0FFFF,  $00FFC0C0,  $00FFC0FF,  $00FFFFC0
  );

// TARColorButton
constructor TARColorButton.Create(AOwner: TComponent);
var i:integer;
    sb:TColorBtn;
    lb:TLabel;
begin
  inherited Create(AOwner);

  FDocColors  :=TAIList.Create('TARColorButton.FDocColors');
  FDocBtns    :=TAIList.Create('TARColorButton.FDocBtns');
  FStdBtns    :=TAIList.Create('TARColorButton.FStdBtns');

  FStdPanel       :=TPanel.Create(Self);
  FStdPanel.Parent:=FPopupForm;
  FStdPanel.Align :=alTop;
  FStdPanel.Height:=89;

  lb:=TLabel.Create(Self);
  lb.Parent :=FStdPanel;
  lb.Caption:='Standard Colors';
  lb.Left   :=8;
  lb.Top    :=8;

  for i:=0 to High(StdColors) do begin
    sb:=TColorBtn.Create(Self);
    sb.Parent:=FStdPanel;
    sb.SetBounds(8 + (i mod 8) * 19, 24 + (i div 8) * 19, 19, 19);
    sb.Color:=StdColors[i];
    sb.Tag:=StdColors[i];

    sb.Flat:=true;
    sb.GroupIndex:=3;
    sb.AllowAllUp:=true;
    FStdBtns.Add(sb);
    sb.OnClick:=ColorBtnClick;
  end;

  FDocPanel       :=TPanel.Create(Self);
  FDocPanel.Parent:=FPopupForm;
  FDocPanel.Align :=alTop;

  lb:=TLabel.Create(Self);
  lb.Parent :=FDocPanel;
  lb.Caption:='Document''s Colors';
  lb.Left   :=8;
  lb.Top    :=8;

  FDocPanel.Height:=23;

  FMorePanel       :=TPanel.Create(Self);
  FMorePanel.Parent:=FPopupForm;
  FMorePanel.Align :=alClient;

  {$IFDEF CLX}
    FMoreBtn:=TButton.Create(Self);
  {$ELSE}
    FMoreBtn:=TSpeedButton.Create(Self);
    {$IFNDEF D3}
      FMoreBtn.Transparent:=false;
    {$ENDIF}
    FMoreBtn.Flat:=true;
  {$ENDIF}
  FMoreBtn.Parent:=FMorePanel;
  FMoreBtn.SetBounds(8, 6, 153, 22);
  FMoreBtn.Caption:='More Colors...';
  FMoreBtn.OnClick:=MoreBtnClick;

  FMorePanel.Height:=MorePanelHeight;

  Width:=37;
end;

procedure   TARColorButton.AddDocColor(AColor:TColor);
var sb:TColorBtn;
    i:integer;
begin
  if (AColor <> clBlack) and (AColor <> clWhite) then begin
    if FDocColors.IndexOf(pointer(AColor)) = -1 then begin
      FDocColors.Add(pointer(AColor));
      sb:=TColorBtn.Create(Self);
      FDocBtns.Add(sb);
      i:=FDocBtns.Count - 1;
      sb.Parent:=FDocPanel;
      sb.SetBounds(8 + (i mod 8) * 19, 24 + (i div 8) * 19, 19, 19);
      sb.Tag:=AColor;
      sb.Color:=AColor;
      sb.OnClick:=ColorBtnClick;
      sb.Flat:=true;
    end;
  end;
end;

destructor  TARColorButton.Destroy;
begin
//sa: 11.06.2001
  FStdBtns.Free;
//--

  FDocColors.Free;
  FDocBtns.Free;

  inherited Destroy;
end;

procedure   TARColorButton.ColorBtnClick(Sender: TObject);
begin
  Color:=TColorBtn(Sender).Color;
  CloseUp(false);
  DoChange;
end;

procedure   TARColorButton.SetColor(AColor:TColor);
begin
  FColor:=AColor;
  Invalidate;
end;

procedure   TARColorButton.Paint;
begin
  inherited;

  if Color <> clNone then begin
    Canvas.Brush.Color:=Color;
    Canvas.Pen.Color  :=Color;
  end else begin
    Canvas.Brush.Color:=clBtnFace;
    Canvas.Pen.Color  :=clBlack;
  end;
  Canvas.Rectangle(2, Height - 6, Height - 2, Height - 2);
end;

procedure   TARColorButton.ClearDocColor;
var i:integer;
begin
  FDocColors.Clear;

  for i:=0 to FDocBtns.Count - 1 do begin
    TColorBtn(FDocBtns[i]).Free;
  end;
  FDocBtns.Clear();
end;

procedure   TARColorButton.RecreatePopupForm;
var i:integer;
begin
  inherited;
	if Assigned(OnGetDocColors) then OnGetDocColors(Self);

  FPopupForm.Width:=170;

  FDocPanel.Visible :=FDocColors.Count > 0;
  FDocPanel.Height  :=(FDocColors.Count div 8 + 1) * 19 + 33;

  FPopupForm.Height:= FStdPanel.Height + MorePanelHeight;
  if FDocPanel.Visible then begin
    FPopupForm.Height:=FPopupForm.Height + FDocPanel.Height;
  end;

  for i:=0 to FStdBtns.Count - 1 do begin
    if TColorBtn(FStdBtns[i]).Color = Color then begin
      TColorBtn(FStdBtns[i]).Down:=true;
      break;
    end;
  end;

  for i:=0 to FDocBtns.Count - 1 do begin
    if TColorBtn(FDocBtns[i]).Color = Color then begin
      TColorBtn(FDocBtns[i]).Down:=true;
      break;
    end;
  end;
end;

procedure   TARColorButton.MoreBtnClick(Sender: TObject);
var cd:TColorDialog;
begin
  CloseUp(false);

  cd:=TColorDialog.Create(Self);
  {$IFDEF CLX}
  {$ELSE}
    cd.Options:=[cdFullOpen];
  {$ENDIF}
  cd.Color:=Color;
  if cd.Execute then begin
    Color:=cd.Color;
    DoChange;
  end;
end;

procedure   TARColorButton.DoClick;
begin
  if Color <> clNone then begin
    inherited;
  end;
end;

////////////////////////////////////////////////////////////////////////////////////////////////
// TARGridButton
procedure   TARGridButton.SetGridWidth(AGridWidth:integer);
begin
  if AGridWidth = 0 then begin
    FGridStyle:=cbsSolid;
  end;

  FGridWidth:=AGridWidth;

  Invalidate;
end;

procedure TARGridButton.SetGridStyle(const Value: TLineStyle);
begin
  if Value <> cbsSolid then begin
    GridWidth:=CellBorderWidths[Value];
  end;
  FGridStyle:=Value;
  Invalidate;
end;

procedure   TARGridButton.SetGridKind(AGridKind:TFrameStyle);
begin
  FGridKind:=AGridKind;
  Invalidate;
end;

type
  TGridBtn = class(TSpeedButton)
  protected
    procedure Paint;override;
  public
    GridWidth :integer;
  end;

  TGridStyleBtn = class(TSpeedButton)
  protected
    procedure Paint;override;
  public
    GridStyle :TLineStyle;
  end;

procedure     TGridBtn.Paint;
var x, y:integer;
begin
  inherited;
  x:=(Width - 70) div 2;
  y:=(Height - 11) div 2;
  Canvas.TextOut(x + 2, y + (11 - Canvas.TextHeight('0')) div 2, IntToStr(GridWidth));
  if GridWidth > 0 then begin
    Canvas.Brush.Style:=bsSolid;
    Canvas.Brush.Color:=clBlack;
    Canvas.FillRect(Rect(x + Canvas.TextWidth('0') + 5, (Height - GridWidth) div 2, Width - 5, (Height - GridWidth) div 2 + GridWidth));
  end;
end;

constructor TARGridButton.Create(AOwner: TComponent);
var i:integer;
    sb:TGridBtn;
    gsb:TGridStyleBtn;
    lb:TLabel;
    si:TLineStyle;
begin
  inherited;

  FGridBtns :=TAIList.Create('TARGridButton.FGridBtns:');
  FStyleBtns:=TAIList.Create('TARGridButton.FStyleBtns:');

  FGridPanel       :=TPanel.Create(Self);
  FGridPanel.Parent:=FPopupForm;
  FGridPanel.Align :=alTop;
  FGridPanel.Height:=109;

  lb:=TLabel.Create(Self);
  lb.Parent :=FGridPanel;
  lb.Caption:='Width (px)';
  lb.Left   :=8;
  lb.Top    :=8;

  for i:=0 to 7 do begin
    sb:=TGridBtn.Create(Self);
    sb.Parent:=FGridPanel;
    sb.SetBounds(8 + (i mod 2) * 76, 24 + (i div 2) * 19, 76, 19);
    sb.GridWidth:=i;
    sb.Tag:=i;

    sb.OnClick:=GridBtnClick;
    sb.Flat:=true;
    sb.GroupIndex:=3;
    sb.AllowAllUp:=true;

    FGridBtns.Add(sb);
  end;

  FStylePanel       :=TPanel.Create(Self);
  FStylePanel.Parent:=FPopupForm;
  FStylePanel.Align :=alTop;
  FStylePanel.Height:=109;

  lb:=TLabel.Create(Self);
  lb.Parent :=FStylePanel;
  lb.Caption:='Border style';
  lb.Left   :=8;
  lb.Top    :=8;

  for si:=Low(TLineStyle) to High(TLineStyle) do begin
    gsb:=TGridStyleBtn.Create(Self);
    gsb.Parent:=FStylePanel;
    gsb.SetBounds(8 + (Ord(si) mod 2) * 76, 24 + (Ord(si) div 2) * 19, 76, 19);
    gsb.GridStyle:=si;
    gsb.Tag:=Ord(si);

    gsb.OnClick:=GridStyleBtnClick;
    gsb.Flat:=true;
    gsb.GroupIndex:=4;
    gsb.AllowAllUp:=true;

    FStyleBtns.Add(gsb);
  end;

end;

destructor  TARGridButton.Destroy;
begin
  FGridBtns.Free;
  FStyleBtns.Free;

  inherited;
end;

procedure   TARGridButton.GridBtnClick(Sender: TObject);
begin
  GridStyle:=cbsSolid;
  GridWidth:=TGridBtn(Sender).GridWidth;
  CloseUp(false);
  DoChange;
end;

procedure   TARGridButton.RecreatePopupForm;
begin
  inherited;
  FPopupForm.Height:= FStdPanel.Height  * integer(FStdPanel.Visible) +
                      FDocPanel.Height  * integer(FDocPanel.Visible) +
                      MorePanelHeight * integer(FMorePanel.Visible) +
                      FGridPanel.Height + FStylePanel.Height;

  if (GridWidth >= 0) and (GridWidth < FGridBtns.Count) then begin
    TGridBtn(FGridBtns[GridWidth]).Down:=true;
  end;

  TGridStyleBtn(FStyleBtns[Ord(GridStyle)]).Down:=true;
end;

procedure   TARGridButton.SetShowColors(AShowColors:boolean);
begin
  FShowColors:=AShowColors;

  FStdPanel.Visible :=FShowColors;
  FDocPanel.Visible :=FShowColors;
  FMorePanel.Visible:=FShowColors;
  RecreatePopupForm;
end;

procedure   TARGridButton.Paint;
var R: TRect;
begin
  inherited;

  if not Checked then begin
    Canvas.Brush.Style:=bsSolid;
    Canvas.Brush.Color:=clBtnFace;
    Canvas.FillRect(Rect(1, 1, Width - DropDownWidth - 2, Height - 1));
  end else begin
    Canvas.Brush.Style:=bsSolid;
    Canvas.Brush.Color:=$00F0F0F0;
    Canvas.FillRect(Rect(1, 1, Width - DropDownWidth - 2, Height - 1));

    R := Rect(-1, -1, Width - DropDownWidth + 0, Height);
    {$IFDEF CLX}
      DrawEdge(Canvas, R, esLowered, esLowered, [ebTop, ebRight, ebBottom]);
      DrawEdge(Canvas, R, esRaised,  esLowered, [ebLeft]);
    {$ELSE}
      DrawEdge(Canvas.Handle, R, BDR_SUNKEN, BF_TOP or BF_RIGHT or BF_BOTTOM);
      DrawEdge(Canvas.Handle, R, EDGE_ETCHED, BF_LEFT);
    {$ENDIF}
  end;

  if Assigned(FImage) then begin
    Canvas.Draw((Height - FImage.Height) div 2 - 1, (Height - FImage.Height) div 2 - 1, FImage);
  end;

  if GridWidth > 0 then begin
    Canvas.Brush.Style:=bsSolid;
    Canvas.Brush.Color:=Color;
    Canvas.Pen.Color  :=Color;

    R:=Rect(4, 4, Height-5, Height-5);

    if frLeft   in GridKind then begin
      DrawBorder(Canvas, R, GridStyle, frLeft, Color, clBtnFace, GridWidth, Ord(frTop in GridKind) * GridWidth, Ord(frBottom in GridKind) * GridWidth);
    end;

    if frRight  in GridKind then begin
      DrawBorder(Canvas, R, GridStyle, frRight, Color, clBtnFace, GridWidth, Ord(frTop in GridKind) * GridWidth, Ord(frBottom in GridKind) * GridWidth);
    end;

    if frTop    in GridKind then begin
      DrawBorder(Canvas, R, GridStyle, frTop, Color, clBtnFace, GridWidth, Ord(frLeft in GridKind) * GridWidth, Ord(frRight in GridKind) * GridWidth);
    end;

    if frBottom in GridKind then begin
      DrawBorder(Canvas, R, GridStyle, frBottom, Color, clBtnFace, GridWidth, Ord(frLeft in GridKind) * GridWidth, Ord(frRight  in GridKind) * GridWidth);
    end;

  end else begin
    Canvas.Brush.Color:=clBtnFace;
    Canvas.Pen.Color  :=clBlack;
    if [frLeft, frRight, frTop, frBottom] = GridKind then begin
      {$IFDEF D3}
        Canvas.Rectangle(4,    4,    Height-4,   Height-4);
        Canvas.Rectangle(4+3,  4+3,  Height-4-3, Height-4-3);
      {$ELSE}
        Canvas.Rectangle(Rect(4,    4,    Height-4,   Height-4));
        Canvas.Rectangle(Rect(4+3,  4+3,  Height-4-3, Height-4-3));
      {$ENDIF}
    end else begin
      {$IFDEF D3}
        if frLeft   in GridKind then Canvas.Rectangle(4,          4,          4+4,      Height-5);
        if frRight  in GridKind then Canvas.Rectangle(Height-5-4, 4,          Height-5, Height-5);
        if frTop    in GridKind then Canvas.Rectangle(4,          4,          Height-5, 4+4);
        if frBottom in GridKind then Canvas.Rectangle(4,          Height-5-4, Height-5, Height-5);
      {$ELSE}
        if frLeft   in GridKind then Canvas.Rectangle(4,          4,          4+4,      Height-5);
        if frRight  in GridKind then Canvas.Rectangle(Height-5-4, 4,          Height-5, Height-5);
        if frTop    in GridKind then Canvas.Rectangle(4,          4,          Height-5, 4+4);
        if frBottom in GridKind then Canvas.Rectangle(4,          Height-5-4, Height-5, Height-5);
      {$ENDIF}
    end;
  end;
end;

// TARComboButton
constructor TARComboButton.Create(AOwner: TComponent);
begin
  inherited;

  Self.OnClick:=SelfClick;

  FMainPanel:=TPanel.Create(Self);
  FMainPanel.Parent     :=FPopupForm;
  FMainPanel.Align      :=alClient;
  FMainPanel.BevelInner :=bvLowered;//bvRaised;
  FMainPanel.BevelOuter :=bvRaised;

  FListBox:=TListBox.Create(Self);
  FListBox.Parent     :=FMainPanel;
  FListBox.BorderStyle:=bsNone;
  FListBox.Align      :=alClient;
  FListBox.OnClick    :=LBClick;

  Width:=50;
  {$IFNDEF D3}
    Constraints.MinWidth:=50;
  {$ENDIF}
end;

procedure   TARComboButton.UpdateWidth;
begin
  inherited;
  Width:=50;
end;

destructor  TARComboButton.Destroy;
begin
  FListBox.Free;
  FMainPanel.Free;

  inherited;
end;

procedure   TARComboButton.SelfClick(Sender:TObject);
begin
  Popup(Self);
end;

function    TARComboButton.GetItems:TStrings;
begin
  Result:=FListBox.Items;
end;

procedure   TARComboButton.SetItems(AItems:TStrings);
begin
  FListBox.Items.Assign(AItems);
end;

function    TARComboButton.GetValue:string;
begin
  if FListBox.ItemIndex >= 0 then begin
    Result:=FListBox.Items[FListBox.ItemIndex];
  end else begin
    Result:='';
  end;
end;

procedure   TARComboButton.SetValue(AValue:string);
begin
  FListBox.ItemIndex:=FListBox.Items.IndexOf(AValue);
  Invalidate;
end;

function    TARComboButton.GetItemIndex:integer;
begin
  Result:=FListBox.ItemIndex;
end;

procedure   TARComboButton.SetItemIndex(AItemIndex:integer);
begin
  FListBox.ItemIndex:=AItemIndex;
end;

function    TARComboButton.GetPopupWidth:integer;
begin
  Result:=FPopupForm.Width;
end;

procedure   TARComboButton.SetPopupWidth(APopupWidth:integer);
begin
  FPopupForm.Width:=APopupWidth;
end;

procedure   TARComboButton.RecreatePopupForm;
begin
  inherited;

  FPopupForm.Width:=Width;
end;

procedure   TARComboButton.LBClick(Sender:TObject);
begin
  CloseUp(false);
  if FListBox.ItemIndex >= 0 then begin
    Value:=FListBox.Items[FListBox.ItemIndex];
    DoChange;
    Invalidate;
  end;
end;

procedure   TARComboButton.Paint;
begin
  inherited;

  Canvas.Brush.Style:=bsSolid;
  Canvas.Brush.Color:=clWhite;
  Canvas.FillRect(Rect(2, 2, Width - DropDownWidth - 4, Height - 3));

  Canvas.TextRect(Rect(2, 2, Width - DropDownWidth - 4, Height - 3),
                  4,
                  (Height - 3 - Canvas.TextHeight(Value)) div 2,
                  Value);
end;

// TARFontButton
constructor TARFontButton.Create(AOwner: TComponent);
begin
  inherited;

  FListBox.Style      :=lbOwnerDrawFixed;
  FListBox.BorderStyle:=bsNone;
  FListBox.ItemIndex  :=0;
  FListBox.OnDrawItem :=LBDrawItem;
  FListBox.Items.Assign(Screen.Fonts);
  Value:='Arial';

  Width:=180;
  {$IFNDEF D3}
    Constraints.MinWidth:=180;
  {$ENDIF}
end;

procedure   TARFontButton.UpdateWidth;
begin
  inherited;
  Width:=180;
end;

{$IFDEF CLX}
procedure   TARFontButton.LBDrawItem(Sender:TObject; Index:integer; Rect:TRect; State:TOwnerDrawState; var Handled:boolean);
begin
  Handled:=true;
  if Index <> -1 then begin
    TListBox(Sender).Canvas.Brush.Color:=clWhite;
    TListBox(Sender).Canvas.FillRect(Rect);

    TListBox(Sender).Canvas.Font.Size:=10;
    TListBox(Sender).Canvas.Font.Name:=FListBox.Items[Index];
    TListBox(Sender).Canvas.TextRect(Rect, Rect.Left + 5, Rect.Top, FListBox.Items[Index]);
    Handled:=true;
  end;
end;
{$ELSE}
procedure   TARFontButton.LBDrawItem(Control:TWinControl; Index:integer; Rect:TRect; State:TOwnerDrawState);
begin
  if Index <> -1 then begin
    TListBox(Control).Canvas.Font.Size:=10;
    TListBox(Control).Canvas.Font.Name:=FListBox.Items[Index];
    TListBox(Control).Canvas.TextRect(Rect, Rect.Left + 5, Rect.Top, FListBox.Items[Index]);
  end;
end;
{$ENDIF}

procedure   TARFontButton.Paint;
begin
  inherited;

  Canvas.Brush.Color:=clWhite;
  Canvas.FillRect(Rect(2, 2, Width - DropDownWidth - 4, Height - 3));

  Canvas.Font.Name:=Value;
  Canvas.Font.Size:=10;
  Canvas.TextRect(Rect(2, 2, Width - DropDownWidth - 4, Height - 3),
                  4,
                  (Height - 3 - Canvas.TextHeight(Value)) div 2,
                  Value);
end;

// TARComboMoreButton
constructor TARComboMoreButton.Create(AOwner: TComponent);
begin
  inherited;

  FMoreBtn            :=TSpeedButton.Create(Self);
  FMoreBtn.Parent     :=FMainPanel;
  FMoreBtn.Caption    :='Define...';
  FMoreBtn.OnClick    :=MoreBtnClick;
  {$IFNDEF D3}
    FMoreBtn.Transparent:=false;
  {$ENDIF}
  FMoreBtn.Flat       :=true;

  Width:=180;
  {$IFNDEF D3}
    Constraints.MinWidth:=180;
  {$ENDIF}
end;

procedure   TARComboMoreButton.UpdateWidth;
begin
  inherited;
  Width:=180;
end;

procedure   TARComboMoreButton.RecreatePopupForm;
begin
  inherited;

  FListBox.Align :=alTop;
  FListBox.Height:=FMainPanel.Height - 30;

  FMoreBtn.SetBounds(3, FMainPanel.Height - 26, FMainPanel.Width - 6, 22);
end;

procedure   TARComboMoreButton.MoreBtnClick(Sender: TObject);
begin
  CloseUp(false);
  if Assigned(FOnMoreBtnClick) then FOnMoreBtnClick(Self);
end;

// TARComboEditButton
constructor TARComboEditButton.Create(AOwner: TComponent);
begin
  inherited;

	FEditForm              :=TForm.Create(Self);
	FEditForm.FormStyle    :=fsStayOnTop;
	FEditForm.Parent   	   :=Application.MainForm;
	FEditForm.BorderIcons  :=[];
  {$IFDEF CLX}
    FEditForm.BorderStyle  :=fbsNone;
  {$ELSE}
    FEditForm.BorderStyle  :=bsNone;
  {$ENDIF}
	FEditForm.Color        :=clGreen;
	FEditForm.OnDeactivate :=EditFormDeactivate;
	FEditForm.KeyPreview   :=true;
	FEditForm.OnKeyPress   :=EditFormKeyPress;

  FEdit:=TEdit.Create(Self);
  FEdit.Parent      :=FEditForm;
  FEdit.BorderStyle :=bsNone;

  Width:=50;
  {$IFNDEF D3}
    Constraints.MinWidth:=50;
  {$ENDIF}
end;

procedure   TARComboEditButton.UpdateWidth;
begin
  inherited;
  Width:=50;
end;

procedure   TARComboEditButton.EditFormDeactivate(Sender: TObject);
begin
	if not FEditFormHiding then begin
		CloseUpEdit(true);
	end;
end;

procedure   TARComboEditButton.SelfClick(Sender:TObject);
begin
	if not FPopupForm.Visible then begin
    PopupEditForm(Self);
  end;
end;

procedure   TARComboEditButton.CloseUpEdit(Action:boolean);
begin
	FEditFormHiding:=true;
  	FEditForm.Close;
	FEditFormHiding:=false;

  Down:=false;

	if Action then begin
    Value:=FEdit.Text;
		DoChange;
	end;
end;

procedure   TARComboEditButton.EditFormKeyPress(Sender:TObject; var Key: Char);
begin
  if Key = #$1b then begin
    CloseUpEdit(false);
    Key:=#$0;
  end else
  if Key = #$0d then begin
    CloseUpEdit(true);
    Key:=#$0;
  end;

  inherited;
end;

procedure   TARComboEditButton.RecreateEditForm;
begin
  inherited;
end;

procedure   TARComboEditButton.PopupEditForm(Sender:TObject);
var P:TPoint;
begin
	P:=ClientToScreen(Point(0, 0));
  FEditForm.SetBounds(P.X + 2, P.Y + 2, Width - DropDownWidth - 4, Height - 4);

  FEdit.SetBounds(0, 0, FEditForm.Width, FEditForm.Height);
  FEdit.Text:=Value;

	FEditForm.Show;
	FEditForm.SetFocus;
end;

function    TARComboEditButton.GetValue:string;
begin
  Result:=FValue;
end;

procedure   TARComboEditButton.SetValue(AValue:string);
begin
  inherited SetValue(AValue);
  FValue:=AValue;
//  Invalidate;
end;

var CurPopupForm:TForm;

procedure RegisterPopup(AForm:TForm);
begin
  if Assigned(CurPopupForm) then begin
    CurPopupForm.Close;
    CurPopupForm:=nil;
  end;

  CurPopupForm:=AForm;
end;

procedure RegisterCloseup(AForm:TForm);
begin
  if CurPopupForm = AForm then begin
    CurPopupForm:=nil;
  end;
end;

procedure CloseupAllPopupForms;
begin
  if Assigned(CurPopupForm) then begin
    CurPopupForm.Close;
    CurPopupForm:=nil;
  end;
end;

{ TGridStyleBtn }

procedure TGridStyleBtn.Paint;
var x, y:integer;
begin
  inherited;

  x:=(Width - 70) div 2;
  y:=(Height - 11) div 2;
//  Canvas.TextOut(x + 2, y + (11 - Canvas.TextHeight('0')) div 2, IntToStr(GridWidth));

  Canvas.Pen.Color:=clBlack;

  case GridStyle of
    cbsSolid      :
      begin
        Canvas.Pen.Style:=psSolid;
        Canvas.MoveTo(5,          (Height - CellBorderWidths[GridStyle]) div 2);
        Canvas.LineTo(Width - 5,  (Height - CellBorderWidths[GridStyle]) div 2);
      end;
    cbsDash       :
      begin
        Canvas.Pen.Style:=psDash;
        Canvas.MoveTo(5,          (Height - CellBorderWidths[GridStyle]) div 2);
        Canvas.LineTo(Width - 5,  (Height - CellBorderWidths[GridStyle]) div 2);
      end;
    cbsDot        :
      begin
        Canvas.Pen.Style:=psDot;
        Canvas.MoveTo(5,          (Height - CellBorderWidths[GridStyle]) div 2);
        Canvas.LineTo(Width - 5,  (Height - CellBorderWidths[GridStyle]) div 2);
      end;
    cbsDashDot    :
      begin
        Canvas.Pen.Style:=psDashDot;
        Canvas.MoveTo(5,          (Height - CellBorderWidths[GridStyle]) div 2);
        Canvas.LineTo(Width - 5,  (Height - CellBorderWidths[GridStyle]) div 2);
      end;
    cbsDashDotDot :
      begin
        Canvas.Pen.Style:=psDashDotDot;
        Canvas.MoveTo(5,          (Height - CellBorderWidths[GridStyle]) div 2);
        Canvas.LineTo(Width - 5,  (Height - CellBorderWidths[GridStyle]) div 2);
      end;
    cbsDouble11   :
      begin
        Canvas.Pen.Style:=psSolid;

        Canvas.MoveTo(5,          (Height - CellBorderWidths[GridStyle]) div 2);
        Canvas.LineTo(Width - 5,  (Height - CellBorderWidths[GridStyle]) div 2);

        Canvas.MoveTo(5,          (Height - CellBorderWidths[GridStyle]) div 2 + 2);
        Canvas.LineTo(Width - 5,  (Height - CellBorderWidths[GridStyle]) div 2 + 2);
      end;
    cbsDouble21   :
      begin
        Canvas.Pen.Style:=psSolid;

        Canvas.MoveTo(5,          (Height - CellBorderWidths[GridStyle]) div 2);
        Canvas.LineTo(Width - 5,  (Height - CellBorderWidths[GridStyle]) div 2);
        Canvas.MoveTo(5,          (Height - CellBorderWidths[GridStyle]) div 2 + 1);
        Canvas.LineTo(Width - 5,  (Height - CellBorderWidths[GridStyle]) div 2 + 1);

        Canvas.MoveTo(5,          (Height - CellBorderWidths[GridStyle]) div 2 + 3);
        Canvas.LineTo(Width - 5,  (Height - CellBorderWidths[GridStyle]) div 2 + 3);
      end;
    cbsDouble12   :
      begin
        Canvas.Pen.Style:=psSolid;

        Canvas.MoveTo(5,          (Height - CellBorderWidths[GridStyle]) div 2);
        Canvas.LineTo(Width - 5,  (Height - CellBorderWidths[GridStyle]) div 2);

        Canvas.MoveTo(5,          (Height - CellBorderWidths[GridStyle]) div 2 + 2);
        Canvas.LineTo(Width - 5,  (Height - CellBorderWidths[GridStyle]) div 2 + 2);
        Canvas.MoveTo(5,          (Height - CellBorderWidths[GridStyle]) div 2 + 3);
        Canvas.LineTo(Width - 5,  (Height - CellBorderWidths[GridStyle]) div 2 + 3);
      end;
  end;
end;

procedure TARGridButton.GridStyleBtnClick(Sender: TObject);
begin

  GridStyle:=TGridStyleBtn(Sender).GridStyle;
  CloseUp(false);
  DoChange;
end;

//sa: 11.06.2001
destructor TARGlyphButton.Destroy;
begin
  if Assigned(FImage) then FImage.Free;

  inherited Destroy;
end;
//--

initialization
  {$IFDEF CLX}
    BtnGlyph                    :=QGraphics.TBitmap.Create;
  {$ELSE}
    BtnGlyph                    :=Graphics.TBitmap.Create;
  {$ENDIF}

  {$IFDEF CLX}
    BtnGlyph.Width            :=7;
    BtnGlyph.Height           :=7;

    BtnGlyph.Canvas.Rectangle (2, 2, 4, 4);
    BtnGlyph.Canvas.MoveTo    (0, 2);
    BtnGlyph.Canvas.LineTo    (6, 2);
    BtnGlyph.Canvas.LineTo    (3, 5);
    BtnGlyph.Canvas.LineTo    (0, 2);
    BtnGlyph.Canvas.LineTo    (3, 3);
  {$ELSE}
    BtnGlyph.Handle           :=LoadBitmap(0, PChar(OBM_COMBO));
  {$ENDIF}
  BtnGlyph.Transparent        :=true;
  {$IFNDEF D3}
    BtnGlyph.TransparentMode    :=tmAuto;
  {$ENDIF}
finalization
  BtnGlyph.Free;
end.
