unit ARCellProp;
//************************************************************//
//                         AND Report                        //
//  Eugene Hrulev (mrHru) Laxian Key, Omsk, Russia 1999,2001 //
//  Oleg Melnikov (muler) Laxian Key, Omsk, Russia 1999,2001 //
//************************************************************//

{$INCLUDE DEFINES.INC}

interface

uses
  SysUtils, Classes,
  {$IFDEF CLX}
    QForms, QStdCtrls, QButtons, QExtCtrls, QControls, QDialogs, QGraphics,
  {$ELSE}
    Forms, StdCtrls, Buttons, ExtCtrls, Controls, Dialogs, Graphics,

    Windows, Messages, ComCtrls, ExtDlgs,
  {$ENDIF}

  ARResStrings, AREditRep;

type
  TARCellPropDlg = class(TForm)
    Image: TGroupBox;
    Path: TEdit;
    LinkToFile: TCheckBox;
    Browse: TBitBtn;
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    ApplyBtn: TBitBtn;
    TextOrientation: TRadioGroup;
    CellTextRG: TGroupBox;
    CellText: TMemo;
    WidthGroupBox: TGroupBox;
    WidthInPercent: TCheckBox;
    HeightRG: TGroupBox;
    eHeight: TEdit;
    WidthRG: TGroupBox;
    eWidth: TEdit;
    procedure FormShow(Sender: TObject);
    procedure CancelBtnClick(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
    procedure ApplyBtnClick(Sender: TObject);
    procedure BrowseClick(Sender: TObject);
    procedure SomeChange(Sender: TObject);
    procedure TextOrientationChanging(Sender: TObject; var AllowChange: Boolean);
    procedure TextOrientationRGClick(Sender: TObject);
    procedure PathChange(Sender: TObject);
    procedure LinkToFileClick(Sender: TObject);
    procedure CellTextChange(Sender: TObject);
    procedure WidthInPercentClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    rep       :TAREditRep;
    Modified  :boolean;
  end;

implementation

{$R *.dfm}

procedure TARCellPropDlg.CancelBtnClick(Sender: TObject);
begin
  ModalResult:=mrCancel;
end;

procedure TARCellPropDlg.OKBtnClick(Sender: TObject);
begin
  ApplyBtnClick(Sender);
  ModalResult:=mrOK;
end;

procedure TARCellPropDlg.BrowseClick(Sender: TObject);
{$IFDEF CLX}
  var dlg:TOpenDialog;
{$ELSE}
  var dlg:TOpenPictureDialog;
{$ENDIF}
begin
  {$IFDEF CLX}
    dlg:=TOpenDialog.Create(Self);
  {$ELSE}
    dlg:=TOpenPictureDialog.Create(Self);
  {$ENDIF}
  try
    {$IFDEF D3}
      dlg.Filter:='All (*.bmp;*.ico;*.emf;*.wmf)|*.bmp;*.ico;*.emf;*.wmf|Bitmaps (*.bmp)|*.bmp|Icons (*.ico)|*.ico|Enhanced Metafiles (*.emf)|*.emf|Metafiles (*.wmf)|*.wmf';
    {$ELSE}
      dlg.Filter:='All (*.jpg;*.jpeg;*.bmp;*.ico;*.emf;*.wmf)|*.jpg;*.jpeg;*.bmp;*.ico;*.emf;*.wmf|JPEG Image File (*.jpg)|*.jpg|JPEG Image File (*.jpeg)|*.jpeg|Bitmaps (*.bmp)|*.bmp|Icons (*.ico)|*.ico|Enhanced Metafiles (*.emf)|*.emf|Metafiles (*.wmf)|*.wmf';
    {$ENDIF}

    dlg.FileName:=Path.Text;
    if dlg.Execute() then begin
      Path.Text:=dlg.FileName;
    end;
  finally
    dlg.Free;
  end;
end;

procedure TARCellPropDlg.SomeChange(Sender: TObject);
begin
  Modified:=true;
  ApplyBtn.Enabled:=true;
end;

procedure TARCellPropDlg.TextOrientationChanging(Sender: TObject; var AllowChange: Boolean);
begin
  AllowChange:=true;

  SomeChange(Sender);
end;

procedure TARCellPropDlg.TextOrientationRGClick(Sender: TObject);
begin
  SomeChange(Sender);
end;

procedure TARCellPropDlg.PathChange(Sender: TObject);
begin
  Path.Color:=clWhite;
  SomeChange(Sender);
end;

procedure TARCellPropDlg.LinkToFileClick(Sender: TObject);
begin
  LinkToFile.AllowGrayed:=false;
  SomeChange(Sender);
end;

procedure TARCellPropDlg.WidthInPercentClick(Sender: TObject);
begin
  WidthInPercent.AllowGrayed:=false;
  SomeChange(Sender);
end;

procedure TARCellPropDlg.CellTextChange(Sender: TObject);
begin
  CellText.Color:=clWhite;
  SomeChange(Sender);
end;

procedure TARCellPropDlg.FormShow(Sender: TObject);
var i:integer;
begin
  if rep.SelCount > 0 then begin
    Path.Text                 :=rep.CurrCell.PictureFileName;
    LinkToFile.Checked        :=rep.CurrCell.LinkPictureToFile;
    CellText.Text             :=rep.CurrCell.Value;
    TextOrientation.ItemIndex :=rep.CurrCell.TextAngle div 90;
    WidthInPercent.Checked    :=rep.CurrCell.WidthRatio <> 0;
    eWidth.Text               :=IntToStr(rep.CurrCell.Width); //CD
    eHeight.Text              :=IntToStr(rep.CurrCell.Owner.Height);  //CD

    if rep.SelCount = 1 then
      CellText.SetFocus; //CD

    for i:=0 to rep.SelCount - 1 do begin
      if rep.Sel[i].PictureFileName <> Path.Text then begin
        Path.Color:=clBtnFace;
      end;
      if rep.Sel[i].LinkPictureToFile <> LinkToFile.Checked then begin
        LinkToFile.State:=cbGrayed;
      end;
      if rep.Sel[i].Value <> CellText.Text then begin
        CellText.Color:=clBtnFace;
      end;
      if rep.Sel[i].TextAngle <> (TextOrientation.ItemIndex * 90) then begin
        TextOrientation.ItemIndex:=-1;
      end;
      if (rep.Sel[i].WidthRatio <> 0) <> WidthInPercent.Checked then begin
        WidthInPercent.State:=cbGrayed;
      end;
    end;

  end else begin
    Close();
  end;
end;

procedure TARCellPropDlg.ApplyBtnClick(Sender: TObject);
var i, j, w:integer;
begin
  for i:=0 to rep.SelCount - 1 do begin
    if Path.Color <> clBtnFace then begin
      rep.Sel[i].PictureFileName:=Path.Text;
    end;
    if LinkToFile.State <> cbGrayed then begin
      rep.Sel[i].LinkPictureToFile:=LinkToFile.Checked;
    end;
    if CellText.Color <> clBtnFace then begin
      rep.Sel[i].Value:=CellText.Text;
    end;
    if TextOrientation.ItemIndex <> -1 then begin
      rep.Sel[i].TextAngle:=TextOrientation.ItemIndex * 90;
    end;
    if WidthInPercent.State <> cbGrayed then begin
      if not WidthInPercent.Checked then begin
        rep.Sel[i].WidthRatio:=0;
      end else begin
        w:=0;
        for j:=0 to rep.Sel[i].Owner.CellCount - 1 do begin
          w:=w + rep.Sel[i].Owner.Cells[j].Width;
        end;
        rep.Sel[i].WidthRatio:=Round(rep.Sel[i].Width * 10000.0 / w);
      end;
    end;
    rep.Sel[i].Width := StrToIntDef(eWidth.Text,rep.Sel[i].Width); //CD
    rep.Sel[i].Owner.Height := StrToIntDef(eHeight.Text,rep.Sel[i].Owner.Height); //CD
  end;
  ApplyBtn.Enabled:=false;
end;

procedure TARCellPropDlg.FormCreate(Sender: TObject);
begin
  Caption                 :=rsARCellProp_ARCellPropDlgCaption;
  ApplyBtn.Caption        :=rsARCellProp_ApplyBtnCaption;
  WidthGroupBox.Caption   :=rsARCellProp_WidthGroupBoxCaption;
  CellTextRG.Caption      :=rsARCellProp_CellTextCaption;
  TextOrientation.Caption :=rsARCellProp_TextOrientationCaption;
  Image.Caption           :=rsARCellProp_ImageCaption;
  LinkToFile.Caption      :=rsARCellProp_LinkToFileCaption;
  WidthInPercent.Caption  :='%';

  OKBtn.Caption     :=rsAR_OkCaption;
  CancelBtn.Caption :=rsAR_CancelCaption;
end;

end.
