unit ARCommonFS;
//************************************************************//
//                         AND Report                        //
//  Eugene Hrulev (mrHru) Laxian Key, Omsk, Russia 1999,2001 //
//  Oleg Melnikov (muler) Laxian Key, Omsk, Russia 1999,2001 //
//************************************************************//

{$INCLUDE DEFINES.INC}

interface

uses
  Classes, SysUtils,
  {$IFDEF CLX}
    QDialogs, QControls, QMenus, QForms,
  {$ELSE}
    Windows, Dialogs, Controls, Menus,
  {$ENDIF}

  {$IFDEF LINUX}
    IniFiles,
  {$ENDIF}
  {$IFDEF MSWINDOWS}
    Registry,
  {$ENDIF}


  {$IFDEF IBFS}
    ARIBFS,
  {$ENDIF}
  {$IFDEF DBFS}
    ARDBFS,
  {$ENDIF}

  ARResStrings, ARUtils, ARVFS, ARSelectFS;


type
  TCommonFileSystem = class(TVirtualFileSystem)
  private
    procedure       SetFileName(const AFileName: string);
    procedure       SetFilter(const AFilter: string);
    procedure       SetInitialDir(const AInitialDir: string);
    procedure       SetOptions(const AOptions: TOpenOptions);
    procedure       SetOnExecuteSetup(const AOnExecuteSetup: TNotifyEvent);
    function        GetOptionsMenuItem: TMenuItem;
    function        GetFileName: string;
    function        GetFiles: TStringList;
    function        GetFilter: string;
    function        GetInitialDir: string;
    function        GetOnExecuteSetup: TNotifyEvent;
    function        GetOptions: TOpenOptions;
    procedure       LoadParams;
    procedure       SaveParams;
  protected
    FSList          :TList;
    CurFS           :TVirtualFileSystem;
    FRegRoot        :string;
    procedure       SetOptionsMenuItem(AOptionsMenuItem:TMenuItem);override;
    procedure       SetParams(AParams:string);override;
    function        GetParams:string;override;
    function        GetMediaName:string;override;
    function        GetDisplayName:string;override;

    function        SelectFSByFileName(AFileName:string):string;
    function        SelectFS:boolean;
  published
    property        Files:TStringList         read  GetFiles;
    property        FileName:string           read  GetFileName         write SetFileName;
    property        Filter:string             read  GetFilter           write SetFilter;
    property        InitialDir:string         read  GetInitialDir       write SetInitialDir;
    property        Options:TOpenOptions      read  GetOptions          write SetOptions;
    property        OptionsMenuItem:TMenuItem read  GetOptionsMenuItem  write SetOptionsMenuItem;
    property        Params:string             read  GetParams           write SetParams;

    property        DisplayName:string        read  GetDisplayName;
    property        MediaName:string          read  GetMediaName;

    function        Caption:string;override;

    function        CreateReadStream(AFileName:string):TStream;override;
    procedure       CloseReadStream;override;

    function        CreateWriteStream(AFileName:string):TStream;override;
    procedure       CloseWriteStream;override;

    function        OpenDialogExecute:boolean;override;
    function        SaveDialogExecute:boolean;override;

    //utilites
    procedure       CreateFile(AFileName:string);override;
    function        FileExists(AFileName:string):boolean;override;
    function        ExtractPath(AFileName:string):string;override;
    function        ExtractFileNameOnly(AFileName:string):string;override;

    procedure       ExecuteSetup(Sender: TObject);override;
    property        OnExecuteSetup:TNotifyEvent read GetOnExecuteSetup      write SetOnExecuteSetup;

    {$IFDEF D3}
      constructor     Create(AOwner:TComponent; ARegRoot:string);
    {$ELSE}
      constructor     Create(AOwner:TComponent; ARegRoot:string);reintroduce;
    {$ENDIF}
    destructor      Destroy;override;
  end;

implementation

//////////////////////////   TCommonFileSystem  /////////////////////

constructor TCommonFileSystem.Create(AOwner: TComponent; ARegRoot:string);
begin
  inherited Create(AOwner);

  FSList:=TList.Create;

  CurFS:=TStandardFileSystem.Create(Self);
  FSList.Add(CurFS);

{$IFDEF DBFS}
  FSList.Add(TDBFileSystem.Create(Self));
{$ENDIF}

{$IFDEF IBFS}
  TIBFileSystem(FSList[FSList.Add(TIBFileSystem.CreateDB(Self))]).SelfDatabase:=true;
{$ENDIF}

  FRegRoot:=ARegRoot;
  LoadParams;
end;

destructor TCommonFileSystem.Destroy;
begin
  SaveParams;

  while FSList.Count > 0 do begin
    TVirtualFileSystem(FSList[0]).Free;
    FSList.Delete(0);
  end;
  FSList.Free;

  inherited;
end;

procedure TCommonFileSystem.LoadParams;
var i:integer;
    {$IFDEF LINUX}
      reg:TIniFile;
    {$ENDIF}
    {$IFDEF MSWINDOWS}
      reg:TRegistry;
    {$ENDIF}
begin
  {$IFDEF LINUX}
    reg:=TIniFile.Create(Application.ExeName + '.ini');
  {$ENDIF}
  {$IFDEF MSWINDOWS}
    reg:=TRegistry.Create;
  {$ENDIF}
  try
    //reg.RootKey := HKEY_CURRENT_USER; // sets by default

    for i:=0 to FSList.Count - 1 do begin
      {$IFDEF LINUX}
        TVirtualFileSystem(FSList[i]).Params:=reg.ReadString('Setup\' + TVirtualFileSystem(FSList[i]).DisplayName + '\',
                                                             'Params', '');
      {$ENDIF}
      {$IFDEF MSWINDOWS}
        reg.OpenKey(FRegRoot + 'Setup\' + TVirtualFileSystem(FSList[i]).DisplayName + '\', True);
        TVirtualFileSystem(FSList[i]).Params:=reg.ReadString('Params');
      {$ENDIF}
    end;
  finally
    {$IFDEF LINUX}
      reg.UpdateFile;
    {$ENDIF}
    reg.Free;
  end;
end;

procedure TCommonFileSystem.SaveParams;
var i:integer;
    {$IFDEF LINUX}
      reg:TIniFile;
    {$ENDIF}
    {$IFDEF MSWINDOWS}
      reg:TRegistry;
    {$ENDIF}
begin
  {$IFDEF LINUX}
    reg:=TIniFile.Create(Application.ExeName + '.ini');
  {$ENDIF}
  {$IFDEF MSWINDOWS}
    reg:=TRegistry.Create;
  {$ENDIF}
  try
    //reg.RootKey := HKEY_CURRENT_USER; // sets by default

    for i:=0 to FSList.Count - 1 do begin
      {$IFDEF LINUX}
        reg.WriteString('Setup\' + TVirtualFileSystem(FSList[i]).DisplayName + '\',
                        'Params',
                        TVirtualFileSystem(FSList[i]).Params);
      {$ENDIF}
      {$IFDEF MSWINDOWS}
        reg.OpenKey(FRegRoot + 'Setup\' + TVirtualFileSystem(FSList[i]).DisplayName + '\', True);
        reg.WriteString('Params', TVirtualFileSystem(FSList[i]).Params);
      {$ENDIF}
    end;
  finally
    {$IFDEF LINUX}
      reg.UpdateFile;
    {$ENDIF}
    reg.Free;
  end;
end;

function    TCommonFileSystem.SelectFSByFileName(AFileName:string):string;
var idx, i:integer;
    s:string;
begin
  Result:=AFileName;
  CurFS:=TVirtualFileSystem(FSList[0]);
  idx:=System.Pos(':', AFileName);
  if idx > 0 then begin
    s:=System.Copy(AFileName, 1, idx - 1);
    for i:=0 to FSList.Count - 1 do begin
      if CT(s, TVirtualFileSystem(FSList[i]).MediaName) then begin
        CurFS:=TVirtualFileSystem(FSList[i]);
        Result:=System.Copy(AFileName, idx + 1, Length(AFileName));
        break;
      end;
    end;
  end;
//  if not Assigned(CurFS) then Error('');
end;

function TCommonFileSystem.Caption: string;
begin
  Result:=CurFS.MediaName + ':' + CurFS.FileName;
end;

procedure TCommonFileSystem.CloseReadStream;
begin
  CurFS.CloseReadStream;
end;

procedure TCommonFileSystem.CloseWriteStream;
begin
  CurFS.CloseWriteStream;
end;

procedure TCommonFileSystem.CreateFile(AFileName: string);
begin
  CurFS.CreateFile(SelectFSByFileName(AFileName));
end;

function TCommonFileSystem.CreateReadStream(AFileName: string): TStream;
begin
  Result:=CurFS.CreateReadStream(SelectFSByFileName(AFileName));
end;

function TCommonFileSystem.CreateWriteStream(AFileName: string): TStream;
begin
  Result:=CurFS.CreateWriteStream(SelectFSByFileName(AFileName));
end;

function TCommonFileSystem.ExtractFileNameOnly(AFileName: string): string;
begin
  Result:=CurFS.ExtractFileNameOnly(SelectFSByFileName(AFileName));
end;

function TCommonFileSystem.ExtractPath(AFileName: string): string;
begin
  Result:=CurFS.ExtractPath(SelectFSByFileName(AFileName));
end;

function TCommonFileSystem.FileExists(AFileName: string): boolean;
begin
  Result:=CurFS.FileExists(SelectFSByFileName(AFileName));
end;

function TCommonFileSystem.GetDisplayName: string;
begin
  Result:=CurFS.DisplayName;
end;

procedure TCommonFileSystem.SetFileName(const AFileName: string);
begin
  CurFS.FileName:=SelectFSByFileName(AFileName);
end;

function TCommonFileSystem.GetFileName: string;
begin
  Result:=CurFS.MediaName + ':' + CurFS.FileName;
end;

function TCommonFileSystem.GetFiles: TStringList;
begin
  Result:=CurFS.Files;
end;

function TCommonFileSystem.GetFilter: string;
begin
  Result:=CurFS.Filter;
end;

procedure TCommonFileSystem.SetFilter(const AFilter: string);
begin
  CurFS.Filter:=AFilter;
end;

function TCommonFileSystem.GetInitialDir: string;
begin
  Result:=CurFS.InitialDir;
end;

procedure TCommonFileSystem.SetInitialDir(const AInitialDir: string);
begin
  CurFS.InitialDir:=AInitialDir;
end;

function TCommonFileSystem.GetMediaName: string;
begin
  Result:=CurFS.MediaName;
end;

function TCommonFileSystem.GetOnExecuteSetup: TNotifyEvent;
begin
  Result:=CurFS.OnExecuteSetup;
end;

procedure TCommonFileSystem.SetOptions(const AOptions: TOpenOptions);
begin
  CurFS.Options:=AOptions;
end;

function TCommonFileSystem.GetOptions: TOpenOptions;
begin
  Result:=CurFS.Options;
end;

function TCommonFileSystem.GetOptionsMenuItem: TMenuItem;
begin
  Result:=CurFS.OptionsMenuItem;
end;

function TCommonFileSystem.GetParams: string;
begin
  Result:=CurFS.Params;
end;

function TCommonFileSystem.OpenDialogExecute: boolean;
var i:integer;
begin
  Result:=false;
  if SelectFS then begin
    Result:=CurFS.OpenDialogExecute;
    if Result then begin
      for i:=0 to Files.Count - 1 do begin
        Files[i]:=CurFS.MediaName + ':' + Files[i];
      end;
    end;
  end;
end;

function TCommonFileSystem.SaveDialogExecute: boolean;
begin
  Result:=SelectFS and CurFS.SaveDialogExecute;
end;

procedure TCommonFileSystem.SetOnExecuteSetup(const AOnExecuteSetup: TNotifyEvent);
begin
  CurFS.OnExecuteSetup:=AOnExecuteSetup;
end;

procedure TCommonFileSystem.SetOptionsMenuItem(AOptionsMenuItem:TMenuItem);
begin
  CurFS.OptionsMenuItem:=AOptionsMenuItem;
end;

procedure TCommonFileSystem.SetParams(AParams: string);
begin
  CurFS.Params:=AParams;
end;

function  TCommonFileSystem.SelectFS:boolean;
var d:TARSelectFSDlg;
    i:integer;
begin
  Result:=false;
  if FSList.Count = 1 then begin
    CurFS:=TVirtualFileSystem(FSList[0]);
    Result:=true;
  end else begin
    if FSList.Count > 1 then begin
      d:=TARSelectFSDlg.Create(Self);
      try
        for i:=0 to FSList.Count - 1 do begin
          d.FSList.Items.AddObject(TVirtualFileSystem(FSList[i]).DisplayName, FSList[i]);
        end;
        d.FSList.ItemIndex:=d.FSList.Items.IndexOfObject(CurFS);
        if d.ShowModal = mrOK then begin
          CurFS:=TVirtualFileSystem(d.FSList.Items.Objects[d.FSList.ItemIndex]);
          Result:=true;
        end;
      finally
        d.Free;
      end;
    end;
  end;
end;

procedure TCommonFileSystem.ExecuteSetup(Sender: TObject);
begin
  if SelectFS then CurFS.ExecuteSetup(Sender);
end;

end.



