unit ARCustomExport;
//************************************************************//
//                         AND Report                        //
//  Eugene Hrulev (mrHru) Laxian Key, Omsk, Russia 1999,2001 //
//  Oleg Melnikov (muler) Laxian Key, Omsk, Russia 1999,2001 //
//************************************************************//

{$INCLUDE DEFINES.INC}

interface

uses
  Classes, Sysutils,
  {$IFDEF CLX}
    QForms,
  {$ELSE}
    Forms,
  {$ENDIF}

  ARGridRep;

type
  TExportRange = (erAllPages, erSelection, erPageNums);

  TARExportProgressStage = (epsStarting, epsRunning, epsEnding);

  TARExportProgressEvent = procedure(Sender:TObject; Stage:TARExportProgressStage; PercentDone:integer; var Abort:boolean) of object;

  TARCustomExport = class(TComponent)
  private
    FRep            :TARGridRep;
    FFileName       :TFileName;
    FOnProgress     :TARExportProgressEvent;

    FMax            :integer;
    FPos            :integer;

    procedure   SetRep(const Value: TARGridRep);
    procedure   SetFileName(const Value: TFileName);
  protected
    function    ExecuteSetupDialog:boolean;virtual;abstract;
    procedure   Execute;virtual;abstract;
    procedure   ProgressStart(AMax:integer);
    function    ProgressStep:boolean;
    procedure   ProgressStop;
  public
    constructor Create(AOwner:TComponent);override;
  published
    property    ARRep:TARGridRep          read  FRep              write SetRep;
    property    FileName:TFileName        read  FFileName         write SetFileName;

    property    OnProgress:TARExportProgressEvent read FOnProgress write FOnProgress;

  end;

implementation

{ TARCustomExport }

constructor TARCustomExport.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

procedure TARCustomExport.ProgressStart(AMax: integer);
var Abort:boolean;
begin
  FMax:=AMax;
  FPos:=0;
  if Assigned(OnProgress) then begin
    Abort:=false;
    OnProgress(Self, epsStarting, 0, Abort);
  end;
end;

function  TARCustomExport.ProgressStep:boolean;
var Abort:boolean;
begin
  Application.ProcessMessages;
  inc(FPos);
  Abort:=false;
  if Assigned(OnProgress) then begin
    OnProgress(Self, epsRunning, Round(FPos / FMax * 100.0), Abort);
  end;
  Result:=not Abort;
end;

procedure TARCustomExport.ProgressStop;
var Abort:boolean;
begin
  if Assigned(OnProgress) then begin
    Abort:=false;
    OnProgress(Self, epsEnding, 100, Abort);
  end;
end;

procedure TARCustomExport.SetFileName(const Value: TFileName);
begin
  FFileName := Value;
end;

procedure TARCustomExport.SetRep(const Value: TARGridRep);
begin
  FRep := Value;
end;

end.
