unit ARDBFS;
//************************************************************//
//                         AND Report                        //
//  Eugene Hrulev (mrHru) Laxian Key, Omsk, Russia 1999,2001 //
//  Oleg Melnikov (muler) Laxian Key, Omsk, Russia 1999,2001 //
//************************************************************//
{$INCLUDE DEFINES.INC}

interface

uses
  Classes, SysUtils, 
  {$IFDEF CLX}
  {$ELSE}
    Menus, Dialogs, DBTables, Controls, DB,
  {$ENDIF}

  ARVFS, ARSQL, ARResStrings, ARSelectDatabase, ARDBOpen;

type

  TDBFileSystem = class(TVirtualFileSystem)
  private
    FDatabase   :TDatabase;
  protected
    FTableName  :string;
    QCreate     :TSQL;
    QOpen       :TSQL;
    QWrite      :TSQL;

    procedure   SetDatabase(ADatabase:TDatabase);virtual;
    procedure   SetTableName(ATableName:string);virtual;
    procedure   CheckTable;virtual;
    procedure   ResetQs;virtual;

    procedure   SetOptionsMenuItem(AOptionsMenuItem:TMenuItem);override;
    procedure   DBFSSelectDBClick(Sender:TObject);
    procedure   DBFSSelectTableName(Sender:TObject);
    function    GetDisplayName:string;override;
    function    GetMediaName:string;override;
  public
  published
    function    OpenFile(AFileName:string):boolean;virtual;
    procedure   CreateFile(AFileName:string);override;

    function    Caption:string;override;

    property    Database:TDatabase  read FDatabase   write SetDatabase;
    property    TableName:string    read FTableName  write SetTableName;

    procedure   SelectDatabase;virtual;
    procedure   SelectTableName;virtual;

    function    CreateReadStream(AFileName:string):TStream;override;
    function    CreateWriteStream(AFileName:string):TStream;override;

    procedure   CloseReadStream;override;
    procedure   CloseWriteStream;override;

    function    OpenDialogExecute:boolean;override;
    function    SaveDialogExecute:boolean;override;

    function    FileExists(AFileName:string):boolean;override;
    function    ExtractPath(AFileName:string):string;override;
    function    ExtractFileNameOnly(AFileName:string):string;override;
  end;

procedure  Register;

implementation
{$R *.res}

//////////////////////////   TDBFileSystem  /////////////////////
procedure   TDBFileSystem.SetOptionsMenuItem(AOptionsMenuItem:TMenuItem);
var mi:TMenuItem;
begin
  inherited  SetOptionsMenuItem(AOptionsMenuItem);

  if csDesigning in ComponentState then exit;

  if Assigned(FOptionsMenuItem) then begin
    mi:=TMenuItem.Create(Self);
    mi.Caption:=rsVFS_SelectDatabaseMenuItem;
    mi.OnClick:=DBFSSelectDBClick;
    FOptionsMenuItem.Add(mi);

    mi:=TMenuItem.Create(Self);
    mi.Caption:=rsVFS_SelectTableNameMenuItem;
    mi.OnClick:=DBFSSelectTableName;
    FOptionsMenuItem.Add(mi);
  end;
end;

procedure   TDBFileSystem.SelectDatabase;
var  d:TSelectDBDlg;
begin
  d:=TSelectDBDlg.CreateDB(Self, FDatabase);
  try
    d.ShowModal;
  finally
    d.Free;
  end;
end;

procedure   TDBFileSystem.SelectTableName;
begin
  TableName:=InputBox(Database.AliasName, rsVFS_EnterTableName, 'Reports');
end;

procedure    TDBFileSystem.DBFSSelectDBClick(Sender:TObject);
begin
  SelectDatabase;
end;

procedure    TDBFileSystem.DBFSSelectTableName(Sender:TObject);
begin
  SelectTableName;
end;

procedure    TDBFileSystem.CheckTable;
var Q:TSQL;
begin
  if (not Assigned(FDatabase)) or (Length(FTableName) = 0) then exit;

  try
    if not FDatabase.Connected then FDatabase.Open;
  except
    MessageDlg(rsVFS_CantOpenDatabase, mtError, [mbOK], 0);
    exit;
  end;

  Q:=TSQL.CreateDB(FDatabase);
  try
    Q.Add('SELECT * FROM ' + TableName);
    try
      Q.Open;
    except
      if MessageDlg(Format(rsVFS_TableNotExistsCreateIt, [TableName, FDatabase.AliasName]), mtConfirmation, [mbYes, mbNo], 0) = mrYes then begin
        Q.New;
        Q.Add('CREATE TABLE ' + TableName + ' ( ');
        Q.Add('    NAME CHAR(80), ');
        Q.Add('    STRVAL BLOB, ');
        Q.Add('    PRIMARY KEY (NAME) ');
        Q.Add(') ');
        try
          Q.Exec;
        except
          on E:Exception do begin
            MessageDlg(Format(rsVFS_CantCreateTable, [TableName, E.Message]), mtError, [mbOK], 0);
          end;
        end;
      end;
    end;
  finally
    Q.Free;
  end;
end;

procedure   TDBFileSystem.ResetQs;
begin
  if Assigned(QOpen) then begin
    QOpen.Free;
    QOpen:=nil;
  end;
  if Assigned(QWrite) then begin
    QWrite.Free;
    QWrite:=nil;
  end;
  if Assigned(QCreate) then begin
    QCreate.Free;
    QCreate:=nil;
  end;
end;

procedure   TDBFileSystem.SetDatabase(ADatabase:TDatabase);
begin
  ResetQs;
  FDatabase:=ADatabase;
  CheckTable;
end;

procedure   TDBFileSystem.SetTableName(ATableName:string);
begin
  ResetQs;
  FTableName:=ATableName;
  CheckTable;
end;

function    TDBFileSystem.Caption:string;
begin
  Result:='[' + FDatabase.AliasName + ']' + FileName;
end;

function    TDBFileSystem.OpenFile(AFileName:string):boolean;
begin
  if not Assigned(QOpen) then begin
    QOpen:=TSQL.CreateDB(FDatabase);
    QOpen.Add('SELECT STRVAL FROM ' + FTableName + ' WHERE NAME = :NAME ');
    QOpen.Prepare;
  end;
  QOpen.Close;
  QOpen.S['NAME']:=AFileName;
  QOpen.Open;

  Result:=not QOpen.EOF;
end;

function    TDBFileSystem.CreateReadStream(AFileName:string):TStream;
begin
  Result:=nil;

  FFileName:=ANSIUpperCase(AFileName);

  if OpenFile(FFileName) then begin
    FReadStream:=TBlobStream.Create(QOpen.DataSet.FieldByName('STRVAL') as TBlobField, bmRead);
    Result:=FReadStream;
  end;
end;

procedure   TDBFileSystem.CloseReadStream;
begin
  inherited;
  QOpen.Close;
end;

procedure   TDBFileSystem.CreateFile(AFileName:string);
begin
  if not Assigned(QCreate) then begin
    QCreate:=TSQL.CreateDB(FDatabase);
    QCreate.Add('INSERT INTO ' + FTableName + '(NAME) VALUES(:NAME) ');
    QCreate.Prepare;
  end;
  try
    QCreate.S['NAME']:=FFileName;
    QCreate.Exec;
  except
    //already exists, do nothing
  end;
end;

function    TDBFileSystem.CreateWriteStream(AFileName:string):TStream;
begin
  FFileName:=ANSIUpperCase(AFileName);

  CreateFile(FFileName);

  if not Assigned(QWrite) then begin
    QWrite:=TSQL.CreateDB(FDatabase);
    QWrite.RequestLive:=true;
    QWrite.Add('SELECT STRVAL FROM ' + FTableName + ' WHERE NAME = :NAME ');
    QWrite.Prepare;
  end;
  QWrite.Close;
  QWrite.S['NAME']:=FFileName;
  QWrite.Open;
  QWrite.Edit;

  FWriteStream:=TBlobStream.Create(QWrite.DataSet.FieldByName('STRVAL') as TBlobField, bmWrite);
  Result:=FWriteStream;
end;

procedure   TDBFileSystem.CloseWriteStream;
begin
  inherited;
  QWrite.Post;
  QWrite.Close;
  QWrite.RequestLive:=false;
end;

function    TDBFileSystem.OpenDialogExecute:boolean;
begin
  Result:=false;
  with TDBOpenDlg.Create(Self) do
  try
    Database:=FDatabase;
    TableName:=FTableName;
    FileName:=FFileName;
    if Execute then begin
      FFileName:=FileName;
      Files.Clear;
      Files.Add(FileName);
      Result:=true;
    end;
  finally
    Free;
  end;
end;

function    TDBFileSystem.SaveDialogExecute:boolean;
begin
  Result:=OpenDialogExecute;
end;

function    TDBFileSystem.FileExists(AFileName:string):boolean;
begin
  Result:=OpenFile(AFileName);
end;

function    TDBFileSystem.ExtractPath(AFileName:string):string;
begin
  Result:='';
end;

function    TDBFileSystem.ExtractFileNameOnly(AFileName:string):string;
begin
  Result:=AFileName;
end;

function    TDBFileSystem.GetDisplayName:string;
begin
  Result:=rsFS_DatabaseDisplayName;
end;

function    TDBFileSystem.GetMediaName:string;
begin
  Result:=rsFS_DatabaseMediaName;
end;

////////////////////////////////////////////////////////////////
procedure Register;
begin
  RegisterComponents('ARComponents', [TDBFileSystem]);
end;

end.



