unit AREditRep;
//************************************************************//
//                         AND Report                        //
//  Eugene Hrulev (mrHru) Laxian Key, Omsk, Russia 1999,2001 //
//  Oleg Melnikov (muler) Laxian Key, Omsk, Russia 1999,2001 //
//************************************************************//

{$INCLUDE DEFINES.INC}

interface

uses
  Classes, Math,
  {$IFDEF CLX}
    QControls, Types,
  {$ELSE}
    Controls, Windows,
  {$ENDIF}

  ARUtils, ARRep, ARGridRep, ARUndo;

type
  TAREditRep = class(TARGridRep)
  private
    FOnResizeGutter:TNotifyEvent;
  protected
    procedure   KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure   MouseMove(Shift: TShiftState; X, Y: Integer);override;
    procedure   MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);override;
    procedure   MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);override;
    procedure   SetCellWidth(ACell:TARCell; AWidth:integer);
  public
    Undo        :TUndo;
    FileName    :string;
    Modified    :boolean;
    PropEditor  :boolean;

    procedure   Load(AFileName:string);
    procedure   Save(AFileName:string);

    constructor Create(AOwner:TComponent);override;
    destructor  Destroy;override;

    procedure   DeleteSelCells;
    function    FrameCellRect(IdxBand:integer;IdxCell:integer):TRect;override;
  published 
    property    OnKeyPress;
    property    OnResizeGutter:TNotifyEvent read FOnResizeGutter  write FOnResizeGutter;
  end;

procedure Register;

implementation
{$R *.res}

constructor  TAREditRep.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);

  ShowRightMargin:=true;
  FShowBandTitle:=true;

  if GridX  = 0 then GridX:=4.0;
  if GridY  = 0 then GridY:=4.0;
  if Gutter = 0 then Gutter:=100;
end;

destructor  TAREditRep.Destroy;
begin
  inherited;
end;

procedure   TAREditRep.Load(AFileName:string);
begin
  try
    SrcRep.Load(AFileName);
  finally
     Repaint;
  end;
end;

procedure   TAREditRep.Save(AFileName:string);
begin
  SrcRep.Save(AFileName);
end;

var //shared all TAREditRep instance
  TmpBandIdx    :integer;
  TmpCellIdx    :integer;

  OldY          :integer;
  OldHeight     :integer;

  OldX          :integer;
  OldWidth      :integer;
  InGutterShift :boolean;

procedure   TAREditRep.MouseMove(Shift: TShiftState; X, Y: Integer);
var idxband, idxcell:integer;
    d, w, w1, w2, h, h1, h2:extended;
    stepx, stepy:extended;
    cell1, cell2:TARCell;
    r:TRect;
    curs:TCursor;
begin
  curs:=Cursor;
  if ssLeft in Shift then begin
    if curs = crVSplit then begin
      d:=AsUnits(OldHeight + (Y - OldY));

      if ssShift in Shift then begin
        stepy:=AsUnits(1);
      end else begin
        stepy:=GridY;
      end;
{$IFDEF D3}
      h:=MaxEx(d, stepy);
{$ELSE}
      h:=Max(d, stepy);
{$ENDIF}
      h1:=AsUnits(BandHeigths[TmpBandIdx]);

      if (ssAlt in Shift) and (TmpBandIdx < SrcRep.BandCount - 1) then begin
        h2:=AsUnits(BandHeigths[TmpBandIdx + 1]);
        if Round(h2 - (h - h1) - stepy) >= 0 then begin
          BandHeigths[TmpBandIdx + 1]:=BandHeigths[TmpBandIdx + 1] - (AsPix(AlignToGrid(h, stepy)) - BandHeigths[TmpBandIdx]);
          BandHeigths[TmpBandIdx]    :=AsPix(AlignToGrid(h, stepy));
        end;
      end else begin
        BandHeigths[TmpBandIdx]:=AsPix(AlignToGrid(h, stepy));
      end;

      UpdateScrolls;
      if Assigned(OnResizeBand) then OnResizeBand(Self, TmpBandIdx);
    end else begin
      if curs = crHSplit then begin
        d:=AsUnits(OldWidth + (X - OldX));

        if ssShift in Shift then begin
          stepx:=AsUnits(1);
        end else begin
          stepx:=GridY;
        end;
{$IFDEF D3}
        w:=MaxEx(d, stepx);
{$ELSE}
        w:=Max(d, stepx);
{$ENDIF}

        if InGutterShift then begin
          Gutter:=AsPix(AlignToGrid(d, stepx));
          if Assigned(OnResizeGutter) then OnResizeGutter(Self);
        end else begin
          cell1:=SrcRep.Bands[TmpBandIdx].Cells[TmpCellIdx];
          w1:=AsUnits(cell1.Width);

          if (ssAlt in Shift) and (TmpCellIdx < SrcRep.Bands[TmpBandIdx].CellCount - 1) then begin
            cell2:=SrcRep.Bands[TmpBandIdx].Cells[TmpCellIdx + 1];
            w2:=AsUnits(cell2.Width);

            if Round(w2 - (w - w1) - stepx) >= 0 then begin
//              cell2.Width:=cell2.Width - (AsPix(AlignToGrid(w, stepx)) - cell1.Width);
              SetCellWidth(cell2, cell2.Width - (AsPix(AlignToGrid(w, stepx)) - cell1.Width));
//              cell1.Width:=AsPix(AlignToGrid(w, stepx));
              SetCellWidth(cell1, AsPix(AlignToGrid(w, stepx)));
            end;
          end else begin
//            cell1.Width:=AsPix(AlignToGrid(w, stepx));
            SetCellWidth(cell1, AsPix(AlignToGrid(w, stepx)));
          end;
          UpdateScrolls;
          Invalidate;
          if Assigned(OnResizeCell) then OnResizeCell(Self, TmpBandIdx, TmpCellIdx);
        end;
      end else begin
        cell1:=MouseToCell(X, Y, idxband, idxcell);
        if Assigned(cell1) and (CurrCell <> cell1) then begin
          CurrBandIdx:=idxband;
          CurrCellIdx:=idxcell;

          SelAdd(CurrBandIdx, CurrCellIdx);
          if Assigned(OnSelectCell) then OnSelectCell(Self, CurrBandIdx, CurrCellIdx);
        end;
      end;
    end;
  end else begin
    InGutterShift:=false;
    cell1:=MouseToCell(X, Y, idxband, idxcell);
    if Abs(X - Gutter) <= 5 then begin
      curs:=crHSplit;
      InGutterShift:=true;
    end else begin
      if Assigned(cell1) and (Abs(X - Gutter) > 5) then begin
        r:=CellRect(idxband, 0);
        if Abs(r.Bottom - Y) < 3 then begin
          TmpBandIdx:=idxband;
          curs:=crVSplit;
        end else begin
          curs:=crDefault;
          for idxcell:=0 to SrcRep.Bands[idxband].CellCount - 1 do begin
            d:=SrcRep.Bands[idxband].Rights[idxcell] + Gutter - LeftX - X;
            if (d >= 0) and (d < 3) then begin
              curs:=crHSplit;
              TmpBandIdx:=idxband;
              TmpCellIdx:=idxcell;
              break;
            end;
          end;
        end
      end else begin
        curs:=crDefault;
      end;
    end;
  end;
  Cursor:=curs;

  if Assigned(OnMouseMove) then OnMouseMove(Self, Shift, X, Y);
end;

procedure   TAREditRep.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var idx, idxband, idxcell:integer;
    cell:TARCell;
begin
  inherited;

  if ssRight in Shift then exit;

  cell:=MouseToCell(X, Y, idxband, idxcell);
  if Abs(X - Gutter) <= 5 then begin
    if Cursor = crHSplit then begin
      OldX:=X;
      OldWidth:=Gutter;
    end;
  end else
  if X < (Gutter - 5) then begin
    if ssShift in Shift then begin
      for idx:=Min(CurrBandIdx, idxband) to Max(CurrBandIdx, idxband) do begin
        SrcRep.Bands[idx].Selected:=true;
      end;
    end else begin
      if not (ssCtrl in Shift) then begin
        ClearSel;
      end;
      SrcRep.Bands[idxband].Selected:=not SrcRep.Bands[idxband].Selected;
    end;
    CurrBandIdx:=idxband;
    if Assigned(OnSelectBand) then OnSelectBand(Self, idxband);
  end else
  if Assigned(cell) then begin
    if Cursor = crVSplit then begin
      OldY     :=Y;
      OldHeight:=SrcRep.Bands[TmpBandIdx].Height;
    end else begin
      if Cursor = crHSplit then begin
        OldX:=X;
        OldWidth:=SrcRep.Bands[TmpBandIdx].Cells[TmpCellIdx].Width;
      end else begin
        CurrBandIdx:=idxband;
        CurrCellIdx:=idxcell;
        if not (ssCtrl in Shift) then ClearSel;
        SelAdd(CurrBandIdx, CurrCellIdx);
        if Assigned(OnSelectCell) then OnSelectCell(Self, CurrBandIdx, CurrCellIdx);
      end;
    end;
  end;
end;

procedure   TAREditRep.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited;

  InGutterShift:=false;
end;

procedure   TAREditRep.DeleteSelCells;
var idxband, idxcell:integer;
begin
  for idxband:=0 to SrcRep.BandCount - 1 do begin
    if SrcRep.Bands[idxband].CellCount > 1 then begin
      for idxcell:=SrcRep.Bands[idxband].CellCount - 1 downto 0 do begin
        if SrcRep.Bands[idxband].Cells[idxcell].Selected then begin
          SrcRep.Bands[idxband].DeleteCell(idxcell);
        end;
      end;
    end;
  end;
  SelList.Clear;
end;

procedure   TAREditRep.KeyDown(var Key: Word; Shift: TShiftState);
var stepx, stepy:extended;
    h, w:extended;
    cell, cell1:TARCell;
begin
  if ssCtrl in Shift then begin
    stepx:=GridX;
    stepy:=GridX;
    if ssShift in Shift then begin
      stepx:=AsUnits(1);
      stepy:=AsUnits(1);
    end;

    case Key of
      VK_UP:    begin
{$IFDEF D3}
                  h:=AlignToGrid(MaxEx(AsUnits(BandHeigths[CurrBandIdx]) - stepy, stepy), stepy);
{$ELSE}
                  h:=AlignToGrid(Max(AsUnits(BandHeigths[CurrBandIdx]) - stepy, stepy), stepy);
{$ENDIF}
                  if (ssAlt in Shift) and (CurrBandIdx < SrcRep.BandCount - 1) then begin
                    BandHeigths[CurrBandIdx + 1]:=BandHeigths[CurrBandIdx + 1] - (AsPix(h) - BandHeigths[CurrBandIdx]);
                  end;
                  BandHeigths[CurrBandIdx]:=AsPix(h);

                  if Assigned(OnResizeBand) then OnResizeBand(Self, CurrBandIdx);
                  Invalidate;
                  Key:=0;
                end;
      VK_DOWN:  begin
                  h:=AlignToGrid(AsUnits(BandHeigths[CurrBandIdx]) + stepy, stepy);
                  if (ssAlt in Shift) and (CurrBandIdx < SrcRep.BandCount - 1) then begin
                    if (AsUnits(BandHeigths[CurrBandIdx + 1]) - (h - AsUnits(BandHeigths[CurrBandIdx])) - stepy) >= 0 then begin
                      BandHeigths[CurrBandIdx + 1]:=BandHeigths[CurrBandIdx + 1] - (AsPix(h) - BandHeigths[CurrBandIdx]);
                      BandHeigths[CurrBandIdx]:=AsPix(h);
                    end;
                  end else begin
                    BandHeigths[CurrBandIdx]:=AsPix(h);
                  end;

                  if Assigned(OnResizeBand) then OnResizeBand(Self, CurrBandIdx);
                  Invalidate;
                  Key:=0;
                end;
      VK_LEFT:  begin
                  cell:=SrcRep.Bands[CurrBandIdx].Cells[CurrCellIdx];
{$IFDEF D3}
                  w:=AlignToGrid(MaxEx(AsUnits(cell.Width) - stepx, stepx), stepx);
{$ELSE}
                  w:=AlignToGrid(Max(AsUnits(cell.Width) - stepx, stepx), stepx);
{$ENDIF}

                  if (ssAlt in Shift) and (CurrCellIdx < SrcRep.Bands[CurrBandIdx].CellCount - 1) then begin
                    cell1:=SrcRep.Bands[CurrBandIdx].Cells[CurrCellIdx + 1];
                    SetCellWidth(cell1, cell1.Width - (AsPix(w) - cell.Width));
                  end;
                  SetCellWidth(cell, AsPix(w));

                  if Assigned(OnResizeCell) then OnResizeCell(Self, CurrBandIdx, CurrCellIdx);
                  Invalidate;
                  Key:=0;
                end;
      VK_RIGHT: begin
                  cell:=SrcRep.Bands[CurrBandIdx].Cells[CurrCellIdx];
                  w:=AlignToGrid(AsUnits(cell.Width) + stepx, stepx);

                  if (ssAlt in Shift) and (CurrCellIdx < SrcRep.Bands[CurrBandIdx].CellCount - 1) then begin
                    cell1:=SrcRep.Bands[CurrBandIdx].Cells[CurrCellIdx + 1];
                    if (AsUnits(cell1.Width) - (w - AsUnits(cell.Width)) - stepx) >= 0 then begin
                      SetCellWidth(cell1, cell1.Width - (AsPix(w) - cell.Width));
                      SetCellWidth(cell, AsPix(w));
                    end;
                  end else begin
                      SetCellWidth(cell, AsPix(w));
                  end;

                  if Assigned(OnResizeCell) then OnResizeCell(Self, CurrBandIdx, CurrCellIdx);
                  Invalidate;
                  Key:=0;
                end;
    end;
  end;

  inherited  KeyDown(Key, Shift);
end;
procedure   TAREditRep.SetCellWidth(ACell:TARCell; AWidth:integer);
begin
  ACell.Width:=AWidth;
end;

function    TAREditRep.FrameCellRect(IdxBand:integer;IdxCell:integer):TRect;
var band:TARBand;
    cell:TARCell;
begin
  if  (IdxBand >= 0) and (IdxBand < SrcRep.BandCount) and
      (IdxCell >= 0) and (IdxCell < SrcRep.Bands[IdxBand].CellCount) then begin

    band:=SrcRep.Bands[IdxBand];
    cell:=band.Cells[IdxCell];

    Result.Left     :=Gutter - LeftX + 1 + band.Lefts[IdxCell];
    Result.Right    :=Result.Left + cell.Width;

    Result.Top    :=SrcRep.Tops[IdxBand];
    Result.Bottom :=Result.Top + SrcRep.Bands[IdxBand].Height;
    if IdxBand >= FixedBands then begin
      Result.Top    :=Result.Top    - TopY;
      Result.Bottom :=Result.Bottom - TopY;
    end;

  end else begin
    Result:=Rect(0, 0, 0, 0);
  end;
end;

/////////////////////////////////////////////////////////////////////////////////////////
procedure Register;
begin
  RegisterComponents('ARComponents', [TAREditRep]);
end;

end.
