unit AREditorMain;
//************************************************************//
//                         AND Report                        //
//  Eugene Hrulev (mrHru) Laxian Key, Omsk, Russia 1999,2001 //
//  Oleg Melnikov (muler) Laxian Key, Omsk, Russia 1999,2001 //
//************************************************************//

// changes
// sa: Sertac Akyuz, mailto:prosoft@turk.net // 11.06.2001
{$INCLUDE DEFINES.INC}

interface

uses
  Classes, Math, SysUtils, IniFiles,
  {$IFDEF CLX}
    QForms, QComCtrls, QMenus, QDialogs, QExtCtrls, QStdCtrls, QControls,
    QGraphics, Types, QClipbrd, QTypes,
  {$ELSE}
    Forms, ComCtrls, Menus, Dialogs, ExtCtrls, StdCtrls, Controls, Graphics,
    Clipbrd, ExtDlgs, Windows, Messages, Grids, ToolWin, Buttons,
    OleCtrls, Printers,
    {$IFDEF D3}
      {$IFDEF D3_CLIENTSERVER}
        isp3,
      {$ENDIF}
    {$ELSE}
      SHDocVw,
    {$ENDIF}
  {$ENDIF}

  {$IFDEF MSWINDOWS}
    Registry,
  {$ENDIF}

  ARResStrings, ARUtils, ARPrinter, AREditorPageSetup, ARRepCommon,
  ARRep, ARGridRep, AREditRep, AREditorAbout, ARUndo,
  ARRulerBar, ARButtons, ARActions, ARToolsCustomize, AREditorOptions,
  ARCommonFS, ARCellProp, AREditorUserVarList;

const
  RegRoot = '\Software\ANDReport\Editor\';

  Category_File     = 'File';
  Category_Edit     = 'Edit';
  Category_Cells    = 'Cells';
  Category_Bands    = 'Bands';
  Category_Format   = 'Format';
  Category_Frame    = 'Frame';
  Category_Options  = 'Options';
  Category_Help     = 'Help';

type
  TAREPageType = (ptNone, ptTemplate, ptScript, ptHelp, ptPreview);

  {$IFDEF CLX}
    TOpenPictureDialog  = class(TOpenDialog);
    TARToolBar          = class(TToolBar);
    TWebBrowser         = class(TTextBrowser);
  {$ELSE}
    TARToolBar          = class(TToolBar);
    {$IFDEF D3}
      TControlBar       = class(TCoolBar);
    {$ENDIF}
  {$ENDIF}

  TAREditorDlg = class(TForm)
    StatusBar: TStatusBar;
    MainMenu: TMainMenu;
    FontDialog: TFontDialog;
    ColorDialog: TColorDialog;
    BarPopupMenu: TPopupMenu;
    RepPopupMenu: TPopupMenu;
    PC: TPageControl;
    Dummy1: TMenuItem;

    procedure FileNewExecute(Sender: TObject);
    procedure FileOpenExecute(Sender: TObject);
    procedure FileSaveExecute(Sender: TObject);
    procedure FileExitExecute(Sender: TObject);
    procedure FileSaveAsExecute(Sender: TObject);
    procedure FileSaveAllExecute(Sender: TObject);
    procedure FileClosePageExecute(Sender: TObject);
    procedure FilePageSetupExecute(Sender: TObject);
    procedure FilePreviewExecute(Sender: TObject);
    procedure FilePrintExecute(Sender: TObject);

    procedure EditSelectAllExecute(Sender: TObject);
    procedure EditInsertPictureExecute(Sender: TObject);
    procedure EditFitToCellExecute(Sender: TObject);
    procedure EditTileExecute(Sender: TObject);
    procedure EditUndoExecute(Sender: TObject);
    procedure EditRedoExecute(Sender: TObject);

    procedure FormatLeftAlignExecute(Sender: TObject);
    procedure FormatCenterAlignExecute(Sender: TObject);
    procedure FormatRightAlignExecute(Sender: TObject);
    procedure FormatFontExecute(Sender: TObject);
    procedure FormatFontColorExecute(Sender: TObject);
    procedure FormatBoldExecute(Sender: TObject);
    procedure FormatItalicExecute(Sender: TObject);
    procedure FormatUnderlineExecute(Sender: TObject);
    procedure FormatToplayoutExecute(Sender: TObject);
    procedure FormatCenterlayoutExecute(Sender: TObject);
    procedure FormatBottomlayoutExecute(Sender: TObject);
    procedure FormatFontNameExecute(Sender: TObject);
    procedure FormatFontSizeExecute(Sender: TObject);
    procedure FormatDecreaseFontSizeExecute(Sender: TObject);
    procedure FormatIncreaseFontSizeExecute(Sender: TObject);

    procedure CellsCellTextExecute(Sender: TObject);
    procedure CellsCellPropertiesExecute(Sender: TObject);
    procedure CellsSplitCellExecute(Sender: TObject);
    procedure CellsAddCellExecute(Sender: TObject);
    procedure CellsDeleteCellExecute(Sender: TObject);
    procedure CellsCopyCellExecute(Sender: TObject);
    procedure CellsCutCellExecute(Sender: TObject);
    procedure CellsPasteCellExecute(Sender: TObject);
    procedure CellsCellMoveLeftExecute(Sender: TObject);
    procedure CellsCellMoveRightExecute(Sender: TObject);
    procedure CellsAutoStretchExecute(Sender: TObject);
    procedure CellsWordWrapExecute(Sender: TObject);
    procedure CellsBgColorExecute(Sender: TObject);

    procedure BandsAddBandExecute(Sender: TObject);
    procedure BandsDeleteBandExecute(Sender: TObject);
    procedure BandsCopyBandsExecute(Sender: TObject);
    procedure BandsCutBandsExecute(Sender: TObject);
    procedure BandsPasteBandsExecute(Sender: TObject);
    procedure BandsBandUpExecute(Sender: TObject);
    procedure BandsBandDownExecute(Sender: TObject);
    procedure BandsBandNameExecute(Sender: TObject);

    procedure FrameNotFrameExecute(Sender: TObject);
    procedure FrameFrameColorExecute(Sender: TObject);
    procedure FrameFrameWidthExecute(Sender: TObject);
    procedure FrameFrameExecuteInc(Sender: TObject);
    procedure FrameFrameExecuteDec(Sender: TObject);

    procedure OptionsUserVarsExecute(Sender: TObject);
    procedure OptionsDefaultFontExecute(Sender: TObject);
    procedure OptionsShowAllExecute(Sender: TObject);
    procedure UnitsInchesExecute(Sender: TObject);
    procedure UnitsMillemetersExecute(Sender: TObject);
    procedure UnitsCentimetersExecute(Sender: TObject);
    procedure UnitsPixelsExecute(Sender: TObject);
    procedure OptionsFileSystemExecute(Sender: TObject);

    procedure HelpContentsExecute(Sender: TObject);
    procedure HelpAboutExecute(Sender: TObject);

    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormDestroy(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure RepResizeCell(Sender: TObject; IdxBand, IdxCell: Integer);
    procedure RepSelectCell(Sender: TObject; IdxBand, IdxCell: Integer);
    procedure RepResizeBand(Sender: TObject; IdxBand: Integer);

    procedure CutMenuItemClick(Sender: TObject);
    procedure CopyMenuItemClick(Sender: TObject);
    procedure PasteMenuItemClick(Sender: TObject);
    procedure DeleteMenuItemClick(Sender: TObject);
    procedure RepResizeGutter(Sender: TObject);
    procedure HRBChange(Sender: TObject);
    procedure RepScroll(Sender: TObject);
    procedure RepKeyPress(Sender: TObject; var Key: Char);
    procedure RepMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure BarPopupMenuPopup(Sender: TObject);
    procedure RepSelectBand(Sender: TObject; IdxBand: Integer);
    procedure MainControlBarDblClick(Sender: TObject);
    procedure PCChange(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure MainControlBarClick(Sender: TObject);
    procedure PCMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    OldApplicationOnHint:TNotifyEvent;
    OldApplicationOnHelp:THelpEvent;

    fEnableUpdateEditText:boolean;

    FSetUndo     :boolean;
    prn         :TARPrinter;
    CellEditor  :TMemo;

    Units       :TARUnits;
    FGridX      :extended;
    FGridY      :extended;

    RecentFiles :TAIStringList;
    OldEditValue:string;

    UserVarList :TAIStringList;

    EditingRep  :TARGridRep;

    FFileSystem :TCommonFileSystem;

    fToPrint, fToView:boolean;

    function    GetGridX:extended;
    function    GetGridY:extended;
    procedure   SetGridX(AGridX:extended);
    procedure   SetGridY(AGridY:extended);

    procedure   OptionsUserVarsDialogExecute(Sender: TObject);
    procedure   UserVarListChange(Sender: TObject);

    procedure   SetUnits(AUnits:TARUnits);

    procedure   LoadReport(const AFileName:string);
    function    SaveReport:boolean;
    function    SaveAll:boolean;
    function    NewReport(const AFileName:string):boolean;
    procedure   UpdateButtons;

    procedure   ShowCellEditor(IdxBand:integer; IdxCell:integer; Key:char);
    procedure   CellEditorChange(Sender: TObject);
    procedure   ShowBandEditor(IdxBand:integer);
    procedure   BandEditorChange(Sender: TObject);

    procedure   CancelEdit;
    procedure   CellEditorExit(Sender: TObject);
    procedure   CellEditorKeyPress(Sender:TObject; var Key: Char);

    procedure   DisplayHint(Sender: TObject);
    {$IFDEF CLX}
      function  DisplayHelp(HelpType:THelpType; HelpContext:THelpContext; HelpKeyword:string; HelpFile:string; var Handled:boolean):boolean;
    {$ELSE}
      function  DisplayHelp(Command:Word; Data:Longint; var CallHelp:Boolean):Boolean;
    {$ENDIF}
    procedure   ShowHelpPage;

    procedure   ShowRecentFiles;
    procedure   LoadRecentFiles;
    procedure   SaveRecentFiles;

    procedure   AddToRecentFiles(const AFileName:string);
    procedure   RecentFilesMenuClick(Sender:TObject);

//    procedure   LoadLastEditedFiles;
    procedure   SaveLastEditedFiles;

    {$IFDEF D3}
    {$ELSE}
      {$IFDEF CLX}
      {$ELSE}
        procedure   ControlStartDock(Sender: TObject; var DragObject: TDragDockObject);
      {$ENDIF}
    {$ENDIF}

    procedure   CreateRulers;

    {$IFNDEF D3}
      procedure   ToolBarEndDock(Sender, Target: TObject; X, Y: Integer);
    {$ENDIF}
    function    CreateToolBar(const ACaption:string):TARToolBar;
    procedure   CreateToolButton(ACategory:string; const ACaption:string);
    procedure   LoadToolBars;
    procedure   SaveToolBars;

    procedure   UpdateRulerBars;
    procedure   CreateActions;
    function    GetMenuItem(const MenuItemName:string):TMenuItem;
    procedure   AddMenuItem(const ACategory:string; const ACaption:string; const AName:string);
    procedure   AddSubMenuItem(const ACategory:string; const AMenuItemName:string; const ACaption:string; const AName:string);
    procedure   AddPopupMenuItem(PopupMenu:TpopupMenu; const ACategory:string; const ACaption:string; const AName:string);
    procedure   CreateMainMenu;

    procedure   CreateBnt           (Sender:TARAction; AOwner:TComponent; Parent:TParControl; var Control:TControl);
    procedure   CreateCheckedBnt    (Sender:TARAction; AOwner:TComponent; Parent:TParControl; var Control:TControl);
    procedure   CreateColorBnt      (Sender:TARAction; AOwner:TComponent; Parent:TParControl; var Control:TControl);
    procedure   CreateGridBnt       (Sender:TARAction; AOwner:TComponent; Parent:TParControl; var Control:TControl);

    procedure   CreateLeftGridBnt   (Sender:TARAction; AOwner:TComponent; Parent:TParControl; var Control:TControl);
    procedure   CreateRightGridBnt  (Sender:TARAction; AOwner:TComponent; Parent:TParControl; var Control:TControl);
    procedure   CreateTopGridBnt    (Sender:TARAction; AOwner:TComponent; Parent:TParControl; var Control:TControl);
    procedure   CreateBottomGridBnt (Sender:TARAction; AOwner:TComponent; Parent:TParControl; var Control:TControl);

    procedure   CreateLeftGridDec   (Sender:TARAction; AOwner:TComponent; Parent:TParControl; var Control:TControl);
    procedure   CreateRightGridDec  (Sender:TARAction; AOwner:TComponent; Parent:TParControl; var Control:TControl);
    procedure   CreateTopGridDec    (Sender:TARAction; AOwner:TComponent; Parent:TParControl; var Control:TControl);
    procedure   CreateBottomGridDec (Sender:TARAction; AOwner:TComponent; Parent:TParControl; var Control:TControl);

    procedure   CreateFontNameBnt   (Sender:TARAction; AOwner:TComponent; Parent:TParControl; var Control:TControl);
    procedure   CreateComboMoreBnt  (Sender:TARAction; AOwner:TComponent; Parent:TParControl; var Control:TControl);
    procedure   CreateFontSizeBnt   (Sender:TARAction; AOwner:TComponent; Parent:TParControl; var Control:TControl);

    procedure   FrameFrameClick(Sender: TObject);

    procedure   ShowToolBarClick(Sender: TObject);
    procedure   ShowCustomizeDlg(Sender: TObject);
    procedure   GetDocColorEvent(Sender:TObject);

    procedure   RepSetFocus;
    procedure   OptionsPreferencesExecute(Sender:TObject);
    procedure   CellsHMergeCellsExecute(Sender:TObject);
    procedure   CellsVMergeCellsExecute(Sender:TObject);

    function    GetUndo:TUndo;
    function    GetFileName:string;
    procedure   SetFileName(const AFileName:string);
    function    GetModified:boolean;
    procedure   SetModified(AModified:boolean);

//    procedure   ShowPropTemplate;
  protected
    HRB         :TARRulerBar;
    VRB         :TARRulerBar;
    FRep        :TAREditRep;
    TemplateList:TAIStringList;

    function    GetRep:TAREditRep;
    function    GetFileSystem:TCommonFileSystem;

    function    NewPage(PageType:TAREPageType):TTabSheet;
    function    NewTemplatePage:TTabSheet;
    function    NewHelpPage:TTabSheet;

    property    Undo:TUndo              read  GetUndo;
    property    FileName:string         read  GetFileName   write SetFileName;
    property    Modified:boolean        read  GetModified   write SetModified;

    procedure   UpdateCaption;
    function    ShowPage(const AFileName:string):boolean;
    function    CloseAllPages:boolean;
    function    ClosePage(SaveChanges:boolean; QueryToSave:boolean):boolean;
  public
    PropEditor    :boolean;
    ActList       :TARActionList;
    MainControlBar:TControlBar;

    procedure   SetEditingRep(ARep:TARGridRep);

    procedure   CreateDefaultButtons;
    procedure   ResizeToolBars;
    property    Rep:TAREditRep               read  GetRep;
    property    FileSystem:TCommonFileSystem read  GetFileSystem;
    procedure   RestoreDefaultTools;

    property    GridX:extended          read  GetGridX      write SetGridX;
    property    GridY:extended          read  GetGridY      write SetGridY;
  end;

var
  AREditorDlg: TAREditorDlg;

procedure ResizeToolBar(tb:TARToolBar);

implementation

{$R *.dfm}

procedure ResizeToolBar(tb:TARToolBar);
var i, w:integer;
begin
  w:=0;
  for i:=0 to tb.ControlCount - 1 do begin
    w:=w + tb.Controls[i].Width;
  end;
  tb.Width:=w;
end;

procedure TAREditorDlg.EditSelectAllExecute(Sender: TObject);
begin
  Rep.SelAll;
end;

procedure TAREditorDlg.FormatLeftAlignExecute(Sender: TObject);
var i:integer;
begin
  for i:=0 to rep.SelCount-1 do begin
    rep.Sel[i].HAlign:=ahLeft;
  end;
  Modified:=true;
end;

procedure TAREditorDlg.FormatCenterAlignExecute(Sender: TObject);
var i:integer;
begin
  for i:=0 to rep.SelCount-1 do begin
    rep.Sel[i].HAlign:=ahCenter;
  end;
  Modified:=true;
end;

procedure TAREditorDlg.FormatRightAlignExecute(Sender: TObject);
var i:integer;
begin
  for i:=0 to rep.SelCount-1 do begin
    rep.Sel[i].HAlign:=ahRight;
  end;
  Modified:=true;
end;

procedure TAREditorDlg.FrameNotFrameExecute(Sender: TObject);
var i:integer;
begin
  for i:=0 to rep.SelCount-1 do begin
    rep.Sel[i].FrameStyle:=[];
  end;
  Modified:=true;
end;

procedure TAREditorDlg.OptionsUserVarsDialogExecute(Sender: TObject);
var d:TUserVarListDialog;//TObjectsDialog;
begin
  d:=TUserVarListDialog.Create(Self);//TObjectsDialog.Create(self);
  try
    d.Objects.Assign(UserVarList);
    if d.ShowModal = mrOK then begin
      UserVarList.Assign(d.Objects);
      Modified:=true;
    end;
  finally
    d.Free;
  end;
end;

procedure TAREditorDlg.OptionsUserVarsExecute(Sender: TObject);
var d:TUserVarListDialog;//TObjectsDialog;
    i:integer;
begin
  if Sender is TMenuItem then begin
    d:=TUserVarListDialog.Create(Self);//TObjectsDialog.Create(self);
    try
      d.Objects.Assign(UserVarList);
      if d.ShowModal = mrOK then begin
        UserVarList.Assign(d.Objects);
        Modified:=true;
      end;
    finally
      d.Free;
    end;
  end else
  if Sender is TARComboButton then begin
    for i:=0 to rep.SelCount-1 do begin
      rep.Sel[i].UserVar:=TARComboButton(Sender).Value;
    end;
    Modified:=true;
    RepSetFocus;
  end;
end;

procedure TAREditorDlg.FormatFontExecute(Sender: TObject);
var i:integer;
begin
  if rep.SelCount > 0 then begin
    FontDialog.Font.Name    :=rep.CurrCell.FontName;
    FontDialog.Font.Size    :=rep.CurrCell.FontSize;
    FontDialog.Font.Color   :=rep.CurrCell.FontColor;
    FontDialog.Font.Style   :=rep.CurrCell.FontStyle;
    FontDialog.Font.Charset :=rep.SrcRep.Font.Charset;
    if FontDialog.Execute then begin
      for i:=0 to rep.SelCount-1 do begin
        rep.Sel[i].FontName     :=FontDialog.Font.Name;
        rep.Sel[i].FontSize     :=FontDialog.Font.Size;
        rep.Sel[i].FontColor    :=FontDialog.Font.Color;
        rep.Sel[i].FontStyle    :=FontDialog.Font.Style;
        rep.SrcRep.Font.Charset :=FontDialog.Font.Charset;
      end;
    end;
  end;
  Modified:=true;
end;

procedure TAREditorDlg.BandsAddBandExecute(Sender: TObject);
begin
  rep.NewBand;
  rep.ClearSel;
  rep.CurrBandIdx:=rep.SrcRep.BandCount - 1;
  rep.SelAdd(rep.CurrBandIdx, rep.CurrCellIdx);
  Modified:=true;
end;

procedure TAREditorDlg.FileSaveExecute(Sender: TObject);
begin
  SaveReport;
end;

procedure TAREditorDlg.BandsDeleteBandExecute(Sender: TObject);
var idxband:integer;
begin
  for idxband:=rep.SrcRep.BandCount - 1 downto 0 do begin
    if rep.SrcRep.Bands[idxband].Selected then begin
      rep.DeleteBand(idxband);
    end;
  end;
  rep.Invalidate;
  Modified:=true;
end;

procedure TAREditorDlg.CellsDeleteCellExecute(Sender: TObject);
begin
  if rep.Focused and (rep.SelCount > 0) then begin
    rep.DeleteSelCells;
    rep.SelAdd(rep.CurrBandIdx, rep.CurrCellIdx);
    rep.Invalidate;
    Modified:=true;
  end;
end;

procedure TAREditorDlg.CellsAddCellExecute(Sender: TObject);
begin
  if Assigned(rep.CurrBand) then begin
    rep.CurrBand.NewCell;
    rep.CurrCellIdx:=rep.CurrBand.CellCount - 1;
    rep.ClearSel;
    rep.SelAdd(rep.CurrBandIdx, rep.CurrCellIdx);
    Modified:=true;
  end;
end;

procedure TAREditorDlg.CellsSplitCellExecute(Sender: TObject);
begin
  if Assigned(rep.CurrBand) and Assigned(rep.CurrCell) then begin
    rep.CurrBand.Split(rep.CurrCell);
    rep.Invalidate;
    Modified:=true;
  end;
end;

procedure TAREditorDlg.BandsBandUpExecute(Sender: TObject);
var idxband:integer;
    ismodified:boolean;
begin
  ismodified:=false;
  for idxband:=0 to rep.SrcRep.BandCount - 1 do begin
    if rep.SrcRep.Bands[idxband].Selected then begin
      if not rep.SrcRep.MoveBand(idxband, idxband - 1) then break;
      ismodified:=true;
    end;
  end;
  if ismodified then begin
    rep.CurrBandIdx:=rep.CurrBandIdx - 1;
    rep.Invalidate;
    Modified:=true;
  end;
end;

procedure TAREditorDlg.BandsBandDownExecute(Sender: TObject);
var idxband:integer;
    ismodified:boolean;
begin
  ismodified:=false;
  for idxband:=rep.SrcRep.BandCount - 1 downto 0 do begin
    if rep.SrcRep.Bands[idxband].Selected then begin
      if not rep.SrcRep.MoveBand(idxband, idxband + 1) then break;
      ismodified:=true;
    end;
  end;
  if ismodified then begin
    rep.CurrBandIdx:=rep.CurrBandIdx + 1;
    rep.Invalidate;
    Modified:=true;
  end;
end;

procedure TAREditorDlg.CellsCellMoveLeftExecute(Sender: TObject);
var idxband, idxcell:integer;
    ismodified:boolean;
begin
  ismodified:=false;
  if rep.SelCount > 0 then begin
    for idxband:=0 to rep.SrcRep.BandCount - 1 do begin
      for idxcell:=1 to rep.SrcRep.Bands[idxband].CellCount - 1 do begin
        if rep.SrcRep.Bands[idxband].Cells[idxcell].Selected then begin
          rep.SrcRep.Bands[idxband].MoveCell(idxcell, idxcell - 1);
          ismodified:=true;
        end;
      end;
    end;
  end else begin
    if rep.CurrCellIdx > 0 then begin
      rep.SrcRep.Bands[rep.CurrBandIdx].MoveCell(rep.CurrCellIdx, rep.CurrCellIdx - 1);
      ismodified:=true;
    end;
  end;
  if ismodified then begin
    rep.CurrCellIdx:=rep.CurrCellIdx - 1;
    rep.Invalidate;
    Modified:=true;
  end;
end;

procedure TAREditorDlg.CellsCellMoveRightExecute(Sender: TObject);
var idxband, idxcell:integer;
    ismodified:boolean;
begin
  ismodified:=false;
  if rep.SelCount > 0 then begin
    for idxband:=0 to rep.SrcRep.BandCount - 1 do begin
      for idxcell:=rep.SrcRep.Bands[idxband].CellCount - 2 downto 0 do begin
        if rep.SrcRep.Bands[idxband].Cells[idxcell].Selected then begin
          rep.SrcRep.Bands[idxband].MoveCell(idxcell, idxcell + 1);
          ismodified:=true;
        end;
      end;
    end;
  end else begin
    if rep.CurrCellIdx < rep.SrcRep.Bands[rep.CurrBandIdx].CellCount - 1 then begin
      rep.SrcRep.Bands[rep.CurrBandIdx].MoveCell(rep.CurrCellIdx, rep.CurrCellIdx + 1);
      ismodified:=true;
    end;
  end;
  if ismodified then begin
    rep.CurrCellIdx:=rep.CurrCellIdx + 1;
    rep.Invalidate;
    Modified:=true;
  end;
end;

procedure TAREditorDlg.EditInsertPictureExecute(Sender: TObject);
var i:integer;
    dlg:TOpenPictureDialog;
begin
  dlg:=TOpenPictureDialog.Create(Self);
  try
    dlg.Filter:='All (*.bmp;*.ico;*.emf;*.wmf)|*.bmp;*.ico;*.emf;*.wmf|Bitmaps (*.bmp)|*.bmp|Icons (*.ico)|*.ico|Enhanced Metafiles (*.emf)|*.emf|Metafiles (*.wmf)|*.wmf';

    (*
    {$IFDEF D3}
      dlg.Filter:='All (*.bmp;*.ico;*.emf;*.wmf)|*.bmp;*.ico;*.emf;*.wmf|Bitmaps (*.bmp)|*.bmp|Icons (*.ico)|*.ico|Enhanced Metafiles (*.emf)|*.emf|Metafiles (*.wmf)|*.wmf';
    {$ELSE}
      dlg.Filter:='All (*.jpg;*.jpeg;*.bmp;*.ico;*.emf;*.wmf)|*.jpg;*.jpeg;*.bmp;*.ico;*.emf;*.wmf|JPEG Image File (*.jpg)|*.jpg|JPEG Image File (*.jpeg)|*.jpeg|Bitmaps (*.bmp)|*.bmp|Icons (*.ico)|*.ico|Enhanced Metafiles (*.emf)|*.emf|Metafiles (*.wmf)|*.wmf';
    {$ENDIF}
    *)

    if dlg.Execute then begin
      for i:=0 to rep.SelCount-1 do begin
        rep.Sel[i].PictureFileName:=dlg.FileName;
      end;
      Modified:=true;
    end;
  finally
    dlg.Free;
  end;
end;

procedure TAREditorDlg.EditFitToCellExecute(Sender: TObject);
var i:integer;
begin
  for i:=0 to rep.SelCount-1 do begin
    rep.Sel[i].AutoSize:=not rep.Sel[i].AutoSize;
  end;
  Modified:=true;
end;

procedure TAREditorDlg.EditTileExecute(Sender: TObject);
var i:integer;
begin
  for i:=0 to rep.SelCount-1 do begin
    rep.Sel[i].Tile:=not rep.Sel[i].Tile;
  end;
  Modified:=true;
end;

procedure TAREditorDlg.CellsAutoStretchExecute(Sender: TObject);
var i:integer;
begin
  TARAction(TControl(Sender).Owner).Checked:=not TARAction(TControl(Sender).Owner).Checked;
  for i:=0 to rep.SelCount-1 do begin
    rep.Sel[i].AutoStretch:=TARAction(TControl(Sender).Owner).Checked;
  end;
  Modified:=true;
end;

procedure TAREditorDlg.FilePrintExecute(Sender: TObject);
begin
  Rep.Print;
end;

procedure TAREditorDlg.FilePreviewExecute(Sender: TObject);
begin
  Rep.Preview;
end;

procedure TAREditorDlg.CellsWordWrapExecute(Sender: TObject);
var i:integer;
begin
  TARAction(TControl(Sender).Owner).Checked:=not TARAction(TControl(Sender).Owner).Checked;

  for i:=0 to rep.SelCount-1 do begin
    rep.Sel[i].WordWrap:=TARAction(TControl(Sender).Owner).Checked;
  end;
  Modified:=true;
end;

procedure TAREditorDlg.FormDestroy(Sender: TObject);
begin
  CellEditor.Free;
  prn.Free;

  ActList.Free;
  UserVarList.Free;
  TemplateList.Free;
  RecentFiles.Free;
end;

procedure TAREditorDlg.FileOpenExecute(Sender: TObject);
var i:integer;
begin
  FileSystem.Options:=FileSystem.Options + [ofAllowMultiSelect];
  if FileSystem.OpenDialogExecute then begin
    for i:=0 to FileSystem.Files.Count - 1 do begin
      LoadReport(FileSystem.Files[i]);
    end;
  end;
end;

procedure TAREditorDlg.DisplayHint(Sender: TObject);
begin
  StatusBar.Panels[2].Text:=GetLongHint(Application.Hint);
//  StatusBar.Panels[2].Text:=Application.Hint;
end;

procedure TAREditorDlg.FormClose(Sender: TObject; var Action: TCloseAction);
var i:integer;
    {$IFDEF LINUX}
      reg:TIniFile;
    {$ENDIF}
    {$IFDEF MSWINDOWS}
      reg:TRegistry;
    {$ENDIF}
begin
  Application.OnHint:=OldApplicationOnHint;
  Application.OnHelp:=OldApplicationOnHelp;

  SaveToolBars;
  SaveRecentFiles;

  Action:= caHide;

  {$IFDEF LINUX}
    reg:=TIniFile.Create(Application.ExeName + '.ini');
  {$ENDIF}
  {$IFDEF MSWINDOWS}
    reg:=TRegistry.Create;
  {$ENDIF}
  try
    {$IFDEF LINUX}
      reg.WriteInteger('Setup\', 'Left',          Left);
      reg.WriteInteger('Setup\', 'Width',         Width);
      reg.WriteInteger('Setup\', 'Top',           Top);
      reg.WriteInteger('Setup\', 'Height',        Height);

      reg.WriteInteger('Setup\', 'UnitsIndex',    Ord(Units));
      reg.WriteFloat  ('Setup\', 'GridX',         GridX);
      reg.WriteFloat  ('Setup\', 'GridY',         GridY);

      reg.WriteString ('Setup\', 'LastEditedFile',FileSystem.FileName);

      reg.WriteString ('Setup\', 'FileSystem',    FileSystem.DisplayName);
      reg.WriteString ('Setup\' + FileSystem.DisplayName + '\', 'Params',  FileSystem.Params);

      for i:=0 to UserVarList.Count - 1 do begin
        reg.WriteInteger('Setup\Objects\', UserVarList[i], integer(UserVarList.Objects[i]));
      end;
    {$ENDIF}
    {$IFDEF MSWINDOWS}
      //reg.RootKey := HKEY_CURRENT_USER;

      reg.OpenKey(RegRoot + 'Setup\', True);
      reg.WriteInteger('Left',            Left);
      reg.WriteInteger('Width',           Width);
      reg.WriteInteger('Top',             Top);
      reg.WriteInteger('Height',          Height);

      reg.WriteInteger('UnitsIndex',      Ord(Units));
      reg.WriteFloat  ('GridX',           GridX);
      reg.WriteFloat  ('GridY',           GridY);

      reg.WriteString ('LastEditedFile',  FileSystem.FileName);

      reg.WriteString ('FileSystem',      FileSystem.DisplayName);
      reg.OpenKey(RegRoot + 'Setup\' + FileSystem.DisplayName + '\', True);
      reg.WriteString ('Params',          FileSystem.Params);

      reg.OpenKey(RegRoot + 'Setup\Objects\', True);
      for i:=0 to UserVarList.Count - 1 do begin
        reg.WriteInteger(UserVarList[i], integer(UserVarList.Objects[i]));
      end;
    {$ENDIF}
  finally
    {$IFDEF LINUX}
      reg.UpdateFile;
    {$ENDIF}
    reg.Free;
  end;
end;

procedure TAREditorDlg.RepResizeBand(Sender: TObject; IdxBand: Integer);
begin
  rep.Repaint;
  Modified:=true;
end;

procedure TAREditorDlg.OptionsDefaultFontExecute(Sender: TObject);
begin
  FontDialog.Font.Assign(rep.SrcRep.Font);
  if FontDialog.Execute then begin
    rep.SrcRep.Font.Assign(FontDialog.Font);
  end;
  Modified:=true;
end;

procedure TAREditorDlg.FormatBoldExecute(Sender: TObject);
var i:integer;
begin
  TARAction(TControl(Sender).Owner).Checked:=not TARAction(TControl(Sender).Owner).Checked;

  for i:=0 to rep.SelCount-1 do begin
    if TARAction(TControl(Sender).Owner).Checked then begin
      rep.Sel[i].FontStyle:=rep.Sel[i].FontStyle + [fsBold];
    end else begin
      rep.Sel[i].FontStyle:=rep.Sel[i].FontStyle - [fsBold];
    end;
  end;
  Modified:=true;
end;

procedure TAREditorDlg.FormatItalicExecute(Sender: TObject);
var i:integer;
begin
  TARAction(TControl(Sender).Owner).Checked:=not TARAction(TControl(Sender).Owner).Checked;

  for i:=0 to rep.SelCount-1 do begin
    if TARAction(TControl(Sender).Owner).Checked then begin
      rep.Sel[i].FontStyle:=rep.Sel[i].FontStyle + [fsItalic];
    end else begin
      rep.Sel[i].FontStyle:=rep.Sel[i].FontStyle - [fsItalic];
    end;
  end;
  Modified:=true;
end;

procedure TAREditorDlg.FormatUnderlineExecute(Sender: TObject);
var i:integer;
begin
  TARAction(TControl(Sender).Owner).Checked:=not TARAction(TControl(Sender).Owner).Checked;

  for i:=0 to rep.SelCount-1 do begin
    if TARAction(TControl(Sender).Owner).Checked then begin
      rep.Sel[i].FontStyle:=rep.Sel[i].FontStyle + [fsUnderline];
    end else begin
      rep.Sel[i].FontStyle:=rep.Sel[i].FontStyle - [fsUnderline  ];
    end;
  end;
  Modified:=true;
end;

procedure TAREditorDlg.FormatToplayoutExecute(Sender: TObject);
var i:integer;
begin
  for i:=0 to rep.SelCount-1 do begin
    rep.Sel[i].VAlign:=avTop;
  end;
  Modified:=true;
end;

procedure TAREditorDlg.FormatCenterlayoutExecute(Sender: TObject);
var i:integer;
begin
  for i:=0 to rep.SelCount-1 do begin
    rep.Sel[i].VAlign:=avCenter;
  end;
  Modified:=true;
end;

procedure TAREditorDlg.UserVarListChange(Sender: TObject);
begin
  if Sender is TAIStringList then begin
    ActList[Category_Options, rsAREditor_OptionsUserVars].StrList:=TAIStringList(Sender);
  end;
end;

procedure TAREditorDlg.FormatBottomlayoutExecute(Sender: TObject);
var i:integer;
begin
  for i:=0 to rep.SelCount-1 do begin
    rep.Sel[i].VAlign:=avBottom;
  end;
  Modified:=true;
end;

procedure TAREditorDlg.FrameFrameClick(Sender: TObject);
var i:integer;
begin
  for i:=0 to rep.SelCount-1 do begin
    if (TARAction(TControl(Sender).Owner).GridKind * rep.Sel[i].FrameStyle) <> [] then begin
      rep.Sel[i].FrameStyle:=rep.Sel[i].FrameStyle - TARAction(TControl(Sender).Owner).GridKind;
    end else begin
      rep.Sel[i].FrameStyle:=rep.Sel[i].FrameStyle + TARAction(TControl(Sender).Owner).GridKind;
    end;
  end;
  Modified:=true;
end;

procedure TAREditorDlg.FrameFrameExecuteInc(Sender: TObject);
var i:integer;
    j:TFrameStyles;
begin
  if Sender is TMenuItem then begin
    for i:=0 to rep.SelCount-1 do begin
      for j:=Low(TFrameStyles) to High(TFrameStyles) do begin
        if j in TARAction(TControl(Sender).Owner).GridKind then begin
          rep.Sel[i].FrameWidths[j]:=Min(rep.Sel[i].FrameWidths[j] + 1, 7);
        end;
      end;
    end;
  end else begin
    if Assigned(rep) then begin
      for i:=0 to rep.SelCount-1 do begin
        for j:=Low(TFrameStyles) to High(TFrameStyles) do begin
          if j in TARGridButton(Sender).GridKind then begin
            rep.Sel[i].FrameWidths[j] :=TARGridButton(Sender).GridWidth;
            rep.Sel[i].FrameColors[j] :=TARGridButton(Sender).Color;
            rep.Sel[i].BorderStyles[j]:=TARGridButton(Sender).GridStyle;
          end;
        end;
      end;
    end;
  end;
  Modified:=true;
end;

procedure TAREditorDlg.FrameFrameExecuteDec(Sender: TObject);
var i:integer;
    j:TFrameStyles;
begin
  if Sender is TMenuItem then begin
    for i:=0 to rep.SelCount-1 do begin
      for j:=Low(TFrameStyles) to High(TFrameStyles) do begin
        if j in TARAction(TControl(Sender).Owner).GridKind then begin
          rep.Sel[i].FrameWidths[j]:=Max(rep.Sel[i].FrameWidths[j] - 1, 0);
        end;
      end;
    end;
  end;
  Modified:=true;
end;

procedure TAREditorDlg.BandsBandNameExecute(Sender: TObject);
begin
  if rep.Focused then begin
    ShowBandEditor(rep.CurrBandIdx);
  end else begin
    if CellEditor.Focused then begin
      repSetFocus;
    end else begin
    end;
  end;
end;

procedure TAREditorDlg.FileExitExecute(Sender: TObject);
begin
  if Active and (ActiveControl = rep)  then begin
    repSetFocus;
    Close;
  end else begin
    {$IFDEF CLX}
    {$ELSE}
      ActiveControl.Perform(WM_CHAR, $1b, 0);
    {$ENDIF}
  end;
end;

procedure TAREditorDlg.CancelEdit;
begin
  CellEditor.Text:=OldEditValue;
end;

procedure TAREditorDlg.CellEditorChange(Sender: TObject);
var i:integer;
begin
  if fEnableUpdateEditText then begin
    for i:=0 to rep.SelCount-1 do begin
      rep.Sel[i].Value:=CellEditor.Text;
      rep.Sel[i].ProcessValue;
    end;
    Modified:=true;
  end;
end;

procedure TAREditorDlg.CellEditorExit(Sender: TObject);
begin
  ActList[Category_Cells, rsAREditor_CellsDeleteCell].ShortCut  :=TextToShortCut('Del');
  ActList[Category_Cells, rsAREditor_CellsCopyCells].ShortCut   :=TextToShortCut('Ctrl+C');
  ActList[Category_Cells, rsAREditor_CellsCutCells].ShortCut    :=TextToShortCut('Ctrl+X');
  ActList[Category_Cells, rsAREditor_CellsPasteCells].ShortCut  :=TextToShortCut('Ctrl+V');
//  ActList[Category_Cells, rsAREditor_CellsDeleteCell].Enabled :=true;
//  ActList[Category_Cells, rsAREditor_CellsCopyCells].Enabled  :=true;
//  ActList[Category_Cells, rsAREditor_CellsCutCells].Enabled   :=true;
//  ActList[Category_Cells, rsAREditor_CellsPasteCells].Enabled :=true;

  CellEditor.Visible:=false;
  CellEditor.Parent:=nil;
  rep.InEditing:=false;
end;

procedure TAREditorDlg.CellEditorKeyPress(Sender:TObject; var Key: Char);
begin
  if Key = #$1b then begin
    CancelEdit;
    repSetFocus;
    Key:=#$0;
  end;
  inherited;
end;

procedure TAREditorDlg.ShowBandEditor(IdxBand:integer);
var r:TRect;
    Band:TARBand;
begin
  RepSetFocus;

  ActList[Category_Cells, rsAREditor_CellsDeleteCell].ShortCut  :=TextToShortCut('');
  ActList[Category_Cells, rsAREditor_CellsCopyCells].ShortCut   :=TextToShortCut('');
  ActList[Category_Cells, rsAREditor_CellsCutCells].ShortCut    :=TextToShortCut('');
  ActList[Category_Cells, rsAREditor_CellsPasteCells].ShortCut  :=TextToShortCut('');
//  ActList[Category_Cells, rsAREditor_CellsDeleteCell].Enabled :=false;
//  ActList[Category_Cells, rsAREditor_CellsCopyCells].Enabled  :=false;
//  ActList[Category_Cells, rsAREditor_CellsCutCells].Enabled   :=false;
//  ActList[Category_Cells, rsAREditor_CellsPasteCells].Enabled :=false;

  rep.CurrBandIdx:=IdxBand;

  band:=rep.SrcRep.Bands[IdxBand];

  r:=rep.CellRect(IdxBand, 0);
  r.Left  :=5;
  r.Right :=rep.Gutter - 5;
  r.Top   :=r.Top + (band.Height - Abs(rep.SrcRep.Font.Height) - 3) div 2 - 2;
  r.Bottom:=r.Top + Abs(rep.SrcRep.Font.Height) + 3;

  CellEditor.Parent:=rep;
  CellEditor.OnChange:=BandEditorChange;
  CellEditor.OnKeyPress:=CellEditorKeyPress;

  CellEditor.SetBounds(  r.Left - 1, r.Top + 2, r.Right - r.Left + 1, r.Bottom - r.Top + 1);
  CellEditor.BorderStyle  :=bsNone;
  CellEditor.Alignment    :=taLeftJustify       ;
  CellEditor.Color        :=clWhite;
  CellEditor.Font.Assign(rep.SrcRep.Font);

  fEnableUpdateEditText:=false;
    OldEditValue:=band.Name;
    CellEditor.Text:=band.Name;
  fEnableUpdateEditText:=true;

  CellEditor.Visible:=true;
  CellEditor.SelectAll;
  CellEditor.SetFocus;
end;

procedure TAREditorDlg.BandEditorChange(Sender: TObject);
var i:integer;
begin
  if fEnableUpdateEditText then begin
    rep.SrcRep.BeginUpdate;
    try
      for i:=0 to rep.SrcRep.BandCount - 1 do begin
        if rep.SrcRep.Bands[i].Selected then begin
          rep.SrcRep.Bands[i].Name:=CellEditor.Text;
          Modified:=true;
        end;
      end;
    finally
      rep.SrcRep.EndUpdate;
      rep.Invalidate;
    end;
  end;
end;

procedure TAREditorDlg.HelpAboutExecute(Sender: TObject);
var ad:TARAboutDlg;
begin
  ad:=TARAboutDlg.Create(Self);
  try
    ad.ShowModal();
  finally
    ad.Free();
  end;
end;

procedure TAREditorDlg.CutMenuItemClick(Sender: TObject);
begin
  CellEditor.CutToClipboard;
end;

procedure TAREditorDlg.CopyMenuItemClick(Sender: TObject);
begin
  CellEditor.CopyToClipboard;
end;

procedure TAREditorDlg.PasteMenuItemClick(Sender: TObject);
begin
  CellEditor.PasteFromClipboard;
end;

procedure TAREditorDlg.DeleteMenuItemClick(Sender: TObject);
begin
  CellEditor.SelText:='';
end;

procedure TAREditorDlg.EditUndoExecute(Sender: TObject);
begin
  try
    FSetUndo:=false;
    if Undo.CanUndo then begin
      Rep.ARTemplate.Text:=Undo.GetUndo;
      Rep.ClearSel;

      Rep.CurrBandIdx :=Min(Rep.CurrBandIdx, Rep.SrcRep.BandCount - 1);
      Rep.CurrCellIdx :=Min(Rep.CurrCellIdx, Rep.CurrBand.CellCount - 1);
      Rep.SelAdd(Rep.CurrBandIdx, Rep.CurrCellIdx);

      Modified:=true;
    end;
  finally
    Rep.Invalidate;
    FSetUndo:=true;
  end;
end;

procedure TAREditorDlg.EditRedoExecute(Sender: TObject);
begin
  try
    FSetUndo:=false;

    if Undo.CanRedo then begin
      Rep.ARTemplate.Text:=Undo.GetRedo;
      Rep.ClearSel;

      Rep.CurrBandIdx :=Min(Rep.CurrBandIdx, Rep.SrcRep.BandCount - 1);
      Rep.CurrCellIdx :=Min(Rep.CurrCellIdx, Rep.CurrBand.CellCount - 1);
      Rep.SelAdd(Rep.CurrBandIdx, Rep.CurrCellIdx);

      Modified:=true;
      UpdateButtons;
    end;
  finally
    Rep.Invalidate;
    FSetUndo:=true;
  end;
end;

procedure TAREditorDlg.FormatFontNameExecute(Sender: TObject);
var i:integer;
begin
  if Sender is TARFontButton then begin
    if rep.SelCount > 0 then begin
      for i:=0 to rep.SelCount - 1 do begin
        rep.Sel[i].FontName:=TARFontButton(Sender).Value;
      end;
      Modified:=true;
    end;
  end;
end;

procedure TAREditorDlg.UpdateButtons;
var fontname, uservar:string;
    i, fontsize:integer;
    fontcolor, bkcolor:integer;
    cellwidth:integer;
    bold, italic, underline:boolean;
    toplayout, centerlayout, bottomlayout:boolean;
    leftalign, centeralign, rightalign:boolean;
    notframe, leftframe, rightframe, topframe, bottomframe:boolean;
    autostretch, wordwrap, fittocell, tile:boolean;
    colors  :TColorArray;
    widths  :TWidthsArray;
    styles  :TBorderStylesArray;

begin
  if Assigned(rep.CurrCell) then begin
    if rep.SelCount > 0 then begin
      fontname  :=rep.Sel[0].FontName;
      uservar   :=rep.Sel[0].UserVar;
      fontsize  :=rep.Sel[0].FontSize;
      fontcolor :=rep.Sel[0].FontColor;

      widths[frLeft]  :=rep.Sel[0].FrameWidths[frLeft];
      widths[frTop]   :=rep.Sel[0].FrameWidths[frTop];
      widths[frRight] :=rep.Sel[0].FrameWidths[frRight];
      widths[frBottom]:=rep.Sel[0].FrameWidths[frBottom];

      colors[frLeft]  :=rep.Sel[0].FrameColors[frLeft];
      colors[frTop]   :=rep.Sel[0].FrameColors[frTop];
      colors[frRight] :=rep.Sel[0].FrameColors[frRight];
      colors[frBottom]:=rep.Sel[0].FrameColors[frBottom];

      styles[frLeft]  :=rep.Sel[0].BorderStyles[frLeft];
      styles[frTop]   :=rep.Sel[0].BorderStyles[frTop];
      styles[frRight] :=rep.Sel[0].BorderStyles[frRight];
      styles[frBottom]:=rep.Sel[0].BorderStyles[frBottom];

      bkcolor   :=rep.Sel[0].Color;
      cellwidth :=rep.Sel[0].Width;
      bold      :=fsBold      in rep.Sel[0].FontStyle;
      italic    :=fsItalic    in rep.Sel[0].FontStyle;
      underline :=fsUnderline in rep.Sel[0].FontStyle;
      toplayout   :=rep.Sel[0].VAlign = avTop;
      centerlayout:=rep.Sel[0].VAlign = avCenter;
      bottomlayout:=rep.Sel[0].VAlign = avBottom;
      leftalign   :=rep.Sel[0].HAlign = ahLeft;
      centeralign :=rep.Sel[0].HAlign = ahCenter;
      rightalign  :=rep.Sel[0].HAlign = ahRight;
      notframe    :=rep.Sel[0].FrameStyle = [];
      leftframe   :=frLeft    in rep.Sel[0].FrameStyle;
      rightframe  :=frRight   in rep.Sel[0].FrameStyle;
      topframe    :=frTop     in rep.Sel[0].FrameStyle;
      bottomframe :=frBottom  in rep.Sel[0].FrameStyle;
      autostretch :=rep.Sel[0].AutoStretch;
      tile        :=rep.Sel[0].Tile;
      wordwrap    :=rep.Sel[0].WordWrap;
      fittocell   :=rep.Sel[0].AutoSize;
      for i:=0 to rep.SelCount-1 do begin
        if not CT(rep.Sel[i].FontName, fontname)  then fontname   :='';
        if not CT(rep.Sel[i].UserVar,  uservar)   then uservar    :='';
        if rep.Sel[i].FontSize   <> fontsize      then fontsize   :=0;
        if rep.Sel[i].FontColor  <> fontcolor     then fontcolor  :=clNone;

        if rep.Sel[i].FrameWidths[frLeft]     <>  widths[frLeft]   then widths[frLeft]   :=-1;
        if rep.Sel[i].FrameWidths[frTop]      <>  widths[frTop]    then widths[frTop]    :=-1;
        if rep.Sel[i].FrameWidths[frRight]    <>  widths[frRight]  then widths[frRight]  :=-1;
        if rep.Sel[i].FrameWidths[frBottom]   <>  widths[frBottom] then widths[frBottom] :=-1;

        if rep.Sel[i].FrameColors[frLeft]     <>  colors[frLeft]   then colors[frLeft]   :=clNone;
        if rep.Sel[i].FrameColors[frTop]      <>  colors[frTop]    then colors[frTop]    :=clNone;
        if rep.Sel[i].FrameColors[frRight]    <>  colors[frRight]  then colors[frRight]  :=clNone;
        if rep.Sel[i].FrameColors[frBottom]   <>  colors[frBottom] then colors[frBottom] :=clNone;

        if rep.Sel[i].BorderStyles[frLeft]    <>  styles[frLeft]   then styles[frLeft]   :=cbsSolid;
        if rep.Sel[i].BorderStyles[frTop]     <>  styles[frTop]    then styles[frTop]    :=cbsSolid;
        if rep.Sel[i].BorderStyles[frRight]   <>  styles[frRight]  then styles[frRight]  :=cbsSolid;
        if rep.Sel[i].BorderStyles[frBottom]  <>  styles[frBottom] then styles[frBottom] :=cbsSolid;

        if rep.Sel[i].Color      <> bkcolor       then bkcolor    :=clNone;
        if rep.Sel[i].Width      <> cellwidth     then cellwidth  :=-1;
        bold        :=bold          and (fsBold       in rep.Sel[i].FontStyle);
        italic      :=italic        and (fsItalic     in rep.Sel[i].FontStyle);
        underline   :=underline     and (fsUnderline  in rep.Sel[i].FontStyle);
        leftframe   :=leftframe     and (frLeft       in rep.Sel[i].FrameStyle);
        rightframe  :=rightframe    and (frRight      in rep.Sel[i].FrameStyle);
        topframe    :=topframe      and (frTop        in rep.Sel[i].FrameStyle);
        bottomframe :=bottomframe   and (frBottom     in rep.Sel[i].FrameStyle);
        notframe    :=notframe      and (rep.Sel[i].FrameStyle  = []);
        toplayout   :=toplayout     and (rep.Sel[i].VAlign      = avTop);
        centerlayout:=centerlayout  and (rep.Sel[i].VAlign      = avCenter);
        bottomlayout:=bottomlayout  and (rep.Sel[i].VAlign      = avBottom);
        leftalign   :=leftalign     and (rep.Sel[i].HAlign      = ahLeft);
        centeralign :=centeralign   and (rep.Sel[i].HAlign      = ahCenter);
        rightalign  :=rightalign    and (rep.Sel[i].HAlign      = ahRight);
        autostretch :=autostretch   and (rep.Sel[i].AutoStretch);
        tile        :=tile          and (rep.Sel[i].Tile);
        wordwrap    :=wordwrap      and (rep.Sel[i].WordWrap);
        fittocell   :=fittocell     and (rep.Sel[i].AutoSize);
      end;

      ActList[Category_Format, rsAREditor_FormatFontName].Value:=fontname;

      if fontsize > 0 then begin
        ActList[Category_Format, rsAREditor_FormatFontSize].Value:=IntToStr(fontsize);
      end else begin
        ActList[Category_Format, rsAREditor_FormatFontSize].Value:='';
      end;

      ActList[Category_Format, rsAREditor_FormatFontColor].Color:=fontcolor;

      ActList[Category_Frame, rsAREditor_FrameNotFrame].Checked:=notframe;

      ActList[Category_Frame, rsAREditor_FrameFrameLeft]  .IntValue:=widths[frLeft];
      ActList[Category_Frame, rsAREditor_FrameFrameTop]   .IntValue:=widths[frTop];
      ActList[Category_Frame, rsAREditor_FrameFrameRight] .IntValue:=widths[frRight];
      ActList[Category_Frame, rsAREditor_FrameFrameBottom].IntValue:=widths[frBottom];

      if (widths[frLeft] = widths[frTop]) and (widths[frTop] = widths[frRight]) and (widths[frRight] = widths[frBottom]) then begin
        ActList[Category_Frame, rsAREditor_FrameFrameWidth].IntValue:=widths[frLeft];
      end else begin
        ActList[Category_Frame, rsAREditor_FrameFrameWidth].IntValue:=-1;
      end;

      ActList[Category_Frame, rsAREditor_FrameFrameLeft]  .Color:=colors[frLeft];
      ActList[Category_Frame, rsAREditor_FrameFrameTop]   .Color:=colors[frTop];
      ActList[Category_Frame, rsAREditor_FrameFrameRight] .Color:=colors[frRight];
      ActList[Category_Frame, rsAREditor_FrameFrameBottom].Color:=colors[frBottom];

      if (colors[frLeft] = colors[frTop]) and (colors[frTop] = colors[frRight]) and (colors[frRight] = colors[frBottom]) then begin
        ActList[Category_Frame, rsAREditor_FrameFrameColor].Color:=colors[frLeft];
      end else begin
        ActList[Category_Frame, rsAREditor_FrameFrameColor].Color:=clNone;
      end;

      ActList[Category_Frame, rsAREditor_FrameFrameLeft]  .BorderStyle:=styles[frLeft];
      ActList[Category_Frame, rsAREditor_FrameFrameTop]   .BorderStyle:=styles[frTop];
      ActList[Category_Frame, rsAREditor_FrameFrameRight] .BorderStyle:=styles[frRight];
      ActList[Category_Frame, rsAREditor_FrameFrameBottom].BorderStyle:=styles[frBottom];

      if (styles[frLeft] = styles[frTop]) and (styles[frTop] = styles[frRight]) and (styles[frRight] = styles[frBottom]) then begin
        ActList[Category_Frame, rsAREditor_FrameFrameWidth].BorderStyle:=styles[frLeft];
      end else begin
        ActList[Category_Frame, rsAREditor_FrameFrameWidth].IntValue:=-1;
      end;

      ActList[Category_Cells, rsAREditor_CellsBgcolor].Color:=bkcolor;
      ActList[Category_Options, rsAREditor_OptionsUserVars].Value:=uservar;
      if cellwidth <> -1 then begin
        StatusBar.Panels[1].Text:=Format('%.2f - %.2f %s', [ConvertUnits(rep.CurrCell.Width, uPix, Units), ConvertUnits(rep.CurrBand.Height, uPix, Units), UnitsShortName(Units)]);
      end else begin
        StatusBar.Panels[1].Text:='';
      end;
      ActList[Category_Format, rsAREditor_FormatBold].Checked     :=bold;
      ActList[Category_Format, rsAREditor_FormatItalic].Checked   :=italic;
      ActList[Category_Format, rsAREditor_FormatUnderLine].Checked:=underline;

      ActList[Category_Format, rsAREditor_FormatTopLayout].Checked    :=toplayout;
      ActList[Category_Format, rsAREditor_FormatCenterLayout].Checked :=centerlayout;
      ActList[Category_Format, rsAREditor_FormatBottomLayout].Checked :=bottomlayout;

      ActList[Category_Format, rsAREditor_FormatLeftAlign].Checked  :=leftalign;
      ActList[Category_Format, rsAREditor_FormatCenterAlign].Checked:=centeralign;
      ActList[Category_Format, rsAREditor_FormatRightAlign].Checked :=rightalign;

      ActList[Category_Cells, rsAREditor_CellsAutoStretch].Checked:=autostretch;
      ActList[Category_Edit,  rsAREditor_EditTile].Checked        :=tile;
      ActList[Category_Cells, rsAREditor_CellsWordWrap].Checked   :=wordwrap;
      ActList[Category_Edit, rsAREditor_EditFitToCell].Checked    :=fittocell;
    end;
  end else begin
    StatusBar.Panels[1].Text:='';
  end;

  ActList[Category_Edit, rsAREditor_EditUndo].Enabled:=Undo.CanUndo;
  ActList[Category_Edit, rsAREditor_EditRedo].Enabled:=Undo.CanRedo;

  CloseupAllPopupForms;
end;

procedure TAREditorDlg.FormatFontSizeExecute(Sender: TObject);
var i:integer;
begin
  if Sender is TARComboEditButton then begin
    for i:=0 to rep.SelCount-1 do begin
      rep.Sel[i].FontSize:=StrToInt(TARComboEditButton(Sender).Value);
    end;
    Modified:=true;
  end;
end;

procedure TAREditorDlg.FormatDecreaseFontSizeExecute(Sender: TObject);
var i:integer;
begin
  for i:=0 to rep.SelCount-1 do begin
    rep.Sel[i].FontSize:=Max(1, rep.Sel[i].FontSize - 1);
  end;
  Modified:=true;
end;

procedure TAREditorDlg.FormatIncreaseFontSizeExecute(Sender: TObject);
var i:integer;
begin
  for i:=0 to rep.SelCount-1 do begin
    rep.Sel[i].FontSize:=rep.Sel[i].FontSize + 1;
  end;
  Modified:=true;
end;

procedure TAREditorDlg.SetUnits(AUnits:TARUnits);
begin
  Units:=AUnits;

  ActList[Category_Options, rsAREditor_UnitsInches].Checked     :=false;
  ActList[Category_Options, rsAREditor_UnitsMillemeters].Checked:=false;
  ActList[Category_Options, rsAREditor_UnitsCentimeters].Checked:=false;
  ActList[Category_Options, rsAREditor_UnitsPixels].Checked     :=false;

  case Units of
    uIn:  ActList[Category_Options, rsAREditor_UnitsInches].Checked     :=true;
    uMM:  ActList[Category_Options, rsAREditor_UnitsMillemeters].Checked:=true;
    uCM:  ActList[Category_Options, rsAREditor_UnitsCentimeters].Checked:=true;
    uPix: ActList[Category_Options, rsAREditor_UnitsPixels].Checked     :=true;
  end;
  if Assigned(Rep) then begin
    Rep.Units:=Units;
    UpdateRulerBars;
    UpdateButtons;
  end;
end;

procedure TAREditorDlg.UnitsInchesExecute(Sender: TObject);
begin
  SetUnits(uIn);
end;

procedure TAREditorDlg.UnitsMillemetersExecute(Sender: TObject);
begin
  SetUnits(uMM);
end;

procedure TAREditorDlg.UnitsCentimetersExecute(Sender: TObject);
begin
  SetUnits(uCM);
end;

procedure TAREditorDlg.UnitsPixelsExecute(Sender: TObject);
begin
  SetUnits(uPix);
end;

procedure TAREditorDlg.FilePageSetupExecute(Sender: TObject);
var PageSetupDlg:TARPageSetupDlg;
begin
  PageSetupDlg:=TARPageSetupDlg.Create(Self);
  try
    PageSetupDlg.Units        :=Units;

    PageSetupDlg.LeftMargin   :=rep.SrcRep.LeftMargin;
    PageSetupDlg.TopMargin    :=rep.SrcRep.TopMargin;
    PageSetupDlg.RightMargin  :=rep.SrcRep.RightMargin;
    PageSetupDlg.BottomMargin :=rep.SrcRep.BottomMargin;

    PageSetupDlg.Orientation  :=rep.SrcRep.Orientation;
//    PageSetupDlg.PaperSize    :=rep.SrcRep.PaperSize;
    PageSetupDlg.PageSize     :=rep.SrcRep.PageSize;

    if PageSetupDlg.Execute then begin
      rep.SrcRep.LeftMargin   :=PageSetupDlg.LeftMargin;
      rep.SrcRep.TopMargin    :=PageSetupDlg.TopMargin;
      rep.SrcRep.RightMargin  :=PageSetupDlg.RightMargin;
      rep.SrcRep.BottomMargin :=PageSetupDlg.BottomMargin;

//      rep.SrcRep.PaperSize      :=PageSetupDlg.PaperSize;
      rep.SrcRep.PageSize     :=PageSetupDlg.PageSize;
      rep.SrcRep.Orientation  :=PageSetupDlg.Orientation;

      rep.Invalidate;
      Modified:=true;
    end;
  finally
    PageSetupDlg.Free;
  end;
end;


procedure TAREditorDlg.OptionsShowAllExecute(Sender: TObject);
begin
  ShowHiddenItems:=not ShowHiddenItems;

  TARAction(TComponent(Sender).Owner).Checked:=ShowHiddenItems;

  rep.Invalidate;
end;

procedure TAREditorDlg.ShowRecentFiles;
var i, j, idx:integer;
    mi, begmi, endmi, filemenu:TMenuItem;
begin
  try
    begmi:=GetMenuItem('RecentFilesBeginMenuItem');
    endmi:=GetMenuItem('RecentFilesEndMenuItem');
    filemenu:=GetMenuItem('FileMenu');

    for i:=0 to filemenu.Count - 1 do begin
      if filemenu.Items[i] = begmi then begin
        idx:=i + 1;

        while filemenu.Items[idx] <> endmi do begin
          mi:=filemenu.Items[idx];
          filemenu.Delete(idx);
          mi.Free;
        end;

        for j:=0 to RecentFiles.Count - 1 do begin
          mi:=TMenuItem.Create(Self);
          mi.Caption:='&' + Chr(Ord('1') + j) + ' ' + RecentFiles[j];
          mi.OnClick:=RecentFilesMenuClick;
          mi.Tag:=j;
          filemenu.Insert(idx + j, mi);
        end;

        break;
      end;
    end;
  except
  end;
end;

procedure TAREditorDlg.RecentFilesMenuClick(Sender:TObject);
begin
  LoadReport(RecentFiles[TMenuItem(Sender).Tag]);
end;

procedure   TAREditorDlg.SaveLastEditedFiles;
var i:integer;
    {$IFDEF LINUX}
      reg:TIniFile;
    {$ENDIF}
    {$IFDEF MSWINDOWS}
      reg:TRegistry;
    {$ENDIF}
begin
  if TemplateList.Count = 0 then exit;

  {$IFDEF LINUX}
    reg:=TIniFile.Create(Application.ExeName + '.ini');
    try
      reg.EraseSection('Setup\LastEditedFiles\');
      for i:=0 to TemplateList.Count - 1 do begin
        if not CT(TemplateList[i], rsAREditor_NoName) then begin
          reg.WriteString('Setup\LastEditedFiles\', TemplateList[i], '');
        end;
      end;
    finally
      reg.UpdateFile;
      reg.Free;
    end;
  {$ENDIF}
  {$IFDEF MSWINDOWS}
    reg:=TRegistry.Create;
    try
      reg.DeleteKey(RegRoot + 'Setup\LastEditedFiles');
      reg.OpenKey(RegRoot + 'Setup\LastEditedFiles', True);
      for i:=0 to TemplateList.Count - 1 do begin
        if not CT(TemplateList[i], rsAREditor_NoName) then begin
          reg.WriteString(TemplateList[i], '');
        end;
      end;
    finally
      reg.Free;
    end;
  {$ENDIF}
end;

procedure   TAREditorDlg.LoadRecentFiles;
var i:integer;
    {$IFDEF LINUX}
      reg:TIniFile;
    {$ENDIF}
    {$IFDEF MSWINDOWS}
      reg:TRegistry;
    {$ENDIF}
begin
  {$IFDEF LINUX}
    reg:=TIniFile.Create(Application.ExeName + '.ini');
    try
      reg.ReadSection('Setup\RecentFiles\', RecentFiles);
    finally
      reg.UpdateFile;
      reg.Free;
    end;
  {$ENDIF}
  {$IFDEF MSWINDOWS}
    reg:=TRegistry.Create;
    try
      reg.OpenKey(RegRoot + 'Setup\RecentFiles', True);
      reg.GetValueNames(RecentFiles);
    finally
      reg.Free;
    end;
  {$ENDIF}

  for i:=RecentFiles.Count - 1 downto 0 do begin
    if CT(RecentFiles[i], rsAREditor_NoName) then RecentFiles.Delete(0);
  end;

  while RecentFiles.Count > 9 do RecentFiles.Delete(9);

  ShowRecentFiles;
end;

procedure TAREditorDlg.SaveRecentFiles;
var i:integer;
    {$IFDEF LINUX}
      reg:TIniFile;
    {$ENDIF}
    {$IFDEF MSWINDOWS}
      reg:TRegistry;
    {$ENDIF}
begin
  {$IFDEF LINUX}
    reg:=TIniFile.Create(Application.ExeName + '.ini');
    try
      reg.EraseSection('Setup\RecentFiles\');
      for i:=0 to Min(RecentFiles.Count - 1, 8) do begin
        if not CT(RecentFiles[i], rsAREditor_NoName) then begin
          reg.WriteString('Setup\RecentFiles\', RecentFiles[i], '');
        end;
      end;
    finally
      reg.UpdateFile;
      reg.Free;
    end;
  {$ENDIF}
  {$IFDEF MSWINDOWS}
    reg:=TRegistry.Create;
    try
      reg.DeleteKey(RegRoot + 'Setup\RecentFiles');
      reg.OpenKey(RegRoot + 'Setup\RecentFiles', True);
      for i:=0 to Min(RecentFiles.Count - 1, 8) do begin
        if not CT(RecentFiles[i], rsAREditor_NoName) then reg.WriteString(RecentFiles[i], '');
      end;
    finally
      reg.Free;
    end;
  {$ENDIF}
end;

procedure TAREditorDlg.AddToRecentFiles(const AFileName:string);
var idx:integer;
begin
  if AFileName <> '' then begin
    idx:=RecentFiles.IndexOf(AFileName);
    if idx <> -1 then RecentFiles.Delete(idx);

    RecentFiles.Insert(0, AFileName);
    while RecentFiles.Count > 10 do RecentFiles.Delete(10);

    ShowRecentFiles;
  end;
end;

procedure TAREditorDlg.RepResizeGutter(Sender: TObject);
begin
  UpdateRulerBars;
end;

procedure TAREditorDlg.RepSelectCell(Sender: TObject; IdxBand, IdxCell: Integer);
begin
  if (IdxBand < rep.SrcRep.BandCount) and (IdxCell < rep.SrcRep.Bands[IdxBand].CellCount) then begin
    UpdateRulerBars;
    UpdateButtons;
  end;
end;

procedure TAREditorDlg.RepResizeCell(Sender: TObject; IdxBand, IdxCell: Integer);
begin
  rep.Repaint;
  Modified:=true;
end;

procedure TAREditorDlg.HRBChange(Sender: TObject);
begin
  Rep.SrcRep.LeftMargin :=HRB.LeftMargin;
  Rep.SrcRep.RightMargin:=HRB.RightMargin;
  Modified:=true;
end;

procedure TAREditorDlg.ShowCellEditor(IdxBand:integer; IdxCell:integer; Key:char);
var r:TRect;
    cell:TARCell;
begin
  RepSetFocus;

  ActList[Category_Cells, rsAREditor_CellsDeleteCell].ShortCut  :=TextToShortCut('');
  ActList[Category_Cells, rsAREditor_CellsCopyCells].ShortCut   :=TextToShortCut('');
  ActList[Category_Cells, rsAREditor_CellsCutCells].ShortCut    :=TextToShortCut('');
  ActList[Category_Cells, rsAREditor_CellsPasteCells].ShortCut  :=TextToShortCut('');

  r:=rep.CellRect(IdxBand, IdxCell);

  cell:=rep.SrcRep.Cells[IdxBand, IdxCell];

  CellEditor.Parent   :=rep;
  CellEditor.OnChange :=CellEditorChange;
  CellEditor.OnKeyPress:=CellEditorKeyPress;

  CellEditor.SetBounds(  r.Left, r.Top, r.Right - r.Left, r.Bottom - r.Top);
  CellEditor.BorderStyle  :=bsNone;
  CellEditor.Alignment    :=TAlignment(cell.HAlign);
  CellEditor.Color        :=cell.Color;
  CellEditor.Font.Size    :=cell.FontSize;
  CellEditor.Font.Style   :=cell.FontStyle;
  CellEditor.Font.Name    :=cell.FontName;
  CellEditor.Font.Color   :=cell.FontColor;
  CellEditor.Font.Charset :=rep.SrcRep.Font.Charset;

  fEnableUpdateEditText:=false;
    OldEditValue:=cell.Value;
    CellEditor.Text:=cell.Value;
  fEnableUpdateEditText:=true;

  CellEditor.Visible:=true;
  CellEditor.SelectAll;

  if Key <> #0 then begin
    {$IFDEF CLX}
    {$ELSE}
      CellEditor.Perform(WM_CHAR, integer(Key), 0);
    {$ENDIF}
  end;

  CellEditor.SetFocus;

  rep.InEditing:=true;
end;

procedure TAREditorDlg.RepKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = #$0d then begin
    ShowCellEditor(rep.CurrBandIdx, rep.CurrCellIdx, #$0);
  end else
  if Key >= ' ' then begin
    ShowCellEditor(rep.CurrBandIdx, rep.CurrCellIdx, Key);
  end;
end;

procedure TAREditorDlg.RepMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
  HRB.Cur:=rep.Left + X + 1;
  VRB.Cur:=Y + 1;
end;

procedure TAREditorDlg.UpdateRulerBars;
var idxband, idxcell, hselbegin, hselend, vselbegin, vselend:integer;
begin
    hselbegin:= 1000000; hselend:=-1;
    vselbegin:= 1000000; vselend:=-1;
    for idxband:=0 to Rep.SrcRep.BandCount - 1 do begin
      for idxcell:=0 to Rep.SrcRep.Bands[idxband].CellCount - 1 do begin
        if Rep.SrcRep.Bands[idxband].Cells[idxcell].Selected then begin
          hselbegin :=Min(hselbegin, Rep.SrcRep.Bands[idxband].Lefts[idxcell]);
          hselend   :=Max(hselend,   Rep.SrcRep.Bands[idxband].Rights[idxcell]);

          vselbegin :=Min(vselbegin, Rep.SrcRep.Tops[idxband]);
          vselend   :=Max(vselend,   Rep.SrcRep.Tops[idxband] + Rep.SrcRep.Bands[idxband].Height);
        end;
      end;
    end;

  HRB.BeginUpdate;

    HRB.Units      :=Units;

    HRB.Gutter     :=rep.Left + Rep.Gutter - Rep.LeftX + 3;

//    HRB.PageWidth  :=cARPaperSizeMetrics[Rep.SrcRep.PaperSize, Ord(Rep.SrcRep.Orientation)] * 10.0;
    HRB.PageWidth  :=Rep.SrcRep.PageSize.Width;

    HRB.LeftMargin :=Rep.SrcRep.LeftMargin;
    HRB.RightMargin:=Rep.SrcRep.RightMargin;


    HRB.SelBegin :=FromPix(hselbegin);
    HRB.SelEnd   :=FromPix(hselend);

  HRB.EndUpdate;

  VRB.BeginUpdate;

    VRB.Units      :=Units;

    VRB.Gutter     :=1 - Rep.TopY;

//    VRB.PageWidth  :=cARPaperSizeMetrics[Rep.SrcRep.PaperSize, 1 - Ord(Rep.SrcRep.Orientation)] * 10.0;
    VRB.PageWidth  :=Rep.SrcRep.PageSize.Height;

    VRB.LeftMargin :=Rep.SrcRep.TopMargin;
    VRB.RightMargin:=Rep.SrcRep.BottomMargin;

    VRB.SelBegin :=FromPix(vselbegin);
    VRB.SelEnd   :=FromPix(vselend);

  VRB.EndUpdate;
end;

procedure TAREditorDlg.FormatFontColorExecute(Sender: TObject);
  procedure SetColor(AColor:TColor);
  var i:integer;
  begin
    for i:=0 to rep.SelCount-1 do begin
      rep.Sel[i].FontColor:=AColor;
    end;
    Modified:=true;
    RepSetFocus;
  end;
begin
  if Sender is TMenuItem then begin
    ColorDialog.Color:=rep.CurrCell.Color;
    if ColorDialog.Execute then begin
      SetColor(ColorDialog.Color);
    end;
  end else
  if Sender is TARColorButton then begin
    if TARColorButton(Sender).Color <> clNone then begin
      SetColor(TARColorButton(Sender).Color);
    end;
  end;
end;

procedure TAREditorDlg.CellsBgColorExecute(Sender: TObject);
  procedure SetColor(AColor:TColor);
  var i:integer;
  begin
    for i:=0 to rep.SelCount-1 do begin
      rep.Sel[i].Color:=AColor;
    end;
    Modified:=true;
    RepSetFocus;
  end;
begin
  if Sender is TMenuItem then begin
    ColorDialog.Color:=rep.CurrCell.Color;
    if ColorDialog.Execute then begin
      SetColor(ColorDialog.Color);
    end;
  end else
  if Sender is TARColorButton then begin
    if TARColorButton(Sender).Color <> clNone then begin
      SetColor(TARColorButton(Sender).Color);
    end;
  end;
end;

procedure TAREditorDlg.FrameFrameColorExecute(Sender: TObject);
  procedure SetColor(AColor:TColor);
  var i:integer;
  begin
    for i:=0 to rep.SelCount-1 do begin
      rep.Sel[i].FrameColor:=AColor;
    end;
    Modified:=true;
    RepSetFocus;
  end;
begin
  if Sender is TMenuItem then begin
    ColorDialog.Color:=rep.CurrCell.Color;
    if ColorDialog.Execute then begin
      SetColor(ColorDialog.Color);
    end;
  end else
  if Sender is TARColorButton then begin
    if TARColorButton(Sender).Color <> clNone then begin
      SetColor(TARColorButton(Sender).Color);
    end;
  end;
end;

procedure TAREditorDlg.RepScroll(Sender: TObject);
begin
  UpdateRulerBars;

  repSetFocus;
end;

procedure TAREditorDlg.FrameFrameWidthExecute(Sender: TObject);
var i:integer;
    idx:TFrameStyles;
begin
  if Sender is TARGridButton then begin
    for i:=0 to rep.SelCount-1 do begin
      for idx:=frLeft to frBottom do begin
        rep.Sel[i].FrameWidths[idx] :=TARGridButton(Sender).GridWidth;
        rep.Sel[i].BorderStyles[idx]:=TARGridButton(Sender).GridStyle;
      end;
    end;
    Modified:=true;
    RepSetFocus;
  end;
end;

procedure TAREditorDlg.ShowToolBarClick(Sender: TObject);
var tb:TARToolBar;
begin
  tb:=TARToolBar(TMenuItem(Sender).Tag);

  if Assigned(tb) then begin
    if tb.Visible then begin
      tb.Visible:=false;
      TMenuItem(Sender).Checked:=false;
    end else begin
      tb.Visible:=true;
      TMenuItem(Sender).Checked:=true;
    end;
  end;
end;

procedure TAREditorDlg.BarPopupMenuPopup(Sender: TObject);
var i:integer;
    mi:TMenuItem;
    tb:TARToolBar;
begin
{$IFDEF D3}
  while BarPopupMenu.Items.Count > 0 do begin
    BarPopupMenu.Items.Delete(0);
  end;
{$ELSE}
  BarPopupMenu.Items.Clear;
{$ENDIF}

  mi:=TMenuItem.Create(Self);
  mi.Caption:='Customize...';
  mi.OnClick:=ShowCustomizeDlg;
  BarPopupMenu.Items.Add(mi);

  mi:=TMenuItem.Create(Self);
  mi.Caption:='-';
  BarPopupMenu.Items.Add(mi);

  for i:=0 to ActList.CategoryList.Count - 1 do begin
    mi:=TMenuItem.Create(Self);
    mi.Caption:=ActList.CategoryList[i];
    mi.OnClick:=ShowToolBarClick;

    tb:=TARToolBar(FindComponent(ActList.CategoryList[i] + 'ToolBar'));
    if Assigned(tb) then begin
      if tb.Visible then begin
        mi.Checked:=true;
      end;
      mi.Tag:=integer(tb);
    end;

    BarPopupMenu.Items.Add(mi);
  end;
end;

procedure TAREditorDlg.ShowCustomizeDlg(Sender: TObject);
begin
  TARToolsCustomizeDlg.Create(Self).Show;
end;

function  TAREditorDlg.GetMenuItem(const MenuItemName:string):TMenuItem;
begin
  Result:=TMenuItem(FindComponent(MenuItemName));

  if not Assigned(Result) then begin
    Result:=TMenuItem(ActList.FindComponent(MenuItemName));
    if not Assigned(Result) then begin
      raise Exception.CreateFmt('Can''t find menu item <%s>', [MenuItemName]);
    end;
  end;
end;

procedure TAREditorDlg.AddMenuItem(const ACategory:string; const ACaption:string; const AName:string);
var mi, topmi:TMenuItem;
    act:TARAction;
{$IFDEF D3}
    i:integer;
{$ENDIF}
begin
  mi:=TMenuItem.Create(Self);
  mi.Name     :=AName;
  mi.Caption  :=ACaption;

  if Length(ACategory) = 0 then begin
    MainMenu.Items.Add(mi);
  end else begin
    if ACaption <> '-' then begin
      act:=ActList.Items[ACategory, ACaption];
{$IFDEF D3}
{$ELSE}
      mi.Bitmap   :=act.Image;
{$ENDIF}
      mi.Hint     :=act.Hint;
      mi.OnClick  :=act.Action;
      mi.ShortCut :=act.ShortCut;
      mi.Checked  :=act.Checked;
      act.AddComponent(mi);
    end;

{$IFDEF D3}
    topmi:=nil;
    for i:=0 to MainMenu.Items.Count - 1 do begin
      if CT(MainMenu.Items[i].Caption, ACategory) then begin
        topmi:=MainMenu.Items[i];
        break;
      end;
    end;
{$ELSE}
    topmi:=MainMenu.Items.Find(ACategory);
{$ENDIF}
    topmi.Add(mi);
  end;
end;

procedure TAREditorDlg.AddSubMenuItem(const ACategory:string; const AMenuItemName:string; const ACaption:string; const AName:string);
var mi, topmi:TMenuItem;
    act:TARAction;
begin
  topmi:=GetMenuItem(AMenuItemName);
  if Assigned(topmi) then begin
    mi:=TMenuItem.Create(Self);
    mi.Name     :=AName;
    mi.Caption  :=ACaption;

    if ACaption <> '-' then begin
      act:=ActList.Items[ACategory, ACaption];
{$IFDEF D3}
{$ELSE}
      mi.Bitmap   :=act.Image;
{$ENDIF}
      mi.Hint     :=act.Hint;
      mi.OnClick  :=act.Action;
      mi.ShortCut :=act.ShortCut;
      mi.Checked  :=act.Checked;
      act.AddComponent(mi);
    end;

    topmi.Add(mi);
  end;
end;

procedure TAREditorDlg.AddPopupMenuItem(PopupMenu:TpopupMenu; const ACategory:string; const ACaption:string; const AName:string);
var mi:TMenuItem;
    act:TARAction;
begin
  mi:=TMenuItem.Create(Self);
  mi.Name     :=AName;
  mi.Caption  :=ACaption;

  if ACaption <> '-' then begin
    act:=ActList.Items[ACategory, ACaption];
{$IFDEF D3}
{$ELSE}
    mi.Bitmap   :=act.Image;
{$ENDIF}
    mi.Hint     :=act.Hint;
    mi.OnClick  :=act.Action;
    mi.ShortCut :=act.ShortCut;
    mi.Checked  :=act.Checked;
    act.AddComponent(mi);
  end;

  PopupMenu.Items.Add(mi);
end;

procedure TAREditorDlg.CreateMainMenu;
var ACategory:string;
  procedure Add(ACaption:string; AName:string);
  begin
    AddMenuItem(ACategory, ACaption, AName);
  end;
  procedure AddSub(AMenuItemName:string; ACaption:string; AName:string);
  begin
    AddSubMenuItem(ACategory, AMenuItemName, ACaption, AName);
  end;
begin
{$IFDEF D3}
  while MainMenu.Items.Count > 0 do begin
    MainMenu.Items.Delete(0);
  end;
{$ELSE}
  MainMenu.Items.Clear;
{$ENDIF}

  AddMenuItem('', rsAREditor_File, 'FileMenu');
  ACategory:=Category_File;
    AddSub('FileMenu', rsAREditor_FileNew,            '');
    AddSub('FileMenu', rsAREditor_FileOpen,           '');
    AddSub('FileMenu', rsAREditor_FileSave,           '');
    AddSub('FileMenu', rsAREditor_FileSaveAs,         '');
    AddSub('FileMenu', rsAREditor_FileSaveAll,        '');
    AddSub('FileMenu', rsAREditor_FileClosePage,      '');
    AddSub('FileMenu', '-',                           '');
    AddSub('FileMenu', rsAREditor_FilePageSetup,      '');
    AddSub('FileMenu', rsAREditor_FilePrint,          '');
    AddSub('FileMenu', rsAREditor_FilePreview,        '');
    AddSub('FileMenu', '-',                           'RecentFilesBeginMenuItem');
    AddSub('FileMenu', '-',                           'RecentFilesEndMenuItem');
    AddSub('FileMenu', rsAREditor_FileExit,           '');

  AddMenuItem('', rsAREditor_Edit, 'EditMenu');
  ACategory:=Category_Edit;
    AddSub('EditMenu', rsAREditor_EditUndo,           '');
    AddSub('EditMenu', rsAREditor_EditRedo,           '');
    AddSub('EditMenu', '-',                           '');
    AddSub('EditMenu', rsAREditor_EditSelectAll,      '');
    AddSub('EditMenu', '-',                           '');
    AddSub('EditMenu', rsAREditor_EditInsertPicture,  '');
    AddSub('EditMenu', rsAREditor_EditFitToCell,      '');
    AddSub('EditMenu', rsAREditor_EditTile,           '');

  AddMenuItem('', rsAREditor_Cells, 'CellsMenu');
  ACategory:=Category_Cells;
    AddSub('CellsMenu', rsAREditor_CellsAddCell,       '');
    AddSub('CellsMenu', rsAREditor_CellsDeleteCell,    '');
    AddSub('CellsMenu', rsAREditor_CellsCellMoveLeft,  '');
    AddSub('CellsMenu', rsAREditor_CellsCellMoveRight, '');
    AddSub('CellsMenu', '-',                           '');
    AddSub('CellsMenu', rsAREditor_CellsCopyCells,     '');
    AddSub('CellsMenu', rsAREditor_CellsCutCells,      '');
    AddSub('CellsMenu', rsAREditor_CellsPasteCells,    '');
    AddSub('CellsMenu', '-',                           '');
    AddSub('CellsMenu', rsAREditor_CellsSplitCell,     '');
    AddSub('CellsMenu', rsAREditor_CellsHMergeCells,   '');
    AddSub('CellsMenu', rsAREditor_CellsVMergeCells,   '');
    AddSub('CellsMenu', '-',                           '');
    AddSub('CellsMenu', rsAREditor_CellsBgcolor,       '');
    AddSub('CellsMenu', rsAREditor_CellsAutoStretch,   '');
    AddSub('CellsMenu', rsAREditor_CellsWordWrap,      '');
    AddSub('CellsMenu', '-',                           '');
    AddSub('CellsMenu', rsAREditor_CellsCellText,      '');
    AddSub('CellsMenu', rsAREditor_CellsProperties,    '');

  AddMenuItem('', rsAREditor_Bands, 'BandsMenu');
  ACategory:=Category_Bands;
    AddSub('BandsMenu', rsAREditor_BandsAddBand,       '');
    AddSub('BandsMenu', rsAREditor_BandsDeleteBand,    '');
    AddSub('BandsMenu', rsAREditor_BandsBandUp,        '');
    AddSub('BandsMenu', rsAREditor_BandsBandDown,      '');
    AddSub('BandsMenu', '-',                           '');
    AddSub('BandsMenu', rsAREditor_BandsCopyBands,     '');
    AddSub('BandsMenu', rsAREditor_BandsCutBands,      '');
    AddSub('BandsMenu', rsAREditor_BandsPasteBands,    '');
    AddSub('BandsMenu', '-',                           '');
    AddSub('BandsMenu', rsAREditor_BandsBandName,      '');

  AddPopupMenuItem(RepPopupMenu, Category_File,   rsAREditor_FileClosePage,   '');
  AddPopupMenuItem(RepPopupMenu, '-',  '-',  '');
  AddPopupMenuItem(RepPopupMenu, Category_Cells,  rsAREditor_CellsCopyCells,  '');
  AddPopupMenuItem(RepPopupMenu, Category_Cells,  rsAREditor_CellsCutCells,   '');
  AddPopupMenuItem(RepPopupMenu, Category_Cells,  rsAREditor_CellsPasteCells, '');
  AddPopupMenuItem(RepPopupMenu, Category_Cells,  rsAREditor_CellsCellText,   '');
  AddPopupMenuItem(RepPopupMenu, '-',  '-',  '');
  AddPopupMenuItem(RepPopupMenu, Category_Bands,  rsAREditor_BandsCopyBands,  '');
  AddPopupMenuItem(RepPopupMenu, Category_Bands,  rsAREditor_BandsCutBands,   '');
  AddPopupMenuItem(RepPopupMenu, Category_Bands,  rsAREditor_BandsPasteBands, '');
  AddPopupMenuItem(RepPopupMenu, Category_Bands,  rsAREditor_BandsBandName,   '');
  AddPopupMenuItem(RepPopupMenu, '-',  '-',  '');
  AddPopupMenuItem(RepPopupMenu, Category_Cells,  rsAREditor_CellsProperties, '');

  AddMenuItem('', rsAREditor_Format, 'FormatMenu');
  ACategory:=Category_Format;
    AddSub('FormatMenu', rsAREditor_FormatFont,         '');
    AddSub('FormatMenu', rsAREditor_FormatFontColor,    '');
    AddSub('FormatMenu', rsAREditor_FormatBold,         '');
    AddSub('FormatMenu', rsAREditor_FormatItalic,       '');
    AddSub('FormatMenu', rsAREditor_FormatUnderline,    '');
    AddSub('FormatMenu', rsAREditor_FormatIncreaseFont, '');
    AddSub('FormatMenu', rsAREditor_FormatDecreaseFont, '');
    AddSub('FormatMenu', '-',                           '');
    AddSub('FormatMenu', rsAREditor_FormatLeftalign,    '');
    AddSub('FormatMenu', rsAREditor_FormatCenteralign,  '');
    AddSub('FormatMenu', rsAREditor_FormatRightalign,   '');
    AddSub('FormatMenu', '-',                           '');
    AddSub('FormatMenu', rsAREditor_FormatToplayout,    '');
    AddSub('FormatMenu', rsAREditor_FormatCenterlayout, '');
    AddSub('FormatMenu', rsAREditor_FormatBottomlayout, '');

  AddMenuItem('', rsAREditor_Frame, 'FrameMenu');
  ACategory:=Category_Frame;
//    AddSub('FrameMenu', rsAREditor_FrameFrameColor,     '');
//    AddSub('FrameMenu', rsAREditor_FrameFrameWidth,     '');
    AddSub('FrameMenu', rsAREditor_FrameNotFrame,       '');

    AddSub('FrameMenu', rsAREditor_FrameFrameLeft,      '');
    AddSub('FrameMenu', rsAREditor_FrameFrameLeftDec,   '');
    AddSub('FrameMenu', rsAREditor_FrameFrameRight,     '');
    AddSub('FrameMenu', rsAREditor_FrameFrameRightDec,  '');
    AddSub('FrameMenu', rsAREditor_FrameFrameTop,       '');
    AddSub('FrameMenu', rsAREditor_FrameFrameTopDec,    '');
    AddSub('FrameMenu', rsAREditor_FrameFrameBottom,    '');
    AddSub('FrameMenu', rsAREditor_FrameFrameBottomDec, '');

  AddMenuItem('', rsAREditor_Options, 'OptionsMenu');
  ACategory:=Category_Options;
    AddSub('OptionsMenu', rsAREditor_OptionsPreferences, '');
    AddSub('OptionsMenu', rsAREditor_OptionsDefaultFont, '');
    AddSub('OptionsMenu', rsAREditor_OptionsUserVars,    '');
    AddSub('OptionsMenu', rsAREditor_OptionsUnits,       'UnitsMenuItem');
    AddSub('UnitsMenuItem', rsAREditor_UnitsInches,        '');
    AddSub('UnitsMenuItem', rsAREditor_UnitsMillemeters,   '');
    AddSub('UnitsMenuItem', rsAREditor_UnitsCentimeters,   '');
    AddSub('UnitsMenuItem', rsAREditor_UnitsPixels,        '');
    AddSub('OptionsMenu', rsAREditor_OptionsShowAll,     '');
    AddSub('OptionsMenu', '-',                           '');
    AddSub('OptionsMenu', rsAREditor_OptionsFileSystem,  'FileSystemMenuItem');

  AddMenuItem('', rsAREditor_Help, 'HelpMenu');
  ACategory:=Category_Help;
    AddSub('HelpMenu', rsAREditor_HelpContents,       '');
    AddSub('HelpMenu', '-',                           '');
    AddSub('HelpMenu', rsAREditor_HelpAbout,          '');
end;

procedure TAREditorDlg.ResizeToolBars;
var i, j, w:integer;
    tb:TARToolBar;
    {$IFDEF D3}
      tw:integer;
    {$ENDIF}
begin
  for i:=0 to ComponentCount - 1 do begin
    if Components[i] is TARToolBar then begin
      tb:=Components[i] as TARToolBar;
      w:=0;
      for j:=0 to tb.ControlCount - 1 do begin
        w:=w + tb.Controls[j].Width;
      end;
      if (tb.Parent = MainControlBar) and ((tb.Left + w) > ClientWidth) then begin
        tb.Top:=tb.Top + tb.Height;
        tb.Left:=0;
      end;
      tb.Width:=w;
    end;
  end;

  {$IFDEF D3}
  tw:=0;
  for i:=0 to MainControlBar.Bands.Count - 1 do begin
    if MainControlBar.Bands[i].Control is TARToolBar then begin
      tb:=MainControlBar.Bands[i].Control as TARToolBar;
      w:=0;
      for j:=0 to tb.ControlCount - 1 do begin
        w:=w + tb.Controls[j].Width + 5;
      end;
      tw:=tw + w;
      if tw > MainControlBar.ClientWidth then begin
        MainControlBar.Bands[i].Break:=true;
        tw:=w;
      end else begin
        MainControlBar.Bands[i].Break:=false;
      end;
      MainControlBar.Bands[i].Width:=w;
    end;
  end;
  {$ENDIF}
end;

procedure   TAREditorDlg.RestoreDefaultTools;
var i:integer;
begin
  for i:=ComponentCount - 1 downto 0 do begin
    if Components[i] is TARToolBar then begin
      Components[i].Free;
    end;
  end;

  CreateDefaultButtons;
  ResizeToolBars;
end;

procedure TAREditorDlg.CreateToolButton(ACategory:string; const ACaption:string);
var act:TARAction;
    tbr:TARToolBar;
    ctrl:TControl;
begin
  if Length(ACaption) > 0 then begin
    ACategory:=Flat(ACategory);

    tbr:=TARToolBar(FindComponent(ACategory + 'ToolBar'));
    if not Assigned(tbr) then begin
      tbr:=CreateToolBar(ACategory);
      tbr.Parent:=MainControlBar;
    end;

    act:=ActList.Action(ACaption);

    ctrl:=act.CreateControl(Self, TParControl(tbr));
    if Assigned(ctrl) then begin

//       ctrl.Parent:=tbr;

      {$IFDEF CLX}
      {$ELSE}
        {$IFDEF D3}
        {$ELSE}
          TDummyControl(ctrl).OnStartDock:=ControlStartDock;
        {$ENDIF}
      {$ENDIF}

      tbr.Width:=400;
    end;
    ResizeToolBar(tbr);
  end;
end;

procedure TAREditorDlg.CreateDefaultButtons;
var Category:string;
  procedure NewBtn(Caption:string);
  begin
    CreateToolButton(Category, Caption);
  end;
begin
  Category:=Category_Help;
//    NewBtn(rsAREditor_HelpAbout);
    NewBtn(rsAREditor_HelpContents);
    NewBtn(rsAREditor_FormatFontSize);

  Category:=Category_Options;
//    NewBtn(rsAREditor_OptionsDefaultFont);
//    NewBtn(rsAREditor_OptionsUnits);
//    NewBtn(rsAREditor_UnitsInches);
//    NewBtn(rsAREditor_UnitsMillemeters);
//    NewBtn(rsAREditor_UnitsCentimeters);
//    NewBtn(rsAREditor_UnitsPixels);
    NewBtn(rsAREditor_OptionsShowAll);
//    NewBtn(rsAREditor_OptionsFileSystem);
//    NewBtn(rsAREditor_OptionsStandard);
//    NewBtn(rsAREditor_OptionsDatabase);
//    NewBtn(rsAREditor_OptionsInterbase);
    NewBtn(rsAREditor_OptionsUserVars);

  Category:=Category_Frame;
    NewBtn(rsAREditor_FrameFrameColor);
    NewBtn(rsAREditor_FrameFrameWidth);

    NewBtn(rsAREditor_FrameFrameBottom);
    NewBtn(rsAREditor_FrameFrameTop);
    NewBtn(rsAREditor_FrameFrameRight);
    NewBtn(rsAREditor_FrameFrameLeft);

    NewBtn(rsAREditor_FrameNotFrame);

  Category:=Category_Format;
    NewBtn(rsAREditor_FormatDecreaseFont);
    NewBtn(rsAREditor_FormatIncreaseFont);
    NewBtn(rsAREditor_FormatFontName);
    NewBtn(rsAREditor_FormatFontColor);
    NewBtn(rsAREditor_FormatFont);
    NewBtn(rsAREditor_FormatUnderline);
    NewBtn(rsAREditor_FormatItalic);
    NewBtn(rsAREditor_FormatBold);
    NewBtn(rsAREditor_FormatBottomlayout);
    NewBtn(rsAREditor_FormatCenterlayout);
    NewBtn(rsAREditor_FormatToplayout);
    NewBtn(rsAREditor_FormatRightalign);
    NewBtn(rsAREditor_FormatCenteralign);
    NewBtn(rsAREditor_FormatLeftalign);

  Category:=Category_Bands;
//    NewBtn(rsAREditor_BandsBandName);
    NewBtn(rsAREditor_BandsBandDown);
    NewBtn(rsAREditor_BandsBandUp);
    NewBtn(rsAREditor_BandsDeleteBand);
    NewBtn(rsAREditor_BandsAddBand);

  Category:=Category_Cells;
//    NewBtn(rsAREditor_CellsCopyCells);
//    NewBtn(rsAREditor_CellsCutCells);
//    NewBtn(rsAREditor_CellsPasteCells);
//    NewBtn(rsAREditor_CellsCellText);
    NewBtn(rsAREditor_CellsWordWrap);
    NewBtn(rsAREditor_CellsAutoStretch);
    NewBtn(rsAREditor_CellsBgcolor);
    NewBtn(rsAREditor_CellsCellMoveRight);
    NewBtn(rsAREditor_CellsCellMoveLeft);
    NewBtn(rsAREditor_CellsDeleteCell);
    NewBtn(rsAREditor_CellsAddCell);
    NewBtn(rsAREditor_CellsSplitCell);
    NewBtn(rsAREditor_CellsHMergeCells);
    NewBtn(rsAREditor_CellsVMergeCells);

  Category:=Category_Edit;
//    NewBtn(rsAREditor_EditSelectAll);
    NewBtn(rsAREditor_EditTile);
    NewBtn(rsAREditor_EditFitToCell);
    NewBtn(rsAREditor_EditInsertPicture);
    NewBtn(rsAREditor_EditRedo);
    NewBtn(rsAREditor_EditUndo);

  Category:=Category_File;
//    NewBtn(rsAREditor_FileExit);
    NewBtn(rsAREditor_FilePreview);
    NewBtn(rsAREditor_FilePrint);
    NewBtn(rsAREditor_FilePageSetup);
    NewBtn(rsAREditor_FileSaveAs);
    NewBtn(rsAREditor_FileSave);
    NewBtn(rsAREditor_FileOpen);
    NewBtn(rsAREditor_FileNew);

//////////////////////////

  ResizeToolBars;
end;

function TAREditorDlg.CreateToolBar(const ACaption:string):TARToolBar;
begin
  Result:=TARToolBar.Create(Self);
  Result.Name                   :=ACaption + 'ToolBar';
  Result.Caption                :=ACaption;
  {$IFDEF D3}
    Result.AutoSize:=False;
    Result.Height:=22;
  {$ELSE}
    {$IFDEF CLX}
    {$ELSE}
      Result.DragKind           :=dkDock;
      Result.OnEndDock          :=ToolBarEndDock;
    {$ENDIF}
    Result.Constraints.MinHeight:=22;
    Result.Constraints.MinWidth :=23;
    Result.OnEndDrag            :=ToolBarEndDock;
    Result.AutoSize             :=true;
    Result.Flat                 :=true;
    Result.Wrapable             :=false;
    Result.ShowCaptions         :=true;
    Result.EdgeBorders          :=[];
  {$ENDIF}
  Result.DragMode               :=dmAutomatic;
  Result.ParentShowHint         :=false;
  Result.ParentShowHint         :=true;
  Result.ShowHint               :=false;
end;

procedure TAREditorDlg.SaveToolBars;
var i, j, tbcnt, ctrlcnt:integer;
    tb:TARToolBar;
    c:TControl;
    sls, sl:TStringList;
    {$IFDEF LINUX}
      reg:TIniFile;
    {$ENDIF}
    {$IFDEF MSWINDOWS}
      reg:TRegistry;
    {$ENDIF}
begin
  MainControlBar.Parent:=Self;
  sls :=TStringList.Create;
  sl  :=TStringList.Create;
  tbcnt:=0;
  {$IFDEF LINUX}
    reg:=TIniFile.Create(Application.ExeName + '.ini');
  {$ENDIF}
  {$IFDEF MSWINDOWS}
    reg:=TRegistry.Create;
  {$ENDIF}
  try
    for i:=0 to ComponentCount - 1 do begin
      if Components[i] is TARToolBar then begin
        tb:=TARToolBar(Components[i]);
        if Length(tb.Caption) > 0 then begin
          if True then begin
            inc(tbcnt);
            sl.Clear();
            sl.Add('VISIBLE=' + IntToStr(Ord(tb.Visible)));
            sl.Add('WIDTH='   + IntToStr(tb.Width));
            sl.Add('HEIGHT='  + IntToStr(tb.Height));
            if tb.Parent is TControlBar then begin
              sl.Add('FLOAT=0');
              sl.Add('LEFT=' + IntToStr(tb.Left));
              sl.Add('TOP='  + IntToStr(tb.Top));
            end else begin
              sl.Add('FLOAT=1');
              sl.Add('LEFT'  + IntToStr(tb.Parent.Left));
              sl.Add('TOP'   + IntToStr(tb.Parent.Top));
            end;

            ctrlcnt:=0;
            while tb.ControlCount > 0 do begin
              for j:=0 to tb.ControlCount - 1 do begin
                c:=tb.Controls[j];
                if c.Left <= 10 then begin
                  sl.Add(IntToStr(ctrlcnt) + '=' + TARAction(c.Owner).Caption);
                  inc(ctrlcnt);
                  c.Free;
                  break;
                end;
              end;
            end;
            sl.Add('COUNT=' + IntToStr(ctrlcnt));
          end;
          sls.Add(tb.Caption + '=' + sl.CommaText);
        end;
      end;
    end;

    {$IFDEF LINUX}
      reg.WriteInteger('ToolBars\', 'COUNT',  tbcnt);
      reg.WriteInteger('ToolBars\', 'HEIGHT', MainControlBar.Height);
      reg.WriteString ('ToolBars\', 'ITEMS',  sls.CommaText);
    {$ENDIF}
    {$IFDEF MSWINDOWS}
      if reg.OpenKey(RegRoot + 'ToolBars\', True) then begin
        reg.WriteInteger('COUNT',   tbcnt);
        reg.WriteInteger('HEIGHT',  MainControlBar.Height);
        reg.WriteString('ITEMS', sls.CommaText);
      end;
    {$ENDIF}
  finally
    sl.Free;
  // sa: 11.06.2001
    sls.Free;
  //--

    {$IFDEF LINUX}
      reg.UpdateFile;
    {$ENDIF}
    Reg.Free;
  end;
end;

function ComparePos(List: TStringList; Index1, Index2: Integer):integer;
begin
  Result:=integer(List.Objects[Index1]) - integer(List.Objects[Index2]);
end;

procedure TAREditorDlg.LoadToolBars;
var i, j:integer;
    sl, sl2:TAIStringList;
    vVisible, vFloat:boolean;
    cCount, vLeft, vTop:integer;
    tb:TARToolBar;
    {$IFDEF LINUX}
      reg:TIniFile;
    {$ENDIF}
    {$IFDEF MSWINDOWS}
      reg:TRegistry;
    {$ENDIF}
begin
  sl:=TAIStringList.Create('TAREditorDlg sl');
  sl2:=TAIStringList.Create('TAREditorDlg sl2');
  {$IFDEF LINUX}
    reg:=TIniFile.Create(Application.ExeName + '.ini');
  {$ENDIF}
  {$IFDEF MSWINDOWS}
    reg:=TRegistry.Create;
  {$ENDIF}
  try
    try
      {$IFDEF D3}
      {$ELSE}
        MainControlBar.AutoSize:=false;
      {$ENDIF}

      {$IFDEF LINUX}
        MainControlBar.Height :=reg.ReadInteger('ToolBars\', 'HEIGHT', 50);
        sl.CommaText          :=reg.ReadString ('ToolBars\', 'ITEMS', '');
      {$ENDIF}
      {$IFDEF MSWINDOWS}
        reg.OpenKey(RegRoot + 'ToolBars\', false);
        MainControlBar.Height:=reg.ReadInteger('HEIGHT');
        sl.CommaText:=reg.ReadString('ITEMS');
      {$ENDIF}

      if sl.Count > 0 then begin
        for i:=0 to sl.Count - 1 do begin
          sl2.CommaText:=sl.Values[sl.Names[i]];
          if (sl2.Values['VISIBLE'] = '1') and (sl2.Values['FLOAT'] = '0') then begin
            sl.Objects[i]:=pointer(StrToInt(sl2.Values['LEFT']) + 10000 * StrToInt(sl2.Values['TOP']));
          end else begin
            sl.Objects[i]:=pointer(0);
          end;
        end;
        {$IFDEF D3}
        {$ELSE}
          sl.CustomSort(ComparePos);
        {$ENDIF}

        for i:=0 to sl.Count - 1 do begin
          sl2.CommaText:=sl.Values[sl.Names[i]];
          cCount  :=StrToInt(sl2.Values['COUNT']);
          vVisible:=boolean(StrToInt(sl2.Values['VISIBLE']));
          vFloat  :=boolean(StrToInt(sl2.Values['FLOAT']));
          vLeft   :=StrToInt(sl2.Values['LEFT']);
          vTop    :=StrToInt(sl2.Values['TOP']);

          tb:=CreateToolBar(sl.Names[i]);

          tb.Visible        :=vVisible;
          if vFloat then begin
            {$IFDEF D3}
            {$ELSE}
              {$IFDEF CLX}
              {$ELSE}
                tb.ManualFloat(Rect(vLeft, vTop, vLeft + 23, vTop + 22));
              {$ENDIF}
            {$ENDIF}
          end else begin
            tb.Top:=vTop;
            tb.Left:=vLeft;
            tb.Parent         :=MainControlBar;
          end;
          for j:=cCount - 1 downto 0 do begin
            CreateToolButton(sl.Names[i], sl2.Values[IntToStr(j)]);
          end;
        end;
      end else begin
        CreateDefaultButtons;
      end;
      ResizeToolBars;
    except
      CreateDefaultButtons;
      ResizeToolBars;
    end;
  finally
{$IFDEF D3}
{$ELSE}
    MainControlBar.AutoSize:=true;
{$ENDIF}
    sl2.Free;
    sl.Free;
    {$IFDEF LINUX}
      reg.UpdateFile;
    {$ENDIF}
    Reg.Free;
  end;
end;

{$IFNDEF D3}
procedure TAREditorDlg.ToolBarEndDock(Sender, Target: TObject; X, Y: Integer);
var tb:TARToolBar;
begin
  tb:=TARToolBar(Sender);

  if Target is TARToolBar then begin
    {$IFDEF D3}
    {$ELSE}
      {$IFDEF CLX}
      {$ELSE}
        tb.ManualDock(MainControlBar);
      {$ENDIF}
    {$ENDIF}
  end;

  ResizeToolBar(tb);
end;
{$ENDIF}

{$IFDEF D3}
{$ELSE}
  {$IFDEF CLX}
  {$ELSE}
procedure TAREditorDlg.ControlStartDock(Sender: TObject; var DragObject: TDragDockObject);
begin
  if TControl(Sender).Parent is TARToolBar then begin
    ResizeToolBar(TARToolBar(TControl(Sender).Parent));
  end;
end;
  {$ENDIF}
{$ENDIF}

procedure TAREditorDlg.CreateActions;
var Category:string;
{$IFDEF D3}
  procedure NewAct(Caption:string;ImageName:string;ShortCut:string;Hint:string;Action:TNotifyEvent; CreateControl:TCreateControlEvent);
{$ELSE}
  procedure NewAct(Caption:string;ImageName:string;ShortCut:string;Hint:string;Action:TNotifyEvent; CreateControl:TCreateControlEvent = nil);
{$ENDIF}
  var act:TARAction;
  begin
    act:=TARAction.Create(Self);

    act.Category        :=Category;
    act.Caption         :=Caption;
    act.ShortCut        :=TextToShortCut(ShortCut);
    act.Hint            :=Hint;
    act.Action          :=Action;
    act.OnCreateControl :=CreateControl;

    if Length(ImageName) > 0 then begin
      act.Image:=GetBitmap(ImageName);
    end;

    ActList.Add(act);
  end;
begin

  Category:=Category_File;
    NewAct(rsAREditor_FileNew,            'NEW',          'Ctrl+N',       rsAREditor_NewHint,               FileNewExecute, CreateBnt);
    NewAct(rsAREditor_FileOpen,           'OPEN',         'Ctrl+O',       rsAREditor_OpenHint,              FileOpenExecute, CreateBnt);
    NewAct(rsAREditor_FileSave,           'SAVE',         'Ctrl+S',       rsAREditor_SaveHint,              FileSaveExecute, CreateBnt);
    NewAct(rsAREditor_FileSaveAs,         'SAVEAS',       '',             rsAREditor_SaveAsHint,            FileSaveAsExecute, CreateBnt);
    NewAct(rsAREditor_FileSaveAll,        '',             '',             rsAREditor_SaveAllHint,           FileSaveAllExecute, CreateBnt);
    NewAct(rsAREditor_FileClosePage,      '',             'Ctrl+F4',      rsAREditor_ClosePageHint,         FileClosePageExecute, CreateBnt);

    NewAct(rsAREditor_FilePageSetup,      'PAGESETUP',    '',             rsAREditor_PageSetupHint,         FilePageSetupExecute, CreateBnt);
    NewAct(rsAREditor_FilePrint,          'PRINT',        'Ctrl+P',       rsAREditor_PrintHint,             FilePrintExecute, CreateBnt);
    NewAct(rsAREditor_FilePreview,        'PREVIEW',      '',             rsAREditor_PreviewHint,           FilePreviewExecute, CreateBnt);
    NewAct(rsAREditor_FileExit,           '',             '',             rsAREditor_ExitHint,              FileExitExecute, CreateBnt);

  Category:=Category_Edit;
    NewAct(rsAREditor_EditUndo,           'UNDO',         'Ctrl+Z',       rsAREditor_UndoHint,              EditUndoExecute, CreateBnt);
    NewAct(rsAREditor_EditRedo,           'REDO',         'Shift+Ctrl+Z', rsAREditor_RedoHint,              EditRedoExecute, CreateBnt);
    NewAct(rsAREditor_EditSelectAll,      '',             'Ctrl+A',       rsAREditor_SelectAllHint,         EditSelectAllExecute, CreateBnt);
    NewAct(rsAREditor_EditInsertPicture,  'INSERTPICTURE','',             rsAREditor_InsertPictureHint,     EditInsertPictureExecute, CreateBnt);
    NewAct(rsAREditor_EditFitToCell,      'FITTOCELL',    '',             rsAREditor_FitToCellHint,         EditFitToCellExecute, CreateCheckedBnt);
    NewAct(rsAREditor_EditTile,           'TILE',         '',             rsAREditor_TileHint,              EditTileExecute, CreateCheckedBnt);

  Category:=Category_Cells;
    NewAct(rsAREditor_CellsAddCell,       'ADDCELL',      'Ins',          rsAREditor_AddCellHint,           CellsAddCellExecute, CreateBnt);
    NewAct(rsAREditor_CellsDeleteCell,    'DELETECELL',   'Del',          rsAREditor_DeleteCellHint,        CellsDeleteCellExecute, CreateBnt);
    NewAct(rsAREditor_CellsSplitCell,     'SPLITCELL',    'Ctrl+Ins',     rsAREditor_SplitCellHint,         CellsSplitCellExecute, CreateBnt);
    NewAct(rsAREditor_CellsHMergeCells,   'HMERGECELL',   'Ctrl+Q',       rsAREditor_HMergeCellsHint,       CellsHMergeCellsExecute, CreateBnt);
    NewAct(rsAREditor_CellsVMergeCells,   'VMERGECELL',   'Ctrl+D',       rsAREditor_VMergeCellsHint,       CellsVMergeCellsExecute, CreateBnt);
    NewAct(rsAREditor_CellsCellMoveLeft,  'CELLMOVELEFT', 'Alt+Left',     rsAREditor_CellMoveLeftHint,      CellsCellMoveLeftExecute, CreateBnt);
    NewAct(rsAREditor_CellsCellMoveRight, 'CELLMOVERIGHT','Alt+Right',    rsAREditor_CellMoveRightHint,     CellsCellMoveRightExecute, CreateBnt);

    NewAct(rsAREditor_CellsCopyCells,     '',             'Ctrl+C',       rsAREditor_CopyCellsHint,         CellsCopyCellExecute, CreateBnt);
    NewAct(rsAREditor_CellsCutCells,      '',             'Ctrl+X',       rsAREditor_CutCellsHint,          CellsCutCellExecute, CreateBnt);
    NewAct(rsAREditor_CellsPasteCells,    '',             'Ctrl+V',       rsAREditor_PasteCellsHint,        CellsPasteCellExecute, CreateBnt);

    NewAct(rsAREditor_CellsBgcolor,       'BGCOLOR',      '',             rsAREditor_BgColorHint,           CellsBgColorExecute, CreateColorBnt);
    NewAct(rsAREditor_CellsAutoStretch,   'AUTOSTRETCH',  '',             rsAREditor_AutoStretchHint,       CellsAutoStretchExecute, CreateCheckedBnt);
    NewAct(rsAREditor_CellsWordWrap,      'WORDWRAP',     '',             rsAREditor_WordWrapHint,          CellsWordWrapExecute, CreateCheckedBnt);
    NewAct(rsAREditor_CellsCellText,      '',             'Enter',        rsAREditor_CellTextHint,          CellsCellTextExecute, CreateBnt);
    NewAct(rsAREditor_CellsProperties,    '',             'Alt+Enter',    rsAREditor_CellPropertiesHint,    CellsCellPropertiesExecute, CreateBnt);

  Category:=Category_Bands;
    NewAct(rsAREditor_BandsAddBand,       'ADDBAND',      'Alt+Ins',      rsAREditor_AddBandHint,           BandsAddBandExecute, CreateBnt);
    NewAct(rsAREditor_BandsDeleteBand,    'DELETEBAND',   'Alt+Del',      rsAREditor_DeleteBandHint,        BandsDeleteBandExecute, CreateBnt);

    NewAct(rsAREditor_BandsCopyBands,     '',             'Shift+Ctrl+C', rsAREditor_CopyBandsHint,         BandsCopyBandsExecute, CreateBnt);
    NewAct(rsAREditor_BandsCutBands,      '',             'Shift+Ctrl+X', rsAREditor_CutBandsHint,          BandsCutBandsExecute, CreateBnt);
    NewAct(rsAREditor_BandsPasteBands,    '',             'Shift+Ctrl+V', rsAREditor_PasteBandsHint,        BandsPasteBandsExecute, CreateBnt);

    NewAct(rsAREditor_BandsBandUp,        'BANDUP',       'Alt+Up',       rsAREditor_BandUpHint,            BandsBandUpExecute, CreateBnt);
    NewAct(rsAREditor_BandsBandDown,      'BANDDOWN',     'Alt+Down',     rsAREditor_BandDownHint,          BandsBandDownExecute, CreateBnt);
    NewAct(rsAREditor_BandsBandName,      '',             'F3',           rsAREditor_BandNameHint,          BandsBandNameExecute, CreateBnt);

  Category:=Category_Format;
    NewAct(rsAREditor_FormatFont,         'FONT',         'Ctrl+F',       rsAREditor_FontHint,              FormatFontExecute, CreateBnt);
    NewAct(rsAREditor_FormatFontColor,    'FONTCOLOR',    '',             rsAREditor_FontColorHint,         FormatFontColorExecute, CreateColorBnt);
    NewAct(rsAREditor_FormatFontName,     'FONT',         '',             rsAREditor_FontNameHint,          FormatFontNameExecute, CreateFontNameBnt);
    NewAct(rsAREditor_FormatFontSize,     'FONT',         '',             rsAREditor_FontSizeHint,          FormatFontSizeExecute, CreateFontSizeBnt);
    NewAct(rsAREditor_FormatBold,         'BOLD',         'Ctrl+B',       rsAREditor_BoldHint,              FormatBoldExecute, CreateCheckedBnt);
    NewAct(rsAREditor_FormatItalic,       'ITALIC',       'Ctrl+I',       rsAREditor_ItalicHint,            FormatItalicExecute, CreateCheckedBnt);
    NewAct(rsAREditor_FormatUnderline,    'UNDERLINE',    'Ctrl+U',       rsAREditor_UnderlineHint,         FormatUnderlineExecute, CreateCheckedBnt);
    NewAct(rsAREditor_FormatIncreaseFont, 'INCREASEFONT', 'Ctrl+]',       rsAREditor_IncreaseFontHint,      FormatIncreaseFontSizeExecute, CreateBnt);
    NewAct(rsAREditor_FormatDecreaseFont, 'DECREASEFONT', 'Ctrl+[',       rsAREditor_DecreaseFontHint,      FormatDecreaseFontSizeExecute, CreateBnt);
    NewAct(rsAREditor_FormatLeftalign,    'LEFTALIGN',    'Ctrl+L',       rsAREditor_LeftalignHint,         FormatLeftalignExecute, CreateCheckedBnt);
    NewAct(rsAREditor_FormatCenteralign,  'CENTERALIGN',  'Ctrl+E',       rsAREditor_CenteralignHint,       FormatCenteralignExecute, CreateCheckedBnt);
    NewAct(rsAREditor_FormatRightalign,   'RIGHTALIGN',   'Ctrl+R',       rsAREditor_RightalignHint,        FormatRightalignExecute, CreateCheckedBnt);
    NewAct(rsAREditor_FormatToplayout,    'TOPLAYOUT',    'Ctrl+T',       rsAREditor_ToplayoutHint,         FormatToplayoutExecute, CreateCheckedBnt);
    NewAct(rsAREditor_FormatCenterlayout, 'CENTERLAYOUT', 'Ctrl+G',       rsAREditor_CenterlayoutHint,      FormatCenterlayoutExecute, CreateCheckedBnt);
    NewAct(rsAREditor_FormatBottomlayout, 'BOTTOMLAYOUT', 'Ctrl+M',       rsAREditor_BottomlayoutHint,      FormatBottomlayoutExecute, CreateCheckedBnt);

  Category:=Category_Frame;
    NewAct(rsAREditor_FrameFrameColor,    'FRAMECOLOR',   '',             rsAREditor_FrameColorHint,        FrameFrameColorExecute,  CreateColorBnt);
    NewAct(rsAREditor_FrameFrameWidth,    'NOTFRAME',     '',             rsAREditor_FrameWidthHint,        FrameFrameWidthExecute,  CreateGridBnt);
    NewAct(rsAREditor_FrameNotFrame,      'NOTFRAME',     'Ctrl+0',       rsAREditor_NotFrameHint,          FrameNotFrameExecute,    CreateCheckedBnt);

    NewAct(rsAREditor_FrameFrameLeft,     'NOTFRAME',     'Ctrl+1',       rsAREditor_FrameLeftHint,         FrameFrameExecuteInc,    CreateLeftGridBnt);
    NewAct(rsAREditor_FrameFrameRight,    'NOTFRAME',     'Ctrl+2',       rsAREditor_FrameRightHint,        FrameFrameExecuteInc,    CreateRightGridBnt);
    NewAct(rsAREditor_FrameFrameTop,      'NOTFRAME',     'Ctrl+3',       rsAREditor_FrameTopHint,          FrameFrameExecuteInc,    CreateTopGridBnt);
    NewAct(rsAREditor_FrameFrameBottom,   'NOTFRAME',     'Ctrl+4',       rsAREditor_FrameBottomHint,       FrameFrameExecuteInc,    CreateBottomGridBnt);

    NewAct(rsAREditor_FrameFrameLeftDec,  'NOTFRAME',     'Shift+Ctrl+1', rsAREditor_FrameLeftHintDec,      FrameFrameExecuteDec,    CreateLeftGridDec);
    NewAct(rsAREditor_FrameFrameRightDec, 'NOTFRAME',     'Shift+Ctrl+2', rsAREditor_FrameRightHintDec,     FrameFrameExecuteDec,    CreateRightGridDec);
    NewAct(rsAREditor_FrameFrameTopDec,   'NOTFRAME',     'Shift+Ctrl+3', rsAREditor_FrameTopHintDec,       FrameFrameExecuteDec,    CreateTopGridDec);
    NewAct(rsAREditor_FrameFrameBottomDec,'NOTFRAME',     'Shift+Ctrl+4', rsAREditor_FrameBottomHintDec,    FrameFrameExecuteDec,    CreateBottomGridDec);

  Category:=Category_Options;
    NewAct(rsAREditor_OptionsPreferences, '',             '',             rsAREditor_PreferencesHint,       OptionsPreferencesExecute,  CreateBnt);
    NewAct(rsAREditor_OptionsDefaultFont, '',             '',             rsAREditor_DefaultFontHint,       OptionsDefaultFontExecute,  CreateBnt);
    NewAct(rsAREditor_OptionsUserVars,    'EMPTY',        '',             rsAREditor_UserVarsHint,          OptionsUserVarsExecute,     CreateComboMoreBnt);
    NewAct(rsAREditor_OptionsUnits,       'UNITS',        '',             '',                               nil, nil);
    NewAct(rsAREditor_UnitsInches,        'INCHES',       '',             rsAREditor_UnitsInchesHint,       UnitsInchesExecute,         CreateCheckedBnt);
    NewAct(rsAREditor_UnitsMillemeters,   'MM',           '',             rsAREditor_UnitsMillemetersHint,  UnitsMillemetersExecute,    CreateCheckedBnt);
    NewAct(rsAREditor_UnitsCentimeters,   'CM',           '',             rsAREditor_UnitsCentimetersHint,  UnitsCentimetersExecute,    CreateCheckedBnt);
    NewAct(rsAREditor_UnitsPixels,        'PIXELS',       '',             rsAREditor_UnitsPixelsHint,       UnitsPixelsExecute,         CreateCheckedBnt);
    NewAct(rsAREditor_OptionsShowAll,     'SHOWALL',      '',             rsAREditor_ShowAllHint,           OptionsShowAllExecute,      CreateCheckedBnt);
    NewAct(rsAREditor_OptionsFileSystem,  '',             '',             rsAREditor_StandardHint,          OptionsFileSystemExecute,   CreateBnt);

  Category:=Category_Help;
    NewAct(rsAREditor_HelpContents,       'CONTENTS',     'F1',           rsAREditor_ContentsHint,          HelpContentsExecute, CreateBnt);
    NewAct(rsAREditor_HelpAbout,          '',             '',             rsAREditor_AboutHint,             HelpAboutExecute, CreateBnt);
end;

procedure TAREditorDlg.CreateBnt(Sender:TARAction; AOwner:TComponent; Parent:TParControl; var Control:TControl);
begin
  Control:=TARGlyphButton.Create(self);
  Control.Parent:=Parent;
end;

procedure TAREditorDlg.CreateCheckedBnt(Sender:TARAction; AOwner:TComponent; Parent:TParControl; var Control:TControl);
begin
  Control:=TARGlyphButton.Create(self);
  Control.Parent:=Parent;
end;

procedure TAREditorDlg.CreateColorBnt(Sender:TARAction; AOwner:TComponent; Parent:TParControl; var Control:TControl);
begin
  Control:=TARColorButton.Create(self);
  Control.Parent:=Parent;
  TARColorButton(Control).OnGetDocColors:=GetDocColorEvent;
end;

procedure TAREditorDlg.CreateGridBnt(Sender:TARAction; AOwner:TComponent; Parent:TParControl; var Control:TControl);
begin
  Control:=TARGridButton.Create(self);
  Control.Parent:=Parent;
  TARGridButton(Control).GridKind:=[frLeft,frRight,frTop,frBottom];
  TARGridButton(Control).ShowColors:=false;
end;

procedure TAREditorDlg.CreateLeftGridBnt(Sender:TARAction; AOwner:TComponent; Parent:TParControl; var Control:TControl);
begin
  CreateGridBnt(Sender, AOwner, Parent, Control);
  Control.Parent:=Parent;
  Sender.GridKind:=[frLeft];
  TARGridButton(Control).GridKind:=[frLeft];
  TARGridButton(Control).ShowColors:=true;
  TARGridButton(Control).OnClick :=FrameFrameClick;
end;
procedure TAREditorDlg.CreateRightGridBnt(Sender:TARAction; AOwner:TComponent; Parent:TParControl; var Control:TControl);
begin
  CreateGridBnt(Sender, AOwner, Parent, Control);
  Control.Parent:=Parent;
  Sender.GridKind:=[frRight];
  TARGridButton(Control).GridKind:=[frRight];
  TARGridButton(Control).ShowColors:=true;
  TARGridButton(Control).OnClick :=FrameFrameClick;
end;
procedure TAREditorDlg.CreateTopGridBnt(Sender:TARAction; AOwner:TComponent; Parent:TParControl; var Control:TControl);
begin
  CreateGridBnt(Sender, AOwner, Parent, Control);
  Control.Parent:=Parent;
  Sender.GridKind:=[frTop];
  TARGridButton(Control).GridKind:=[frTop];
  TARGridButton(Control).ShowColors:=true;
  TARGridButton(Control).OnClick :=FrameFrameClick;
end;
procedure TAREditorDlg.CreateBottomGridBnt(Sender:TARAction; AOwner:TComponent; Parent:TParControl; var Control:TControl);
begin
  CreateGridBnt(Sender, AOwner, Parent, Control);
  Control.Parent:=Parent;
  Sender.GridKind:=[frBottom];
  TARGridButton(Control).GridKind:=[frBottom];
  TARGridButton(Control).ShowColors:=true;
  TARGridButton(Control).OnClick :=FrameFrameClick;
end;

procedure TAREditorDlg.CreateLeftGridDec(Sender:TARAction; AOwner:TComponent; Parent:TParControl; var Control:TControl);
begin
  Sender.GridKind:=[frLeft];
end;
procedure TAREditorDlg.CreateRightGridDec(Sender:TARAction; AOwner:TComponent; Parent:TParControl; var Control:TControl);
begin
  Sender.GridKind:=[frRight];
end;
procedure TAREditorDlg.CreateTopGridDec(Sender:TARAction; AOwner:TComponent; Parent:TParControl; var Control:TControl);
begin
  Sender.GridKind:=[frTop];
end;
procedure TAREditorDlg.CreateBottomGridDec(Sender:TARAction; AOwner:TComponent; Parent:TParControl; var Control:TControl);
begin
  Sender.GridKind:=[frBottom];
end;

procedure TAREditorDlg.CreateFontNameBnt(Sender:TARAction; AOwner:TComponent; Parent:TParControl; var Control:TControl);
begin
  Control:=TARFontButton.Create(self);
  Control.Parent:=Parent;
end;

procedure TAREditorDlg.CreateComboMoreBnt(Sender:TARAction; AOwner:TComponent; Parent:TParControl; var Control:TControl);
begin
  Control:=TARComboMoreButton.Create(self);
  Control.Parent:=Parent;
  TARComboMoreButton(Control).OnMoreBtnClick:=OptionsUserVarsDialogExecute;
end;

procedure TAREditorDlg.CreateFontSizeBnt(Sender:TARAction; AOwner:TComponent; Parent:TParControl; var Control:TControl);
begin
  Control:=TARComboEditButton.Create(self);
  Control.Parent:=Parent;
  TARComboEditButton(Control).Items.Add('8');
  TARComboEditButton(Control).Items.Add('9');
  TARComboEditButton(Control).Items.Add('10');
  TARComboEditButton(Control).Items.Add('11');
  TARComboEditButton(Control).Items.Add('12');
  TARComboEditButton(Control).Items.Add('14');
  TARComboEditButton(Control).Items.Add('16');
  TARComboEditButton(Control).Items.Add('18');
  TARComboEditButton(Control).Items.Add('20');
  TARComboEditButton(Control).Items.Add('22');
  TARComboEditButton(Control).Items.Add('24');
  TARComboEditButton(Control).Items.Add('26');
  TARComboEditButton(Control).Items.Add('28');
  TARComboEditButton(Control).Items.Add('36');
  TARComboEditButton(Control).Items.Add('48');
  TARComboEditButton(Control).Items.Add('72');
end;

procedure  TAREditorDlg.GetDocColorEvent(Sender:TObject);
var idxband, idxcell:integer;
    idx:TFrameStyles;
    cell:TARCell;
begin
  TARColorButton(Sender).ClearDocColor;
  if Sender is TARColorButton then begin
    for idxband:=0 to rep.SrcRep.BandCount - 1 do begin
      for idxcell:=0 to rep.SrcRep.Bands[idxband].CellCount - 1 do begin
        cell:=rep.SrcRep.Bands[idxband].Cells[idxcell];
        TARColorButton(Sender).AddDocColor(cell.Color);
        TARColorButton(Sender).AddDocColor(cell.FontColor);
        for idx:=Low(TFrameStyles) to High(TFrameStyles) do begin
          TARColorButton(Sender).AddDocColor(cell.FrameColors[idx]);
        end;
      end;
    end;
  end;
end;

procedure TAREditorDlg.RepSelectBand(Sender: TObject; IdxBand: Integer);
begin
  ShowBandEditor(rep.CurrBandIdx);
end;

procedure TAREditorDlg.CellsCellTextExecute(Sender: TObject);
begin
  if rep.Focused then begin
    ShowCellEditor(rep.CurrBandIdx, rep.CurrCellIdx, #0);
  end else begin
    if CellEditor.Focused then begin
      repSetFocus;
    end else begin
    end;
  end;
end;

procedure TAREditorDlg.BandsCopyBandsExecute(Sender: TObject);
var idxband:integer;
    sl:TAIStringList;
begin
  sl:=TAIStringList.Create('TAREditorDlg sl');
  try
    for idxband:=0 to rep.SrcRep.BandCount - 1 do begin
      if rep.SrcRep.Bands[idxband].Selected then begin
        sl.AddStrings(rep.SrcRep.Bands[idxband].ARTemplate);
      end;
    end;
    Clipboard.AsText:=sl.Text;
  finally
    sl.Free;
  end;
end;

procedure TAREditorDlg.BandsCutBandsExecute(Sender: TObject);
begin
  BandsCopyBandsExecute(Sender);
  BandsDeleteBandExecute(Sender);
end;

procedure TAREditorDlg.BandsPasteBandsExecute(Sender: TObject);
var sl:TAIStringList;
//    slb:TAIStringList;
    slb:TARTemplate;
    idxband:integer;
begin
  sl:=TAIStringList.Create('TAREditorDlg sl');
  slb:=TARTemplate.Create('TAREditorDlg slb');
  try
    sl.Text:=Clipboard.AsText;
    if (sl.Count > 0) and CT(Trim(sl[0]), 'BAND') then begin
      idxband:=rep.CurrBandIdx;
      while sl.Count > 0 do begin
        slb.Add(sl[0]);
        if CT(Trim(sl[0]), 'ENDBAND') then begin
          rep.NewBand;
          rep.SrcRep.MoveBand(rep.SrcRep.BandCount - 1, idxband);
          rep.CurrBandIdx:=idxband;
          rep.CurrBand.ARTemplate:=slb;
          inc(idxband);
          slb.Clear;
        end;
        sl.Delete(0);
      end;
      Rep.Invalidate;
      Modified:=true;
    end;
  finally
    slb.Free;
    sl.Free;
  end;
end;

procedure TAREditorDlg.CellsCopyCellExecute(Sender: TObject);
var i:integer;
    sl:TAIStringList;
begin
  sl:=TAIStringList.Create('TAREditorDlg sl');
  try
    for i:=0 to rep.SelCount-1 do begin
      sl.AddStrings(rep.Sel[i].ARTemplate);
    end;
    Clipboard.AsText:=sl.Text;
  finally
    sl.Free;
  end;
end;

procedure TAREditorDlg.CellsCutCellExecute(Sender: TObject);
begin
  CellsCopyCellExecute(Sender);
  CellsDeleteCellExecute(Sender);
end;

procedure TAREditorDlg.CellsPasteCellExecute(Sender: TObject);
var sl:TAIStringList;
    slc:TARTemplate;
    idxcell:integer;
begin
  sl:=TAIStringList.Create('TAREditorDlg sl');
  slc:=TARTemplate.Create('TAREditorDlg slc');
  try
    sl.Text:=Clipboard.AsText;
    if (sl.Count > 0) and CT(Trim(sl[0]), 'CELL') then begin
      idxcell:=rep.CurrCellIdx;
      while sl.Count > 0 do begin
        slc.Add(sl[0]);
        if CT(Trim(sl[0]), 'ENDCELL') then begin
          rep.CurrBand.NewCell;
          rep.CurrBand.MoveCell(rep.CurrBand.CellCount - 1, idxcell);
          rep.CurrCellIdx:=idxcell;
          rep.CurrCell.ARTemplate:=slc;
          inc(idxcell);
          slc.Clear;
        end;
        sl.Delete(0);
      end;
      Rep.Invalidate;
      Modified:=true;
    end;
  finally
    slc.Free;
    sl.Free;
  end;
end;

procedure   TAREditorDlg.CellsHMergeCellsExecute(Sender:TObject);
begin
  if rep.MergeSelectedCells then begin
    rep.Invalidate;
    Modified:=true;
  end;
end;


procedure TAREditorDlg.MainControlBarDblClick(Sender: TObject);
begin
//  ShowAIListCounts;
end;

procedure   TAREditorDlg.OptionsPreferencesExecute(Sender:TObject);
var d:TARPreferecesDlg;
begin
  d:=TARPreferecesDlg.Create(Self);
  d.Units:=Units;
  d.GridX:=GridX;
  d.GridY:=GridY;

  if d.ShowModal = mrOK then begin
    GridX:=d.GridX;
    GridY:=d.GridY;
  end;
end;

function    TAREditorDlg.GetGridX:extended;
begin
  Result:=FGridX;//Rep.GridX;
end;

function    TAREditorDlg.GetGridY:extended;
begin
  Result:=FGridY;//Rep.GridY;
end;

procedure   TAREditorDlg.SetGridX(AGridX:extended);
begin
  FGridX:=AGridX;
  if Assigned(Rep) then Rep.GridX:=AGridX;
end;

procedure   TAREditorDlg.SetGridY(AGridY:extended);
begin
  FGridY:=AGridY;
  if Assigned(Rep) then Rep.GridY:=AGridY;
end;

procedure TAREditorDlg.RepSetFocus;
begin
  if Assigned(Rep) then begin
    PC.ActivePage:=TTabSheet(Rep.Parent);
    if Self.Visible and Assigned(Rep.Parent) then begin
      Rep.SetFocus;
    end;
  end;
  CloseupAllPopupForms;
end;

function    TAREditorDlg.GetRep:TAREditRep;
begin
  Result:=FRep;
end;

function    TAREditorDlg.GetFileSystem:TCommonFileSystem;
begin
  Result:=FFileSystem;
end;

function    TAREditorDlg.GetUndo:TUndo;
begin
  Result:=FRep.Undo;
end;

function    TAREditorDlg.GetFileName:string;
begin
  Result:=FRep.FileName;
end;

function    TAREditorDlg.GetModified:boolean;
begin
  Result:=FRep.Modified;
end;

procedure   TAREditorDlg.SetModified(AModified:boolean);
begin
  FRep.Modified:=AModified;

  if AModified then begin
    if FSetUndo then begin
      Undo.Add(Rep.ARTemplate.Text);
    end;
    StatusBar.Panels[0].Text:=rsAREditor_ReportChanged;
  end else begin
    StatusBar.Panels[0].Text:='';
  end;

  CloseupAllPopupForms;
  UpdateButtons;
  UpdateRulerBars;
end;

procedure   TAREditorDlg.UpdateCaption;
begin
  if Assigned(FileSystem) then begin
    PC.ActivePage.Caption:=FileSystem.ExtractFileNameOnly(FileName);
  end;
  Caption:=FileName;
end;

procedure   TAREditorDlg.PCChange(Sender: TObject);
var ps:TTabSheet;
    i:integer;
begin
  ps:=PC.ActivePage;
  if Assigned(ps) then begin
    case TAREPageType(ps.Tag) of
      ptTemplate: begin
                    FRep:=nil;
                    for i:=0 to ps.ComponentCount - 1 do begin
                      if ps.Components[i] is TAREditRep then begin
                        FRep:=ps.Components[i] as TAREditRep;
                        FRep.Units:=Units;
                        FRep.GridX:=GridX;
                        FRep.GridY:=GridY;

//                        MainControlBar.Parent:=ps;

                        VRB.Parent:=ps;
                        HRB.Parent:=ps;
//                        MainControlBar.Top:=0;

                        Modified:=Modified;

                        RepSetFocus;
                        Caption:=FRep.FileName;
                        break;
                      end;
                    end;
                  end;
      ptHelp:     begin
                    for i:=0 to ps.ComponentCount - 1 do begin
                      {$IFDEF D3}
                        {$IFDEF D3_CLIENTSERVER}
                          if ps.Components[i] is THTML then begin
                            Caption:=THTML(ps.Components[i]).URL;
                            break;
                          end;
                        {$ENDIF}
                      {$ELSE}
                        if ps.Components[i] is TWebBrowser then begin
                          {$IFDEF CLX}
                            Caption:=TWebBrowser(ps.Components[i]).FileName;
                          {$ELSE}
                            Caption:=TWebBrowser(ps.Components[i]).LocationURL;
                          {$ENDIF}
                          break;
                        end;
                      {$ENDIF}
                    end;
                  end;
    end;
  end;
end;

function    TAREditorDlg.NewPage(PageType:TAREPageType):TTabSheet;
begin
  Result:=nil;
  case PageType of
    ptTemplate: Result:=NewTemplatePage;
    ptHelp:     Result:=NewHelpPage;
    ptPreview:  Result:=nil;
    ptScript:   Result:=nil;
  end;

  ActList[Category_File, rsAREditor_FileClosePage].Enabled:=PC.PageCount > 1;
end;

procedure   TAREditorDlg.SetFileName(const AFileName:string);
begin
  FRep.FileName:=AFileName;
  FileSystem.FileName:=FileName;
  UpdateCaption;
end;

function  TAREditorDlg.NewReport(const AFileName:string):boolean;
var s:string;
begin
  s:=Trim(AFileName);
  TemplateList.AddObject(s, NewPage(ptTemplate));
  FileName:=s;

  rep.DeleteBands;
  rep.NewBand;
  rep.NewBand;
  rep.NewBand;
  rep.NewBand;
  rep.Invalidate;
  Modified:=false;

  PCChange(Self);

  Result:=true;
end;

procedure TAREditorDlg.LoadReport(const AFileName:string);
var s:string;
begin
  s:=Trim(AFileName);
  if not ShowPage(s) then begin
    NewReport(s);
    Modified:=false;

    if FileSystem.FileExists(s) then begin
      rep.FileSystem:=FileSystem;
      rep.Load(s);
      if rep.SrcRep.UserVarList.Count > 0 then begin
        UserVarList.Assign(rep.SrcRep.UserVarList);
      end else begin
        rep.SrcRep.UserVarList.Assign(UserVarList);
      end;
    end else begin
      rep.SrcRep.UserVarList.Assign(UserVarList);
    end;

    PCChange(Self);
//    CreateCanvasImage;

    Undo.Clear;
    Undo.Add(Rep.ARTemplate.Text);

    AddToRecentFiles(s);
    UpdateRulerBars;
  end;
end;

function  TAREditorDlg.ShowPage(const AFileName:string):boolean;
var idx:integer;
begin
  idx:=TemplateList.IndexOf(AFileName);
  if idx <> -1 then begin
    PC.ActivePage:=TTabSheet(TemplateList.Objects[idx]);
    PCChange(Self);
  end;

  Result:=idx <> -1;
end;

procedure TAREditorDlg.FileNewExecute(Sender: TObject);
begin
  NewReport(rsAREditor_NoName);
end;

procedure TAREditorDlg.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  if TemplateList.Count > 0 then begin
    SaveLastEditedFiles;
  end;
  CanClose:=CloseAllPages;
end;

procedure TAREditorDlg.FileClosePageExecute(Sender: TObject);
begin
  ClosePage(true, true);
end;

function  TAREditorDlg.SaveAll:boolean;
var i:integer;
    ps:TTabSheet;
begin
  Result:=true;

  ps:=PC.ActivePage;
  try
    for i:=0 to TemplateList.Count - 1 do begin
      ShowPage(TemplateList[i]);
      if not SaveReport then begin
        Result:=false;
        break;
      end;
    end;
  finally
    PC.ActivePage:=ps;
    PCChange(self);
  end;
end;

procedure TAREditorDlg.FileSaveAllExecute(Sender: TObject);
begin
  SaveAll;
end;

function  TAREditorDlg.CloseAllPages:boolean;
var res:integer;
    QueryToSave:boolean;
begin
  Result:=true;

  QueryToSave:=true;
  while TemplateList.Count > 0 do begin
    ShowPage(TemplateList[0]);
    if Modified then begin
      if QueryToSave then begin
        {$IFDEF CLX}
          res:=MessageDlg(rsAREditor_SaveChangesQuery, mtConfirmation, [mbYes, mbNo, mbCancel], 0);
        {$ELSE}
          res:=MessageDlg(rsAREditor_SaveChangesQuery, mtConfirmation, [mbYes, mbNo, mbYesToAll, mbCancel], 0);
        {$ENDIF}
        case res of
          mrYes:      begin
                        ClosePage(true, false);
                      end;
          mrYesToAll: begin
                        ClosePage(true, false);
                        QueryToSave:=false;
                      end;
          mrNo:       begin
                        ClosePage(false, false);
                      end;
          mrCancel:   begin
                        Result:=false;
                        exit;
                      end;
        end;
      end else begin
        ClosePage(true, false);
      end;
    end else begin
      ClosePage(true, false);
    end;
  end;

  while pc.PageCount > 0 do begin
    ClosePage(true, QueryToSave);
  end;
end;

function    TAREditorDlg.NewTemplatePage:TTabSheet;
begin
  Result:=TTabSheet.Create(PC);
  Result.PageControl:=PC;
  Result.Tag:=Ord(ptTemplate);

  FRep:=TAREditRep.Create(Result);
  FRep.Parent:=Result;
  FRep.Align:=alClient;

  FRep.SelectedCellStyle:=scsGray;
  FRep.FocusedCellStyle :=fcsRect;// fcsBlackAnt;

  FRep.TabStop        :=True;
  FRep.TabOrder       :=3;
  FRep.PopupMenu      :=RepPopupMenu;
  FRep.Options        :=[roEditing];
  FRep.Color          :=clBtnFace;

  FRep.OnHScroll      :=RepScroll;
  FRep.OnVScroll      :=RepScroll;
  FRep.OnKeyPress     :=RepKeyPress;
  FRep.OnMouseMove    :=RepMouseMove;
  FRep.OnResizeBand   :=RepResizeBand;
  FRep.OnResizeCell   :=RepResizeCell;
  FRep.OnResizeGutter :=RepResizeGutter;
  FRep.OnSelectBand   :=RepSelectBand;
  FRep.OnSelectCell   :=RepSelectCell;
  FRep.OnDblClick     :=CellsCellPropertiesExecute;
  FRep.OnEnter        :=FormResize;

  FRep.Undo:=TUndo.Create(UndoLimitDefault);
  Undo.Clear;
  Undo.Add(Rep.ARTemplate.Text);

  PC.ActivePage:=Result;
  PCChange(Self);
end;

function    TAREditorDlg.NewHelpPage:TTabSheet;
begin
  Result:=TTabSheet.Create(PC);
  Result.PageControl:=PC;
  Result.Tag:=Ord(ptHelp);
  Result.Caption:='';

  PC.ActivePage:=Result;
  PCChange(Self);
end;

procedure TAREditorDlg.HelpContentsExecute(Sender: TObject);
//var cmd:string;
begin
//  Application.HelpJump('ARE_Menus');
//  cmd:='explorer ' + ExtractFilePath(ParamStr(0)) + '..\help\areditor.html';
//  WinExec(PChar(cmd), SW_SHOW);

  ShowHelpPage;
end;

{$IFDEF CLX}
function  TAREditorDlg.DisplayHelp(HelpType:THelpType; HelpContext:THelpContext; HelpKeyword:string; HelpFile:string; var Handled:boolean):boolean;
begin
  HelpContentsExecute(Self);
  Handled:=true;
  Result:=true;
end;
{$ELSE}
function  TAREditorDlg.DisplayHelp(Command:Word; Data:Longint; var CallHelp:Boolean):Boolean;
begin
  CallHelp:=false;
  HelpContentsExecute(Self);
  Result:=true;
end;
{$ENDIF}

procedure TAREditorDlg.ShowHelpPage;
  {$IFDEF D3}
    {$IFDEF D3_CLIENTSERVER}
      procedure ShowHelp(ps:TTabSheet; WB:THTML);
      begin
        WB.RequestDoc(ExtractFilePath(ParamStr(0)) + '..\HELP\areditor.html');
        Caption:=WB.RequestURL;
      end;
    {$ENDIF}
  {$ELSE}
    procedure ShowHelp(ps:TTabSheet; WB:TWebBrowser);
    begin
      {$IFDEF CLX}
        WB.FileName:=ExtractFilePath(ParamStr(0)) + '..\HELP\areditor.html';
        Caption:=WB.FileName;
      {$ELSE}
        WB.Navigate(ExtractFilePath(ParamStr(0)) + '..\HELP\areditor.html');
        Caption:=WB.LocationURL;
      {$ENDIF}
      ps.Caption:=Flat(rsAREditor_Help);
    end;
  {$ENDIF}

var i, j:integer;
  {$IFDEF D3}
    {$IFDEF D3_CLIENTSERVER}
      WB:THTML;
    {$ENDIF}
  {$ELSE}
    WB:TWebBrowser;
  {$ENDIF}
    ps:TTabSheet;
begin
  for i:=0 to PC.PageCount - 1 do begin
    ps:=PC.Pages[i];
    if ps.Tag = Ord(ptHelp) then begin
      PC.ActivePage:=ps; PCChange(Self);
      for j:=0 to ps.ControlCount - 1 do begin
        {$IFDEF D3}
          {$IFDEF D3_CLIENTSERVER}
            if ps.Controls[j] is THTML then begin
              ShowHelp(ps, THTML(ps.Controls[j]));
              exit;
            end;
          {$ENDIF}
        {$ELSE}
          if ps.Controls[j] is TWebBrowser then begin
            ShowHelp(ps, TWebBrowser(ps.Controls[j]));
            exit;
          end;
        {$ENDIF}
      end;
    end;
  end;
  {$IFDEF D3}
    {$IFDEF D3_CLIENTSERVER}
      ps:=NewPage(ptHelp);
      WB:=THTML.Create(ps);
      TControl(WB).Parent:=ps;
      WB.Align:=alClient;

      ShowHelp(ps, WB);
    {$ENDIF}
  {$ELSE}
    ps:=NewPage(ptHelp);
    WB:=TWebBrowser.Create(ps);
    TControl(WB).Parent:=ps;
    WB.Align:=alClient;

    ShowHelp(ps, WB);
  {$ENDIF}
end;

function  TAREditorDlg.ClosePage(SaveChanges:boolean; QueryToSave:boolean):boolean;
var res:integer;
    ps:TTabSheet;
    idx:integer;
begin
  Result:=true;
  ps:=PC.ActivePage;
  if ps.Tag in [Ord(ptTemplate), Ord(ptScript)] then begin
    if SaveChanges and Modified then begin
      if QueryToSave then begin
        res:=MessageDlg(rsAREditor_SaveChangesQuery, mtConfirmation, [mbYes, mbNo, mbCancel], 0);
        case res of
          mrYes:    if not SaveReport then begin
                      Result:=false;
                      exit;
                    end;
          mrNo:     ;
          mrCancel: begin
                      Result:=false;
                      exit;
                    end;
        end;
      end else begin
        if not SaveReport then begin
          Result:=false;
          exit;
        end;
      end;
    end;

    idx:=TemplateList.IndexOfObject(ps);
    if idx >=0 then begin
 // sa: 11.06.2001
       for res := 0 to TTabSheet(TemplateList.Objects[idx]).ControlCount - 1 do begin
         if TTabSheet(TemplateList.Objects[idx]).Controls[res] is TAREditRep then begin
           TAREditRep(TTabSheet(TemplateList.Objects[idx]).Controls[res]).Undo.Free;
         end;
       end;
 //--
      TemplateList.Delete(idx);
    end;
  end;

  HRB.Parent:=nil;
  VRB.Parent:=nil;

  PC.SelectNextPage(false);

  ps.Free;
  ActList[Category_File, rsAREditor_FileClosePage].Enabled:=PC.PageCount > 1;

  PCChange(Self);
end;

{
procedure TAREditorDlg.ShowPropTemplate;
var i:integer;
    ps:TTabSheet;
begin
  for i:=0 to PC.PageCount - 1 do begin
    ps:=PC.Pages[i];
    if ps.Tag = Ord(ptTemplate) then begin
      PC.ActivePage:=ps; PCChange(Self);
      if Rep.PropEditor then exit;
    end;
  end;
end;
}
function  TAREditorDlg.SaveReport:boolean;
begin
  if not Modified then begin
    Result:=true;
    exit;
  end;

  Result:=false;

  rep.SrcRep.UserVarList.Assign(UserVarList);

  if rep.PropEditor then begin
    EditingRep.ARTemplate:=Rep.ARTemplate;
    Result:=true; // do nothing
  end else begin
    if (FileName <> '') and (FileName <> rsAREditor_NoName) then begin
      rep.FileSystem:=FileSystem;
      rep.Save(FileName);
      UpdateCaption;
      Modified:=false;
      Result:=true;
    end else begin
      FileSystem.FileName:=FileName;
      if FileSystem.SaveDialogExecute then begin
        FileName:=FileSystem.FileName;
        rep.FileSystem:=FileSystem;
        rep.Save(FileSystem.FileName);
        Caption:=FileSystem.Caption;
        Modified:=false;
        Result:=true;
      end;
    end;
  end;
end;

procedure   TAREditorDlg.SetEditingRep(ARep:TARGridRep);
begin
  EditingRep:=ARep;
  PropEditor:=true;

  NewTemplatePage;
  Rep.PropEditor:=true;
  Rep.ARTemplate:=ARep.ARTemplate;
  Rep.FileName          :='property:' + ARep.Name;  //'$' + ARep.Name;
  PC.ActivePage.Caption :='property:' + ARep.Name;  //'$' + ARep.Name;
end;

procedure   TAREditorDlg.CellsVMergeCellsExecute(Sender:TObject);
var idxband, idxcell, idxband2, idxcell2, l, r:integer;
    cell:TARCell;
begin
  try
    for idxband:=0 to Rep.SrcRep.BandCount - 1 do begin
      if Rep.SrcRep.Bands[idxband].Selected then begin
        for idxcell:=0 to Rep.SrcRep.Bands[idxband].CellCount - 1 do begin
          if Rep.SrcRep.Bands[idxband].Cells[idxcell].Selected then begin
            cell:=Rep.SrcRep.Bands[idxband].Cells[idxcell];
            l:=Rep.SrcRep.Bands[idxband].Lefts[idxcell];
            r:=Rep.SrcRep.Bands[idxband].Rights[idxcell];

            for idxband2:=idxband + 1 to Rep.SrcRep.BandCount - 1 do begin
              if not Rep.SrcRep.Bands[idxband2].Selected then begin
                cell.Focused:=true;
                exit;
              end;

              for idxcell2:=0 to Rep.SrcRep.Bands[idxband2].CellCount - 1 do begin
                if  Rep.SrcRep.Bands[idxband2].Cells[idxcell2].Selected and
                   (Rep.SrcRep.Bands[idxband2].Lefts[idxcell2] = l) and
                   (Rep.SrcRep.Bands[idxband2].Rights[idxcell2] = r)
                then begin
                  Rep.SrcRep.Bands[idxband2].Cells[idxcell2].Shared:=true;
                  Rep.SrcRep.Bands[idxband2].BandChange(Self);
                  break;
                end;
              end;
            end;
          end;
        end;
      end;
    end;
  finally
    Modified:=true;
    RepSetFocus;
    Rep.Repaint;
  end;
end;

procedure TAREditorDlg.CreateRulers;
begin
  HRB             :=TARRulerBar.Create(Self);
  HRB.Parent      :=Self;
  HRB.Orientation :=rbHorizontal;
  HRB.SetBounds(0, 20, 702, 35);

  HRB.Gutter      := 100;
  HRB.PageWidth   := 1500;
  HRB.LeftMargin  := 100;
  HRB.RightMargin := 100;
  HRB.Units       := uCm;
  HRB.SelBegin    := 100;
  HRB.SelEnd      := 300;
  HRB.Cur         := 0;
  HRB.SelColor    := clAqua;
  HRB.RulerColor  := clInfoBk;
  HRB.Font.Charset:= DEFAULT_CHARSET;
  HRB.Font.Color  := clWindowText;
  HRB.Font.Height := -11;
  HRB.Font.Name   := 'Arial';
  HRB.Font.Style  := [];
  HRB.OnChange    := HRBChange;

  VRB             :=TARRulerBar.Create(Self);
  VRB.Parent      :=Self;
  VRB.SetBounds(0, 55, 35, 417);
  VRB.Orientation := rbVertical;
  VRB.Gutter      := 0;
  VRB.PageWidth   := 1500;
  VRB.LeftMargin  := 100;
  VRB.RightMargin := 100;
  VRB.Units       := uCm;
  VRB.SelBegin    := 100;
  VRB.SelEnd      := 300;
  VRB.Cur         := 0;
  VRB.SelColor    := clAqua;
  VRB.RulerColor  := clInfoBk;
  VRB.Font.Charset:= ANSI_CHARSET;
  VRB.Font.Color  := clWindowText;
  VRB.Font.Height := -11;
  VRB.Font.Name   := 'Arial';
  VRB.Font.Style  := [];
  VRB.OnChange    := HRBChange;
end;

{
procedure TAREditorDlg.FileSystemStandardExecute(Sender: TObject);
var i:integer;
    s:string;
begin
  FileSystem:=TStandardFileSystem(Sender);
  FileSystem.Options:=FileSystem.Options + [ofOverwritePrompt];
  TStandardFileSystem(FileSystem).DefaultExt:='';

  if ExtList.Count > 0 then begin
    FileSystem.Filter:='';
    s:='';
    for i:=0 to ExtList.Count - 2 do begin
      s:=s + '*.' + ExtList[i] + ';';
    end;
    s:=s + '*.' + ExtList[ExtList.Count - 1];
    FileSystem.Filter:='Reports (' + s + ')|' + s + '|';

    TStandardFileSystem(FileSystem).DefaultExt:=ExtList[0];
  end;
  FileSystem.Filter:=FileSystem.Filter + 'All files (*.*)|*.*';

  FileSystem.OptionsMenuItem:=GetMenuItem('OptionsMenu');//OptionsMenu;
end;
}

procedure TAREditorDlg.OptionsFileSystemExecute(Sender: TObject);
begin
  FFileSystem.ExecuteSetup(Sender);
end;

procedure TAREditorDlg.FileSaveAsExecute(Sender: TObject);
begin
  FileSystem.FileName:=FileName;
  if FileSystem.SaveDialogExecute then begin
    FileName:=FileSystem.FileName;
    rep.FileSystem:=FileSystem;
    rep.Save(FileName);
    UpdateCaption;
    Modified:=false;
  end;
end;

procedure TAREditorDlg.FormActivate(Sender: TObject);
begin
  RepSetFocus();
  CloseupAllPopupForms;
end;

procedure TAREditorDlg.CellsCellPropertiesExecute(Sender: TObject);
var d:TARCellPropDlg;
begin
  d:=TARCellPropDlg.Create(self);
  try
    d.rep:=rep;
    if d.ShowModal() = mrOK then begin
      Modified:=Modified or d.Modified;
    end;
  finally
    d.Free();
  end;
  Rep.Repaint;  //CD
end;

procedure TAREditorDlg.FormShow(Sender: TObject);
{$IFDEF LINUX}
  var reg:TIniFile;
{$ENDIF}
{$IFDEF MSWINDOWS}
  var reg:TRegistry;
{$ENDIF}
begin
  if fToPrint or fToView then begin
    Close();
  end else begin
    {$IFDEF LINUX}
      reg:=TIniFile.Create(Application.ExeName + '.ini');
      try
        try
          SetBounds(reg.ReadInteger('Setup\', 'Left',   0),
                    reg.ReadInteger('Setup\', 'Top',    0),
                    reg.ReadInteger('Setup\', 'Width',  Screen.Width),
                    reg.ReadInteger('Setup\', 'Height', Screen.Height));
        except
        end;
      finally
        reg.UpdateFile;
        reg.Free;
      end;
    {$ENDIF}
    {$IFDEF MSWINDOWS}
      reg:=TRegistry.Create;
      try
        reg.OpenKey(RegRoot + 'Setup\', True);
        try
          SetBounds(reg.ReadInteger('Left'),
                    reg.ReadInteger('Top'),
                    reg.ReadInteger('Width'),
                    reg.ReadInteger('Height'));
        except
        end;
      finally
        reg.Free;
      end;
    {$ENDIF}
  end;
end;

procedure TAREditorDlg.FormResize(Sender: TObject);
begin
  CloseupAllPopupForms;
  ResizeToolBars;
end;

procedure TAREditorDlg.MainControlBarClick(Sender: TObject);
begin
  CloseupAllPopupForms;
end;

procedure TAREditorDlg.PCMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  CloseupAllPopupForms;
end;

procedure TAREditorDlg.FormCreate(Sender: TObject);
var FileName:string;
    i:integer;
    {$IFDEF LINUX}
      reg:TIniFile;
    {$ENDIF}
    {$IFDEF MSWINDOWS}
      reg:TRegistry;
    {$ENDIF}
begin
  MainControlBar          :=TControlBar.Create(Self);
  MainControlBar.Align    :=alTop;
  MainControlBar.Parent   :=Self;
  MainControlBar.PopupMenu:=BarPopupMenu;
  MainControlBar.AutoSize :=true;

  FFileSystem:=TCommonFileSystem.Create(Self, RegRoot);

  CreateRulers;
  UserVarList :=TAIStringList.Create('TAREditorDlg.UserVarList');
  TemplateList:=TAIStringList.Create('TAREditorDlg.TemplateList');
  RecentFiles:=TAIStringList.Create('TAREditorDlg.RecentFiles');

  FSetUndo:=true;

  prn:=TARPrinter.Create(Self);

  CellEditor:=TMemo.Create(Self);
  CellEditor.OnExit    :=CellEditorExit;
  CellEditor.OnChange  :=CellEditorChange;

  Caption                 :=rsAREditor_Caption;

  UserVarList.OnChange:=UserVarListChange;

  ActList:=TARActionList.Create;
  CreateActions;

  {$IFDEF LINUX}
    reg:=TIniFile.Create(Application.ExeName + '.ini');
  {$ENDIF}
  {$IFDEF MSWINDOWS}
    reg:=TRegistry.Create;
  {$ENDIF}
  try
    {$IFDEF MSWINDOWS}
      reg.OpenKey(RegRoot + 'Setup\', True);
    {$ENDIF}
    try
      {$IFDEF LINUX}
        SetUnits(TARUnits(reg.ReadInteger('Setup\', 'UnitsIndex', 0)));
        GridX:=reg.ReadFloat('Setup\', 'GridX', 4.0);
        GridY:=reg.ReadFloat('Setup\', 'GridY', 4.0);
      {$ENDIF}
      {$IFDEF MSWINDOWS}
        SetUnits(TARUnits(reg.ReadInteger('UnitsIndex')));
        GridX:=reg.ReadFloat('GridX');
        GridY:=reg.ReadFloat('GridY');
      {$ENDIF}
      if (GridX = 0) or (GridY = 0) then begin
        SetUnits(TARUnits(0));
        GridX:=4.0;
        GridY:=4.0;
      end;
    except
      SetUnits(TARUnits(0));
      GridX:=4.0;
      GridY:=4.0;
    end;

/////////
    {$IFDEF LINUX}
      reg.ReadSection('Setup\Objects\', UserVarList);
    {$ENDIF}
    {$IFDEF MSWINDOWS}
      reg.OpenKey(RegRoot + 'Setup\Objects', True);
      reg.GetValueNames(UserVarList);
    {$ENDIF}
    if UserVarList.Count = 0 then begin
      UserVarList.Add('...');
      UserVarList.Add('Object 1');
      UserVarList.Add('Object 2');
      UserVarList.Add('Object 3');
      UserVarList.Add('Object 4');
    end;

/////////
    if not PropEditor then begin
      fToPrint:=false;
      fToView :=false;
      if ParamCount > 0 then begin
        FileName      :='';
        for i:=1 to ParamCount do begin
          if CT(ParamStr(i), '/p') then begin
            fToPrint:=true;
          end else
          if CT(ParamStr(i), '/v') then begin
            fToView:=true;
          end else begin
            FileName:=FileName + ParamStr(i) + ' ';
          end;
        end;
        try
          LoadReport(FileName);
        except
          fToPrint:=false;
          fToView :=false;
//          NewReport(rsAREditor_NoName);
        end;
        if fToPrint then begin
          Rep.Print;
        end else
        if fToView then begin
          Rep.Preview;
        end;
      end else begin
        try
          if not Assigned(Rep) then NewReport(rsAREditor_NoName);
        except
        end;
      end;
    end;
    RepSetFocus;
  finally
    {$IFDEF LINUX}
      reg.UpdateFile;
    {$ENDIF}
    reg.Free;
  end;

  LoadToolBars;

  CreateToolButton(Category_Frame, rsAREditor_FrameFrameBottomDec);
  CreateToolButton(Category_Frame, rsAREditor_FrameFrameTopDec);
  CreateToolButton(Category_Frame, rsAREditor_FrameFrameRightDec);
  CreateToolButton(Category_Frame, rsAREditor_FrameFrameLeftDec);
  CreateMainMenu;

  OldApplicationOnHint:=Application.OnHint;
  OldApplicationOnHelp:=Application.OnHelp;

  Application.OnHint:=DisplayHint;
  {$IFDEF CLX}
  {$ELSE}
    Application.OnHelp:=DisplayHelp;
  {$ENDIF}

//  LoadLastEditedFiles;
  LoadRecentFiles;
end;

initialization
  Classes.RegisterClasses([TOpenPictureDialog, TControlBar]);
end.


