unit AREditorPageSetup;
//************************************************************//
//                         AND Report                        //
//  Eugene Hrulev (mrHru) Laxian Key, Omsk, Russia 1999,2001 //
//  Oleg Melnikov (muler) Laxian Key, Omsk, Russia 1999,2001 //
//************************************************************//

{$INCLUDE DEFINES.INC}

interface

uses
  SysUtils, Classes,
  {$IFDEF CLX}
    QStdCtrls, QExtCtrls, QButtons, QControls, QPrinters, QForms, QComCtrls,
  {$ELSE}
    Windows, Messages, Graphics, Controls, Forms, Dialogs, ExtCtrls, StdCtrls,
    Buttons, Printers,
  {$ENDIF}

  ARResStrings, ARRepCommon, ARPrinter, ARPreviewForm;

type
  TARPageSetupDlg = class(TForm)
    MarginsGB: TGroupBox;
    LeftEdit: TEdit;
    TopEdit: TEdit;
    RightEdit: TEdit;
    BottomEdit: TEdit;
    LeftLabel: TLabel;
    TopLabel: TLabel;
    RightLabel: TLabel;
    BottomLabel: TLabel;
    CancelBtn: TBitBtn;
    OKBtn: TBitBtn;
    OrientationRG: TRadioGroup;
    PaperSizeGB: TGroupBox;
    PaperSizeCB: TComboBox;
    MmLabel1: TLabel;
    MmLabel2: TLabel;
    WidthLabel: TLabel;
    HeightLabel: TLabel;
    procedure OKBtnClick(Sender: TObject);
    procedure CancelBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure PaperSizeCBSelect(Sender: TObject);
  private
    WidthEdit     :TSpinEdit;
    HeightEdit    :TSpinEdit;
    FPageSize     :TARPageSize;

    FIsPageSize   :boolean;
    procedure     SetPageSize(const Value: TARPageSize);
    procedure     WidthHeightChanged(Sender: TObject; NewValue: Integer);
  public
    Units         :TARUnits;

    LeftMargin    :extended;
    TopMargin     :extended;
    RightMargin   :extended;
    BottomMargin  :extended;

    Orientation   :TPrinterOrientation;

    property      PageSize:TARPageSize read FPageSize write SetPageSize;

    function      Execute:boolean;
    constructor   Create(AOwner:TComponent);override;
    destructor    Destroy;override;
  end;

implementation

{$R *.dfm}

constructor   TARPageSetupDlg.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);

  FPageSize            :=TARPageSize.Create;

  WidthEdit           :=TSpinEdit.Create(Self);
  WidthEdit.Parent    :=PaperSizeGB;
  WidthEdit.SetBounds(88, 48, 57, 21);
  WidthEdit.TabOrder  :=1;
  WidthEdit.Min       :=0;
  WidthEdit.Max       :=10000;
  WidthEdit.OnChanged :=WidthHeightChanged;

  HeightEdit          :=TSpinEdit.Create(Self);
  HeightEdit.Parent    :=PaperSizeGB;
  HeightEdit.SetBounds(88, 80, 57, 21);
  HeightEdit.TabOrder :=2;
  HeightEdit.Min      :=0;
  HeightEdit.Max      :=10000;
  HeightEdit.OnChanged:=WidthHeightChanged;
end;

destructor TARPageSetupDlg.Destroy;
begin
  FPageSize.Free;

  inherited;
end;

function      TARPageSetupDlg.Execute:boolean;
  function SizeToText(bound:extended):string;
  begin
    Result:=Format('%.2f',[ConvertTo(bound, Units)]);
  end;
begin
  MarginsGB.Caption:=Format(rsAREditorPageSetup_Margins + ' (%s)', [UnitsShortName(Units)]);

  LeftEdit.Text   :=SizeToText(LeftMargin);
  TopEdit.Text    :=SizeToText(TopMargin);
  RightEdit.Text  :=SizeToText(RightMargin);
  BottomEdit.Text :=SizeToText(BottomMargin);

  OrientationRG.ItemIndex  :=integer(Orientation);

  PaperSizeCB.Items.Assign(PageSize.PageNames);
  PaperSizeCB.ItemIndex :=PaperSizeCB.Items.IndexOf(PageSize.Name);
  WidthEdit.Value       :=Round(PageSize.Width  / 10.0);
  HeightEdit.Value      :=Round(PageSize.Height / 10.0);

  Result:=ShowModal = mrOK;
end;

procedure TARPageSetupDlg.OKBtnClick(Sender: TObject);
  function TextToSize(bound:string):extended;
  begin
    Result:=ConvertFrom(StrToFloat(bound), Units);
  end;
var Edit:TEdit;
begin
  Edit:=LeftEdit;
  try
                      LeftMargin    :=TextToSize(Edit.Text);
    Edit:=TopEdit;    TopMargin     :=TextToSize(Edit.Text);
    Edit:=RightEdit;  RightMargin   :=TextToSize(Edit.Text);
    Edit:=BottomEdit; BottomMargin  :=TextToSize(Edit.Text);
  except
    Edit.SelectAll;
    Edit.SetFocus;
    exit;
  end;

  Orientation  :=TPrinterOrientation(OrientationRG.ItemIndex);

  ModalResult:=mrOK;
end;

procedure TARPageSetupDlg.CancelBtnClick(Sender: TObject);
begin
  ModalResult:=mrCancel;
end;

procedure TARPageSetupDlg.PaperSizeCBSelect(Sender: TObject);
begin
  if not FIsPageSize then begin
    FIsPageSize:=true;
    try
      PageSize.Name   :=PaperSizeCB.Text;

      WidthEdit.Value :=Round(PageSize.Width  / 10.0);
      HeightEdit.Value:=Round(PageSize.Height / 10.0);
    finally
      FIsPageSize:=false;
    end;
  end;
end;

procedure TARPageSetupDlg.WidthHeightChanged(Sender:TObject; NewValue:Integer);
begin
  if not FIsPageSize then begin
    FIsPageSize:=true;
    try
      PageSize.Width  :=WidthEdit.Value  * 10.0;
      PageSize.Height :=HeightEdit.Value * 10.0;

      PaperSizeCB.ItemIndex:=PaperSizeCB.Items.IndexOf(PageSize.Name);
    finally
      FIsPageSize:=false;
    end;
  end;
end;

procedure TARPageSetupDlg.SetPageSize(const Value: TARPageSize);
begin
  FPageSize.Assign(Value);
end;

procedure TARPageSetupDlg.FormCreate(Sender: TObject);
begin
  Caption               :=rsAREditorPageSetup_PageSetup;
  MarginsGB.Caption     :=rsAREditorPageSetup_Margins;
  LeftLabel.Caption     :=rsAREditorPageSetup_LeftLabel;
  TopLabel.Caption      :=rsAREditorPageSetup_TopLabel;
  RightLabel.Caption    :=rsAREditorPageSetup_RightLabel;
  BottomLabel.Caption   :=rsAREditorPageSetup_BottomLabel;
  OrientationRG.Caption :=rsAREditorPageSetup_Orientation;
  PaperSizeGB.Caption   :=rsAREditorPageSetup_PaperSize;
  WidthLabel.Caption    :=rsAREditorPageSetup_WidthLabelCaption;
  HeightLabel.Caption   :=rsAREditorPageSetup_HeightLabelCaption;
  MmLabel1.Caption      :=rsAREditorPageSetup_MmLabelCaption;
  MmLabel2.Caption      :=rsAREditorPageSetup_MmLabelCaption;

  OrientationRG.Items.Clear;
  OrientationRG.Items.Add(rsAREditorPageSetup_Portrait);
  OrientationRG.Items.Add(rsAREditorPageSetup_Landscape);

  OKBtn.Caption         :=rsAR_OkCaption;
  CancelBtn.Caption     :=rsAR_CancelCaption;

  {$IFDEF CLX}
    PaperSizeCB.OnSelect:=PaperSizeCBSelect;
  {$ELSE}
    {$IFDEF D6}
      PaperSizeCB.OnSelect:=PaperSizeCBSelect;
    {$ELSE}
      PaperSizeCB.OnClick :=PaperSizeCBSelect;
    {$ENDIF}
  {$ENDIF}
end;

end.


