unit ARExcelExport;
//************************************************************//
//                         AND Report                        //
//  Eugene Hrulev (mrHru) Laxian Key, Omsk, Russia 1999,2001 //
//  Oleg Melnikov (muler) Laxian Key, Omsk, Russia 1999,2001 //
//************************************************************//

{$INCLUDE DEFINES.INC}

interface

uses
  SysUtils, Classes,
  {$IFDEF CLX}
    QDialogs,
  {$ELSE}
    Windows, Messages, Graphics, Controls, Forms, Dialogs,
    ComObj, ComCtrls, AxCtrls, Clipbrd,
    {$IFDEF D6}
      Variants,
    {$ENDIF}
  {$ENDIF}

  ARCustomExport, ARRepCommon, ARGridRep, ARUserRep, ARRep;

const //from Excel2000.pas (Excel97.pas)
  xlContinuous    = $00000001;

  xlHairline      = $00000001;
  xlMedium        = $FFFFEFD6;
  xlThick         = $00000004;
  xlThin          = $00000002;

  xlHAlignLeft    = $FFFFEFDD;
  xlHAlignCenter  = $FFFFEFF4;
  xlHAlignRight   = $FFFFEFC8;

  xlEdgeBottom    = $00000009;
  xlEdgeLeft      = $00000007;
  xlEdgeRight     = $0000000A;
  xlEdgeTop       = $00000008;

  xlDownward      = $FFFFEFB6;
  xlHorizontal    = $FFFFEFE0;
  xlUpward        = $FFFFEFB5;
  xlVertical      = $FFFFEFBA;

type
  TARExcelExport = class(TARCustomExport)
  private
    FCloseAfterDone: boolean;
  protected
  public
    procedure   Execute;override;
  published
    property    CloseAfterDone:boolean    read FCloseAfterDone  write FCloseAfterDone;
  end;

procedure Register;

implementation
{$R *.res}

procedure Register;
begin
  RegisterComponents('ARComponents', [TARExcelExport]);
end;

{ TARExcelExport }

{$IFDEF D3}
function  EmptyParam:OleVariant;
begin
  VarClear(Result);
end;
{$ENDIF}

function CompareBounds(Item1, Item2: Pointer): Integer;
begin
  Result := integer(Item1) - integer(Item2);
end;

procedure TARExcelExport.Execute;
var Template:OleVariant;
    A, B:OleVariant;  // corners of a excel cell
    C, D:OleVariant;  //

    idxband, idxcell:integer;
    s:string;
    i:integer;
    bl:TList;
    w:cardinal;
    Ex:Variant;
    band, btmband:TARBand;
    cell:TARCell;

  procedure PrepareBoundList(bl:TList);
  var idxband, idxcell:integer;
      i:integer;
  begin
    bl.Clear();
    bl.Add(pointer(0));
    for idxband:=0 to ARRep.SrcRep.BandCount - 1 do begin
      band:=ARRep.SrcRep.Bands[idxband];
      for idxcell:=0 to band.CellCount - 1 do begin
        i:=band.Lefts[idxcell];
        if bl.IndexOf(pointer(i)) = -1 then begin
          bl.Add(pointer(i));
        end;
      end;

      i:=band.Rights[band.CellCount - 1];
      if bl.IndexOf(pointer(i)) = -1 then begin
        bl.Add(pointer(i));
      end;
    end;
    bl.Sort(CompareBounds);
  end;
  function  GetColumnName(n:integer):string;
  begin
    if n < 26 then begin
      Result:=Chr(Ord('A') + n);
    end else begin
      Result:=Chr(Ord('A') + (n div 26 - 1)) + Chr(Ord('A') + n mod 26);
    end;
  end;

var
  WBk, WS, exCell:OleVariant;

  procedure SetBorder(fr:TFrameStyles; xl:integer);
  begin
    if cell.FrameWidths[fr] > 0 then begin
      WS.Range[A, D].Borders[xl].Color     :=cell.FrameColors[fr];
      WS.Range[A, D].Borders[xl].LineStyle :=xlContinuous;
      case cell.FrameWidths[fr] of
        1:    WS.Range[A, D].Borders[xl].Weight:=xlThin;
        2:    WS.Range[A, D].Borders[xl].Weight:=xlMedium;
        else  WS.Range[A, D].Borders[xl].Weight:=xlThick;
      end;
    end;
  end;

{$IFDEF CLX}
{$ELSE}
var AFormat: Word;
    AData: THandle;
    APalette: HPALETTE;
    TextAngle:integer;
{$ENDIF}
begin
  {$IFDEF CLX}
    ShowMessage('Excel export can''t work under CLX application.');
  {$ELSE}
    if Assigned(ARRep) then begin
      bl:=TList.Create();

//      try
//        Ex:=GetActiveOleObject('Excel.Application');
//      except
        Ex:=CreateOleObject('Excel.Application');
//      end;

      try
        if ARRep is TARUserRep then begin
          Ex.Caption := TARUserRep(ARRep).Title;
        end;
        Template := EmptyParam;
        WBk:=Ex.Workbooks.Add;
        WS :=Ex.ActiveSheet;

        PrepareBoundList(bl);
        for i:=0 to bl.Count - 2 do begin
          A:=GetColumnName(i) + '1';
          w:=integer(bl[i + 1]) - integer(bl[i]);
          WS.Range[A, A].ColumnWidth:=w div 7;
        end;

        ProgressStart(ARRep.SrcRep.BandCount);

        for idxband:=0 to ARRep.SrcRep.BandCount - 1 do begin
          band:=ARRep.SrcRep.Bands[idxband];
          try
            for idxcell:=0 to band.CellCount - 1 do begin
              cell:=band.Cells[idxcell];
              if cell.Shared then continue;

              A:=GetColumnName(bl.IndexOf(pointer(band.Lefts[idxcell])))      + IntToStr(idxband + 1);
              B:=GetColumnName(bl.IndexOf(pointer(band.Rights[idxcell])) - 1) + IntToStr(idxband + 1);

              exCell:=WS.Range[A, A];

              exCell.Font.Color     :=cell.FontColor;
              exCell.Font.Size      :=cell.FontSize;
              exCell.Font.Bold      :=fsBold in cell.FontStyle;
              exCell.Font.Italic    :=fsItalic in cell.FontStyle;
              exCell.Font.Name      :=cell.FontName;
              if cell.Color <> clWhite then begin
                exCell.Interior.Color :=cell.Color;
              end;

              s:=cell.S;
              exCell.Formula:=s;

              exCell.NumberFormat       :='@';
              case cell.HAlign of
                ahLeft:   exCell.HorizontalAlignment:=xlHAlignLeft;
                ahCenter: exCell.HorizontalAlignment:=xlHAlignCenter;
                ahRight:  exCell.HorizontalAlignment:=xlHAlignRight;
              end;

              case cell.TextAngle of
                  0:  ; //exCell.Orientation:=xlHorizontal;
                 90:  exCell.Orientation:=xlUpward;
                180:  ; // not supported
                270:  exCell.Orientation:=xlDownward;
                else begin
                  TextAngle:=cell.TextAngle mod 360;
                  if cell.TextAngle < 90 then begin
                    exCell.Orientation:=TextAngle;
                  end else begin
                    if cell.TextAngle >=270 then begin
                      exCell.Orientation:=360 - TextAngle;
                    end else begin
                      // not supported
                    end;
                  end;
                end;
              end;

              btmband:=cell.GetBottomBand;
              if btmband = band then begin
                C:=A;
                D:=B;
              end else begin
                C:=GetColumnName(bl.IndexOf(pointer(band.Lefts[idxcell])))      + IntToStr(btmband.Owner.IndexOfBand(btmband) + 1);
                D:=GetColumnName(bl.IndexOf(pointer(band.Rights[idxcell])) - 1) + IntToStr(btmband.Owner.IndexOfBand(btmband) + 1);
              end;

              WS.Range[A, D].Merge(Template);

              SetBorder(frLeft,   xlEdgeLeft);
              SetBorder(frTop,    xlEdgeTop);
              SetBorder(frRight,  xlEdgeRight);
              SetBorder(frBottom, xlEdgeBottom);

              if Assigned(cell.Picture.Graphic) and (not cell.Picture.Graphic.Empty) then begin
                cell.Picture.Bitmap.SaveToClipboardFormat(AFormat, AData, APalette);
                Clipboard.SetAsHandle(AFormat, AData);

                WS.Paste(WS.Range[A, D]);
              end;

            end;
          except
            raise;
            break;
          end;

          WS.Range[A, A].RowHeight  :=(band.Height * 7) div 9;

          if not ProgressStep then break;
        end;

        if Length(FileName) > 0 then begin
          WBk.SaveAs(FileName);
        end;
      finally
        bl.Free();
        if CloseAfterDone then begin
          Ex.Quit;
        end else begin
          Ex.Visible:=true;
        end;
        ProgressStop;
      end;
    end;
  {$ENDIF}
end;

end.

