unit ARFrame;
//************************************************************//
//                         AND Report                        //
//  Eugene Hrulev (mrHru) Laxian Key, Omsk, Russia 1999,2001 //
//  Oleg Melnikov (muler) Laxian Key, Omsk, Russia 1999,2001 //
//************************************************************//

{$INCLUDE DEFINES.INC}

interface

uses
  Classes,
  {$IFDEF CLX}
    QControls, QGraphics, QImgList, SysUtils, SyncObjs,
  {$ELSE}
    Windows, Controls, Graphics,
    {$IFNDEF D3}
      ImgList,
    {$ENDIF}
  {$ENDIF}

  ARUtils, ARGridRep, ARRepCommon;

type
  TARFrameThread = class(TThread)
  private
    ARGridRepList :TAIList;
    {$IFDEF CLX}
      CS            :TCriticalSection;
    {$ELSE}
      CS            :TRTLCriticalSection;
    {$ENDIF}
  protected
    procedure     Execute; override;
    procedure     ShowFrame;
  public
    constructor   Create(CreateSuspended: Boolean);
    destructor    Destroy;override;
    procedure     Add(Rep:TARGridRep);
    procedure     Delete(Rep:TARGridRep);
  end;


implementation
//$R ARGridRepFR8.res}

var
    FrameBMPList:TImageList;
    FrameBMP:TBitmap;
    FrameBmpIdx :integer;

constructor   TARFrameThread.Create(CreateSuspended: Boolean);
begin
  ARGridRepList:=TAIList.Create('TARFrameThread ARGridList');
  {$IFDEF CLX}
    CS:=TCriticalSection.Create;
  {$ELSE}
    InitializeCriticalSection(CS);
  {$ENDIF}
  inherited;
  FreeOnTerminate:=false;
end;

destructor    TARFrameThread.Destroy;
begin
  {$IFDEF CLX}
    CS.Enter;
  {$ELSE}
    EnterCriticalSection(CS);
  {$ENDIF}
  Terminate;
  WaitFor;
  {$IFDEF CLX}
    CS.Leave;
    CS.Free;
  {$ELSE}
    LeaveCriticalSection(CS);
    DeleteCriticalSection(CS);
  {$ENDIF}
  ARGridRepList.Free;
  inherited;
end;

procedure   TARFrameThread.ShowFrame;
{$IFDEF CLX}
{$ELSE}
var OldClipRgn, ClipRgn:HRGN;
    r:TRect;
    i:integer;
    Rep:TARGridRep;
    FocusedCellStyle:TFocusedCellStyle;
const d = 0;
{$ENDIF}
begin
{$IFDEF CLX}
{$ELSE}
  FocusedCellStyle:=fcsBlackAnt;
  {$IFDEF CLX}
    CS.Enter;
  {$ELSE}
    EnterCriticalSection(CS);
  {$ENDIF}
  try
    if ARGridRepList.Count > 0 then begin
      for i:=0 to ARGridRepList.Count - 1 do begin
        Rep:=TARGridRep(ARGridRepList[i]);

        if  (not Assigned(Rep.Parent)) or
            (not (Rep.Parent is TWinControl)) or
            (csDestroying	in Rep.ComponentState) or
            (not Rep.Focused) or
            (not Rep.Visible) or
            (not (Rep.FocusedCellStyle in [fcsBlackAnt, fcsRedAnt]))
        then continue;

        if (Rep.FOldCurrBandIdx <> Rep.CurrBandIdx) or (Rep.FOldCurrCellIdx <> Rep.CurrCellIdx) then begin
          if  (Rep.FOldCurrBandIdx >= 0) and (Rep.FOldCurrBandIdx <= Rep.SrcRep.BandCount - 1) and
              (Rep.FOldCurrCellIdx >= 0) and (Rep.FOldCurrCellIdx <= Rep.SrcRep.Bands[Rep.FOldCurrBandIdx].CellCount - 1) then begin

            //force repaint last cell
            with Rep.SrcRep.Bands[Rep.FOldCurrBandIdx].Cells[Rep.FOldCurrCellIdx] do begin
              BeginUpdate;
                Changed:=true;
              EndUpdate;
            end;

          end;
          Rep.FOldCurrBandIdx:=Rep.CurrBandIdx;
          Rep.FOldCurrCellIdx:=Rep.CurrCellIdx;
        end;

        r:=Rep.FrameCellClipRect(Rep.CurrBandIdx, Rep.CurrCellIdx);
        if not IsRectEmpty(r) then begin
          ClipRgn:=CreateRectRgn(r.Left, r.Top,  r.Right, r.Bottom);
          OldClipRgn:=SelectObject(Rep.Canvas.Handle, ClipRgn);
          try
            Rep.Canvas.Brush.Color  :=clRed;
            FrameBMPList.GetBitmap(FrameBmpIdx, FrameBMP);
            Rep.Canvas.Brush.Bitmap:=FrameBMP;

            r:=Rep.FrameCellRect(Rep.CurrBandIdx, Rep.CurrCellIdx);
            if not IsRectEmpty(r) then begin
              r.Left  :=r.Left + d;
              r.Top   :=r.Top + d;
              r.Right :=r.Right - d;
              r.Bottom:=r.Bottom - d;
              Rep.Canvas.FrameRect(r);
            end;
            Rep.Canvas.Brush.Color:=clBlack;
          finally
            SelectObject(Rep.Canvas.Handle, OldClipRgn);
            DeleteObject(ClipRgn);
          end;
        end;
        FocusedCellStyle:=Rep.FocusedCellStyle;
      end;

      FrameBmpIdx:=(FrameBmpIdx + 1) mod 4;
      if FocusedCellStyle = fcsRedAnt then begin
        FrameBmpIdx:=FrameBmpIdx + 8;
      end;
    end;
  finally
    LeaveCriticalSection(CS);
  end;
{$ENDIF}
end;

procedure   TARFrameThread.Execute;
begin
  while not Terminated do begin
    Synchronize(ShowFrame);
    Sleep(100);
  end;
end;

procedure   TARFrameThread.Add(Rep:TARGridRep);
begin
  {$IFDEF CLX}
    CS.Enter;
  {$ELSE}
    EnterCriticalSection(CS);
  {$ENDIF}
  if ARGridRepList.IndexOf(Rep) = -1 then begin
    ARGridRepList.Add(Rep);
  end;
  {$IFDEF CLX}
    CS.Leave;
  {$ELSE}
    LeaveCriticalSection(CS);
  {$ENDIF}
end;

procedure   TARFrameThread.Delete(Rep:TARGridRep);
var idx:integer;
begin
  {$IFDEF CLX}
    CS.Enter;
  {$ELSE}
    EnterCriticalSection(CS);
  {$ENDIF}
    idx:=ARGridRepList.IndexOf(Rep);
    if idx <> -1 then begin
      ARGridRepList.Delete(idx);
    end;
  {$IFDEF CLX}
    CS.Leave;
  {$ELSE}
    LeaveCriticalSection(CS);
  {$ENDIF}
end;

initialization
//sa: 11.06.2001
//  IsMultiThread:=true;
//--

  FrameBMPList:=TImageList.CreateSize(8, 8);
{$IFDEF CLX}
{$ELSE}
  FrameBMPList.GetResource(rtBitmap, 'FRAMES', 8, [], clBlue);
{$ENDIF}

  FrameBMP    :=TBitmap.Create();

finalization
  FrameBMP.Free();
  FrameBMPList.Free();
end.


