unit ARGridRep;
//************************************************************//
//                         AND Report                        //
//  Eugene Hrulev (mrHru) Laxian Key, Omsk, Russia 1999,2001 //
//  Oleg Melnikov (muler) Laxian Key, Omsk, Russia 1999,2001 //
//************************************************************//

// changes
// CD: Claudio Driussi cldriussi@swit.net

{$INCLUDE DEFINES.INC}

interface

uses
  Classes, Math, SysUtils,
  {$IFDEF CLX}
    QControls, QGraphics, QForms, Types, QExtCtrls, Qt,
  {$ELSE}
    Controls, Windows, Messages, Graphics, Forms,
  {$ENDIF}

  ARRepCommon, ARVFS, ARUtils, ARPrinter, ARRep, ARScrollingWidget;

var
  GridRepCnt:integer = 0;

type

{$IFDEF CLX}
  TARGridRep = class(TARScrollingWidget)
{$ELSE}
  TARGridRep = class(TCustomControl)
{$ENDIF}
  private
    FUnits              :TARUnits;
    FGridX              :extended;
    FGridY              :extended;

    FIdxTopBand         :integer;
    FIdxBottomBand      :integer;
    FLeftX              :integer;
    FTopY               :integer;
    FGutter             :integer;
    FFixedBands         :integer;
    FFixedCells         :integer;
    FScrollUpdate       :boolean;

//    FEditor           :TARInplaceEditorDlg;

    FOnSelectCell       :TARSelectCellEvent;
    FOnSelectBand       :TARSelectBandEvent;
    FOnResizeCell       :TARResizeCellEvent;
    FOnResizeBand       :TARResizeBandEvent;
    FOnCanEdit          :TARCanEditEvent;
    FOnCellChange       :TARCellChangeEvent;
    FOnShowEditor       :TARShowEditorEvent;
    FOnAfterEdit        :TNotifyEvent;
    FOnDrawBandTitle    :TARDrawBandTitleEvent;
    FOnExpandBand       :TARExpandBandEvent;
    FOnCollapseBand     :TARCollapseBandEvent;
    FOnHScroll          :TNotifyEvent;
    FOnVScroll          :TNotifyEvent;

    FShowRightMargin    :boolean;

    FSrcRep             :TARRep;

    FOptions            :TARRepOptions;

    FCurrBandIdx        :integer;
    FCurrCellIdx        :integer;

    FShowHiddenItems    :boolean;
    FSelectedCellStyle  :TSelectedCellStyle;
    FFocusedCellStyle   :TFocusedCellStyle;

    FRTF                :TARRTF;

    FColumns            :integer;
    FShowPrintProgress  :boolean;

    procedure   AddFrameThread;
    procedure   ReleaseFrameThread;
    procedure   SetSelectedCellStyle(ASelectedCellStyle:TSelectedCellStyle);
    procedure   SetFocusedCellStyle(AFocusedCellStyle:TFocusedCellStyle);

    procedure   SetShowHiddenItems(AShowHiddenItems:boolean);

    procedure   SetShowRightMargin(AShowRightMargin:boolean);

    procedure   SetLeftX(ALeftX:integer);
    procedure   SetTopY(ATopY:integer);
    procedure   SetGutter(AGutter:integer);
    procedure   SetFixedBands(AFixedBands:integer);
    procedure   SetFixedCells(AFixedCells:integer);

    function    GetSel(index:integer):TARCell;

    function    GetBandHeight(IdxBand:integer):integer;
    procedure   SetBandHeight(IdxBand:integer; h:integer);

    procedure   SetVS(AVSPos:integer; AVSPage:integer; AVSMax:integer);
    procedure   SetHS(AHSPos:integer; AHSPage:integer; AHSMax:integer);

    {$IFDEF CLX}
    {$ELSE}
      function    GetVSTrackPos:integer;
      function    GetVSPos:integer;
      procedure   SetVSPos(AVSPos:integer);
      function    GetVSMax:integer;
      procedure   SetVSMax(AVSMax:integer);
      function    GetVSPage:integer;
      procedure   SetVSPage(AVSPage:integer);

      function    GetHSTrackPos:integer;
      function    GetHSPos:integer;
      procedure   SetHSPos(AHSPos:integer);
      function    GetHSMax:integer;
      procedure   SetHSMax(AHSMax:integer);
      function    GetHSPage:integer;
      procedure   SetHSPage(AHSPage:integer);
    {$ENDIF}

    function    GetFileSystem:TVirtualFileSystem;
    procedure   SetFileSystem(AFileSystem:TVirtualFileSystem);

    procedure   SetShowBandTitle(AShowBandTitle:boolean);

    function    GetCurrBandIdx:integer;
    function    GetCurrCellIdx:integer;
    procedure   SetCurrBandIdx(ACurrBandIdx:integer);
    procedure   SetCurrCellIdx(ACurrCellIdx:integer);
    function    GetCurrBand:TARBand;
    function    GetCurrCell:TARCell;

    procedure   PaintCell(IdxBand:integer; IdxCell:integer);

    {$IFNDEF D3}
      procedure   MouseWheelUp(Sender:TObject; Shift:TShiftState; MousePos:TPoint; var Handled:boolean);virtual;
      procedure   MouseWheelDown(Sender:TObject; Shift:TShiftState; MousePos:TPoint; var Handled:boolean);virtual;
    {$ENDIF}

    procedure   BandChange(band:TObject);
    procedure   CellChange(band:TObject; cell:TObject);

    function    GetSelCount:integer;
    function    GetBandCount:integer;

  protected
    FInEditing  :boolean;
    FShowBandTitle:boolean;

    {$IFDEF CLX}
    {$ELSE}
      procedure   CreateParams(var p: TCreateParams);override;
      procedure   WMGetDlgCode(var Message: TWMGetDlgCode); message WM_GETDLGCODE;
      procedure   WMVSCROLL(var m: TWMScroll ); message WM_VSCROLL;
      procedure   WMHSCROLL(var m: TWMScroll ); message WM_HSCROLL;
      procedure   WMSize(var Message: TWMSize);message WM_SIZE;
      procedure   WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
    {$ENDIF}

    procedure   Paint;override;
    procedure   PaintBandTitle(IdxBand:integer);virtual;
    {$IFNDEF D3}
      procedure   Resize();override;
    {$ENDIF}

    procedure   SetARRTF;
    procedure   Loaded;override;
    procedure   UpdateScrolls;

    {$IFDEF CLX}
    {$ELSE}
      property    VSPos :integer          read GetVSPos         write SetVSPos;
      property    VSMax :integer          read GetVSMax         write SetVSMax;
      property    VSPage:integer          read GetVSPage        write SetVSPage;
      property    HSPos :integer          read GetHSPos         write SetHSPos;
      property    HSMax :integer          read GetHSMax         write SetHSMax;
      property    HSPage:integer          read GetHSPage        write SetHSPage;
    {$ENDIF}

    function    GetTemplate:TARTemplate;virtual;
    procedure   SetTemplate(ATemplate:TARTemplate);virtual;

    procedure   MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);override;
    procedure   KeyDown(var Key: Word; Shift: TShiftState);override;
    procedure   KeyPress(var Key: Char);override;

    procedure   SetUnits(AUnits:TARUnits);
    procedure   ApplyValues(src:TARCell; dst:TARCell);virtual;
    procedure   PrinterChange(Sender:TObject);virtual;

    procedure   SetI(VarName:string; VarValue:integer);virtual;
    function    AlignToGrid(u:extended; grid:extended):extended;
    function    GridXPix:integer;
    function    GridYPix:integer;
    function    AsUnits(pix:integer):extended;
    function    AsPix(u:extended):integer;

    procedure   MoveToPriorBand;virtual;
    procedure   MoveToNextBand;virtual;

  public
    FOldCurrBandIdx:integer;
    FOldCurrCellIdx:integer;

    SelList     :TAIList;

    prn         :TARPrinter;
    OrgRep      :TARRep;

    {$IFDEF CLX}
    {$ELSE}
      property    Canvas;
    {$ENDIF}

    property    TopY:integer        read FTopY            write SetTopY;
    property    LeftX:integer       read FLeftX           write SetLeftX;

    property    BandHeigths[IdxBand:integer]:integer  read GetBandHeight write SetBandHeight;
    property    Sel[index:integer]:TARCell  read GetSel;

    property    InEditing:boolean read FInEditing write FInEditing;

    procedure   SelAdd(IdxBand:integer; IdxCell:integer);
    procedure   ClearSel;
    procedure   SelAll;
    procedure   SelRepaint;

    procedure   DeleteBands;
    function    NewBand:TARBand;
    procedure   DeleteBand(IdxBand:integer);
    function    MergeSelectedCells:boolean;

    function    MouseToCell(X:integer;Y:integer;var IdxBand:integer;var IdxCell:integer):TARCell;

    constructor Create(AOwner:TComponent);override;
    destructor  Destroy;override;

    function    CellRect(IdxBand:integer;IdxCell:integer):TRect;
    function    CellClipRect(IdxBand:integer; IdxCell:integer): TRect;
    function    FrameCellRect(IdxBand:integer;IdxCell:integer):TRect;virtual;
    function    FrameCellClipRect(IdxBand:integer; IdxCell:integer): TRect;virtual;

    function    BandRect(IdxBand:integer):TRect;
    function    BandClipRect(IdxBand:integer):TRect;

    procedure   PageUp;
    procedure   PageDown;

    function    ShowEditor(ch:char):boolean;

    property    SrcRep:TARRep       read  FSrcRep         write FSrcRep;
    property    CurrBandIdx:integer read  GetCurrBandIdx  write SetCurrBandIdx;
    property    CurrCellIdx:integer read  GetCurrCellIdx  write SetCurrCellIdx;
    property    CurrBand:TARBand    read  GetCurrBand;
    property    CurrCell:TARCell    read  GetCurrCell;
    property    SelCount:integer    read  GetSelCount;
    property    BandCount:integer   read  GetBandCount;

    function    CreateCanvasImage(JobName:string):boolean;
    procedure   Print;
    procedure   Preview;
    procedure   PrintDialog;
  published
    property    Align;
{$IFDEF CLX}
{$ELSE}
  {$IFNDEF D3}
    property    BevelInner;
    property    BevelOuter;
  {$ENDIF}
{$ENDIF}
    property    TabStop;
    property    TabOrder;
    property    PopupMenu;
    property    ShowHint;
    property    ParentShowHint;
    property    OnDblClick;
    property    OnMouseUp;
    property    OnMouseDown;
    property    OnMouseMove;
    property    OnKeyDown;
    property    OnEnter;
    property    OnExit;

    property    SelectedCellStyle:TSelectedCellStyle      read  FSelectedCellStyle  write SetSelectedCellStyle default scsGray;
    property    FocusedCellStyle:TFocusedCellStyle        read  FFocusedCellStyle   write SetFocusedCellStyle default fcsRect;

    property    Units:TARUnits                            read  FUnits              write SetUnits;
    property    GridX:extended                            read  FGridX              write FGridX;
    property    GridY:extended                            read  FGridY              write FGridY;

    property    Options:TARRepOptions                     read  FOptions            write FOptions;

    property    Color stored true default clWhite;

    property    FileSystem:TVirtualFileSystem             read  GetFileSystem       write SetFileSystem;
    property    Gutter:integer                            read  FGutter             write SetGutter default 100;
    property    ShowBandTitle:boolean                     read  FShowBandTitle      write SetShowBandTitle default true;
    property    FixedBands:integer                        read  FFixedBands         write SetFixedBands default 0;
    property    FixedCells:integer                        read  FFixedCells         write SetFixedCells default 0;
    property    ARTemplate:TARTemplate                    read  GetTemplate         write SetTemplate;

    property    ShowRightMargin:boolean                   read  FShowRightMargin    write SetShowRightMargin;
    property    ShowHiddenItems:boolean                   read  FShowHiddenItems    write SetShowHiddenItems;

    property    Columns:integer                           read  FColumns            write FColumns default 1;
    property    ShowPrintProgress:boolean                 read  FShowPrintProgress  write FShowPrintProgress default true;

    property    OnSelectCell    :TARSelectCellEvent       read  FOnSelectCell       write FOnSelectCell;
    property    OnSelectBand    :TARSelectBandEvent       read  FOnSelectBand       write FOnSelectBand;
    property    OnResizeBand    :TARResizeBandEvent       read  FOnResizeBand       write FOnResizeBand;
    property    OnResizeCell    :TARResizeCellEvent       read  FOnResizeCell       write FOnResizeCell;
    property    OnCanEdit       :TARCanEditEvent          read  FOnCanEdit          write FOnCanEdit;
    property    OnCellChange    :TARCellChangeEvent       read  FOnCellChange       write FOnCellChange;
    property    OnShowEditor    :TARShowEditorEvent       read  FOnShowEditor       write FOnShowEditor;
    property    OnAfterEdit     :TNotifyEvent             read  FOnAfterEdit        write FOnAfterEdit;
    property    OnDrawBandTitle :TARDrawBandTitleEvent    read  FOnDrawBandTitle    write FOnDrawBandTitle;
    property    OnExpandBand    :TARExpandBandEvent       read  FOnExpandBand       write FOnExpandBand;
    property    OnCollapseBand  :TARCollapseBandEvent     read  FOnCollapseBand     write FOnCollapseBand;
    property    OnHScroll       :TNotifyEvent             read  FOnHScroll          write FOnHScroll;
    property    OnVScroll       :TNotifyEvent             read  FOnVScroll          write FOnVScroll;
  end;

implementation

{$R ARGridRep.res}

uses
  ARInplaceEditor, ARFrame, ARPrintProgress, ARResStrings;

var CollapsedBMP:TBitmap;
    ExpandedBMP :TBitmap;

function    TARGridRep.GetSelCount:integer;
begin
  Result:=SelList.Count;
end;

procedure   TARGridRep.SelRepaint;
var IdxBand, IdxCell:integer;
begin
  for IdxBand:=0 to SrcRep.BandCount - 1 do begin
    for IdxCell:=0 to SrcRep.Bands[IdxBand].CellCount - 1 do begin
      if SrcRep.Bands[IdxBand].Cells[IdxCell].Selected then begin
        PaintCell(IdxBand, IdxCell);
      end;
    end;
  end;
end;

procedure    TARGridRep.ClearSel;
var IdxBand, IdxCell:integer;
    cell:TARCell;
begin
  for IdxBand:=0 to SrcRep.BandCount - 1 do begin
    for IdxCell:=0 to SrcRep.Bands[IdxBand].CellCount - 1 do begin
      cell:=SrcRep.Bands[IdxBand].Cells[IdxCell];
      if cell.Selected then begin
        cell.Selected:=false;
//        PaintCell(IdxBand, IdxCell);
      end;
    end;
  end;
  SelList.Clear;
end;

procedure     TARGridRep.SelAdd(IdxBand:integer; IdxCell:integer);
var idx:integer;
    cell:TARCell;
begin
  if Assigned(CurrBand) and Assigned(CurrCell) then begin
    cell:=SrcRep.Bands[IdxBand].Cells[IdxCell];

    idx:=SelList.IndexOf(cell);
    if idx = -1 then begin
      SelList.Add(cell);
      cell.Selected:=true;
    end else begin
      SelList.Delete(idx);
      cell.Selected:=false;
    end;

//    PaintCell(IdxBand, IdxCell);
  end;
end;

procedure    TARGridRep.SelAll;
var IdxBand,IdxCell:integer;
begin
  ClearSel;
  for IdxBand:=0 to SrcRep.BandCount-1 do begin
    for IdxCell:=0 to SrcRep.Bands[IdxBand].CellCount - 1 do begin
      SelAdd(IdxBand, IdxCell);
    end;
  end;
  Invalidate;
end;

function    TARGridRep.GetSel(index:integer):TARCell;
begin
  Result:=SelList[index];
end;

function    TARGridRep.GetBandHeight(IdxBand:integer):integer;
begin
  Result:=0;
  if (IdxBand >= 0) and (IdxBand < SrcRep.BandCount) then begin
    Result:=SrcRep.Bands[IdxBand].Height;
  end;
end;

procedure   TARGridRep.UpdateScrolls;
var IdxBand,maxwidth,cellcnt:integer;
begin
  if not HandleAllocated then exit;
  maxwidth:=0;
  for IdxBand:=0 to SrcRep.BandCount - 1 do begin
    cellcnt:=SrcRep.Bands[IdxBand].CellCount;
    if cellcnt > 0 then begin
      maxwidth:=Max(maxwidth, SrcRep.Bands[IdxBand].Lefts[cellcnt - 1] + SrcRep.Bands[IdxBand].Cells[cellcnt - 1].Width);
    end;
  end;

  SetHS(LeftX, Max(0, ClientWidth - FGutter), maxwidth);

  if SrcRep.BandCount > 0 then begin
    SetVS(TopY,
          ClientHeight - SrcRep.Tops[FixedBands],
          SrcRep.Tops[SrcRep.BandCount - 1] + SrcRep.Bands[SrcRep.BandCount - 1].Height - SrcRep.Tops[FixedBands]
         );
  end;
end;

procedure   TARGridRep.SetBandHeight(IdxBand:integer; h:integer);
begin
  SrcRep.Bands[IdxBand].Height :=h;
  UpdateScrolls;
  Invalidate;
end;

procedure    TARGridRep.DeleteBands;
begin
  ClearSel;

  SrcRep.DeleteAll;
  FixedBands  :=0;
  
  CurrBandIdx :=0;
  CurrCellIdx :=0;

//  VSPos :=0;
//  VSMax :=0;
//  VSPage:=0;
//  HSPos :=0;
//  HSMax :=0;
//  HSPage:=0;
  UpdateScrolls;

  FTopY :=0;
  FLeftX:=0;
  FIdxTopBand:=0;
  FIdxBottomBand:=0;
  UpdateScrolls;
  Invalidate;
end;

{$IFDEF CLX}
{$ELSE}
procedure   TARGridRep.CreateParams(var p: TCreateParams);
begin
  inherited CreateParams(p);
  with p do begin
    Style   :=(Style or WS_VSCROLL or WS_HSCROLL) or WS_TABSTOP;

    ExStyle :=ExStyle or WS_EX_CLIENTEDGE;
  end;
end;
{$ENDIF}

///////////////////////////////////////////////////////////////////
{$IFDEF CLX}
{$ELSE}
function    TARGridRep.GetVSTrackPos:integer;
var si: TScrollInfo;
begin
  si.cbSize:=SizeOf(si);
  si.fMask:=SIF_ALL;
  si.nTrackPos:=0;
  if HandleAllocated then begin
    GetScrollInfo(Handle, SB_VERT, si);
  end;
  Result:=si.nTrackPos;
end;
function    TARGridRep.GetVSPos:integer;
var si: TScrollInfo;
begin
  si.cbSize:=SizeOf(si);
  si.fMask:=SIF_ALL;
  si.nPos:=0;
  if HandleAllocated then begin
    GetScrollInfo(Handle, SB_VERT, si);
  end;
  Result:=si.nPos;
end;

procedure   CheckScrollInfo(var si:TScrollInfo);
begin
  si.nMin:=0;
  si.nPage:=Max(0, si.nPage);
  si.nPos :=Max(0, si.nPos);
end;

procedure   TARGridRep.SetVSPos(AVSPos:integer);
var si: TScrollInfo;
begin
  si.cbSize:=SizeOf(si);
  si.fMask:=SIF_POS;
  si.nPos:=AVSPos;
  if HandleAllocated then begin
    CheckScrollInfo(si);
    SetScrollInfo(Handle, SB_VERT, si, true);
  end;
end;

function    TARGridRep.GetVSMax:integer;
var si: TScrollInfo;
begin
  si.cbSize:=SizeOf(si);
  si.fMask:=SIF_ALL;
  si.nMax:=0;
  if HandleAllocated then begin
    GetScrollInfo(Handle, SB_VERT, si);
  end;
  Result:=si.nMax;
end;
procedure   TARGridRep.SetVSMax(AVSMax:integer);
var si: TScrollInfo;
begin
  si.cbSize:=SizeOf(si);
  si.fMask:=SIF_RANGE;
  si.nMin:=0;
  si.nMax:=AVSMax;
  if HandleAllocated then begin
    CheckScrollInfo(si);
    SetScrollInfo(Handle, SB_VERT, si, true);
  end;
end;

function    TARGridRep.GetVSPage:integer;
var si: TScrollInfo;
begin
  si.cbSize:=SizeOf(si);
  si.fMask:=SIF_ALL;
  si.nPage:=0;
  if HandleAllocated then begin
    GetScrollInfo(Handle, SB_VERT, si);
  end;
  Result:=integer(si.nPage);
end;
procedure   TARGridRep.SetVSPage(AVSPage:integer);
var si: TScrollInfo;
begin
  si.cbSize:=SizeOf(si);
  si.fMask:=SIF_PAGE;
  si.nPage:=cardinal(Max(0, AVSPage));
  if HandleAllocated then begin
    CheckScrollInfo(si);
    SetScrollInfo(Handle, SB_VERT, si, true);
  end;
end;

function    TARGridRep.GetHSTrackPos:integer;
var si: TScrollInfo;
begin
  si.cbSize:=SizeOf(si);
  si.fMask:=SIF_ALL;
  si.nTrackPos:=0;
  if HandleAllocated then begin
    GetScrollInfo(Handle, SB_HORZ, si);
  end;
  Result:=si.nTrackPos;
end;
function    TARGridRep.GetHSPos:integer;
var si: TScrollInfo;
begin
  si.cbSize:=SizeOf(si);
  si.fMask:=SIF_ALL;
  si.nPos:=0;
  if HandleAllocated then begin
    GetScrollInfo(Handle, SB_HORZ, si);
  end;
  Result:=si.nPos;
end;
procedure   TARGridRep.SetHSPos(AHSPos:integer);
var si: TScrollInfo;
begin
  si.cbSize:=SizeOf(si);
  si.fMask:=SIF_POS;
  si.nPos:=AHSPos;
  if HandleAllocated then begin
    CheckScrollInfo(si);
    SetScrollInfo(Handle, SB_HORZ, si, true);
  end;
end;

function    TARGridRep.GetHSMax:integer;
var si: TScrollInfo;
begin
  si.cbSize:=SizeOf(si);
  si.fMask:=SIF_ALL;
  si.nMax:=0;
  if HandleAllocated then begin
    GetScrollInfo(Handle, SB_HORZ, si);
  end;
  Result:=si.nMax;
end;
procedure   TARGridRep.SetHSMax(AHSMax:integer);
var si: TScrollInfo;
begin
  si.cbSize:=SizeOf(si);
  si.fMask:=SIF_RANGE;
  si.nMin:=0;
  si.nMax:=AHSMax;
  if HandleAllocated then begin
    CheckScrollInfo(si);
    SetScrollInfo(Handle, SB_HORZ, si, true);
  end;
end;

function    TARGridRep.GetHSPage:integer;
var si: TScrollInfo;
begin
  si.cbSize:=SizeOf(si);
  si.fMask:=SIF_ALL;
  si.nPage:=0;
  if HandleAllocated then begin
    GetScrollInfo(Handle, SB_HORZ, si);
  end;
  Result:=integer(si.nPage);
end;
procedure   TARGridRep.SetHSPage(AHSPage:integer);
var si: TScrollInfo;
begin
  si.cbSize:=SizeOf(si);
  si.fMask:=SIF_PAGE;
  si.nPage:=Max(0, AHSPage);
  if HandleAllocated then begin
    CheckScrollInfo(si);
    SetScrollInfo(Handle, SB_HORZ, si, true);
  end;
end;
{$ENDIF}

procedure   TARGridRep.SetHS(AHSPos:integer; AHSPage:integer; AHSMax:integer);
{$IFDEF CLX}
{$ELSE}
  var si: TScrollInfo;
{$ENDIF}
begin
  {$IFDEF CLX}
    SetHSParams(AHSPos, AHSPage, AHSMax);
  {$ELSE}
    si.cbSize:=SizeOf(si);
    si.fMask:=SIF_POS or SIF_PAGE or SIF_RANGE ;
    si.nPos:=AHSPos;
    si.nPage:=Max(0, AHSPage);
    si.nMin:=0;
    si.nMax:=AHSMax;
    if HandleAllocated then begin
      CheckScrollInfo(si);
      SetScrollInfo(Handle, SB_HORZ, si, true);
    end;
  {$ENDIF}
end;

procedure   TARGridRep.SetVS(AVSPos:integer; AVSPage:integer; AVSMax:integer);
{$IFDEF CLX}
{$ELSE}
  var si: TScrollInfo;
{$ENDIF}
begin
  {$IFDEF CLX}
    SetVSParams(AVSPos, AVSPage, AVSMax);
  {$ELSE}
    si.cbSize:=SizeOf(si);
    si.fMask:=SIF_POS or SIF_PAGE or SIF_RANGE ;
    si.nPos:=AVSPos;
    si.nPage:=Max(0, AVSPage);
    si.nMin:=0;
    si.nMax:=AVSMax;
    if HandleAllocated then begin
      CheckScrollInfo(si);
      SetScrollInfo(Handle, SB_VERT, si, true);
    end;
  {$ENDIF}
end;

///////////////////////////////////////////////////////////////////
procedure   TARGridRep.SetLeftX(ALeftX:integer);
var x:integer;
//    dx:integer;
//    r, cr:TRect;
begin
  x:=Max(Min(Max(ALeftX, 0), HSMax - integer(HSPage)), 0);
  if FLeftX <> x then begin
//    dx:=FLeftX - x;
    FLeftX:=x;
      Invalidate;
    HSPos:=FLeftX;
  end;
end;

procedure   TARGridRep.SetTopY(ATopY:integer);
var y, dy:integer;
    r:TRect;
begin
  y:=Max(0, Min(ATopY, VSMax - integer(VSPage)));
  if FTopY <> y then begin
    dy:=FTopY - y;
    FTopY:=y;
    if Abs(dy) < (ClientHeight - SrcRep.Tops[FixedBands]) then begin
      r :=Rect(0, SrcRep.Tops[FixedBands], ClientWidth, ClientHeight);
      {$IFDEF CLX}
//        Invalidate;
        if dy > 0 then begin
          Canvas.CopyRect(Rect(r.Left, r.Top + dy, r.Right, r.Bottom), Canvas, Rect(r.Left, r.Top, r.Right, r.Bottom - dy));
          InvalidateRect (Rect(r.Left, r.Top, r.Right, r.Top + dy), false);
        end else begin
          Canvas.CopyRect(Rect(r.Left, r.Top, r.Right, r.Bottom + dy), Canvas, Rect(r.Left, r.Top - dy, r.Right, r.Bottom));
          InvalidateRect (Rect(r.Left, r.Bottom + dy, r.Right, r.Bottom), false);
        end;
      {$ELSE}
        ScrollWindow(Handle, 0, dy, @r, @r);
      {$ENDIF}
    end else begin
      Invalidate;
    end;
    VSPos:=FTopY;
  end;
end;

procedure   TARGridRep.SetGutter(AGutter:integer);
begin
  FGutter:=Max(AGutter, 0);
  Invalidate;
end;


function  TARGridRep.MouseToCell(X:integer;Y:integer;var IdxBand:integer;var IdxCell:integer):TARCell;
var  ib,ic, T:integer;
begin
  IdxBand:=-1;
  IdxCell:=-1;
  Result:=nil;

  if Y < SrcRep.Tops[FixedBands] then begin
    for ib:=0 to FixedBands - 1 do begin
      if Between(Y, SrcRep.Tops[ib], SrcRep.Bands[ib].Height) then begin
        IdxBand:=ib;
        if X <= FGutter then exit;
        for ic:=0 to SrcRep.Bands[ib].CellCount - 1 do begin
          if Between(X - FGutter + LeftX, SrcRep.Bands[ib].Lefts[ic], SrcRep.Bands[ib].Cells[ic].Width) then begin
            Result:=SrcRep.Bands[ib].Cells[ic];
            IdxBand:=ib;
            IdxCell:=ic;
            exit;
          end;
        end;
      end;
    end;
  end else begin
    T:=SrcRep.Tops[FixedBands] - TopY;
    for ib:=FixedBands to SrcRep.BandCount - 1 do begin
      if Between(Y, T, SrcRep.Bands[ib].Height) then begin
        IdxBand:=ib;
        if X <= FGutter then exit;
        for ic:=0 to SrcRep.Bands[ib].CellCount - 1 do begin
          if Between(X - FGutter + LeftX, SrcRep.Bands[ib].Lefts[ic], SrcRep.Bands[ib].Cells[ic].Width) then begin
            Result:=SrcRep.Bands[ib].Cells[ic];
            IdxCell:=ic;
            exit;
          end;
        end;
      end;
      T:=T + BandHeigths[ib];
    end;
  end;
end;

procedure    TARGridRep.DeleteBand(IdxBand:integer);
var i,idx:integer;
begin
  // delete from selected cells
  if  (IdxBand >= 0) and (IdxBand < SrcRep.BandCount) then begin
    for i:=0 to SrcRep.Bands[IdxBand].CellCount - 1 do begin
      idx:=SelList.IndexOf(SrcRep.Bands[IdxBand].Cells[i]);
      if idx <> -1 then SelList.Delete(idx);
    end;

    SrcRep.DeleteBand(IdxBand);

    CurrBandIdx:=Min(CurrBandIdx, SrcRep.BandCount - 1);
    CurrCellIdx:=Min(CurrCellIdx, SrcRep.Bands[CurrBandIdx].CellCount - 1);

    UpdateScrolls;
    Invalidate;
  end;
end;

{$IFDEF CLX}
{$ELSE}
procedure   TARGridRep.WMHSCROLL(var m: TWMScroll );
begin
  if FScrollUpdate then exit;
  FScrollUpdate:=true;

  case m.ScrollCode of
    SB_LINEDOWN:begin
                  LeftX:=LeftX + 10;
                end;
    SB_LINEUP  :begin
                  LeftX:=LeftX - 10;
                end;
    SB_PAGEUP  :begin
                  LeftX:=LeftX - (ClientWidth + FGutter);
                end;
    SB_PAGEDOWN:begin
                  LeftX:=LeftX + (ClientWidth + FGutter);
                end;
    SB_THUMBPOSITION:begin
                  LeftX:=HSPos;
                end;
    SB_THUMBTRACK:begin
                  LeftX:=GetHSTrackPos;
                end;
  end;
  if Assigned(FOnHScroll) then FOnHScroll(self);
  FScrollUpdate:=false;

end;

procedure   TARGridRep.WMGetDlgCode(var Message: TWMGetDlgCode);
begin
  inherited;
  Message.Result := DLGC_DEFPUSHBUTTON or DLGC_WANTARROWS or DLGC_WANTCHARS;// or DLGC_WANTALLKEYS;// or DLGC_WANTTAB;
end;

procedure   TARGridRep.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
  Message.Result:=1;
end;

procedure   TARGridRep.WMVSCROLL(var m: TWMScroll );
begin
  if FScrollUpdate then exit;
  FScrollUpdate:=true;

  case m.ScrollCode of
    SB_LINEDOWN:begin
                  TopY:=TopY + 10;
                end;
    SB_LINEUP  :begin
                  TopY:=TopY - 10;
                end;
    SB_PAGEUP  :begin
                  PageUp;
                end;
    SB_PAGEDOWN:begin
                  PageDown;
                end;
    SB_THUMBPOSITION:begin
                  TopY:=VSPos;
                end;
    SB_THUMBTRACK:begin
                  TopY:=GetVSTrackPos;
                end;
  end;
  if Assigned(FOnVScroll) then FOnVScroll(self);
  FScrollUpdate:=false;
end;

procedure   TARGridRep.WMSize(var Message: TWMSize);
begin
  inherited;
  {$IFNDEF D3}
    Resize();
  {$ENDIF}
end;
{$ENDIF}

procedure   TARGridRep.PageUp;
var idxband, y:integer;
begin
  y:=BandRect(CurrBandIdx).Top;

  TopY:=TopY - (ClientHeight - SrcRep.Tops[FixedBands]);

  idxband:=CurrBandIdx;
  while (idxband >=0) and (BandRect(idxband).Top > y) do begin
    dec(idxband);
  end;
  CurrBandIdx:=idxband;
end;

procedure   TARGridRep.PageDown;
var idxband, y:integer;
begin
  if  (CurrBandIdx >= 0) and (CurrBandIdx < SrcRep.BandCount) then begin
    y:=BandRect(CurrBandIdx).Top;

    TopY:=TopY + (ClientHeight - SrcRep.Tops[FixedBands]);

    idxband:=CurrBandIdx;
    while (idxband < SrcRep.BandCount) and (BandRect(idxband).Top < y) do begin
      inc(idxband);
    end;
    CurrBandIdx:=idxband;
  end;
end;

procedure    TARGridRep.SetShowRightMargin(AShowRightMargin:boolean);
begin
  FShowRightMargin:=AShowRightMargin;
  Invalidate;
end;

function      TARGridRep.GetTemplate:TARTemplate;
begin
  Result:=SrcRep.ARTemplate;
end;

procedure     TARGridRep.SetTemplate(ATemplate:TARTemplate);
begin
  SrcRep.ARTemplate:=ATemplate;
  Invalidate;
end;

procedure     TARGridRep.SetShowBandTitle(AShowBandTitle:boolean);
begin
  FShowBandTitle:=AShowBandTitle;
  Invalidate;
end;

procedure   TARGridRep.Loaded;
begin
  inherited Loaded;                                     { call the inherited method first}

  if (not (csDesigning in ComponentState)) and (not Assigned(FileSystem)) then begin
    FileSystem:=TStandardFileSystem.Create(Self);
  end;

  Realign;
  LeftX:=0;
  TopY:=0;
  UpdateScrolls;
end;

function    TARGridRep.GetFileSystem:TVirtualFileSystem;
begin
  if Assigned(SrcRep) then begin
    Result:=SrcRep.FileSystem;
  end else begin
    Result:=nil;
  end;
end;

procedure   TARGridRep.SetFileSystem(AFileSystem:TVirtualFileSystem);
begin
  if Assigned(SrcRep) then begin
    SrcRep.FileSystem:=AFileSystem;
  end;
end;

//procedure   TARGridRep.DblClick;
//begin
//  inherited;
//end;

procedure   TARGridRep.KeyPress(var Key: Char);
begin
  inherited KeyPress(Key);
end;

function    TARGridRep.ShowEditor(ch:char):boolean;
var CanEdit:boolean;
begin
  Result:=false;
  if Assigned(CurrCell) and (roEditing in Options) then begin
    CanEdit:=true;
    if Assigned(OnCanEdit) then begin
      OnCanEdit(Self, CurrBandIdx, CurrCellIdx, CanEdit);
    end;
    if CanEdit then begin
      if Assigned(OnShowEditor) then begin
        OnShowEditor(Self, ch);
      end else begin
        TARInplaceEditorDlg.Create(Self).RunEdit(ch);
      end;
      Result:=true;
    end;
  end;
end;

procedure   TARGridRep.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited;

  if not Focused then SetFocus;
end;

procedure   TARGridRep.SetCurrBandIdx(ACurrBandIdx:integer);
var tmpidx:integer;
begin
  tmpidx:=Max(0, Min(ACurrBandIdx, SrcRep.BandCount - 1));
  if FCurrBandIdx <> tmpidx then begin
    if Assigned(CurrCell) then CurrCell.Focused:=false;
    FCurrBandIdx:=tmpidx;
    CurrCellIdx:=CurrCellIdx;
    TopY:=TopY;
    if Assigned(CurrCell) then CurrCell.Focused:=true;
  end;
end;

procedure   TARGridRep.SetCurrCellIdx(ACurrCellIdx:integer);
var tmpidx:integer;
begin
  if Assigned(CurrBand) then begin
    tmpidx:=Max(0, Min(ACurrCellIdx, CurrBand.CellCount - 1));
    if FCurrCellIdx <> tmpidx then begin
      if Assigned(CurrCell) then CurrCell.Focused:=false;
      FCurrCellIdx:=Max(0, Min(ACurrCellIdx, CurrBand.CellCount - 1));
      LeftX:=LeftX;
      if Assigned(CurrCell) then CurrCell.Focused:=true;
    end;
  end;
end;

function    TARGridRep.GetCurrBandIdx:integer;
begin
//  Result:=FCurrBandIdx;
  Result:=Max(-1, Min(FCurrBandIdx, SrcRep.BandCount - 1));
end;

function    TARGridRep.GetCurrCellIdx:integer;
begin
//  Result:=FCurrCellIdx;
  if Assigned(CurrBand) then begin
    Result:=Max(0, Min(FCurrCellIdx, CurrBand.CellCount - 1));
  end else begin
    Result:=-1;
  end;
end;

function    TARGridRep.GetCurrBand:TARBand;
begin
  if (not Assigned(SrcRep)) or (SrcRep.BandCount = 0) then begin
    Result:=nil;
  end else begin
    FCurrBandIdx:=Max(0, Min(SrcRep.BandCount - 1, FCurrBandIdx));
    Result:=SrcRep.Bands[CurrBandIdx];
  end;
end;

function    TARGridRep.GetCurrCell:TARCell;
var cb:TARBand;
begin
  cb:=CurrBand;
  if Assigned(cb) and (cb.CellCount > 0) then begin
    FCurrCellIdx:=Max(0, Min(cb.CellCount - 1, FCurrCellIdx));
    Result:=cb.Cells[FCurrCellIdx];
  end else begin
    Result:=nil;
  end;
end;

procedure   TARGridRep.KeyDown(var Key: Word; Shift: TShiftState);
var n, curleftx, curtopy, d:integer;
    OldCurrCellIdx, OldCurrBandIdx:integer;
    r:TRect;
begin
  if SrcRep.BandCount = 0 then exit;

  OldCurrCellIdx:=CurrCellIdx;
  OldCurrBandIdx:=CurrBandIdx;

  if ssCtrl in Shift then begin
    case Key of
      VK_UP:    begin
                  TopY:=TopY - 10;
                  exit;
                end;
      VK_DOWN:  begin
                  TopY:=TopY + 10;
                  exit;
                end;
      VK_LEFT:  begin
                  LeftX:=LeftX - 10;
                  exit;
                end;
      VK_RIGHT: begin
                  LeftX:=LeftX + 10;
                  exit;
                end;
      VK_HOME:  begin
                  TopY:=0;
                  CurrCellIdx :=0;
                  CurrBandIdx :=0;
                end;
      VK_END:   begin
                  CurrBandIdx :=SrcRep.BandCount - 1;
                  CurrCellIdx :=SrcRep.Bands[CurrBandIdx].CellCount - 1;
                end;
      VK_PRIOR: begin
                  n:=CurrBandIdx;
                  while BandRect(n).Top > SrcRep.Tops[FixedBands] do begin
                    dec(n);
                  end;
                  CurrBandIdx:=n;
                  CurrCellIdx :=0;
                end;
      VK_NEXT:  begin
                  n:=CurrBandIdx;
                  while (n < SrcRep.BandCount) and (BandRect(n).Bottom < ClientHeight) do begin
                    inc(n);
                  end;
                  CurrBandIdx:=n;
                  CurrCellIdx :=SrcRep.Bands[CurrBandIdx].CellCount - 1;
                end;
    else  exit;
    end;
  end else begin
    case Key of
      VK_UP:    begin
                  MoveToPriorBand;
                end;
      VK_DOWN:  begin
                  MoveToNextBand;
                end;
      VK_LEFT:  begin
                  CurrCellIdx:=Max(CurrCellIdx - 1, 0);
                end;
      VK_RIGHT: begin
                  CurrCellIdx:=Min(CurrCellIdx + 1, SrcRep.Bands[CurrBandIdx].CellCount - 1);
                end;
      VK_HOME:  begin
                  CurrCellIdx:=0;
                end;
      VK_END:   begin
                  CurrCellIdx:=SrcRep.Bands[CurrBandIdx].CellCount - 1;
                end;
      VK_PRIOR: begin
                  PageUp;
                end;
      VK_NEXT:  begin
                  PageDown;
                end;
      VK_F2,
      VK_RETURN:begin
                  if ShowEditor(Char(Key)) then begin
                    Key:=0;
                  end;
                end;
      VK_ADD, VK_SUBTRACT:
                begin
                  if SrcRep[CurrBandIdx].Expandable then begin
                    SrcRep[CurrBandIdx].Expanded:=not SrcRep[CurrBandIdx].Expanded;
                    Invalidate;
                  end;
                end;
    else  exit;
    end;
  end;

  CurrCellIdx:=Min(CurrCellIdx, SrcRep.Bands[CurrBandIdx].CellCount - 1);

  curleftx:=LeftX;
  curtopy :=TopY;

  r:=CellRect(CurrBandIdx, CurrCellIdx);
  if r.Bottom > ClientHeight  then begin
    d       :=r.Bottom - ClientHeight;
    curtopy :=curtopy  + d;
    r.Top   :=r.Top    - d;
  end;
  if r.Right  >= ClientWidth   then begin
    d       :=r.Right  - ClientWidth;
    curleftx:=curleftx + d;
    r.Left  :=r.Left   - d;
  end;

  if r.Top  < SrcRep.Tops[FixedBands] then curtopy :=curtopy  + (r.Top - SrcRep.Tops[FixedBands]);

  if (FFixedCells = 0) or (CurrCellIdx < FFixedCells) then begin
    if r.Left < Gutter                 then curleftx:=curleftx + (r.Left - Gutter) - 1;
  end else begin
    if r.Left < Gutter + SrcRep.Bands[CurrBandIdx].Rights[Min(CurrCellIdx, Max(0, FFixedCells - 1))] then begin
      curleftx:=curleftx + (r.Left - Gutter - SrcRep.Bands[CurrBandIdx].Rights[Min(CurrCellIdx, Max(0, FFixedCells - 1))]) - 1;
    end;
  end;

  LeftX :=curleftx;
  TopY  :=curtopy;

  if Assigned(OnKeyDown) then OnKeyDown(Self, Key, Shift);

  if (OldCurrBandIdx <> CurrBandIdx) or (OldCurrCellIdx <> CurrCellIdx) then begin
    if not ((SelCount = 1) and (Sel[0] = SrcRep.Bands[CurrBandIdx].Cells[CurrCellIdx])) then begin
      if not (ssShift in Shift) then ClearSel;
      SelAdd(CurrBandIdx, CurrCellIdx);
    end;

    if Assigned(OnSelectCell) then OnSelectCell(Self, CurrBandIdx, CurrCellIdx);

    Key:=0;
    UpdateScrolls;
  end;
end;

function    TARGridRep.NewBand:TARBand;
begin
  Result:=SrcRep.NewBand;
  Result.OnBandChange:=BandChange;
  Result.OnCellChange:=CellChange;
//  VSMax:=SrcRep.BandCount;
  UpdateScrolls;
  Invalidate;
end;

{$IFNDEF D3}
procedure   TARGridRep.MouseWheelUp(Sender:TObject; Shift:TShiftState; MousePos:TPoint; var Handled:boolean);
begin
  TopY:=TopY - 10;
end;

procedure   TARGridRep.MouseWheelDown(Sender:TObject; Shift:TShiftState; MousePos:TPoint; var Handled:boolean);
begin
  TopY:=TopY + 10;
end;
{$ENDIF}

procedure   TARGridRep.SetShowHiddenItems(AShowHiddenItems:boolean);
begin
  if FShowHiddenItems <> AShowHiddenItems then begin
    FShowHiddenItems:=AShowHiddenItems;
    Invalidate;
  end;
end;

procedure   TARGridRep.BandChange(band:TObject);
begin
  Invalidate;
end;

procedure   TARGridRep.CellChange(band:TObject; cell:TObject);
var idxband, idxcell:integer;
    {$IFDEF CLX}
    {$ELSE}
      r:TRect;
    {$ENDIF}
begin
  idxband:=SrcRep.IndexOfBand(TARBandList(band));
  if idxband <> -1 then begin
    PaintBandTitle(idxband);
    idxcell:=TARBand(band).IndexOfCell(TARCell(cell));
    if idxcell <> -1 then begin
      {$IFDEF CLX}
        PaintCell(idxband, idxcell);
      {$ELSE}
        r:=CellRect(idxband, idxcell);
        r.Left:=r.Left - 1;
        InvalidateRect(Handle, @r, false);
      {$ENDIF}
    end;
  end;
end;

function    TARGridRep.GetBandCount:integer;
begin
  Result:=SrcRep.BandCount;
end;

function    TARGridRep.MergeSelectedCells:boolean;
var idxband, idxcell:integer;
    band:TARBand;
    cell:TARCell;
begin
  Result:=false;

  for idxband:=0 to SrcRep.BandCount - 1 do begin
    band:=SrcRep.Bands[idxband];

    idxcell:=0;
    while idxcell < band.CellCount do begin
      if band.Cells[idxcell].Selected then begin
        cell:=band.Cells[idxcell];
        inc(idxcell);
        while (idxcell < band.CellCount) and (band.Cells[idxcell].Selected) do begin
          cell.Value:=cell.Value + band.Cells[idxcell].Value;
          cell.Width:=cell.Width + band.Cells[idxcell].Width;
          SelAdd(idxband, idxcell);
          band.DeleteCell(idxcell);
          Result:=true;
        end;
      end else begin
        inc(idxcell);
      end;
    end;
  end;
end;

function    TARGridRep.GridXPix:integer;
begin
  Result:=Round(ConvertUnits(GridX, Units, uPix));
end;

function    TARGridRep.GridYPix:integer;
begin
  Result:=Round(ConvertUnits(GridY, Units, uPix));
end;

function    TARGridRep.AsUnits(pix:integer):extended;
begin
  Result:=ConvertUnits(pix, uPix, Units);
end;

function    TARGridRep.AsPix(u:extended):integer;
begin
  Result:=Round(ConvertUnits(u, Units, uPix));
end;

function    TARGridRep.AlignToGrid(u:extended; grid:extended):extended;
begin
  if grid = 0.0 then begin
    Result:=u;
  end else begin
    Result:=Round(u / grid) * grid;
  end;
end;

procedure   TARGridRep.SetUnits(AUnits:TARUnits);
begin
  if FUnits <> AUnits then begin
    GridX :=ConvertUnits(GridX, FUnits, AUnits);
    GridY :=ConvertUnits(GridY, FUnits, AUnits);
    FUnits:=AUnits;
  end;
end;

procedure   TARGridRep.SetSelectedCellStyle(ASelectedCellStyle:TSelectedCellStyle);
begin
  if FSelectedCellStyle <> ASelectedCellStyle then begin
    FSelectedCellStyle:=ASelectedCellStyle;
//    SelRepaint;
//    PaintCell(CurrBandIdx, CurrCellIdx);
  end;
end;

procedure   TARGridRep.PaintCell(IdxBand:integer; IdxCell:integer);
var r:TRect;
{$IFDEF CLX}
{$ELSE}
    OldClipRect, ClipRect:HRGN;
{$ENDIF}
begin
  if InEditing and (IdxBand = CurrBandIdx) and (IdxCell = CurrCellIdx) then exit;

  SetARRTF;

  r:=CellClipRect(IdxBand, IdxCell);
{$IFDEF CLX}
  Canvas.SetClipRect(r);
{$ELSE}
  ClipRect:=CreateRectRgn(r.Left, r.Top, r.Right, r.Bottom);
  OldClipRect:=SelectObject(Canvas.Handle, ClipRect);
{$ENDIF}
  try
    SrcRep.Bands[IdxBand].Cells[IdxCell].PaintCell(Self.Canvas, CellRect(IdxBand, IdxCell), r, SelectedCellStyle, FocusedCellStyle);
  finally
{$IFDEF CLX}
  Canvas.ResetClipRegion();
{$ELSE}
    SelectObject(Canvas.Handle, OldClipRect);
    DeleteObject(ClipRect);
{$ENDIF}
  end;
end;

procedure   TARGridRep.SetARRTF;
begin
  if not(csDesigning in ComponentState) then begin
    if not Assigned(FRTF) then begin
      FRTF:=TARRTF.Create(Self);
    end;

    ARRepCommon.RTF:=FRTF;
  end;
end;

procedure   TARGridRep.ApplyValues(src:TARCell; dst:TARCell);
begin
end;

function    TARGridRep.CreateCanvasImage(JobName:string):boolean;
var idxband, pagecnt,footerheight, headerheight:integer;
    LeftMargin, TopMargin, RightMargin, BottomMargin:integer;
    PPD:TPrintProgressDlg;
    band:TARBand;
    curcol, rightmgn:integer;
    rLeft:integer;
    CellRect:TRect;
    lastfooterheight: integer; //CD height of lastFooter
    OldOrgRep, CurOrgRep, NextOrgRep:TARRep;
    NextTopBandState:TARBandState;
  function HFAddBand(Band:TARBand):TARBand;
  var  idxcell:integer;
      H:integer;
  begin
    Result:=TARBand.Create(nil);
    {$IFDEF D3}
      Result.AssignBand(Band);
    {$ELSE}
      Result.Assign(Band);
    {$ENDIF}
    for idxcell:=0 to Result.CellCount - 1 do begin
      ApplyValues(Band.Cells[idxcell], Result.Cells[idxcell]);
      Result.Cells[idxcell].ProcessValue;

      if Result.Cells[idxcell].AutoStretch then begin
        Canvas.Font.Size    :=Result.Cells[idxcell].FontSize;
        Canvas.Font.Style   :=Result.Cells[idxcell].FontStyle;
        Canvas.Font.Name    :=Result.Cells[idxcell].FontName;
        Canvas.Font.Charset :=SrcRep.Font.Charset;
        H:=Result.Cells[idxcell].Height(Canvas);
        if H > Result.Height then begin
          Result.Height:=H;
        end;
      end;
    end;
  end;
  procedure PrintClipBand(band:TARBand; PrintBandPos:integer; ClipTop:integer; ClipBottom:integer);
  var idxcell, OldLeft:integer;
      {$IFDEF CLX}
      {$ELSE}
        OldClipReg, ClipReg:HRGN;
        dc:HDC;
      {$ENDIF}
      r:TRect;
      bottomband:TARBand;
  begin
    SetARRTF;
    OldLeft         :=CellRect.Left;
    CellRect.Right  :=CellRect.Left;

    CellRect.Top    :=CellRect.Top - PrintBandPos;
    CellRect.Bottom :=CellRect.Top + band.Height;
    for idxcell:=0 to band.CellCount - 1 do begin
      CellRect.Right:=CellRect.Right + band.Cells[idxcell].Width;
      if not band.Cells[idxcell].InUse then begin
        {$IFDEF CLX}
        {$ELSE}
          dc:=prn.Canvas.Handle;
        {$ENDIF}
        bottomband:=band.Cells[idxcell].GetBottomBand;
        if bottomband <> band then begin
          r:=Rect(CellRect.Left, CellRect.Top, CellRect.Right, CellRect.Top + (SrcRep.Tops[SrcRep.IndexOfBand(bottomband)] - SrcRep.Tops[SrcRep.IndexOfBand(band)]) + bottomband.Height);
          {$IFDEF CLX}
//??            prn.Canvas.SetClipRect(Rect(CellRect.Left, r.Top, CellRect.Right, r.Bottom));
          {$ELSE}
            ClipReg:=CreateRectRgn(CellRect.Left, r.Top, CellRect.Right, r.Bottom);
            OldClipReg:=SelectObject(dc, ClipReg);
          {$ENDIF}
          try
            band.Cells[idxcell].PaintCell(prn.Canvas, r, Rect(CellRect.Left, r.Top, CellRect.Right, r.Bottom), scsNone, fcsNone);
          finally
            {$IFDEF CLX}
//??              prn.Canvas.ResetClipRegion();
            {$ELSE}
              SelectObject(dc, OldClipReg);
              DeleteObject(ClipReg);
            {$ENDIF}
          end;
        end else begin
          {$IFDEF CLX}
//??            prn.Canvas.SetClipRect(Rect(CellRect.Left, ClipTop, CellRect.Right, ClipBottom));
          {$ELSE}
            ClipReg:=CreateRectRgn(CellRect.Left, ClipTop, CellRect.Right, ClipBottom);
            OldClipReg:=SelectObject(dc, ClipReg);
          {$ENDIF}
          try
            band.Cells[idxcell].PaintCell(prn.Canvas, CellRect, Rect(CellRect.Left, ClipTop, CellRect.Right, ClipBottom), scsNone, fcsNone);
          finally
            {$IFDEF CLX}
//??              prn.Canvas.ResetClipRegion();
            {$ELSE}
              SelectObject(dc, OldClipReg);
              DeleteObject(ClipReg);
            {$ENDIF}
          end;
        end;
      end;
      CellRect.Left:=CellRect.Left + band.Cells[idxcell].Width;
    end;
    rightmgn:=Max(rightmgn, CellRect.Right);
    CellRect.Left :=OldLeft;
    CellRect.Top  :=CellRect.Bottom;
  end;

  procedure  PrintBands(BandName:string);
  var idxband:integer;
  begin
    if Assigned(CurOrgRep) then begin
      for idxband:=0 to CurOrgRep.BandCount - 1 do begin
        if CT(CurOrgRep.Bands[idxband].Name, BandName) then begin
          band:=HFAddBand(CurOrgRep.Bands[idxband]);
          try
            CellRect.Left  :=rLeft;
            CellRect.Bottom :=CellRect.Top + band.Height;
            band.SetWidth(prn.Canvas, RightMargin - LeftMargin);
            PrintClipBand(band, 0, CellRect.Top, CellRect.Bottom);
          finally
            band.Free;
          end;
        end;
      end;
    end;
  end;

  function  SplittedBand(band:TARBand):boolean;
  var idxcell:integer;
  begin
    Result:=false;
    for idxcell:=0 to band.CellCount - 1 do begin
      if band.Cells[idxcell].IsRTF then begin
        Result:=true;
        break;
      end;
    end;
  end;

  procedure SetCurrentOrgRep(AOrgRep:TARRep);
  var idxband:integer;
  begin
    if (not Assigned(AOrgRep)) or (CurOrgRep <> AOrgRep) then begin
      OldOrgRep:=CurOrgRep;
      CurOrgRep:=AOrgRep;

      //calc footer height
      footerheight:=0;
      headerheight:=0;
      lastfooterheight := 0; //CD initialize lastfooterheight
      if Assigned(CurOrgRep) then begin
        for idxband:=0 to CurOrgRep.BandCount - 1 do begin
          if CT(CurOrgRep.Bands[idxband].Name, 'Footer') then begin
            inc(footerheight, CurOrgRep.Bands[idxband].Height);
          end;
        end;
        for idxband:=0 to CurOrgRep.BandCount - 1 do begin
          if CT(CurOrgRep.Bands[idxband].Name, 'Header') then begin
            inc(headerheight, CurOrgRep.Bands[idxband].Height);
          end;
        end;

        //CD calc greather between Footer and LastFooter bands
        for idxband:=0 to CurOrgRep.BandCount - 1 do begin
          if CT(CurOrgRep.Bands[idxband].Name, 'LastFooter') then begin
            inc(lastfooterheight, CurOrgRep.Bands[idxband].Height);
          end;
        end;
        if lastfooterheight > footerheight then begin
          footerheight := lastfooterheight;
        end;
      end;

{
      if Assigned(CurOrgRep) then begin
        prn.Orientation :=CurOrgRep.Orientation;
        prn.PageSize.Name :=CurOrgRep.PageSize.Name;
        prn.RefreshPrinter;

        LeftMargin  :=Round(ToPix(CurOrgRep.LeftMargin)) - prn.LeftMargin;
        RightMargin :=prn.PaperWidth - Round(ToPix(CurOrgRep.LeftMargin));
        TopMargin   :=Round(ToPix(CurOrgRep.TopMargin)) - prn.TopMargin;
        BottomMargin:=prn.PaperHeight - prn.TopMargin - footerheight - Round(ToPix(CurOrgRep.BottomMargin));
      end else begin
        prn.Orientation :=SrcRep.Orientation;
        prn.PageSize    :=SrcRep.PageSize;
        prn.RefreshPrinter;

        LeftMargin  :=Round(ToPix(SrcRep.LeftMargin)) - prn.LeftMargin;
//        RightMargin :=prn.PaperWidth - Round(ToPix(CurOrgRep.LeftMargin));
        RightMargin :=prn.PaperWidth - LeftMargin;

        TopMargin   :=Round(ToPix(SrcRep.TopMargin)) - prn.TopMargin;
        BottomMargin:=prn.PaperHeight - prn.TopMargin - footerheight - Round(ToPix(SrcRep.BottomMargin));
      end;
}
    end;
  end;

  procedure NewPage;
  begin
    inc(curcol);
    if curcol < Columns then begin
      CellRect:=Rect(LeftMargin + rightmgn, TopMargin, LeftMargin + rightmgn, TopMargin);
      rLeft:=LeftMargin + rightmgn;
    end else begin
      curcol:=0;
      rightmgn:=0;
      rLeft:=LeftMargin;

      CellRect.Top:=BottomMargin;
      PrintBands('Footer');

      prn.NewPage;
      SetCurrentOrgRep(NextOrgRep);

      prn.Canvas.Font.Charset:=SrcRep.Font.Charset;
      inc(pagecnt);
      SetI('Page', pagecnt);
      if ShowPrintProgress then PPD.PagesLabel.Caption:=Format(rsARReportPreview_Pages, [pagecnt]);
      Application.ProcessMessages;

      CellRect:=Rect(LeftMargin, TopMargin, LeftMargin, TopMargin);
    end;

    if not (arbsResetPageNumber in NextTopBandState) then begin
      PrintBands('Header');
    end;
  end;

  function  ContinuePrintBand(band:TARBand; ClipTop:integer; ClipBottom:integer):integer;
  var idxcell, OldLeft:integer;
      {$IFDEF CLX}
      {$ELSE}
        OldClipReg, ClipReg:HRGN;
        dc:HDC;
      {$ENDIF}
      r, ClipRect:TRect;
      bottomband:TARBand;
  begin
    Result:=ClipTop;
    OldLeft        :=CellRect.Left;
    CellRect.Right :=CellRect.Left;


    CellRect.Bottom:=0;
    for idxcell:=0 to band.CellCount - 1 do begin
      CellRect.Bottom :=Max(CellRect.Bottom, band.Cells[idxcell].Height(prn.Canvas));
    end;
    CellRect.Bottom:=CellRect.Bottom + CellRect.Top;

//    CellRect.Bottom:=band.Height + CellRect.Top;

    for idxcell:=0 to band.CellCount - 1 do begin
      CellRect.Right:=CellRect.Right + band.Cells[idxcell].Width;

      if not band.Cells[idxcell].InUse then begin
        {$IFDEF CLX}
        {$ELSE}
          dc:=prn.Canvas.Handle;
        {$ENDIF}
        bottomband:=band.Cells[idxcell].GetBottomBand;
        if bottomband <> band then begin
          r:=Rect(CellRect.Left, CellRect.Top, CellRect.Right, Min(BottomMargin, CellRect.Top + (SrcRep.Tops[SrcRep.IndexOfBand(bottomband)] - SrcRep.Tops[SrcRep.IndexOfBand(band)]) + bottomband.Height));
          {$IFDEF CLX}
//??            prn.Canvas.SetClipRect(Rect(CellRect.Left, r.Top, CellRect.Right, r.Bottom));
          {$ELSE}
            ClipReg:=CreateRectRgn(CellRect.Left, r.Top, CellRect.Right, r.Bottom);
            OldClipReg:=SelectObject(dc, ClipReg);
          {$ENDIF}
          try
            band.Cells[idxcell].PaintCell(prn.Canvas, r, Rect(CellRect.Left, r.Top, CellRect.Right, r.Bottom), scsNone, fcsNone);
          finally
            {$IFDEF CLX}
//??              prn.Canvas.ResetClipRegion();
            {$ELSE}
              SelectObject(dc, OldClipReg);
              DeleteObject(ClipReg);
            {$ENDIF}
          end;
        end else begin
          ClipRect:=Rect(CellRect.Left, ClipTop, CellRect.Right, ClipBottom);
          {$IFDEF CLX}
//??            prn.Canvas.SetClipRect(ClipRect);
          {$ELSE}
            dc:=prn.Canvas.Handle;
            ClipReg:=CreateRectRgn(ClipRect.Left, ClipRect.Top, ClipRect.Right, ClipRect.Bottom);
            OldClipReg:=SelectObject(dc, ClipReg);
          {$ENDIF}
          try
            Result:=Max(Result, band.Cells[idxcell].ContinuePrint(prn.Canvas, CellRect, ClipRect));
          finally
            {$IFDEF CLX}
//??              prn.Canvas.ResetClipRegion();
            {$ELSE}
              SelectObject(dc, OldClipReg);
              DeleteObject(ClipReg);
            {$ENDIF}
          end;
        end;
      end;
      CellRect.Left:=CellRect.Left + band.Cells[idxcell].Width;
    end;
    rightmgn:=Max(rightmgn, CellRect.Right);
    CellRect.Left :=OldLeft;
    CellRect.Top  :=CellRect.Bottom;//Result;//
  end;

  procedure PrintBand(band:TARBand);
  var idxcell:integer;
      idxband2, curtop:integer;
  begin
    if arbsPageBreak in band.BandState then begin
      NewPage;
    end;

    if arbsBeginPara in band.BandState then begin
      curtop:=CellRect.Top;// + SrcRep.Bands[idxband].Height;
      for idxband2:=idxband to SrcRep.BandCount - 1 do begin
        curtop:=curtop + SrcRep.Bands[idxband2].Height;
        if curtop >= BottomMargin then begin
          NewPage;
          break;
        end else begin
          if arbsEndPara in SrcRep.Bands[idxband2].BandState then begin
            break;
          end;
        end;
      end;
    end;
///////////////

    CellRect.Bottom :=CellRect.Top + band.Height;
    if (CellRect.Top + SrcRep.MaxHeight(band)) >= BottomMargin then begin
      if SplittedBand(band) then begin
        for idxcell:=0 to band.CellCount - 1 do band.Cells[idxcell].PreparePrint;

        ContinuePrintBand(band, CellRect.Top, BottomMargin);
        while CellRect.Top >= BottomMargin do begin
          NewPage;
          ContinuePrintBand(band, CellRect.Top, BottomMargin);
        end;
        for idxcell:=0 to band.CellCount - 1 do band.Cells[idxcell].PreparePrint;
      end else begin
        if SrcRep.MaxHeight(band) > (BottomMargin - TopMargin - headerheight) then begin
          ContinuePrintBand(band, CellRect.Top, BottomMargin);
        end else begin
          NewPage;
          PrintBand(band);
        end;
      end;
    end else begin
      PrintClipBand(band, 0, CellRect.Top, CellRect.Bottom);
    end;
  end;
begin
  Result:=false;

  ClearSel;

  OldOrgRep :=nil;
  CurOrgRep :=nil;
  NextOrgRep:=nil;

//  prn.PageSize    :=SrcRep.PageSize;
//  prn.Orientation :=SrcRep.Orientation;

  if ShowPrintProgress then PPD:=TPrintProgressDlg.Create(Self, JobName);
  try
    try
      if ShowPrintProgress then begin
        PPD.Show;
        PPD.PB.Max:=SrcRep.BandCount;
        PPD.Terminate:=false;
      end;

      pagecnt:=1;
      SetI('Page', pagecnt);

     if ShowPrintProgress then PPD.PagesLabel.Caption:=Format(rsARReportPreview_Pages, [pagecnt]);

      curcol:=0;
      rightmgn:=0;

      if SrcRep.BandCount > 0 then begin
        SetCurrentOrgRep(SrcRep.Bands[0].OrgRep);

{
        if Assigned(CurOrgRep) then begin
          prn.Orientation :=CurOrgRep.Orientation;
          prn.PageSize    :=CurOrgRep.PageSize;
          prn.RefreshPrinter;

          LeftMargin  :=Round(ToPix(CurOrgRep.LeftMargin)) - prn.LeftMargin;
          RightMargin :=prn.PaperWidth - Round(ToPix(CurOrgRep.LeftMargin));
          TopMargin   :=Round(ToPix(CurOrgRep.TopMargin)) - prn.TopMargin;
          BottomMargin:=prn.PaperHeight - prn.TopMargin - footerheight - Round(ToPix(CurOrgRep.BottomMargin));
        end else begin
}
          prn.Orientation :=SrcRep.Orientation;
          prn.PageSize    :=SrcRep.PageSize;
          prn.RefreshPrinter;

          LeftMargin  :=Round(ToPix(SrcRep.LeftMargin)) - prn.LeftMargin;
          RightMargin :=prn.PaperWidth - LeftMargin;

          TopMargin   :=Round(ToPix(SrcRep.TopMargin)) - prn.TopMargin;
          BottomMargin:=prn.PaperHeight - prn.TopMargin - footerheight - Round(ToPix(SrcRep.BottomMargin));
//        end;

        NextOrgRep:=SrcRep.Bands[Min(1, SrcRep.BandCount - 1)].OrgRep;
        NextTopBandState:=SrcRep.Bands[Min(1, SrcRep.BandCount - 1)].BandState;

        CellRect:=Rect(LeftMargin, TopMargin, LeftMargin, TopMargin);
        rLeft:=LeftMargin;
      end;

      prn.BeginDoc;
      prn.Canvas.Font.Charset:=SrcRep.Font.Charset;

//////////
      for idxband:=0 to SrcRep.BandCount - 1 do begin
        SrcRep.Bands[idxband].SetWidth(prn.Canvas, RightMargin - LeftMargin);
        PrintBand(SrcRep.Bands[idxband]);

        SetCurrentOrgRep(SrcRep.Bands[idxband].OrgRep);
        NextOrgRep:=SrcRep.Bands[Min(idxband + 1, SrcRep.BandCount - 1)].OrgRep;
        NextTopBandState:=SrcRep.Bands[Min(idxband + 1, SrcRep.BandCount - 1)].BandState;

        // because ResetPageNumber() applies to NEXT added band, reset page number AFTER printing band
        if arbsResetPageNumber in SrcRep.Bands[idxband].BandState then begin
          pagecnt:=1;
          SetI('Page', pagecnt);
        end;

        if ShowPrintProgress then begin
          PPD.PB.StepIt;
          if PPD.Terminate then begin
            prn.Abort;
            PPD.Close;
            exit;
          end;
        end;
      end;
      CellRect.Top:=BottomMargin;

//      PrintBands('Footer');
      if lastfooterheight > 0 then begin//CD print LastFooter if exist else print Footer
        PrintBands('LastFooter')
      end else begin
        PrintBands('Footer');
      end;

      prn.EndDoc;
      Result:=true;
    except
      prn.Abort;
    end;
  finally
    if ShowPrintProgress then begin
      PPD.Close;
      PPD.Free;
    end;
  end;
end;

procedure   TARGridRep.PrinterChange(Sender:TObject);
begin
  SrcRep.PageSize   :=prn.PageSize;
  SrcRep.Orientation:=prn.Orientation;
  CreateCanvasImage(rsARReportPreview_JobBreakPages);
end;

procedure   TARGridRep.Print;
begin
  if CreateCanvasImage(rsARReportPreview_JobPrint) then prn.Print;
end;

procedure   TARGridRep.Preview;
begin
  if CreateCanvasImage(rsARReportPreview_JobBreakPages) then prn.Preview;
end;

procedure   TARGridRep.PrintDialog;
begin
  if CreateCanvasImage(rsARReportPreview_JobBreakPages) then prn.PrintDialog;
end;

procedure   TARGridRep.SetI(VarName:string; VarValue:integer);
begin
end;

procedure   TARGridRep.MoveToPriorBand;
begin
  CurrBandIdx:=Max(CurrBandIdx - 1, 0);
end;

procedure   TARGridRep.MoveToNextBand;
begin
  CurrBandIdx:=Min(CurrBandIdx + 1, SrcRep.BandCount - 1);
end;

function    TARGridRep.FrameCellRect(IdxBand:integer;IdxCell:integer):TRect;
begin
  Result:=CellRect(IdxBand, IdxCell);
end;

function    TARGridRep.FrameCellClipRect(IdxBand:integer; IdxCell:integer): TRect; 
begin
  Result:=CellClipRect(IdxBand, IdxCell);
end;

var
  frtrd:TARFrameThread;

procedure     TARGridRep.AddFrameThread;
begin
  if (not (csDesigning in ComponentState)) then begin
    if GridRepCnt = 0 then frtrd:=TARFrameThread.Create(false);
    frtrd.Add(Self);
    inc(GridRepCnt);

    FOldCurrBandIdx:=-1;
    FOldCurrCellIdx:=-1;
  end;
end;

procedure     TARGridRep.ReleaseFrameThread;
begin
  if not (csDesigning in ComponentState) then begin
    if GridRepCnt > 0 then begin
      frtrd.Delete(Self);
      dec(GridRepCnt);
      if GridRepCnt = 0 then frtrd.Free;
    end;
  end;
end;

constructor   TARGridRep.Create(AOwner:TComponent);
begin
  prn                 :=TARPrinter.Create(Self);
  prn.OnPrinterChange :=PrinterChange;

  ShowPrintProgress:=true;

  SelectedCellStyle :=scsGray;
  FocusedCellStyle  :=fcsRect;

  SelList     :=TAIList.Create('TARGridRep SelList');
  SrcRep      :=TARRep.Create;
  SrcRep.OnBandChange:=BandChange;
  SrcRep.OnCellChange:=CellChange;

  {$IFNDEF D3}
    OnMouseWheelUp  :=MouseWheelUp;
    OnMouseWheelDown:=MouseWheelDown;
  {$ENDIF}

  inherited Create(AOwner);

  {$IFDEF CLX}
    InputKeys:=[ikArrows, ikChars, ikReturns, ikEdit, ikNav, ikEsc];
  {$ENDIF}

  if Width = 0 then Width:= 180;
  if Height= 0 then Height:=120;
end;

destructor  TARGridRep.Destroy;
begin
  ReleaseFrameThread;

  SrcRep.Free;
  SrcRep:=nil;
  SelList.Free;
  prn.Free;

  inherited Destroy;
end;

procedure   TARGridRep.SetFocusedCellStyle(AFocusedCellStyle:TFocusedCellStyle);
begin
  if FFocusedCellStyle <> AFocusedCellStyle then begin
    if (FFocusedCellStyle in [fcsNone, fcsRect, fcsRoundRect]) and (AFocusedCellStyle in [fcsBlackAnt, fcsRedAnt]) then begin
      AddFrameThread;
    end else
    if (FFocusedCellStyle in [fcsBlackAnt, fcsRedAnt]) and (AFocusedCellStyle in [fcsNone, fcsRect, fcsRoundRect]) then begin
      ReleaseFrameThread;
    end;
    FFocusedCellStyle:=AFocusedCellStyle;

    if Assigned(CurrCell) then begin
      CurrCell.BeginUpdate;
      CurrCell.Changed:=true;
      CurrCell.EndUpdate;
    end;
  end;
end;

{$IFNDEF D3}
procedure TARGridRep.Resize;
begin
  inherited;

  Realign;
  UpdateScrolls;
end;
{$ENDIF}

procedure   TARGridRep.SetFixedBands(AFixedBands:integer);
begin
  FFixedBands:=Max(0, Min(AFixedBands, SrcRep.BandCount - 1));
  Invalidate;
end;

procedure TARGridRep.SetFixedCells(AFixedCells: integer);
begin
  FFixedCells:=AFixedCells;
  Invalidate;
end;

function    TARGridRep.CellRect(IdxBand:integer; IdxCell:integer): TRect;
var IdxTopBand, IdxBottomBand:integer;
    band, topband, bottomband:TARBand;
    cell:TARCell;
begin
  if  (IdxBand >= 0) and (IdxBand < SrcRep.BandCount) and
      (IdxCell >= 0) and (IdxCell < SrcRep.Bands[IdxBand].CellCount) then begin

    band:=SrcRep.Bands[IdxBand];
    cell:=band.Cells[IdxCell];

    Result.Left     :=Gutter + 1 + band.Lefts[IdxCell];
    if (FFixedCells = 0) or (IdxCell >= FFixedCells) then begin
      Result.Left   :=Result.Left - LeftX;
    end;
    Result.Right    :=Result.Left + cell.Width;

    topband   :=cell.GetTopBand;
    IdxTopBand:=SrcRep.IndexOfBand(topband);
    bottomband:=cell.GetBottomBand;
    IdxBottomBand:=SrcRep.IndexOfBand(bottomband);

    Result.Top :=SrcRep.Tops[IdxTopBand];
    Result.Bottom:=SrcRep.Tops[IdxBottomBand] + SrcRep.Bands[IdxBottomBand].Height;
    if IdxBand >= FixedBands then begin
      Result.Top    :=Result.Top    - TopY;
      Result.Bottom :=Result.Bottom - TopY;
    end;

  end else begin
    Result:=Rect(0, 0, 0, 0);
  end;
end;

function    TARGridRep.CellClipRect(IdxBand:integer; IdxCell:integer): TRect;
begin
  if  (IdxBand >= 0) and (IdxBand < SrcRep.BandCount) and
      (IdxCell >= 0) and (IdxCell < SrcRep.Bands[IdxBand].CellCount) then begin
      Result:=CellRect(IdxBand, IdxCell);

    if (FFixedCells = 0) or (IdxCell < FFixedCells) then begin
      Result.Left   :=Max(Result.Left, Gutter);
    end else begin
      Result.Left   :=Max(Result.Left, Gutter + 1 + SrcRep.Bands[IdxBand].Rights[Min(IdxCell, Max(0, FFixedCells - 1))]);
    end;

    Result.Right  :=Min(Result.Right, ClientWidth);

    if IdxBand >= FixedBands then begin
      Result.Top  :=Max(Result.Top, SrcRep.Tops[FixedBands]);
    end;

    Result.Bottom :=Min(Result.Bottom, ClientHeight);
  end else begin
    Result:=Rect(0, 0, 0, 0);
  end;
end;

procedure   TARGridRep.PaintBandTitle(IdxBand:integer);
var r, textr:TRect;
    {$IFDEF CLX}
    {$ELSE}
    OldClipRgn, ClipRgn:HRGN;
    {$ENDIF}
    band:TARBand;
begin
  if (FGutter = 0) or (not ((IdxBand >= 0) and (IdxBand < SrcRep.BandCount))) then exit;

  band:=SrcRep.Bands[IdxBand];

  r:=BandClipRect(IdxBand);
  r.Right:=FGutter + 1;

  {$IFDEF CLX}
    Canvas.SetClipRect(r);
  {$ELSE}
    ClipRgn:=CreateRectRgn(r.Left, r.Top,  r.Right, r.Bottom);
    OldClipRgn:=SelectObject(Canvas.Handle, ClipRgn);
  {$ENDIF}
  try
///////////
    Canvas.Brush.Style:=bsSolid;
    if Canvas.Brush.Color <> clBtnFace then Canvas.Brush.Color:=clBtnFace;
    Canvas.FillRect(r);

    Canvas.Font.Assign(SrcRep.Font);

    if FShowBandTitle then begin
      {$IFDEF CLX}
      {$ELSE}
        SetBkMode(Canvas.Handle, TRANSPARENT);
      {$ENDIF}
      textr:=Rect(r.Left, r.Top + 1, r.Right - 3, r.Bottom - 1);
      if band.Selected then begin
        Canvas.Font.Style:=Canvas.Font.Style + [fsBold];
        Canvas.Font.Color:=clBtnHighlight;
//        Canvas.TextOut(4, r.Top + (band.Height - Canvas.TextHeight(band.Name)) div 2 - 1, band.Name);
        Canvas.TextRect(textr, 4, r.Top + (band.Height - Canvas.TextHeight(band.Name)) div 2 - 1, band.Name);
        Canvas.Font.Color:=clBlack;
//        Canvas.TextOut(5, r.Top + (band.Height - Canvas.TextHeight(band.Name)) div 2, band.Name);
        Canvas.TextRect(textr, 5, r.Top + (band.Height - Canvas.TextHeight(band.Name)) div 2, band.Name);
      end else begin
        Canvas.Font.Style:=Canvas.Font.Style - [fsBold];
        Canvas.Font.Color:=clBtnHighlight;
//        Canvas.TextOut(6, r.Top + (band.Height - Canvas.TextHeight(band.Name)) div 2 + 1, band.Name);
        Canvas.TextRect(textr, 6, r.Top + (band.Height - Canvas.TextHeight(band.Name)) div 2 + 1, band.Name);
        Canvas.Font.Color:=clBlack;
//        Canvas.TextOut(5, r.Top + (band.Height - Canvas.TextHeight(band.Name)) div 2, band.Name);
        Canvas.TextRect(textr, 5, r.Top + (band.Height - Canvas.TextHeight(band.Name)) div 2, band.Name);
      end;
      {$IFDEF CLX}
      {$ELSE}
        SetBkMode(Canvas.Handle, OPAQUE);
      {$ENDIF}

    end else begin
      if Assigned(OnDrawBandTitle) then OnDrawBandTitle(Self, IdxBand, Canvas, r);
      if band.Expandable then begin
        if not band.Expanded then begin
          Canvas.Draw(r.Right - 4 - CollapsedBMP.Width, r.Top  + (r.Bottom - r.Top - CollapsedBMP.Height) div 2, CollapsedBMP);
        end else begin
          Canvas.Draw(r.Right - 4 - ExpandedBMP.Width, r.Top  + (r.Bottom - r.Top - ExpandedBMP.Height) div 2, ExpandedBMP);
        end;
      end;
    end;
    if band.Marked then Canvas.Draw(0, r.Top + (band.Height - band.Mark.Height) div 2, band.Mark);
    //boundares

    if band.Selected then begin
      Canvas.Pen.Color:=clBtnShadow;    Canvas.MoveTo(r.Left + 0, r.Top);         Canvas.LineTo(r.Left + 0,   r.Bottom - 1);
      Canvas.Pen.Color:=clBtnShadow;    Canvas.MoveTo(r.Left,     r.Top);         Canvas.LineTo(r.Right - 3,  r.Top);
      Canvas.Pen.Color:=clBtnHighlight; Canvas.MoveTo(r.Left,     r.Bottom - 1);  Canvas.LineTo(r.Right - 3,  r.Bottom - 1);
    end else begin
      Canvas.Pen.Color:=clBtnHighlight; Canvas.MoveTo(r.Left + 0, r.Top);         Canvas.LineTo(r.Left + 0,   r.Bottom - 1);
      Canvas.Pen.Color:=clBtnHighlight; Canvas.MoveTo(r.Left,     r.Top);         Canvas.LineTo(r.Right - 3,  r.Top);
      Canvas.Pen.Color:=clBtnShadow;    Canvas.MoveTo(r.Left,     r.Bottom - 1);  Canvas.LineTo(r.Right - 3,  r.Bottom - 1);
    end;

    //gutter
    Canvas.Pen.Color:=clWhite;        Canvas.MoveTo(FGutter - 3, r.Top - 1);  Canvas.LineTo(FGutter - 3, r.Bottom + 1);
    Canvas.Pen.Color:=clBtnFace;      Canvas.MoveTo(FGutter - 2, r.Top - 1);  Canvas.LineTo(FGutter - 2, r.Bottom + 1);
    Canvas.Pen.Color:=clDkGray;       Canvas.MoveTo(FGutter - 1, r.Top - 1);  Canvas.LineTo(FGutter - 1, r.Bottom + 1);
///////////
  finally
    {$IFDEF CLX}
      Canvas.ResetClipRegion();
    {$ELSE}
      SelectObject(Canvas.Handle, OldClipRgn);
      DeleteObject(ClipRgn);
    {$ENDIF}
  end;
end;

procedure   TARGridRep.Paint;
var IdxBand, T, left, FixedTop:integer;
    r, rc, ClipR:TRect;
    {$IFDEF CLX}
    {$ELSE}
      OldClipRgn, ClipRgn:HRGN;
    {$ENDIF}
const pd = 4;
  function  IntPaintBand(top:integer):boolean;
  var IdxCell:integer;
  begin
    if top > ClientHeight then begin
      Result:=false;
      exit;
    end;
    Result:=true;

    PaintBandTitle(IdxBand);

    left:=0;

    SetARRTF;
    for IdxCell:=0 to SrcRep.Bands[IdxBand].CellCount - 1 do begin
      rc:=CellClipRect(IdxBand, IdxCell);
      left:=Max(left, rc.Right);
      {$IFDEF CLX}
        Canvas.SetClipRect(rc);
      {$ELSE}
        ClipRgn:=CreateRectRgn(rc.Left, rc.Top, rc.Right, rc.Bottom);
        OldClipRgn:=SelectObject(Canvas.Handle, ClipRgn);
      {$ENDIF}
      try
        SrcRep.Bands[IdxBand].Cells[IdxCell].PaintCell(Self.Canvas, CellRect(IdxBand, IdxCell), rc, SelectedCellStyle, FocusedCellStyle);
      finally
        {$IFDEF CLX}
          Canvas.ResetClipRegion();
        {$ELSE}
          SelectObject(Canvas.Handle, OldClipRgn);
          DeleteObject(ClipRgn);
        {$ENDIF}
      end;
    end;

    r:=BandClipRect(IdxBand);
    {$IFDEF CLX}
      Canvas.SetClipRect(r);
    {$ELSE}
      ClipRgn:=CreateRectRgn(r.Left, r.Top,  r.Right, r.Bottom);
      OldClipRgn:=SelectObject(Canvas.Handle, ClipRgn);
    {$ENDIF}
    try
      r.Left  :=left;
      Canvas.Brush.Style:=bsSolid;
      if Canvas.Brush.Color <> Color then Canvas.Brush.Color:=Color;
      Canvas.FillRect(r);
    finally
      {$IFDEF CLX}
        Canvas.ResetClipRegion();
      {$ELSE}
        SelectObject(Canvas.Handle, OldClipRgn);
        DeleteObject(ClipRgn);
      {$ENDIF}
    end;
  end;
begin
  if not HandleAllocated then exit;

  ClipR:=Canvas.ClipRect;

  {$IFDEF CLX}
    FLeftX:=HSPos;
    FTopY :=VSPos;
  {$ELSE}
    UpdateScrolls;
  {$ENDIF}

  r:=Rect(0, 0, 0, 0);

  Canvas.Font.Charset:=SrcRep.Font.Charset;

  for IdxBand:=0 to Min(FixedBands - 1, SrcRep.BandCount - 1) do begin
    if not IntPaintBand(SrcRep.Tops[IdxBand]) then break;
  end;

  if SrcRep.BandCount > FixedBands then begin
    FixedTop:=SrcRep.Tops[FixedBands];

    T:=FixedTop - TopY;
    for IdxBand:=FixedBands to SrcRep.BandCount - 1 do begin
      if T > ClientHeight then break;

      if (T + BandHeigths[IdxBand]) >= FixedTop then begin
        if not IntPaintBand(T) then break;
      end;
      T:=T + BandHeigths[IdxBand];
    end;
  end else begin
    T:=SrcRep.Tops[SrcRep.BandCount - 1] + BandHeigths[SrcRep.BandCount - 1];
  end;

  if T <= ClientHeight then begin
    r:=ClientRect;
    r.Top:=T;
    {$IFDEF CLX}
      Canvas.SetClipRect(r);
    {$ELSE}
      ClipRgn:=CreateRectRgn(r.Left, r.Top,  r.Right, r.Bottom);
      OldClipRgn:=SelectObject(Canvas.Handle, ClipRgn);
    {$ENDIF}
    try
      Canvas.Brush.Style:=bsSolid;
      if Canvas.Brush.Color <> Color then Canvas.Brush.Color:=Color;
      Canvas.FillRect(r);
    finally
      {$IFDEF CLX}
        Canvas.ResetClipRegion();
      {$ELSE}
        SelectObject(Canvas.Handle, OldClipRgn);
        DeleteObject(ClipRgn);
      {$ENDIF}
    end;
  end;

  if ShowRightMargin then begin
    r:=ClientRect;
    {$IFDEF CLX}
      Canvas.SetClipRect(r);
    {$ELSE}
      ClipRgn:=CreateRectRgn(r.Left, r.Top,  r.Right, r.Bottom);
      OldClipRgn:=SelectObject(Canvas.Handle, ClipRgn);
    {$ENDIF}
    try
//      T:=Round((cARPaperSizeMetrics[SrcRep.PaperSize, Ord(SrcRep.Orientation)] - SrcRep.LeftMargin * 10.0) / 25.4 * Screen.PixelsPerInch);
      T:=Round((SrcRep.PageSize.Width - SrcRep.LeftMargin) / 10.0 / 25.4 * Screen.PixelsPerInch);
      Canvas.Pen.Color:=clRed;
      Canvas.MoveTo(FGutter + T - LeftX, 0);
      Canvas.LineTo(FGutter + T - LeftX, ClientHeight);
    finally
      {$IFDEF CLX}
        Canvas.ResetClipRegion();
      {$ELSE}
        SelectObject(Canvas.Handle, OldClipRgn);
        DeleteObject(ClipRgn);
      {$ENDIF}
    end;
  end;
end;

function    TARGridRep.BandRect(IdxBand:integer):TRect;
begin
  if (IdxBand >= 0) and (IdxBand < SrcRep.BandCount) then begin
    if IdxBand < FixedBands then begin
      Result.Top    :=SrcRep.Tops[IdxBand];
      Result.Bottom :=Result.Top + SrcRep.Bands[IdxBand].Height;// - 1;
    end else begin
      Result.Top    :=SrcRep.Tops[IdxBand] - TopY;
      Result.Bottom :=Result.Top + SrcRep.Bands[IdxBand].Height;// - 1;
    end;
    Result.Left     :=0;
    Result.Right    :=ClientWidth;
  end else begin
    Result:=Rect(0, 0, 0, 0);
  end;
end;

function    TARGridRep.BandClipRect(IdxBand:integer):TRect;
begin
  if (IdxBand >= 0) and (IdxBand < SrcRep.BandCount) then begin
    Result:=BandRect(IdxBand);
    if IdxBand >= FixedBands then begin
      Result.Top    :=Max(Result.Top, SrcRep.Tops[FixedBands]);
    end;
  end else begin
    Result:=Rect(0, 0, 0, 0);
  end;
end;

initialization
  CollapsedBMP:=TBitmap.Create;
  ExpandedBMP :=TBitmap.Create;

  CollapsedBMP.LoadFromResourceName(HInstance, 'Collapsed');
  ExpandedBMP. LoadFromResourceName(HInstance, 'Expanded');
finalization
  CollapsedBMP.Free;
  ExpandedBMP.Free;
end.


