unit ARHTMLExport;

{$INCLUDE DEFINES.INC}

interface

uses
  SysUtils, Classes,
  {$IFDEF CLX}
    QGraphics, Qt,
  {$ELSE}
    Windows, Messages, Graphics, Controls, Forms, Dialogs,
    ComObj, ComCtrls, FileCtrl,
    {$IFDEF D6}
      Variants,
    {$ENDIF}
  {$ENDIF}

  ARCustomExport, ARUtils, ARRepCommon, ARRep, ARGridRep, ARUserRep, ARPrinter;

type
  TARHTMLExport = class(TARCustomExport)
  private
  protected
  public
    procedure   Execute;override;
  published
  end;

procedure Register;

implementation
{$R *.res}

procedure Register;
begin
  RegisterComponents('ARComponents', [TARHTMLExport]);
end;

{ TARHTMLExport }

function CompareBounds(Item1, Item2: Pointer): Integer;
begin
  Result := integer(Item1) - integer(Item2);
end;

const
{$IFDEF MSWINDOWS}
  PathDelim = '\';
{$ENDIF}

{$IFDEF LINUX}
  PathDelim = '/';
{$ENDIF}

procedure TARHTMLExport.Execute;
  function  RGBtoBGR(color:TColor):TColor;
  begin
    Result:=(color and $00FF00) or ((color and $0000FF) shl 16) or ((color and $FF0000) shr 16);
  end;
  function  FontSize(size:integer):string;
  begin
    case size of
       1.. 8: Result:='1';
       9..10: Result:='2';
      11..12: Result:='3';
      13..14: Result:='4';
      15..18: Result:='5';
      19..24: Result:='6';
      25..36: Result:='7';
    else      Result:='7';
    end;
  end;

  procedure PrepareBoundList(bl:TList);
  var idxband, idxcell:integer;
      i:integer;
  begin
    bl.Clear();
    bl.Add(pointer(0));
    for idxband:=0 to ARRep.SrcRep.BandCount - 1 do begin
      for idxcell:=0 to ARRep.SrcRep.Bands[idxband].CellCount - 1 do begin
        i:=ARRep.SrcRep.Bands[idxband].Lefts[idxcell];
        if bl.IndexOf(pointer(i)) = -1 then begin
          bl.Add(pointer(i));
        end;
      end;

      i:=ARRep.SrcRep.Bands[idxband].Rights[ARRep.SrcRep.Bands[idxband].CellCount - 1];
      if bl.IndexOf(pointer(i)) = -1 then begin
        bl.Add(pointer(i));
      end;
    end;
    bl.Sort(CompareBounds);
  end;
var idxband, idxcell, imgidx:integer;
    sl, imglist:TStringList;
    bl:TList;
    band, btmband:TARBand;
    cell:TARCell;
    s, halign, valign, width, height, colspan, rowspan, bgcolor, fontcolor:string;
    RootDir, ImgDir, RelImgDir, imgname, n1,n2:string;
    bmp:TBitmap;
begin
  if Assigned(ARRep) then begin
    sl      :=TStringList.Create;
    bl      :=TList.Create();
    imglist :=TStringList.Create;
    imgidx  :=1;
    RootDir :=ExtractFilePath(FileName);
    ImgDir  :=ExtractFilePathAndNameOnly(FileName) + '_files';
    ForceDirectories(ExpandFileName(ImgDir));
    RelImgDir:=ExtractFileNameOnly(FileName) + '_files';
    try
      PrepareBoundList(bl);

      sl.Add('<html>');
      sl.Add('');
      sl.Add('  <head>');
//      sl.Add('    <meta http-equiv="Content-Language" content="en-us">');
//      sl.Add('    <meta http-equiv="Content-Type" content="text/html; charset=windows-1252">');
      sl.Add('    <meta name="GENERATOR" content="AND Report Generator">');
      sl.Add('    <meta name="ProgId" content="ARHTMLExport.Document">');
      if ARRep is TARUserRep then begin
        sl.Add('    <title>' + TARUserRep(ARRep).Title + '</title>');
      end;
      sl.Add('  </head>');
      sl.Add('');
      sl.Add('  <body> <div align="center"> <center>');

      ProgressStart(ARRep.SrcRep.BandCount);
      sl.Add('    <table border="1" width="' + IntToStr(ARRep.SrcRep.Bands[0].Width) + '" cellspacing="0" cellpadding="0">');
      for idxband:=0 to ARRep.SrcRep.BandCount - 1 do begin
        band  :=ARRep.SrcRep.Bands[idxband];
        height:=IntToStr(band.Height);
        sl.Add('      <tr>');

        for idxcell:=0 to band.CellCount - 1 do begin
          cell:=band.Cells[idxcell];
          if cell.Shared then continue;

          btmband:=cell.GetBottomBand;
          if btmband = band then begin
            rowspan:='1';
          end else begin
            rowspan:=IntToStr(btmband.Owner.IndexOfBand(btmband) - idxband + 1);
          end;

          colspan:=IntToStr(
                      bl.IndexOf(pointer(ARRep.SrcRep.Bands[idxband].Rights[idxcell])) -
                      bl.IndexOf(pointer(ARRep.SrcRep.Bands[idxband].Lefts[idxcell]))
                   );


          case cell.HAlign of
            ahLeft:   halign:='left';
            ahCenter: halign:='center';
            ahRight:  halign:='right';
          end;
          case cell.VAlign of
            avTop:    valign:='top';
            avCenter: valign:='middle';
            avBottom: valign:='bottom';
          end;
          width   :=IntToStr(cell.Width);
          s:='<td width="' + width      + '"' +
             ' align="'    + halign     + '"' +
             ' height="'   + height     + '"' +
             ' valign="'   + valign     + '"';

          if colspan <> '1' then begin
            s:=s + ' colspan="'  + colspan    + '"';
          end;

          if rowspan <> '1' then begin
            s:=s + ' rowspan="'  + rowspan    + '"';
          end;

          if cell.Color <> clWhite then begin
            bgcolor :=IntToHex(RGBtoBGR(cell.Color), 6);
            s:=s + ' bgcolor="#' + bgcolor + '"';
          end;
          s:=s + '>';

          imgname:='';
          if cell.LinkPictureToFile and (Length(cell.PictureFileName) > 0) then begin
            imgname:=cell.PictureFileName;
          end else begin
            if Assigned(cell.Picture.Graphic) and (not cell.Picture.Graphic.Empty) then begin
              bmp:=TBitmap.Create;
              try
                bmp.Width :=cell.Picture.Width;
                bmp.Height:=cell.Picture.Height;
                bmp.Canvas.Draw(0, 0, cell.Picture.Graphic);
                imgname:=Format('%d.bmp', [imgidx]);
                inc(imgidx);
                bmp.SaveToFile(ImgDir + PathDelim + imgname);//full path
                imgname:='./' + RelImgDir + '/' + imgname;
              finally
                bmp.Free;
              end;
            end;
          end;
          if Length(imgname) > 0 then begin

            n1 := ExtractFileName(imgname);        //CD always copy the source picture.
            n2 := ImgDir + '\' + n1;
            copyfile(pchar(imgname),pchar(n2),false);

            s:=s + '<img border="0"';
//            s:=s + ' src="'  + './' + RelImgDir + '/' + n1 + '"';
            s:=s + ' src="'  + n1 + '"';
            if cell.AutoSize then begin  //CD if autosize the imaged is forced to cell dimensions
              s:=s + ' width="'  + IntToStr(cell.Width)  + '"';
              s:=s + ' height="' + IntToStr(band.Height) + '"';
            end else begin
              s:=s + ' width="'  + IntToStr(cell.Picture.Width)  + '"';
              s:=s + ' height="' + IntToStr(cell.Picture.Height) + '"';
            end;
            s:=s + '>';

{ //CD modified
            s:=s + '<img border="0"';
            s:=s + ' src="'  + imgname + '"';
            s:=s + ' width="'  + IntToStr(cell.Picture.Width)  + '"';
            s:=s + ' height="' + IntToStr(cell.Picture.Height) + '"';
            s:=s + '>';
}
          end;

          s:=s + '<font face="' + cell.FontName + '"';
          if cell.FontColor <> clBlack then begin
            fontcolor:=IntToHex(RGBtoBGR(cell.FontColor), 6);
            s:=s + ' color="#' + fontcolor + '"';
          end;
          s:=s + ' size="' + FontSize(cell.FontSize) + '"';

          s:=s + '>';

          if fsBold in cell.FontStyle then s:=s + '<b>';
            if fsItalic in cell.FontStyle then s:=s + '<i>';
              if fsUnderline in cell.FontStyle then s:=s + '<u>';
                s:=s + cell.Value;
              if fsUnderline in cell.FontStyle then s:=s + '</u>';
            if fsItalic in cell.FontStyle then s:=s + '</i>';
          if fsBold in cell.FontStyle then s:=s + '</b>';

          s:=s + '</font></td>';
          
          sl.Add('        ' + s);
        end;
        sl.Add('      </tr>');
        if not ProgressStep then break;
      end;

      sl.Add('');
      sl.Add('    </table>');
      sl.Add('  </center> </div> </body>');
      sl.Add('');
      sl.Add('</html>');

      if Length(FileName) > 0 then begin
        sl.SaveToFile(FileName);
      end;
    finally
      ProgressStop;
      imglist.Free;
      bl.Free;
      sl.Free;
    end;
  end;
end;

end.

