unit ARIBFS;
//************************************************************//
//                         AND Report                        //
//  Eugene Hrulev (mrHru) Laxian Key, Omsk, Russia 1999,2001 //
//  Oleg Melnikov (muler) Laxian Key, Omsk, Russia 1999,2001 //
//************************************************************//

{$INCLUDE DEFINES.INC}

interface

uses
  Classes, SysUtils,
  {$IFDEF CLX}
  {$ELSE}
    Menus, Dialogs, DBTables, Controls, DB, IBDatabase, 
  {$ENDIF}


  ARVFS, ARUtils, ARResStrings, SQLIB, ARIBFSSetup, ARSelectFS;

type

  TIBFileSystem = class(TVirtualFileSystem)
  private
    FIBDatabase   :TIBDatabase;
    FSelfDatabase :boolean;
  protected
    FTableName    :string;
    QCreate       :TISQL;
    FAliases      :TStringList;
    FCurrAlias    :string;

    procedure     CheckDatabase;
    procedure     SetDatabase(ADatabase:TIBDatabase);virtual;
    procedure     SetTableName(ATableName:string);virtual;
    procedure     ResetQs;
    procedure     SetParams(AParams:string);override;
    function      GetParams:string;override;
    function      GetDisplayName:string;override;
    function      GetMediaName:string;override;
    procedure     SetSelfDatabase(ASelfDatabase:boolean);
    function      SelectAliasByFileName(AFileName:string):string;
    function      SelectAlias():boolean;
    function      GetCurrAlias:string;
    procedure     SetCurrAlias(ACurrAlias:string);
  public
    QOpen         :TISQL;
    QWrite        :TISQL;
  published
    property      Database:TIBDatabase  read  FIBDatabase   write SetDatabase;
    property      TableName:string      read  FTableName    write SetTableName;
    property      SelfDatabase:boolean  read  FSelfDatabase write SetSelfDatabase;

    property      CurrAlias:string      read  GetCurrAlias  write SetCurrAlias;
    procedure     ExecuteSetup(Sender: TObject);override;

    function      OpenFile(AFileName:string):boolean;virtual;
    procedure     CreateFile(AFileName:string);override;

    function      CreateReadStream(AFileName:string):TStream;override;
    procedure     CloseReadStream;override;

    function      CreateWriteStream(AFileName:string):TStream;override;
    procedure     CloseWriteStream;override;

    procedure     SelectDatabase;
    procedure     SelectTableName;

    function      OpenDialogExecute:boolean;override;
    function      SaveDialogExecute:boolean;override;

    function      FileExists(AFileName:string):boolean;override;
    function      ExtractPath(AFileName:string):string;override;
    function      ExtractFileNameOnly(AFileName:string):string;override;
    procedure     SplitFileName(AFileName:string; s:TAIStringList);virtual;

    constructor   Create(AOwner:TComponent);override;
    constructor   CreateDB(AOwner:TComponent; Dummy:integer = 0);
    destructor    Destroy;override;
  end;

procedure  Register;

implementation
{$R *.res}

uses ARIBOpen;

//////////////////////////   TIBFileSystem  /////////////////////
procedure    TIBFileSystem.CheckDatabase;
begin
  if not SelfDatabase then exit;

//  if  (FIBDatabase.DatabaseName = '') or (TableName = '') then begin
  if not FIBDatabase.Connected then begin
    try
      FIBDatabase.Connected:=true;
    except
      ExecuteSetup(Self);
    end;
  end;
end;

procedure   TIBFileSystem.SplitFileName(AFileName:string; s:TAIStringList);
var  p:integer;
begin
  AFileName:=ANSIUpperCase(AFileName);

  s.Clear;
  p:=System.Pos('.', AFileName);
  while p > 0 do begin
    if p > 1 then s.Add(System.Copy(AFileName, 1, p-1));
    System.Delete(AFileName, 1, p);
    p:=System.Pos('.', AFileName);
  end;
  s.Add(AFileName);
end;

function    TIBFileSystem.OpenFile(AFileName:string):boolean;
var sl:TAIStringList;
    par, i:integer;
    fname:string;
begin
  Result:=false;

  fname:=SelectAliasByFileName(AFileName);

  CheckDatabase;

  sl:=TAIStringList.Create('TIBFileSystem.OpenFile s');

  try
    if Length(fname) > 0 then begin
      SplitFileName(fname, sl);

      if not Assigned(QOpen) then begin
        QOpen:=TISQL.Create(Database);
        QOpen.Add('SELECT * ');
        QOpen.Add('FROM ' + TableName + ' ');
        QOpen.Add('WHERE PARENT = :PARENT AND NAME = :NAME ');
        QOpen.Prepare;
      end;

      par:=0;
      for i:=0 to sl.Count - 1 do begin
        QOpen.Close;
        QOpen.I['PARENT']  :=par;
        QOpen.S['NAME']    :=sl[i];
        QOpen.Open;

        if QOpen.EOF then exit;

        par:=QOpen.I['ID'];
      end;
      Result:=true;
    end;
  finally
    sl.Free;
  end;
end;

function    TIBFileSystem.OpenDialogExecute:boolean;
var d:TIBOpenFileDlg;
begin
  Result:=false;

//  CheckDatabase;

  if SelectAlias() then begin
    d:=TIBOpenFileDlg.Create(Self);
    d.Filename:=FileName;
    try
      d.IBFS:=Self;

      Result:=d.Execute;
      if Result then begin
        FileName:=CurrAlias + ':' + d.FileName;
        Files.Clear;
        Files.Add(FileName);
      end;
    finally
      d.Free;
    end;
  end;
end;

function    TIBFileSystem.ExtractPath(AFileName:string):string;
var sl:TAIStringList;
    i:integer;
begin
  sl:=TAIStringList.Create('TIBFileSystem.ExtractPath sl');
  try
    SplitFileName(AFileName, sl);
    Result:='';
    for i:=0 to sl.Count - 2 do Result:=Result + '.' + sl[i];
  finally
    sl.Free;
  end;
end;

function    TIBFileSystem.ExtractFileNameOnly(AFileName:string):string;
var sl:TAIStringList;
begin
  sl:=TAIStringList.Create('TIBFileSystem.ExtractFileNameOnly sl');
  try
    SplitFileName(AFileName, sl);
    Result:='';
    if sl.Count > 0 then Result:=sl[sl.Count - 1];
  finally
    sl.Free;
  end;
end;

procedure   TIBFileSystem.SetDatabase(ADatabase:TIBDatabase);
begin
  ResetQs;
  FIBDatabase:=ADatabase;
//  CheckDatabase;
end;

procedure   TIBFileSystem.SetTableName(ATableName:string);
begin
  ResetQs;
  FTableName:=ATableName;
//  CheckDatabase;
end;

procedure   TIBFileSystem.ResetQs;
begin
  if Assigned(QOpen) then begin
    QOpen.Free;
    QOpen:=nil;
  end;
  if Assigned(QWrite) then begin
    QWrite.Free;
    QWrite:=nil;
  end;
  if Assigned(QCreate) then begin
    QCreate.Free;
    QCreate:=nil;
  end;
end;

procedure   TIBFileSystem.CreateFile(AFileName:string);
var sl:TAIStringList;
    par, i:integer;
    fname:string;
begin
  fname:=SelectAliasByFileName(AFileName);

  if FileExists(fname) then exit;

  sl:=TAIStringList.Create('TIBFileSystem.CreateFile s');
  try
    SplitFileName(fname, sl);

    if not Assigned(QCreate) then begin
      QCreate:=TISQL.Create(FIBDatabase);
      QCreate.Add('INSERT INTO ' + FTableName + '(PARENT, NAME) VALUES(:PARENT, :NAME) ');
      QCreate.Prepare;
    end;

    par:=0;
    for i:=0 to sl.Count - 1 do begin
      try
        QCreate.I['PARENT']:=par;
        QCreate.S['NAME']:=sl[i];
        QCreate.Exec;
      except
        //already exists, do nothing
      end;
      QOpen.Close;
      QOpen.I['PARENT']  :=par;
      QOpen.S['NAME']    :=sl[i];
      QOpen.Open;
      par:=QOpen.I['ID'];
    end;

  finally
    sl.Free;
  end;
end;

function    TIBFileSystem.CreateWriteStream(AFileName:string):TStream;
begin
  FFileName:=ANSIUpperCase(SelectAliasByFileName(AFileName));

  CreateFile(FFileName);
  FWriteStream:=TStringStream.Create('');
  Result:=FWriteStream;
end;

procedure   TIBFileSystem.CloseWriteStream;
begin
  QWrite:=TISQL.Create(Database);
	QWrite.Add('UPDATE ' + FTableName + ' SET STRVAL = :STRVAL WHERE ID = :ID');
  QWrite.I['ID']    :=QOpen.I['ID'];
	QWrite.S['STRVAL']:=TStringStream(FWriteStream).DataString;
  QWrite.Exec();
  FWriteStream.Free();
end;

function    TIBFileSystem.CreateReadStream(AFileName:string):TStream;
begin
  Result:=nil;

  FFileName:=ANSIUpperCase(SelectAliasByFileName(AFileName));

  if OpenFile(FFileName) then begin
    FReadStream:=TStringStream.Create(QOpen.S['STRVAL']);
    Result:=FReadStream;
    QOpen.Close();
  end;
end;

procedure   TIBFileSystem.CloseReadStream;
begin
  FReadStream.Free();
end;

procedure   TIBFileSystem.SelectDatabase;
var od:TOpenDialog;
begin
  od:=TOpenDialog.Create(Self);
  try
    od.DefaultExt :='gdb';
    od.Filter     :='Interbase DB (*.gdb)|*.gdb';
    od.FileName   :=FIBDatabase.DatabaseName;
    if od.Execute then begin
      FIBDatabase.Connected   :=false;
      FIBDatabase.DatabaseName:=od.FileName;
      try
        FIBDatabase.Connected   :=true;
      except
        FIBDatabase.DatabaseName:=InputBox(rsARSelectDatabase_Caption, '', od.FileName);
        FIBDatabase.Connected   :=true;
      end;
    end;
  finally
    od.Free();
  end;
end;

procedure   TIBFileSystem.SelectTableName;
begin
  TableName:=InputBox(FIBDatabase.DatabaseName, rsVFS_EnterTableName, 'Reports');
end;

constructor TIBFileSystem.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);

  FAliases:=TStringList.Create();
end;

constructor TIBFileSystem.CreateDB(AOwner:TComponent; Dummy:integer);
begin
  inherited Create(AOwner);

  FAliases    :=TStringList.Create();
  FIBDatabase :=TIBDatabase.Create(Self);
end;

destructor  TIBFileSystem.Destroy;
begin
  FAliases.Free();

  inherited;
end;

function    TIBFileSystem.GetDisplayName:string;
begin
  Result:=rsFS_InterbaseDisplayName;
end;

function    TIBFileSystem.GetMediaName:string;
begin
  Result:=rsFS_InterbaseMediaName;
end;

procedure   TIBFileSystem.ExecuteSetup(Sender: TObject);
var d:TARIBFSSetupDlg;
begin
  d:=TARIBFSSetupDlg.Create(self);
  try
    d.IBDatabase:=FIBDatabase;
    d.AliasList.AddStrings(FAliases);
    d.ShowModal;
    FAliases.Clear();
    FAliases.AddStrings(d.AliasList);
  finally
  end;
end;

procedure   TIBFileSystem.SetSelfDatabase(ASelfDatabase:boolean);
begin
  if FSelfDatabase <> ASelfDatabase then begin
    FSelfDatabase:=ASelfDatabase;
    if FSelfDatabase and not Assigned(FIBDatabase) then begin
      FIBDatabase:=TIBDatabase.Create(Self);
    end;
  end;
end;

function    TIBFileSystem.FileExists(AFileName:string):boolean;
begin
  Result:=OpenFile(SelectAliasByFileName(AFileName));
end;

function    TIBFileSystem.SaveDialogExecute:boolean;
begin
  Result:=OpenDialogExecute();
end;

procedure   TIBFileSystem.SetParams(AParams:string);
begin
  FAliases.CommaText:=AParams;
end;

function    TIBFileSystem.GetParams:string;
begin
  Result:=FAliases.CommaText;
end;

function    TIBFileSystem.SelectAliasByFileName(AFileName:string):string;
var idx, i:integer;
    s:string;
begin
  Result:=AFileName;
  idx:=System.Pos(':', AFileName);
  if idx > 0 then begin
    s:=System.Copy(AFileName, 1, idx - 1);
    for i:=0 to FAliases.Count - 1 do begin
      if CT(s, FAliases.Names[i]) then begin
        CurrAlias:=FAliases[i];
        Result:=System.Copy(AFileName, idx + 1, Length(AFileName));
        break;
      end;
    end;
  end else begin
//    CurrAlias:=FAliases[0];
  end
end;

function      TIBFileSystem.GetCurrAlias:string;
var idx:integer;
begin
  Result:=FCurrAlias;
  idx:=System.Pos(':', FCurrAlias);
  if idx > 0 then begin
    Result:=System.Copy(FCurrAlias, 1, idx - 1);
  end;
end;

procedure     TIBFileSystem.SetCurrAlias(ACurrAlias:string);
var sl:TStringList;
    idx, v, code:integer;
begin
//  if FCurrAlias <> ACurrAlias then begin

    FCurrAlias:=ACurrAlias;
    idx:=System.Pos('=', ACurrAlias);
    if idx > 0 then begin
      FCurrAlias:=System.Copy(ACurrAlias, 1, idx - 1);
    end;

    sl:=TStringList.Create();
    try
      sl.CommaText        :=FAliases.Values[FCurrAlias];

      FIBDatabase.Connected       :=false;
      FIBDatabase.DatabaseName    :=sl.Values['DATABASENAME'];
      TableName                   :=sl.Values['TABLENAME'];

      FIBDatabase.Params.Clear();
      FIBDatabase.Params.Add('user_name=' + sl.Values['USER']);
      FIBDatabase.Params.Add('password='  + sl.Values['PASSWORD']);

      System.Val(sl.Values['LOGINPROMPT'], v, code);
      if code = 0 then begin
        FIBDatabase.LoginPrompt:=boolean(v);
      end;

      try
        FIBDatabase.Connected  :=true;
      except
        ExecuteSetup(Self);
      end;

    finally
      sl.Free();
    end;
//  end;
end;

function      TIBFileSystem.SelectAlias():boolean;
var d:TARSelectFSDlg;
    i:integer;
begin
  Result:=false;
  if FAliases.Count = 1 then begin
    CurrAlias:=FAliases.Names[0];
    Result:=true;
  end else begin
    if FAliases.Count > 1 then begin
      d:=TARSelectFSDlg.Create(Self);
      try
        for i:=0 to FAliases.Count - 1 do begin
          d.FSList.Items.Add(FAliases.Names[i]);
        end;
        d.FSList.ItemIndex:=d.FSList.Items.IndexOf(CurrAlias);
        if d.ShowModal() = mrOK then begin
          CurrAlias:=d.FSList.Items[d.FSList.ItemIndex];
          Result:=true;
        end;
      finally
        d.Free();
      end;
    end;
  end;
end;

////////////////////////////////////////////////////////////////
procedure Register;
begin
  RegisterComponents('ARComponents', [TIBFileSystem]);
end;

initialization
  RegisterClass(TIBFileSystem);
finalization
  UnRegisterClass(TIBFileSystem);
end.



