unit ARIBFSSetup;
//************************************************************//
//                         AND Report                        //
//  Eugene Hrulev (mrHru) Laxian Key, Omsk, Russia 1999,2001 //
//  Oleg Melnikov (muler) Laxian Key, Omsk, Russia 1999,2001 //
//************************************************************//

{$INCLUDE DEFINES.INC}

interface

uses
  SysUtils, Classes,
  {$IFDEF CLX}
    QDialogs, QStdCtrls, QButtons, QControls,
  {$ELSE}
    Windows, Messages, Graphics, Controls, Forms, Dialogs, StdCtrls,
    Buttons, IBDatabase,
  {$ENDIF}

  ARResStrings, ARUtils, SQLIB;

type
  TARIBFSSetupDlg = class(TForm)
    NewBtn: TBitBtn;
    ApplyBtn: TBitBtn;
    DeleteBtn: TBitBtn;
    CloseBtn: TBitBtn;
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    MsgMemo: TMemo;
    Aliases: TComboBox;
    DBPath: TEdit;
    BrowseBtn: TBitBtn;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    TableName: TComboBox;
    UserGB: TGroupBox;
    User: TEdit;
    Password: TEdit;
    Label4: TLabel;
    Label5: TLabel;
    LoginPrompt: TCheckBox;
    OpenDialog: TOpenDialog;
    TestBtn: TBitBtn;
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure NewBtnClick(Sender: TObject);
    procedure ApplyBtnClick(Sender: TObject);
    procedure AliasesClick(Sender: TObject);
    procedure BrowseBtnClick(Sender: TObject);
    procedure PropChange(Sender: TObject);
    procedure CloseBtnClick(Sender: TObject);
    procedure TestBtnClick(Sender: TObject);
    procedure DeleteBtnClick(Sender: TObject);
  private
    FPropChanged:boolean;
    function    TestConnection():boolean;
    procedure   GetTableList();
    procedure   CheckTable();
    procedure   GetAliasParams();
    procedure   SetAliasParams();
    procedure   LoadAliases();

    procedure   SetPropChanged(APropChanged:boolean);
  public
    AliasList   :TStringList;
    IBDatabase  :TIBDatabase;

    property    PropChanged:boolean read FPropChanged write SetPropChanged;
  end;

implementation

{$R *.dfm}

procedure TARIBFSSetupDlg.FormCreate(Sender: TObject);
begin
  AliasList:=TStringList.Create();
end;

procedure TARIBFSSetupDlg.FormDestroy(Sender: TObject);
begin
  AliasList.Free();
end;

procedure TARIBFSSetupDlg.FormShow(Sender: TObject);
begin
  LoadAliases();
end;

procedure TARIBFSSetupDlg.LoadAliases();
var i:integer;
begin
  Aliases.Items.Clear();
  for i:=0 to AliasList.Count - 1 do begin
    Aliases.Items.Add(AliasList.Names[i]);
  end;
  Aliases.ItemInDex:=0;
  AliasesClick(Self);

  PropChanged:=false;
end;

function  TARIBFSSetupDlg.TestConnection():boolean;
begin
  MsgMemo.Text:='Connecting...';
  Application.ProcessMessages();
  IBDatabase.Connected:=false;
  IBDatabase.DatabaseName :=DBPath.Text;
  IBDatabase.Params.Clear();
  IBDatabase.Params.Add('user_name=' + User.Text);
  IBDatabase.Params.Add('password='  + Password.Text);
  IBDatabase.LoginPrompt  :=LoginPrompt.Checked;
  try
    IBDatabase.Connected:=true;
    MsgMemo.Text:='OK';
    GetTableList();
    Result:=true;
  except
    on E:Exception do begin
      MsgMemo.Text:=E.Message;
      Result:=true;
    end;
  end;
end;

procedure TARIBFSSetupDlg.CheckTable();
var Q:TISQL;
begin
  Q:=TISQL.Create(IBDatabase);
  try
    Q.Add('SELECT * FROM ' + TableName.Text);
    try
      Q.Open;
    except
      if MessageDlg(Format(rsVFS_TableNotExistsCreateIt, [TableName.Text, IBDatabase.DatabaseName]), mtConfirmation, [mbYes, mbNo], 0) = mrYes then begin
        Q.New;
        Q.Add('CREATE TABLE ' + TableName.Text + ' ( ');
        Q.Add('  ID       INTEGER NOT NULL, ');
        Q.Add('  PARENT   INTEGER NOT NULL, ');
        Q.Add('  NAME     CHAR(64) DEFAULT "NEW ITEM" NOT NULL, ');
        Q.Add('  INTVAL   INTEGER, ');
        Q.Add('  NUMVAL   DOUBLE PRECISION, ');
        Q.Add('  STRVAL   BLOB SUB_TYPE TEXT SEGMENT SIZE 64, ');
        Q.Add('  PRIMARY KEY (ID) ');
        Q.Add(') ');
        Q.Exec;

        Q.New;
        Q.Add('CREATE UNIQUE ASCENDING INDEX ' + TableName.Text + 'INDEX_PARENT_NAME ');
        Q.Add(' ON ' + TableName.Text + ' (PARENT, NAME) ');
        Q.Exec;

        Q.New;
        Q.Add('CREATE GENERATOR GEN_' + TableName.Text + '_ID ');
        Q.Exec;

        Q.New;
        Q.Add('CREATE TRIGGER ' + TableName.Text + 'TRIGGER_BI FOR ' + TableName.Text);
        Q.Add('  BEFORE INSERT POSITION 0 AS ');
        Q.Add('BEGIN ');
        Q.Add(' IF( NEW.ID IS NULL ) THEN NEW.ID = GEN_ID(GEN_' + TableName.Text + '_ID, 1); ');
        Q.Add('END ');
        Q.Exec;

        Q.Commit;
      end;
    end;
  finally
    Q.Free;
  end;
end;

procedure TARIBFSSetupDlg.NewBtnClick(Sender: TObject);
begin
  LoginPrompt.Checked :=false;
  ApplyBtn.Enabled    :=true;
  DeleteBtn.Enabled   :=false;
  CloseBtn.Caption    :='Cancel';
  Aliases.Style       :=csDropDown;
  Aliases.SelectAll();
  Aliases.SetFocus();
end;

procedure TARIBFSSetupDlg.ApplyBtnClick(Sender: TObject);
begin
  if TestConnection() then begin
    CheckTable();
    SetAliasParams();
    PropChanged:=false;
  end;
end;

procedure TARIBFSSetupDlg.SetAliasParams();
var sl:TStringList;
begin
  sl:=TStringList.Create();
  try
    sl.Add('DATABASENAME=' + DBPath.Text);
    sl.Add('TABLENAME='    + TableName.Text);
    sl.Add('USER='         + User.Text);
    sl.Add('PASSWORD='     + Password.Text);
    sl.Add('LOGINPROMPT='  + IntToStr(Ord(LoginPrompt.Checked)));

    if AliasList.IndexOfName(Aliases.Text) <> -1 then begin
      AliasList.Values[Aliases.Text]:=sl.CommaText;
    end else begin
      AliasList.Add(Aliases.Text + '=' + sl.CommaText);
    end;
  finally
    sl.Free();
  end;
end;

procedure TARIBFSSetupDlg.GetAliasParams();
var sl:TStringList;
    s:string;
    v, code:integer;
begin
  sl:=TStringList.Create();
  try
    sl.CommaText        :=AliasList.Values[Aliases.Text];
    DBPath.Text         :=sl.Values['DATABASENAME'];
    TableName.Text      :=sl.Values['TABLENAME'];
    User.Text           :=sl.Values['USER'];
    Password.Text       :=sl.Values['PASSWORD'];
    LoginPrompt.Checked :=false;
    s:=sl.Values['LOGINPROMPT'];
    System.Val(s, v, code);
    if code = 0 then begin
      LoginPrompt.Checked :=boolean(v);
    end;

    TestConnection();
  finally
    sl.Free();
  end;
end;

procedure TARIBFSSetupDlg.AliasesClick(Sender: TObject);
begin
  GetAliasParams();
  PropChanged:=false;
end;

procedure TARIBFSSetupDlg.BrowseBtnClick(Sender: TObject);
begin
  OpenDialog.FileName:=DBPath.Text;
  if OpenDialog.Execute then begin
    DBPath.Text:=OpenDialog.FileName;
    PropChanged:=true;
  end;
end;

procedure   TARIBFSSetupDlg.SetPropChanged(APropChanged:boolean);
begin
  FPropChanged:=APropChanged;
  ApplyBtn.Enabled  :=FPropChanged;
  NewBtn.Enabled    :=not FPropChanged;
  DeleteBtn.Enabled :=not FPropChanged;
  if FPropChanged then begin
    CloseBtn.Caption:='&Cancel';
  end else begin
    CloseBtn.Caption:='&Close';
  end;
end;

procedure TARIBFSSetupDlg.PropChange(Sender: TObject);
begin
  PropChanged:=true;
end;

procedure TARIBFSSetupDlg.CloseBtnClick(Sender: TObject);
begin
  if PropChanged then begin
    GetAliasParams();
    PropChanged:=false;
  end else begin
    Close();
  end;
end;

procedure TARIBFSSetupDlg.TestBtnClick(Sender: TObject);
begin
  TestConnection();
end;

procedure TARIBFSSetupDlg.GetTableList();
var Q:TISQL;
begin
  Q:=TISQL.Create(IBDatabase);
  try
    TableName.Items.Clear();
    Q.Add('SELECT   DISTINCT RDB$RELATION_NAME');
    Q.Add('FROM     RDB$RELATION_FIELDS');
    Q.Add('WHERE    RDB$VIEW_CONTEXT IS NULL AND RDB$SYSTEM_FLAG = 0');
    Q.Add('ORDER BY RDB$RELATION_NAME');
    Q.Open;
    while not Q.EOF do begin
      TableName.Items.Add(Upper(Trim(Q.S['RDB$RELATION_NAME'])));
      Q.Next;
    end;
    Q.Close;
  finally
    Q.Free;
  end;
end;

procedure TARIBFSSetupDlg.DeleteBtnClick(Sender: TObject);
var idx:integer;
begin
  idx:=AliasList.IndexOfName(Aliases.Text);
  if idx <> -1 then begin
    AliasList.Delete(idx);
    LoadAliases();
  end;
end;

end.
