unit ARIBOpen;
//************************************************************//
//                         AND Report                        //
//  Eugene Hrulev (mrHru) Laxian Key, Omsk, Russia 1999,2001 //
//  Oleg Melnikov (muler) Laxian Key, Omsk, Russia 1999,2001 //
//************************************************************//

{$INCLUDE DEFINES.INC}

interface

uses
  SysUtils, Classes,
  {$IFDEF CLX}
    QStdCtrls, QButtons, QControls, QComCtrls, QExtCtrls,
  {$ELSE}
    Windows, Messages, Graphics, Controls, Forms, Dialogs,
    StdCtrls, Buttons, ExtCtrls, ComCtrls,
  {$ENDIF}

  SQLIB, ARUtils, ARResStrings, ARVFS, ARIBFS;

type
  TIBOpenFileDlg = class(TForm)
    TV: TTreeView;
    Bevel1: TBevel;
    Bevel2: TBevel;
    FileNameEdit: TEdit;
    CancelBtn: TBitBtn;
    OkBtn: TBitBtn;
    procedure OkBtnClick(Sender: TObject);
    procedure CancelBtnClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure TVChange(Sender: TObject; Node: TTreeNode);
    procedure FormCreate(Sender: TObject);
  private
  public
    FileName    :string;
    IBFS        :TIBFileSystem;

    function    Execute:boolean;
  end;

var
  IBOpenFileDlg: TIBOpenFileDlg;

implementation

{$R *.dfm}

function    TIBOpenFileDlg.Execute:boolean;
begin
  FileNameEdit.Text:=FileName;
  TV.Items.Clear;
  ShowModal;
  Result:=ModalResult = mrOK;
end;

procedure TIBOpenFileDlg.OkBtnClick(Sender: TObject);
begin
  FileName:=FileNameEdit.Text;
  ModalResult:=mrOK;
end;

procedure TIBOpenFileDlg.CancelBtnClick(Sender: TObject);
begin
  ModalResult:=mrCancel;
end;

procedure TIBOpenFileDlg.TVChange(Sender: TObject; Node: TTreeNode);
var id:integer;
    Q:TISQL;
    s:string;
begin
  id:=integer(Node.Data);

  Node.DeleteChildren;

  Q:=TISQL.Create(IBFS.Database);
  try
    Q.Add('SELECT ID, NAME FROM ' + IBFS.TableName);
    Q.Add('WHERE  PARENT = :PARENT');
    Q.Add('ORDER  BY NAME');
    Q.I['PARENT']:=id;
    Q.Open;
    while not Q.EOF do begin
      TV.Items.AddChildObject(Node, Trim(Q.S['NAME']), pointer(Q.I['ID']));
      Q.Next;
    end;

    Node.Expand(false);

    Q.New;
    Q.Add('SELECT PARENT, NAME FROM ' + IBFS.TableName);
    Q.Add('WHERE  ID = :ID');
    Q.Add('ORDER  BY NAME');
    Q.Prepare;

    s:='';
    while id <> 0 do begin
      Q.Close;
      Q.I['ID']:=id;
      Q.Open;
      id:=Q.I['PARENT'];
      s:='.' + Trim(Q.S['NAME']) + s;
    end;

    FileNameEdit.Text:=s;
    Caption:=FileNameEdit.Text;
  finally
    Q.Free;
  end;
end;

procedure TIBOpenFileDlg.FormShow(Sender: TObject);
var i,j:integer;
    sl:TAIStringList;
    tn:TTreeNode;
begin
  if IBFS.OpenFile(IBFS.InitialDir) then begin
    TV.Selected:=TV.Items.AddObject(TV.Selected, Trim(IBFS.QOpen.S['NAME']), pointer(IBFS.QOpen.I['ID']));
  end else begin
    TV.Selected:=TV.Items.AddObject(TV.Selected, 'ROOT', pointer(0));
  end;

  sl:=TAIStringList.Create('TIBOpenFileDlg.FormShow sl');
  Self.IBFS.SplitFileName(FileName, sl);
  try
    tn:=TV.Selected;
    for i:=0 to sl.Count - 1 do begin
      for j:=0 to tn.Count - 1 do begin
        if CT(sl[i], tn.Item[j].Text) then begin
          TV.Selected:=tn.Item[j];
          break;
        end;
      end;
      if tn = TV.Selected then break;
      tn:=TV.Selected;
    end;
  finally
    sl.Free;
  end;
end;

procedure TIBOpenFileDlg.FormCreate(Sender: TObject);
begin
  Caption           :=rsARIBOpen_Caption;
  OkBtn.Caption     :=rsAR_OkCaption;
  CancelBtn.Caption :=rsAR_CancelCaption;
end;

end.
