unit ARJPEGExport;
//************************************************************//
//                         AND Report                        //
//  Eugene Hrulev (mrHru) Laxian Key, Omsk, Russia 1999,2001 //
//  Oleg Melnikov (muler) Laxian Key, Omsk, Russia 1999,2001 //
//************************************************************//

{$INCLUDE DEFINES.INC}

interface

uses
  SysUtils, Classes,
  {$IFDEF CLX}
    QGraphics, QDialogs,
  {$ELSE}
    Windows, Messages, Graphics, Controls, Forms, Dialogs, Jpeg,
  {$ENDIF}

  ARCustomExport, ARGridRep, ARUserRep, ARJPEGSetupExport;

type
  TARJPEGExport = class(TARCustomExport)
  private
    FGrayscale          :Boolean;
    FProgressiveEncoding:Boolean;
    {$IFDEF CLX}
    {$ELSE}
      FPerformance        :TJPEGPerformance;
      FCompressionQuality :TJPEGQualityRange;
    {$ENDIF}
    FShowProgress       :boolean;
    FShowSetupDialog    :boolean;
    FFromPage           :integer;
    FToPage             :integer;
    FExportRange        :TExportRange;
    FScaleFactor        :Integer;       //CD scale factor for jpg

    {$IFDEF CLX}
    {$ELSE}
      procedure   SetCompressionQuality(const Value: TJPEGQualityRange);
      procedure   SetPerformance(const Value: TJPEGPerformance);
    {$ENDIF}
    procedure   SetGrayscale(const Value: Boolean);
    procedure   SetProgressiveEncoding(const Value: Boolean);
    procedure   SetExportRange(const Value: TExportRange);
    procedure   SetFromPage(const Value: integer);
    procedure   SetShowProgress(const Value: boolean);
    procedure   SetShowSetupDialog(const Value: boolean);
    procedure   SetToPage(const Value: integer);
  protected
  public
    function    ExecuteSetupDialog:boolean;override;
    procedure   Execute;override;
  published
    property    ShowProgress:boolean      read  FShowProgress     write SetShowProgress;
    property    ShowSetupDialog:boolean   read  FShowSetupDialog  write SetShowSetupDialog;
    property    FromPage:integer          read  FFromPage         write SetFromPage;
    property    ToPage:integer            read  FToPage           write SetToPage;
    property    ExportRange:TExportRange  read  FExportRange      write SetExportRange;

    {$IFDEF CLX}
    {$ELSE}
      property    CompressionQuality:TJPEGQualityRange read FCompressionQuality  write SetCompressionQuality;
      property    Performance:TJPEGPerformance         read FPerformance         write SetPerformance;
    {$ENDIF}
    property    Grayscale:boolean                    read FGrayscale           write SetGrayscale default false;
    property    ProgressiveEncoding:boolean          read FProgressiveEncoding write SetProgressiveEncoding;
  end;

procedure Register;

implementation
{$R *.res}

procedure Register;
begin
  RegisterComponents('ARComponents', [TARJPEGExport]);
end;

{ TARJPEGExport }

procedure TARJPEGExport.Execute;
{$IFDEF CLX}
{$ELSE}
  var OldShowProgress:boolean;
      jpg: TJpegImage;
      bmp: TBitmap;
      i, pagefrom, pageto, p:integer;
      fpath, fname, fext:string;
{$ENDIF}
begin
  {$IFDEF CLX}
    ShowMessage('JPEG export can''t work under CLX application.');
  {$ELSE}
    if Assigned(ARRep) then begin

      if ShowSetupDialog then begin
        if not ExecuteSetupDialog then exit;
      end;

      OldShowProgress         :=ARRep.ShowPrintProgress;
      ARRep.ShowPrintProgress :=ShowProgress;
      ARRep.ShowPrintProgress :=OldShowProgress;

      if ARRep is TARUserRep then begin
        ARRep.CreateCanvasImage(TARUserRep(ARRep).Title);
      end else begin
        ARRep.CreateCanvasImage('Test');
      end;

      if ARRep.prn.PageCount = 0 then exit;

      case ExportRange of
        erAllPages: begin
                      pagefrom:=1;
                      pageto  :=ARRep.prn.PageCount;
                    end;
        erPageNums: begin
                      pagefrom:=FromPage;
                      pageto  :=ToPage;
                    end;
        else begin
          pagefrom:=1;
          pageto  :=ARRep.prn.PageCount;
        end;
      end;

      fpath:=ExtractFilePath(FileName);
      fname:=ExtractFileName(FileName);

      fext:=ExtractFileExt(FileName);
      if Length(fext) = 0 then begin
        fext:='.jpg';
      end else begin
        p:=System.Pos(fext, fname);
        if p > 0 then begin
          System.Delete(fname, p, Length(fname) - p + 1);
        end;
      end;

      p:=System.Pos('?', fname);
      if p > 0 then begin
        System.Delete(fname, p, 1);
        System.Insert('%d', fname, p);
      end else begin
        fname:=fname + '%d';
      end;

      jpg:=TJpegImage.Create;
      jpg.Grayscale           :=Grayscale;
      jpg.ProgressiveEncoding :=ProgressiveEncoding;
      jpg.Performance         :=Performance;
      jpg.CompressionQuality  :=CompressionQuality;
      try
        ProgressStart(pageto - pagefrom + 1);
        for i:=pagefrom to pageto do begin
          ARRep.prn.CurrPage:=i;

          bmp:=TBitmap.Create();
          try
            //CD draw with scale factor
            bmp.Width :=ARRep.prn.Metafile.Width * FScaleFactor;
            bmp.Height:=ARRep.prn.Metafile.Height * FScaleFactor;
            if FScaleFactor = 1 then begin
              bmp.Canvas.Draw( 0 , 0, ARRep.prn.Metafile);
            end else begin
              bmp.Canvas.StretchDraw( rect(0,0,bmp.Width,bmp.Height), ARRep.prn.Metafile);
            end;
            jpg.Assign(bmp);
            jpg.SaveToFile(Format(fpath + fname + fext, [i]));
          finally
            bmp.Free;
          end;
          if not ProgressStep then break;
        end;
      finally
        ProgressStop;
        jpg.free;
      end;
    end;
  {$ENDIF}
end;

function  TARJPEGExport.ExecuteSetupDialog: boolean;
var dlg:TJPEGSetupExportDlg;
begin
  Result:=false;
  dlg:=TJPEGSetupExportDlg.Create(Self);
  try
    {$IFDEF CLX}
    {$ELSE}
      dlg.CompressionQuality  :=CompressionQuality;
      dlg.Performance         :=Performance;
    {$ENDIF}
    dlg.Grayscale           :=Grayscale;
    dlg.ProgressiveEncoding :=ProgressiveEncoding;
    dlg.FileName            :=FileName;
    if dlg.Execute then begin
      {$IFDEF CLX}
      {$ELSE}
        CompressionQuality  :=dlg.CompressionQuality;
        Performance         :=dlg.Performance;
      {$ENDIF}
      ProgressiveEncoding :=dlg.ProgressiveEncoding;
      Grayscale           :=dlg.Grayscale;
      FileName            :=dlg.FileName;
      FScaleFactor        :=dlg.eScale.Value;  //CD scale factor from dialog

      Result:=true;
    end;
  finally
    dlg.Free;
  end;
end;

{$IFDEF CLX}
{$ELSE}
procedure TARJPEGExport.SetCompressionQuality(const Value: TJPEGQualityRange);
begin
  FCompressionQuality := Value;
end;
procedure TARJPEGExport.SetPerformance(const Value: TJPEGPerformance);
begin
  FPerformance := Value;
end;
{$ENDIF}

procedure TARJPEGExport.SetExportRange(const Value: TExportRange);
begin
  FExportRange := Value;
end;

procedure TARJPEGExport.SetFromPage(const Value: integer);
begin
  FFromPage := Value;
end;

procedure TARJPEGExport.SetGrayscale(const Value: Boolean);
begin
  FGrayscale := Value;
end;

procedure TARJPEGExport.SetProgressiveEncoding(const Value: Boolean);
begin
  FProgressiveEncoding := Value;
end;

procedure TARJPEGExport.SetShowProgress(const Value: boolean);
begin
  FShowProgress := Value;
end;

procedure TARJPEGExport.SetShowSetupDialog(const Value: boolean);
begin
  FShowSetupDialog := Value;
end;

procedure TARJPEGExport.SetToPage(const Value: integer);
begin
  FToPage := Value;
end;

end.

