unit ARJPEGSetupExport;
//************************************************************//
//                         AND Report                        //
//  Eugene Hrulev (mrHru) Laxian Key, Omsk, Russia 1999,2001 //
//  Oleg Melnikov (muler) Laxian Key, Omsk, Russia 1999,2001 //
//************************************************************//


{$INCLUDE DEFINES.INC}

interface

uses
  SysUtils, Classes,
  {$IFDEF CLX}
    QForms, QButtons, QStdCtrls, QExtCtrls, QDialogs, QComCtrls, QControls,
  Dialogs, ExtCtrls, StdCtrls, ComCtrls, Controls,
  {$ELSE}
    Windows, Graphics, Forms, Controls, StdCtrls, Buttons, ExtCtrls, ComCtrls,
    Jpeg, Dialogs,
  {$ENDIF}

  ARResStrings, Spin;

type
  TJPEGSetupExportDlg = class(TForm)
    OKBtn: TButton;
    CancelBtn: TButton;
    Bevel1: TBevel;
    ProgressiveCB: TCheckBox;
    CompQualityGB: TGroupBox;
    PerfomanceRG: TRadioGroup;
    CompQualityTB: TTrackBar;
    HighQualityLabel: TLabel;
    LowQualityLabel: TLabel;
    CompQualEdit: TEdit;
    GrayscaleCB: TCheckBox;
    FileNameGB: TGroupBox;
    FileNameEdit: TEdit;
    FileNameBrowseBtn: TButton;
    SaveDialog: TSaveDialog;
    SerNumPosLabel: TLabel;
    eScale: TSpinEdit;    //CD scalefactor
    Label1: TLabel;
    procedure FileNameBrowseBtnClick(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
    procedure CompQualityTBChange(Sender: TObject);
    procedure CompQualEditChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    {$IFDEF CLX}
    {$ELSE}
      function  GetCompressionQuality: TJPEGQualityRange;
      function  GetPerformance: TJPEGPerformance;
      procedure SetCompressionQuality(const Value: TJPEGQualityRange);
      procedure SetPerformance(const Value: TJPEGPerformance);
    {$ENDIF}
    function  GetGrayscale: boolean;
    function  GetProgressiveEncoding: boolean;
    procedure SetGrayscale(const Value: boolean);
    procedure SetProgressiveEncoding(const Value: boolean);
    function  GetFileName: TFileName;
    procedure SetFileName(const Value: TFileName);
  public
    function  Execute:boolean;
  published
    {$IFDEF CLX}
    {$ELSE}
      property CompressionQuality:TJPEGQualityRange read GetCompressionQuality  write SetCompressionQuality;
      property Performance:TJPEGPerformance         read GetPerformance         write SetPerformance;
    {$ENDIF}
    property Grayscale:boolean                    read GetGrayscale           write SetGrayscale;
    property ProgressiveEncoding:boolean          read GetProgressiveEncoding write SetProgressiveEncoding;
    property FileName:TFileName                   read GetFileName            write SetFileName;
  end;

implementation

{$R *.dfm}

{ TJPEGSetupExportDlg }

{$IFDEF CLX}
{$ELSE}
function TJPEGSetupExportDlg.GetCompressionQuality: TJPEGQualityRange;
begin
  Result:=CompQualityTB.Position;
end;
procedure TJPEGSetupExportDlg.SetCompressionQuality(const Value: TJPEGQualityRange);
begin
  CompQualityTB.Position:=Ord(Value);
end;

function TJPEGSetupExportDlg.GetPerformance: TJPEGPerformance;
begin
  Result:=TJPEGPerformance(PerfomanceRG.ItemIndex);
end;
procedure TJPEGSetupExportDlg.SetPerformance(const Value: TJPEGPerformance);
begin
  PerfomanceRG.ItemIndex:=Ord(Value);
end;
{$ENDIF}

function TJPEGSetupExportDlg.GetGrayscale: boolean;
begin
  Result:=GrayscaleCB.Checked;
end;
procedure TJPEGSetupExportDlg.SetGrayscale(const Value: boolean);
begin
  GrayscaleCB.Checked:=Value;
end;

function TJPEGSetupExportDlg.GetProgressiveEncoding: boolean;
begin
  Result:=ProgressiveCB.Checked;
end;

procedure TJPEGSetupExportDlg.SetProgressiveEncoding(const Value: boolean);
begin
  ProgressiveCB.Checked:=Value;
end;

function TJPEGSetupExportDlg.Execute: boolean;
begin
  Result:=ShowModal = mrOk;
end;

function TJPEGSetupExportDlg.GetFileName: TFileName;
begin
  Result:=FileNameEdit.Text;
end;

procedure TJPEGSetupExportDlg.SetFileName(const Value: TFileName);
begin
  FileNameEdit.Text:=Value;
end;

procedure TJPEGSetupExportDlg.FileNameBrowseBtnClick(Sender: TObject);
begin
  SaveDialog.FileName:=FileNameEdit.Text;
  if SaveDialog.Execute then begin
    FileNameEdit.Text:=SaveDialog.FileName;
  end;
end;

procedure TJPEGSetupExportDlg.OKBtnClick(Sender: TObject);
begin
  ModalResult:=mrOK;
end;

procedure TJPEGSetupExportDlg.CompQualityTBChange(Sender: TObject);
begin
  CompQualEdit.Text:=IntToStr(CompQualityTB.Position);
end;

procedure TJPEGSetupExportDlg.CompQualEditChange(Sender: TObject);
begin
  CompQualityTB.Position:=StrToInt(CompQualEdit.Text);
end;

procedure TJPEGSetupExportDlg.FormCreate(Sender: TObject);
begin
  Caption                   :=rsARJPEGSetupExport_Caption;
  ProgressiveCB.Caption     :=rsARJPEGSetupExport_ProgressiveCaption;
  CompQualityGB.Caption     :=rsARJPEGSetupExport_CompQualityCaption;
  HighQualityLabel.Caption  :=rsARJPEGSetupExport_HighQualityLabelCaption;
  LowQualityLabel.Caption   :=rsARJPEGSetupExport_LowQualityLabelCaption;
  PerfomanceRG.Caption      :=rsARJPEGSetupExport_PerfomanceCaption;
  GrayscaleCB.Caption       :=rsARJPEGSetupExport_GrayscaleCaption;
  FileNameGB.Caption        :=rsARJPEGSetupExport_FileNameCaption;
  SerNumPosLabel.Caption    :=rsARJPEGSetupExport_SerNumPosLabelCaption;
  SaveDialog.Filter         :=rsARJPEGSetupExport_SaveDialogFilter;

  PerfomanceRG.Items.Clear;
  PerfomanceRG.Items.Add(rsARJPEGSetupExport_PerfomanceItem1);
  PerfomanceRG.Items.Add(rsARJPEGSetupExport_PerfomanceItem2);

  OKBtn.Caption             :=rsAR_OkCaption;
  CancelBtn.Caption         :=rsAR_CancelCaption;
end;

end.
