unit ARPreviewForm;
//************************************************************//
//                         AND Report                        //
//  Eugene Hrulev (mrHru) Laxian Key, Omsk, Russia 1999,2001 //
//  Oleg Melnikov (muler) Laxian Key, Omsk, Russia 1999,2001 //
//************************************************************//

{$INCLUDE DEFINES.INC}

interface

uses
  Classes, SysUtils,
  {$IFDEF CLX}
    QComCtrls, QStdCtrls, QMask, QControls, QExtCtrls, QButtons, QForms,
  {$ELSE}
    Controls, StdCtrls, ComCtrls, Forms, Buttons, ExtCtrls, Graphics, ToolWin,
  {$ENDIF}

  ARPrinter, ARResStrings;

{$IFDEF CLX}
{$ELSE}
type
  TSEChangedEvent = procedure(Sender:TObject; NewValue:Integer) of object;

  TSpinEdit = class(TCustomControl)
  private
    FEdit       :TEdit;
    FUpDown     :TUpDown;
    FOnChanged  :TSEChangedEvent;
    function    GetIncrement: integer;
    function    GetMax: integer;
    function    GetMin: integer;
    function    GetValue: integer;
    procedure   SetIncrement(const Value: integer);
    procedure   SetMax(const Value: integer);
    procedure   SetMin(const Value: integer);
    procedure   SetValue(const Value: integer);
    procedure   FUpDownOnClick(Sender:TObject; Button:TUDBtnType);
    procedure   FEditChange(Sender: TObject);
  public
    constructor Create(AOwner:TComponent);override;
  published
    property    Increment:integer         read  GetIncrement  write SetIncrement;
		property    Max:integer               read  GetMax        write SetMax;
		property    Min:integer               read  GetMin        write SetMin;
		property    Value:integer             read  GetValue      write SetValue;
    property    OnChanged:TSEChangedEvent read  FOnChanged    write FOnChanged;
  end;
{$ENDIF}

type
  TARPreviewDialog = class(TForm)
    ToolBar: TToolBar;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    StatusBar: TStatusBar;
    CloseButton: TButton;
    PrinterSetupBtn: TSpeedButton;
    FirstPageBtn: TSpeedButton;
    ZoomTo100Btn: TSpeedButton;
    ZoomToWidthBtn: TSpeedButton;
    ZoomToFitBtn: TSpeedButton;
    PrintBtn: TSpeedButton;
    PriorPageBtn: TSpeedButton;
    NextPageBtn: TSpeedButton;
    LastPageBtn: TSpeedButton;
    CloseBtn: TSpeedButton;
    ShowNPABtn: TSpeedButton;
    ToolButton5: TToolButton;
    Panel1: TPanel;
    procedure PrinterSetupBtnClick(Sender: TObject);
    procedure PrintBtnClick(Sender: TObject);
    procedure ZoomToFitBtnClick(Sender: TObject);
    procedure ZoomToWidthBtnClick(Sender: TObject);
    procedure ZoomTo100BtnClick(Sender: TObject);
    procedure FirstPageBtnClick(Sender: TObject);
    procedure PriorPageBtnClick(Sender: TObject);
    procedure NextPageBtnClick(Sender: TObject);
    procedure LastPageBtnClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure CloseBtnClick(Sender: TObject);
    procedure CloseButtonClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ShowNPABtnClick(Sender: TObject);
  private
    function    GetARPrinter:TARPrinter;
    procedure   SetARPrinter(AARPrinter:TARPrinter);
    procedure   UpdateStatusBar;
    procedure   ARPreviewChangeView(Sender: TObject);
    procedure   ZoomUDChanged(Sender:TObject; NewValue:Integer);
  public
    ARPreview   :TARPreview;
    ZoomUD      :TSpinEdit;

    property    ARPrinter:TARPrinter  read  GetARPrinter  write SetARPrinter;
  end;

implementation

{$R *.dfm}

{ TSpinEdit }

{$IFDEF CLX}
{$ELSE}
constructor TSpinEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  Color:=clBtnFace;

  FEdit             :=TEdit.Create(Self);
  FEdit.Parent      :=Self;
  FEdit.Left        :=0;
  FEdit.Top         :=0;
  FEdit.Width       :=35;
  FEdit.OnChange    :=FEditChange;

  FUpDown           :=TUpDown.Create(Self);
  FUpDown.Parent    :=Self;
  FUpDown.Associate :=FEdit;
  FUpDown.OnClick   :=FUpDownOnClick;

  {$IFNDEF D3}
    BorderWidth :=0;
  {$ENDIF}
  Width :=FEdit.Width + FUpDown.Width;
  Height:=FEdit.Height + 2;
end;

procedure TSpinEdit.FEditChange(Sender: TObject);
begin
  if Assigned(FOnChanged) then begin
    FOnChanged(Sender, FUpDown.Position);
  end;
end;

procedure TSpinEdit.FUpDownOnClick(Sender: TObject; Button: TUDBtnType);
begin
  if Assigned(FOnChanged) then begin
    FOnChanged(Sender, FUpDown.Position);
  end;
end;

function TSpinEdit.GetIncrement: integer;
begin
  Result:=FUpDown.Increment;
end;

function TSpinEdit.GetMax: integer;
begin
  Result:=FUpDown.Max;
end;

function TSpinEdit.GetMin: integer;
begin
  Result:=FUpDown.Min;
end;

function TSpinEdit.GetValue: integer;
begin
  Result:=FUpDown.Position;
end;

procedure TSpinEdit.SetIncrement(const Value: integer);
begin
  FUpDown.Increment:=Value;
end;

procedure TSpinEdit.SetMax(const Value: integer);
begin
  FUpDown.Max:=Value;
end;

procedure TSpinEdit.SetMin(const Value: integer);
begin
  FUpDown.Min:=Value;
end;

procedure TSpinEdit.SetValue(const Value: integer);
begin
  FUpDown.Position:=Value;
end;
{$ENDIF}

{ TARPreviewDialog }

procedure TARPreviewDialog.FormResize(Sender: TObject);
begin
  ZoomUD.Value :=ARPreview.Zoom;
end;

procedure TARPreviewDialog.PrinterSetupBtnClick(Sender: TObject);
begin
  ARPreview.ARPrinter.PrinterSetupDialog;
  ARPreview.Invalidate;
  UpdateStatusBar;
end;

procedure TARPreviewDialog.PrintBtnClick(Sender: TObject);
begin
  ARPreview.ARPrinter.PrintDialog;
  ARPreview.Invalidate;
  UpdateStatusBar;
end;

procedure TARPreviewDialog.ZoomToFitBtnClick(Sender: TObject);
begin
  ARPreview.ZoomToFit;
  ZoomUD.Value:=ARPreview.Zoom;
end;

procedure TARPreviewDialog.ZoomToWidthBtnClick(Sender: TObject);
begin
  ARPreview.ZoomToWidth;
  ZoomUD.Value:=ARPreview.Zoom;
end;

procedure TARPreviewDialog.ZoomTo100BtnClick(Sender: TObject);
begin
  ARPreview.Zoom:=100;
  ZoomUD.Value:=ARPreview.Zoom;
end;

procedure TARPreviewDialog.FirstPageBtnClick(Sender: TObject);
begin
  ARPreview.CurrPage:=1;
  UpdateStatusBar;
end;

procedure TARPreviewDialog.PriorPageBtnClick(Sender: TObject);
begin
  ARPreview.CurrPage:=ARPreview.CurrPage - 1;
  UpdateStatusBar;
end;

procedure TARPreviewDialog.NextPageBtnClick(Sender: TObject);
begin
  ARPreview.CurrPage:=ARPreview.CurrPage + 1;
  UpdateStatusBar;
end;

procedure TARPreviewDialog.LastPageBtnClick(Sender: TObject);
begin
  ARPreview.CurrPage:=100000;
  UpdateStatusBar;
end;

function    TARPreviewDialog.GetARPrinter:TARPrinter;
begin
  Result:=ARPreview.ARPrinter
end;

procedure    TARPreviewDialog.SetARPrinter(AARPrinter:TARPrinter);
begin
  ARPreview.ARPrinter:=AARPrinter;
end;

procedure   TARPreviewDialog.CloseBtnClick(Sender: TObject);
begin
  ModalResult:=mrOK;
end;

procedure    TARPreviewDialog.UpdateStatusBar;
begin
  StatusBar.Panels[0].Text:=Format(rsARPreviewForm_Pages, [ARPreview.CurrPage, ARPreview.PageCount]);
end;

procedure TARPreviewDialog.CloseButtonClick(Sender: TObject);
begin
  ModalResult:=mrOK;
end;

procedure TARPreviewDialog.ARPreviewChangeView(Sender: TObject);
begin
  ZoomUD.Value:=ARPreview.Zoom;
  UpdateStatusBar;
end;

procedure TARPreviewDialog.FormCreate(Sender: TObject);
begin
  ARPreview:=TARPreview.Create(Self);
  ARPreview.Parent:=Self;
  ARPreview.Align:=alClient;

  ZoomUD          :=TSpinEdit.Create(Self);
  ZoomUD.Parent   :=Panel1;
  ZoomUD.Width    :=50;
//  ZoomUD.Left     :=180;
  ZoomUD.Min      :=ARPreviewZoomMin;
  ZoomUD.Max      :=ARPreviewZoomMax;
  ZoomUD.Increment:=10;

  ZoomUD.OnChanged:=ZoomUDChanged;

  ARPreview.OnChangeView:=ARPreviewChangeView;

  Caption               :=rsARPreviewForm_Caption;
  PrinterSetupBtn.Hint  :=rsARPreviewForm_PrinterSetupBtn;
  FirstPageBtn.Hint     :=rsARPreviewForm_FirstPageBtn;
  ZoomTo100Btn.Hint     :=rsARPreviewForm_ZoomTo100Btn;
  ZoomToWidthBtn.Hint   :=rsARPreviewForm_ZoomToWidthBtn;
  ZoomToFitBtn.Hint     :=rsARPreviewForm_ZoomToFitBtn;
  PrintBtn.Hint         :=rsARPreviewForm_PrintBtn;
  PriorPageBtn.Hint     :=rsARPreviewForm_PriorPageBtn;
  NextPageBtn.Hint      :=rsARPreviewForm_NextPageBtn;
  LastPageBtn.Hint      :=rsARPreviewForm_LastPageBtn;
  CloseBtn.Hint         :=rsAR_CloseCaption;
  ZoomUD.Hint           :=rsARPreviewForm_Zoom;
  ShowNPABtn.Hint       :=rsARPreviewForm_ShowNPA;
end;

procedure TARPreviewDialog.FormShow(Sender: TObject);
begin
  ARPreview.CurrPage:=1;
  UpdateStatusBar;
  ARPreview.Zoom:=100;
end;

procedure TARPreviewDialog.ShowNPABtnClick(Sender: TObject);
begin
  ARPreview.ShowNonPrintingArea:=ShowNPABtn.Down;
end;

procedure TARPreviewDialog.ZoomUDChanged(Sender:TObject; NewValue:Integer);
begin
  ARPreview.Zoom:=ZoomUD.Value;
end;

end.


