unit ARPrinter;
//************************************************************//
//                         AND Report                        //
//  Eugene Hrulev (mrHru) Laxian Key, Omsk, Russia 1999,2001 //
//  Oleg Melnikov (muler) Laxian Key, Omsk, Russia 1999,2001 //
//************************************************************//

// changes
// sa: Sertac Akyuz, mailto:prosoft@turk.net // 11.06.2001


{$INCLUDE DEFINES.INC}

interface

uses
  Classes, Math, SysUtils,
  {$IFDEF CLX}
    QGraphics, QPrinters, QExtCtrls, QControls, QForms, QDialogs, Types, Qt,
  {$ELSE}
    Graphics, Printers, ExtCtrls, Controls, Forms, Dialogs, Windows, Messages,
  {$ENDIF}

  ARUtils, ARScrollingWidget;


type
  TARTemplate = class(TAIStringList)
  end;

{$IFDEF CLX}
type
  TARPaperSizeCLX = TPageSize; // from QPrinter;

  //windows page types
const
  DMPAPER_LETTER                  = 1;  { Letter 8 12 x 11 in                   }
  DMPAPER_FIRST                   = DMPAPER_LETTER;
  DMPAPER_LETTERSMALL             = 2;  { Letter Small 8 12 x 11 in             }
  DMPAPER_TABLOID                 = 3;  { Tabloid 11 x 17 in                    }
  DMPAPER_LEDGER                  = 4;  { Ledger 17 x 11 in                     }
  DMPAPER_LEGAL                   = 5;  { Legal 8 12 x 14 in                    }
  DMPAPER_STATEMENT               = 6;  { Statement 5 12 x 8 12 in              }
  DMPAPER_EXECUTIVE               = 7;  { Executive 7 14 x 10 12 in             }
  DMPAPER_A3                      = 8;  { A3 297 x 420 mm                       }
  DMPAPER_A4                      = 9;  { A4 210 x 297 mm                       }
  DMPAPER_A4SMALL                 = 10; { A4 Small 210 x 297 mm                 }
  DMPAPER_A5                      = 11; { A5 148 x 210 mm                       }
  DMPAPER_B4                      = 12; { B4 (JIS) 250 x 354                    }
  DMPAPER_B5                      = 13; { B5 (JIS) 182 x 257 mm                 }
  DMPAPER_FOLIO                   = 14; { Folio 8 12 x 13 in                    }
  DMPAPER_QUARTO                  = 15; { Quarto 215 x 275 mm                   }
  DMPAPER_10X14                   = 16; { 10x14 in                              }
  DMPAPER_11X17                   = 17; { 11x17 in                              }
  DMPAPER_NOTE                    = 18; { Note 8 12 x 11 in                     }
  DMPAPER_ENV_9                   = 19; { Envelope #9 3 78 x 8 78               }
  DMPAPER_ENV_10                  = 20; { Envelope #10 4 18 x 9 12              }
  DMPAPER_ENV_11                  = 21; { Envelope #11 4 12 x 10 38             }
  DMPAPER_ENV_12                  = 22; { Envelope #12 4 \276 x 11              }
  DMPAPER_ENV_14                  = 23; { Envelope #14 5 x 11 12                }
  DMPAPER_CSHEET                  = 24; { C size sheet                          }
  DMPAPER_DSHEET                  = 25; { D size sheet                          }
  DMPAPER_ESHEET                  = 26; { E size sheet                          }
  DMPAPER_ENV_DL                  = 27; { Envelope DL 110 x 220mm               }
  DMPAPER_ENV_C5                  = 28; { Envelope C5 162 x 229 mm              }
  DMPAPER_ENV_C3                  = 29; { Envelope C3  324 x 458 mm             }
  DMPAPER_ENV_C4                  = 30; { Envelope C4  229 x 324 mm             }
  DMPAPER_ENV_C6                  = 31; { Envelope C6  114 x 162 mm             }
  DMPAPER_ENV_C65                 = 32; { Envelope C65 114 x 229 mm             }
  DMPAPER_ENV_B4                  = 33; { Envelope B4  250 x 353 mm             }
  DMPAPER_ENV_B5                  = 34; { Envelope B5  176 x 250 mm             }
  DMPAPER_ENV_B6                  = 35; { Envelope B6  176 x 125 mm             }
  DMPAPER_ENV_ITALY               = 36; { Envelope 110 x 230 mm                 }
  DMPAPER_ENV_MONARCH             = 37; { Envelope Monarch 3.875 x 7.5 in       }
  DMPAPER_ENV_PERSONAL            = 38; { 6 34 Envelope 3 58 x 6 12 in          }
  DMPAPER_FANFOLD_US              = 39; { US Std Fanfold 14 78 x 11 in          }
  DMPAPER_FANFOLD_STD_GERMAN      = 40; { German Std Fanfold 8 12 x 12 in       }
  DMPAPER_FANFOLD_LGL_GERMAN      = 41; { German Legal Fanfold 8 12 x 13 in     }
  DMPAPER_ISO_B4                  = 42; { B4 (ISO) 250 x 353 mm                 }
  DMPAPER_JAPANESE_POSTCARD       = 43; { Japanese Postcard 100 x 148 mm        }
  DMPAPER_9X11                    = 44; { 9 x 11 in                             }
  DMPAPER_10X11                   = 45; { 10 x 11 in                            }
  DMPAPER_15X11                   = 46; { 15 x 11 in                            }
  DMPAPER_ENV_INVITE              = 47; { Envelope Invite 220 x 220 mm          }
  DMPAPER_RESERVED_48             = 48; { RESERVED--DO NOT USE                  }
  DMPAPER_RESERVED_49             = 49; { RESERVED--DO NOT USE                  }
  DMPAPER_LETTER_EXTRA            = 50; { Letter Extra 9 \275 x 12 in           }
  DMPAPER_LEGAL_EXTRA             = 51; { Legal Extra 9 \275 x 15 in            }
  DMPAPER_TABLOID_EXTRA           = 52; { Tabloid Extra 11.69 x 18 in           }
  DMPAPER_A4_EXTRA                = 53; { A4 Extra 9.27 x 12.69 in              }
  DMPAPER_LETTER_TRANSVERSE       = 54; { Letter Transverse 8 \275 x 11 in      }
  DMPAPER_A4_TRANSVERSE           = 55; { A4 Transverse 210 x 297 mm            }
  DMPAPER_LETTER_EXTRA_TRANSVERSE = 56; { Letter Extra Transverse 9\275 x 12 in }
  DMPAPER_A_PLUS                  = 57; { SuperASuperAA4 227 x 356 mm           }
  DMPAPER_B_PLUS                  = 58; { SuperBSuperBA3 305 x 487 mm           }
  DMPAPER_LETTER_PLUS             = 59; { Letter Plus 8.5 x 12.69 in            }
  DMPAPER_A4_PLUS                 = 60; { A4 Plus 210 x 330 mm                  }
  DMPAPER_A5_TRANSVERSE           = 61; { A5 Transverse 148 x 210 mm            }
  DMPAPER_B5_TRANSVERSE           = 62; { B5 (JIS) Transverse 182 x 257 mm      }
  DMPAPER_A3_EXTRA                = 63; { A3 Extra 322 x 445 mm                 }
  DMPAPER_A5_EXTRA                = 64; { A5 Extra 174 x 235 mm                 }
  DMPAPER_B5_EXTRA                = 65; { B5 (ISO) Extra 201 x 276 mm           }
  DMPAPER_A2                      = 66; { A2 420 x 594 mm                       }
  DMPAPER_A3_TRANSVERSE           = 67; { A3 Transverse 297 x 420 mm            }
  DMPAPER_A3_EXTRA_TRANSVERSE     = 68; { A3 Extra Transverse 322 x 445 mm      }
{$ELSE}
type
  TARPaperSizeCLX = ( // from QPrinter;
    psA4,     psB5,       psLetter, psLegal,  psExecutive,  psA0,   psA1,
    psA2,     psA3,       psA5,     psA6,     psA7,         psA8,   psA9,
    psB0,     psB1,       psB10,    psB2,     psB3,         psB4,   psB6,
    psB7,     psB8,       psB9,     psC5E,    psComm10E,    psDLE,  psFolio,
    psLedger, psTabloid,  psNPageSize
  );
{$ENDIF}

const
  DMPAPER_CUSTOM                  = 0;  { 0 x 0 }

type
  TARPaperSizeWin = integer;

type
  TARPageSizeDef = record
    Name    :string;
    WinType :integer;
    CLXType :integer;
    Width   :integer; //mm
    Height  :integer; //mm
  end;

const
  cARPageSizeDefs:array [0..83] of TARPageSizeDef = (
    (Name:'------';                   WinType:DMPAPER_CUSTOM;                   CLXType:Ord(psNPageSize); Width:   0;  Height:    0),
    (Name:'Letter';                   WinType:DMPAPER_LETTER;                   CLXType:Ord(psLetter);    Width: 216;  Height:  279),
    (Name:'Letter Small';             WinType:DMPAPER_LETTERSMALL;              CLXType:Ord(psLetter);    Width: 216;  Height:  279),
    (Name:'Tabloid';                  WinType:DMPAPER_TABLOID;                  CLXType:Ord(psTabloid);   Width: 279;  Height:  432),
    (Name:'Ledger';                   WinType:DMPAPER_LEDGER;                   CLXType:Ord(psLedger);    Width: 432;  Height:  279),
    (Name:'Legal';                    WinType:DMPAPER_LEGAL;                    CLXType:Ord(psLegal);     Width: 216;  Height:  356),
    (Name:'Statement';                WinType:DMPAPER_STATEMENT;                CLXType:-1;               Width: 140;  Height:  216),
    (Name:'Executive';                WinType:DMPAPER_EXECUTIVE;                CLXType:Ord(psExecutive); Width: 184;  Height:  267),
    (Name:'A3';                       WinType:DMPAPER_A3;                       CLXType:Ord(psA3);        Width: 297;  Height:  420),
    (Name:'A4';                       WinType:DMPAPER_A4;                       CLXType:Ord(psA4);        Width: 210;  Height:  297),
    (Name:'A4 Small';                 WinType:DMPAPER_A4SMALL;                  CLXType:-1;               Width: 210;  Height:  297),
    (Name:'A5';                       WinType:DMPAPER_A5;                       CLXType:Ord(psA5);        Width: 148;  Height:  210),
    (Name:'B4 (JIS)';                 WinType:DMPAPER_B4;                       CLXType:Ord(psB4);        Width: 257;  Height:  364),
    (Name:'B5 (JIS)';                 WinType:DMPAPER_B5;                       CLXType:Ord(psB5);        Width: 182;  Height:  257),
    (Name:'Folio';                    WinType:DMPAPER_FOLIO;                    CLXType:Ord(psFolio);     Width: 216;  Height:  330),
    (Name:'Quarto';                   WinType:DMPAPER_QUARTO;                   CLXType:-1;               Width: 215;  Height:  275),
    (Name:'10x14';                    WinType:DMPAPER_10X14;                    CLXType:-1;               Width: 254;  Height:  356),
    (Name:'11x17';                    WinType:DMPAPER_11X17;                    CLXType:-1;               Width: 279;  Height:  432),
    (Name:'Note';                     WinType:DMPAPER_NOTE;                     CLXType:-1;               Width: 216;  Height:  279),
    (Name:'Envelope #9';              WinType:DMPAPER_ENV_9;                    CLXType:-1;               Width:  98;  Height:  225),
    (Name:'Envelope #10';             WinType:DMPAPER_ENV_10;                   CLXType:Ord(psComm10E);   Width: 105;  Height:  241),
    (Name:'Envelope #11';             WinType:DMPAPER_ENV_11;                   CLXType:-1;               Width: 114;  Height:  263),
    (Name:'Envelope #12';             WinType:DMPAPER_ENV_12;                   CLXType:-1;               Width: 121;  Height:  279),
    (Name:'Envelope #14';             WinType:DMPAPER_ENV_14;                   CLXType:-1;               Width: 127;  Height:  292),
    (Name:'CSheet';                   WinType:DMPAPER_CSHEET;                   CLXType:-1;               Width: 432;  Height:  559),
    (Name:'DSheet';                   WinType:DMPAPER_DSHEET;                   CLXType:-1;               Width: 559;  Height:  864),
    (Name:'ESheet';                   WinType:DMPAPER_ESHEET;                   CLXType:-1;               Width: 864;  Height: 1118),
    (Name:'Envelope DL';              WinType:DMPAPER_ENV_DL;                   CLXType:Ord(psDLE);       Width: 110;  Height:  220),
    (Name:'Envelope C5';              WinType:DMPAPER_ENV_C5;                   CLXType:Ord(psC5E);       Width: 162;  Height:  229),
    (Name:'Envelope C3';              WinType:DMPAPER_ENV_C3;                   CLXType:-1;               Width: 324;  Height:  458),
    (Name:'Envelope C4';              WinType:DMPAPER_ENV_C4;                   CLXType:-1;               Width: 229;  Height:  324),
    (Name:'Envelope C6';              WinType:DMPAPER_ENV_C6;                   CLXType:-1;               Width: 114;  Height:  162),
    (Name:'Envelope C65';             WinType:DMPAPER_ENV_C65;                  CLXType:-1;               Width: 114;  Height:  229),
    (Name:'Envelope B4';              WinType:DMPAPER_ENV_B4;                   CLXType:-1;               Width: 250;  Height:  353),
    (Name:'Envelope B5';              WinType:DMPAPER_ENV_B5;                   CLXType:-1;               Width: 176;  Height:  250),
    (Name:'Envelope B6';              WinType:DMPAPER_ENV_B6;                   CLXType:-1;               Width: 176;  Height:  125),
    (Name:'Envelope';                 WinType:DMPAPER_ENV_ITALY;                CLXType:-1;               Width: 110;  Height:  230),
    (Name:'Envelope Monarch';         WinType:DMPAPER_ENV_MONARCH;              CLXType:-1;               Width:  98;  Height:  190),
    (Name:'Envelope personal';        WinType:DMPAPER_ENV_PERSONAL;             CLXType:-1;               Width:  92;  Height:  165),
    (Name:'US Std Fanfold';           WinType:DMPAPER_FANFOLD_US;               CLXType:-1;               Width: 378;  Height:  279),
    (Name:'German Std Fanfold';       WinType:DMPAPER_FANFOLD_STD_GERMAN;       CLXType:-1;               Width: 216;  Height:  305),
    (Name:'German Legal Fanfold';     WinType:DMPAPER_FANFOLD_LGL_GERMAN;       CLXType:-1;               Width: 216;  Height:  330),
    (Name:'B4 (ISO)';                 WinType:DMPAPER_ISO_B4;                   CLXType:-1;               Width: 250;  Height:  353),
    (Name:'Japanese Postcard';        WinType:DMPAPER_JAPANESE_POSTCARD;        CLXType:-1;               Width: 100;  Height:  148),
    (Name:'9 x 11';                   WinType:DMPAPER_9X11;                     CLXType:-1;               Width: 229;  Height:  279),
    (Name:'10 x 11';                  WinType:DMPAPER_10X11;                    CLXType:-1;               Width: 254;  Height:  279),
    (Name:'15 x 11';                  WinType:DMPAPER_15X11;                    CLXType:-1;               Width: 381;  Height:  279),
    (Name:'Envelope Invite';          WinType:DMPAPER_ENV_INVITE;               CLXType:-1;               Width: 220;  Height:  220),
    (Name:'RESERVED--DO NOT USE';     WinType:DMPAPER_RESERVED_48;              CLXType:-1;               Width:   0;  Height:    0),
    (Name:'RESERVED--DO NOT USE';     WinType:DMPAPER_RESERVED_49;              CLXType:-1;               Width:   0;  Height:    0),
    (Name:'Letter Extra 9';           WinType:DMPAPER_LETTER_EXTRA;             CLXType:-1;               Width: 241;  Height:  305),
    (Name:'Legal Extra 9';            WinType:DMPAPER_LEGAL_EXTRA;              CLXType:-1;               Width: 241;  Height:  381),
    (Name:'Tabloid Extra';            WinType:DMPAPER_TABLOID_EXTRA;            CLXType:-1;               Width: 305;  Height:  457),
    (Name:'A4 Extra';                 WinType:DMPAPER_A4_EXTRA;                 CLXType:-1;               Width: 235;  Height:  322),
    (Name:'Letter Transverse';        WinType:DMPAPER_LETTER_TRANSVERSE;        CLXType:-1;               Width: 216;  Height:  279),
    (Name:'A4 Transverse';            WinType:DMPAPER_A4_TRANSVERSE;            CLXType:-1;               Width: 210;  Height:  297),
    (Name:'Letter Extra Transverse';  WinType:DMPAPER_LETTER_EXTRA_TRANSVERSE;  CLXType:-1;               Width: 241;  Height:  305),
    (Name:'SuperASuperAA4';           WinType:DMPAPER_A_PLUS;                   CLXType:-1;               Width: 227;  Height:  356),
    (Name:'SuperBSuperBA3';           WinType:DMPAPER_B_PLUS;                   CLXType:-1;               Width: 305;  Height:  487),
    (Name:'Letter Plus';              WinType:DMPAPER_LETTER_PLUS;              CLXType:-1;               Width: 216;  Height:  322),
    (Name:'A4 Plus';                  WinType:DMPAPER_A4_PLUS;                  CLXType:-1;               Width: 210;  Height:  330),
    (Name:'A5 Transverse';            WinType:DMPAPER_A5_TRANSVERSE;            CLXType:-1;               Width: 148;  Height:  210),
    (Name:'B5 (JIS) Transverse';      WinType:DMPAPER_B5_TRANSVERSE;            CLXType:-1;               Width: 182;  Height:  257),
    (Name:'A3 Extra';                 WinType:DMPAPER_A3_EXTRA;                 CLXType:-1;               Width: 322;  Height:  445),
    (Name:'A5 Extra';                 WinType:DMPAPER_A5_EXTRA;                 CLXType:-1;               Width: 174;  Height:  235),
    (Name:'B5 (ISO) Extra';           WinType:DMPAPER_B5_EXTRA;                 CLXType:-1;               Width: 201;  Height:  276),
    (Name:'A2';                       WinType:DMPAPER_A2;                       CLXType:Ord(psA2);        Width: 420;  Height:  594),
    (Name:'A3';                       WinType:DMPAPER_A3_TRANSVERSE;            CLXType:Ord(psA3);        Width: 297;  Height:  420),
    (Name:'A3 Extra Transverse';      WinType:DMPAPER_A3_EXTRA_TRANSVERSE;      CLXType:-1;               Width: 322;  Height:  445),

    (Name:'A0';                       WinType:-1;                               CLXType:Ord(psA0);        Width: 841;  Height: 1189),
    (Name:'A1';                       WinType:-1;                               CLXType:Ord(psA1);        Width: 594;  Height:  841),
    (Name:'A6';                       WinType:-1;                               CLXType:Ord(psA6);        Width: 105;  Height:  148),
    (Name:'A7';                       WinType:-1;                               CLXType:Ord(psA7);        Width:  74;  Height:  105),
    (Name:'A8';                       WinType:-1;                               CLXType:Ord(psA8);        Width:  52;  Height:   74),
    (Name:'A9';                       WinType:-1;                               CLXType:Ord(psA9);        Width:  37;  Height:   52),
    (Name:'B0';                       WinType:-1;                               CLXType:Ord(psB0);        Width:1030;  Height: 1456),
    (Name:'B1';                       WinType:-1;                               CLXType:Ord(psB1);        Width: 728;  Height: 1030),
    (Name:'B10';                      WinType:-1;                               CLXType:Ord(psB10);       Width:  32;  Height:   45),
    (Name:'B2';                       WinType:-1;                               CLXType:Ord(psB2);        Width: 515;  Height:  728),
    (Name:'B3';                       WinType:-1;                               CLXType:Ord(psB3);        Width: 364;  Height:  515),
    (Name:'B6';                       WinType:-1;                               CLXType:Ord(psB6);        Width: 128;  Height:  182),
    (Name:'B7';                       WinType:-1;                               CLXType:Ord(psB7);        Width:  91;  Height:  128),
    (Name:'B8';                       WinType:-1;                               CLXType:Ord(psB8);        Width:  64;  Height:   91),
    (Name:'B9';                       WinType:-1;                               CLXType:Ord(psB9);        Width:  45;  Height:   64)
  );

type
  TARPageSize = class(TPersistent)
  private
    FARTemplate     :TARTemplate;
    FCurPageSizeIdx :integer;
    FHeight         :extended;
    FWidth          :extended;
    FName           :string;
    FWinPageSize    :TARPaperSizeWin;
    FCLXPageSize    :TARPaperSizeCLX;
    FOrientation    :TPrinterOrientation;
    FPageNames      :TStringList;

    procedure   ARTemplateChange(Sender:TObject);
    procedure   SetHeight(const Value: extended);
    procedure   SetName(const Value: string);
    procedure   SetWidth(const Value: extended);
    procedure   SetCLXPageSize(const Value:TARPaperSizeCLX);
    procedure   SetWinPageSize(const Value:TARPaperSizeWin);
    procedure   SetOrientation(const Value:TPrinterOrientation);
    function    GetPageNames:TStrings;
    function    GetARTemplate:TARTemplate;
    procedure   SetARTemplate(const AARTemplate:TARTemplate);
  public
    property    Name:string                     read  FName         write SetName;
    property    Height:extended                 read  FHeight       write SetHeight;  //mm*10
    property    Width:extended                  read  FWidth        write SetWidth;   //mm*10
    property    PageNames:TStrings              read  GetPageNames;
    property    ARTemplate:TARTemplate          read  GetARTemplate write SetARTemplate;
    property    WinPageSize:TARPaperSizeWin     read  FWinPageSize  write SetWinPageSize;
    property    CLXPageSize:TARPaperSizeCLX     read  FCLXPageSize  write SetCLXPageSize;
    property    Orientation:TPrinterOrientation read  FOrientation  write SetOrientation;

    procedure   Assign(Source: TPersistent);override;

    constructor Create;
    destructor  Destroy;override;
  end;

type
  TZoomMode = (zmFixed, zmToWidth, zmToFit);

const
  ARPreviewZoomMin = 10;
  ARPreviewZoomMax = 500;

type
{$IFDEF CLX}
    TMetafile = class(TDrawing)
    end;
{$ENDIF}

  TARPrinter = class(TComponent)
  private
    PageList    :TAIList;

    FCanvas     :TCanvas;
    FCurPage    :integer;

//    FPaperSize  :TARPaperSize;
    FPageSize   :TARPageSize;

    FFromPage   :integer;
    FToPage     :integer;
    FCopies     :integer;
    FOrientation:TPrinterOrientation;
    FTitle      :string;

    FPPIX       :integer;
    FPPIY       :integer;
    FLeftMargin :integer;
    FTopMargin  :integer;
    FPageWidth  :integer;
    FPageHeight :integer;
    FPrintingWidth  :integer;
    FPrintingHeight :integer;

    FOnPrinterChange:TNotifyEvent;

//    function    GetCanvas:TCanvas;
    function    GetMetafile:TMetafile;

    function    GetCopies:integer;
    procedure   SetCopies(ACopies:integer);

    function    GetCollation:boolean;

    function    GetOrientation:TPrinterOrientation;
    procedure   SetOrientation(AOrientation:TPrinterOrientation);

    function    GetTitle:string;
    procedure   SetTitle(ATitle:string);
    procedure   SetCurPage(ACurPage:integer);
    function    GetPageCount:integer;

    function    GetPaperWidth:integer;
    function    GetPaperHeight:integer;
    function    GetPrintWidth:integer;
    function    GetPrintHeight:integer;

    function    GetLeftMargin:integer;
    function    GetTopMargin :integer;
    function    GetRightMargin:integer;
    function    GetBottomMargin :integer;

    procedure   SetPageSize(const Value: TARPageSize);
    procedure   LoadPrinterSetting;
    procedure   SavePrinterSetting;

    function    GetFromPage: integer;
    procedure   SetFromPage(const Value: integer);
    function    GetToPage: integer;
    procedure   SetToPage(const Value: integer);
  public
    function    RefreshPrinter:boolean;

    property    PageSize:TARPageSize            read  FPageSize       write SetPageSize;

    property    PrinterPPIX           :integer  read  FPPIX;
    property    PrinterPPIY           :integer  read  FPPIY;
    property    PrinterLeftMargin     :integer  read  FLeftMargin;
    property    PrinterTopMargin      :integer  read  FTopMargin;
    property    PrinterPrintingWidth  :integer  read  FPrintingWidth;
    property    PrinterPrintingHeight :integer  read  FPrintingHeight;

    property    PaperHeight:integer             read  GetPaperHeight;
    property    PaperWidth:integer              read  GetPaperWidth;
    property    PrintHeight:integer             read  GetPrintHeight;
    property    PrintWidth:integer              read  GetPrintWidth;
    property    LeftMargin:integer              read  GetLeftMargin;
    property    TopMargin:integer               read  GetTopMargin;
    property    RightMargin:integer             read  GetRightMargin;
    property    BottomMargin:integer            read  GetBottomMargin;

    property    CurrPage:integer                read  FCurPage        write SetCurPage;
    property    PageCount:integer               read  GetPageCount;
    property    Canvas:TCanvas                  read  FCanvas;
    property    Metafile:TMetafile              read  GetMetafile;

    property    Title:string                    read  GetTitle        write SetTitle;

    property    FromPage:integer                read  GetFromPage     write SetFromPage;
    property    ToPage:integer                  read  GetToPage       write SetToPage;
    property    Copies:integer                  read  GetCopies       write SetCopies;
    property    Orientation:TPrinterOrientation read  GetOrientation  write SetOrientation;
    property    Collation:boolean               read  GetCollation;

    procedure   BeginDoc;
    procedure   NewPage;
    procedure   EndDoc;
    procedure   Abort;

    procedure   Preview;
    procedure   Print;
    procedure   PrintDialog;
    procedure   PrinterSetupDialog;

    property    OnPrinterChange:TNotifyEvent  read  FOnPrinterChange  write FOnPrinterChange;
    constructor Create(AOwner:TComponent);override;
    destructor  Destroy;override;
  end;

{$IFDEF CLX}
  TARPreview  = class(TARScrollingWidget)
{$ELSE}
  TARPreview  = class(TPanel)
{$ENDIF}
  private
    FScrollUpdate :boolean;

    FZoom       :integer;

    FARPrinter  :TARPrinter;

    FZoomMode   :TZoomMode;

    FOnChangeView:TNotifyEvent;
    FShowNonPrintingArea:boolean;

    procedure   SetShowNonPrintingArea(AShowNonPrintingArea:boolean);

    procedure   SetARPrinter(AARPrinter:TARPrinter);

    function    GetCurPage:integer;
    procedure   SetCurPage(ACurPage:integer);
    function    GetPageCount:integer;

    function    GetViewPageDX:integer;
    function    GetViewPageDY:integer;

    procedure   SetZoom(AZoom:integer);

    procedure   SetTopY(ATopY:integer);
    procedure   SetLeftX(ALeftX:integer);

    {$IFDEF CLX}
    {$ELSE}
      function    GetVSTrackPos:integer;
      function    GetVSPos:integer;
      procedure   SetVSPos(AVSPos:integer);
      function    GetVSMax:integer;
      procedure   SetVSMax(AVSMax:integer);
      function    GetVSPage:integer;
      procedure   SetVSPage(AVSPage:integer);

      function    GetHSTrackPos:integer;
      function    GetHSPos:integer;
      procedure   SetHSPos(AHSPos:integer);
      function    GetHSMax:integer;
      procedure   SetHSMax(AHSMax:integer);
      function    GetHSPage:integer;
      procedure   SetHSPage(AHSPage:integer);
    {$ENDIF}

    procedure   SetPageDX(APageDX:integer);
    procedure   SetPageDY(APageDY:integer);

    {$IFNDEF D3}
      procedure   MouseWheelUp(Sender:TObject; Shift:TShiftState; MousePos:TPoint; var Handled:boolean);
      procedure   MouseWheelDown(Sender:TObject; Shift:TShiftState; MousePos:TPoint; var Handled:boolean);
    {$ENDIF}
  private
    FTopY     :integer;
    FLeftX    :integer;
    FPageDX   :integer;
    FPageDY   :integer;
  protected

    {$IFDEF CLX}
      procedure   Resize;override;
    {$ELSE}
      procedure   CreateParams(var p: TCreateParams);override;
      procedure   WMGetDlgCode(var Message: TWMGetDlgCode); message WM_GETDLGCODE;
      procedure   WMVScroll   (var Message: TWMScroll);     message WM_VSCROLL;
      procedure   WMHScroll   (var Message: TWMScroll);     message WM_HSCROLL;
      procedure   WMSize      (var Message: TWMSize);       message WM_SIZE;
      procedure   WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;

      property    VSTrackPos:integer    read  GetVSTrackPos;
      property    VSPos :integer        read  GetVSPos     write SetVSPos;
      property    VSMax :integer        read  GetVSMax     write SetVSMax;
      property    VSPage:integer        read  GetVSPage    write SetVSPage;

      property    HSTrackPos:integer    read  GetHSTrackPos;
      property    HSPos :integer        read  GetHSPos     write SetHSPos;
      property    HSMax :integer        read  GetHSMax     write SetHSMax;
      property    HSPage:integer        read  GetHSPage    write SetHSPage;
  {$ENDIF}

    procedure   Paint;override;

    procedure   KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure   MouseMove(Shift: TShiftState; X, Y: Integer);override;
    procedure   MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);override;

    procedure   UpdateScrolls;

    property    ViewPageDX:integer          read GetViewPageDX;
    property    ViewPageDY:integer          read GetViewPageDY;
  public
    property    ARPrinter:TARPrinter        read FARPrinter           write SetARPrinter;

    property    TopY :integer               read FTopY                write SetTopY;
    property    LeftX:integer               read FLeftX               write SetLeftX;

    property    PageDX:integer              read FPageDX              write SetPageDX;
    property    PageDY:integer              read FPageDY              write SetPageDY;

    property    PageCount:integer           read GetPageCount;

    procedure   ZoomToFit;
    procedure   ZoomToWidth;

    constructor Create(AOwner:TComponent);override;
    destructor  Destroy;override;
  published
    property    Zoom:integer                read FZoom                write SetZoom default 100;
    property    CurrPage:integer            read GetCurPage           write SetCurPage;
    property    ShowNonPrintingArea:boolean read FShowNonprintingArea write SetShowNonprintingArea;
    property    OnChangeView:TNotifyEvent   read  FOnChangeView       write FOnChangeView;
  end;

procedure Register;

implementation
{$R *.res}

uses ARPreviewForm;

procedure Register;
begin
  RegisterComponents('ARComponents', [TARPreview]);
end;

function    PrinterInstalled:boolean;
begin
  Result:=Printer.Printers.Count > 0;
end;

{$IFDEF CLX}
function    qph:QPrinterH;
begin
  Result:=QPrinterH(Printer.PrintAdapter.Handle);
end;
{$ENDIF}
///////////////////////////////////////////////////////////////////////
{ TARPrinter }

constructor  TARPrinter.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  PageList:=TAIList.Create('TARPrinter PageList');
  FCurPage:=0;
  FPPIX:=Screen.PixelsPerInch;
  FPPIY:=Screen.PixelsPerInch;
  FPageSize :=TARPageSize.Create;
end;

destructor  TARPrinter.Destroy;
begin
  Abort;

  FPageSize.Free;
  PageList.Free;
  inherited;
end;

function      TARPrinter.RefreshPrinter:boolean;
  {$IFDEF CLX}
  {$ELSE}
var
    DummyInt      :integer;
    NewPPIX       :integer;
    NewPPIY       :integer;
    NewLeftMargin :integer;
    NewTopMargin  :integer;
    NewPageWidth  :integer;
    NewPageHeight :integer;
  {$ENDIF}
begin
  Result:=false;

//  FPPIX       :=Screen.PixelsPerInch;
//  FPPIY       :=Screen.PixelsPerInch;
  if Printer.Printers.Count = 0 then begin
    FPPIX       :=Screen.PixelsPerInch;
    FPPIY       :=Screen.PixelsPerInch;
    FLeftMargin :=0;
    FTopMargin  :=0;
    PageSize.Orientation:=Orientation;
    FPageWidth  :=Round(PageSize.Width  / 10.0 / 25.4 * FPPIX);
    FPageHeight :=Round(PageSize.Height / 10.0 / 25.4 * FPPIY);
  end else begin
    if FOrientation <> Printer.Orientation then begin
      Orientation :=Printer.Orientation;
      Result      :=true;
    end;
    {$IFDEF CLX}
      if PageSize.CLXPageSize <> Printer.PrintAdapter.PageSize then begin
        PageSize.CLXPageSize:=Printer.PrintAdapter.PageSize;
        Result:=true;
      end;
      if FPPIX <> printer.YDPI then begin
        FPPIX :=printer.YDPI;
        Result:=true;
      end;
      if FPPIY <> printer.XDPI then begin
        FPPIY :=printer.XDPI;
        Result:=true;
      end;
      if FLeftMargin <> printer.Margins.cx then begin
        FLeftMargin :=printer.Margins.cx;
        Result:=true;
      end;
      if FTopMargin <> printer.Margins.cy then begin
        FTopMargin  :=printer.Margins.cy;
        Result:=true;
      end;
      if FPageWidth <> printer.PageWidth then begin
        FPageWidth := printer.PageWidth;
        Result:=true;
      end;
      if FPageHeight <> printer.PageHeight then begin
        FPageHeight := printer.PageHeight;
        Result:=true;
      end;
    {$ELSE}
      DummyInt:=printer.PageHeight; // FORCE set IC, ie. Printer.Handle is IC now

      NewPPIX           :=GetDeviceCaps(Printer.Handle, LOGPIXELSX);
      if FPPIX <> NewPPIX then begin
        FPPIX := NewPPIX;
        Result:=true;
      end;
      NewPPIY           :=GetDeviceCaps(Printer.Handle, LOGPIXELSY);
      if FPPIY <> NewPPIY then begin
        FPPIY := NewPPIY;
        Result:=true;
      end;
      NewLeftMargin     :=GetDeviceCaps(Printer.Handle, PHYSICALOFFSETX);
      if FLeftMargin <> NewLeftMargin then begin
        FLeftMargin := NewLeftMargin;
        Result:=true;
      end;
      NewTopMargin      :=GetDeviceCaps(Printer.Handle, PHYSICALOFFSETY);
      if FTopMargin <> NewTopMargin then begin
        FTopMargin := NewTopMargin;
        Result:=true;
      end;
      NewPageWidth  :=GetDeviceCaps(Printer.Handle, PHYSICALWIDTH);
      if FPageWidth <> NewPageWidth then begin
        FPageWidth := NewPageWidth;
        Result:=true;
      end;
      NewPageHeight :=GetDeviceCaps(Printer.Handle, PHYSICALHEIGHT);
      if FPageHeight <> NewPageHeight then begin
        FPageHeight := NewPageHeight;
        Result:=true;
      end;
    {$ENDIF}
  end;
end;

(*
function      TARPrinter.GetCanvas:TCanvas;
begin
  {$IFDEF CLX}
    Result    :=Metafile.Canvas;
  {$ELSE}
    if not Assigned(FCanvas) then begin
      FCanvas :=TMetafileCanvas.Create(Metafile, 0);
    end;
    Result    :=FCanvas;
  {$ENDIF}
end;
*)
function      TARPrinter.GetPaperWidth:integer;
begin
  Result:=Round(FPageWidth * Screen.PixelsPerInch / PrinterPPIX);
end;

function      TARPrinter.GetPaperHeight:integer;
begin
  Result:=Round(FPageHeight * Screen.PixelsPerInch / PrinterPPIY);
end;

function      TARPrinter.GetPrintWidth:integer;
begin
  Result:=Round(Printer.PageWidth * Screen.PixelsPerInch / PrinterPPIX);
end;

function      TARPrinter.GetPrintHeight:integer;
begin
  Result:=Round(Printer.PageHeight * Screen.PixelsPerInch / PrinterPPIY);
end;

function      TARPrinter.GetLeftMargin:integer;
begin
  Result:=Round(PrinterLeftMargin * Screen.PixelsPerInch / PrinterPPIX);
end;

function      TARPrinter.GetTopMargin :integer;
begin
  Result:=Round(PrinterTopMargin  * Screen.PixelsPerInch / PrinterPPIY);
end;

function      TARPrinter.GetRightMargin:integer;
begin
  Result:=PaperWidth - LeftMargin;
end;

function      TARPrinter.GetBottomMargin :integer;
begin
  Result:=PaperHeight - TopMargin;
end;

procedure     TARPrinter.BeginDoc;
begin
  Abort;
  RefreshPrinter;
  NewPage;
end;

procedure    TARPrinter.NewPage;
begin
  {$IFDEF CLX}
    if Assigned(Metafile) then Metafile.Canvas.Stop;
  {$ENDIF}

  PageList.Add(TMetafile.Create);
  FCurPage        :=PageCount;

  Metafile.Width  :=PaperWidth;
  Metafile.Height :=PaperHeight;
  {$IFDEF CLX}
    FCanvas:=Metafile.Canvas;
    Metafile.Canvas.Start;
  {$ELSE}
    if Assigned(FCanvas) then FCanvas.Free;
    FCanvas   :=TMetafileCanvas.Create(Metafile, 0);
  {$ENDIF}
end;

procedure    TARPrinter.EndDoc;
begin
  {$IFDEF CLX}
    if Assigned(Metafile) then Metafile.Canvas.Stop;
  {$ENDIF}

  {$IFDEF CLX}
  {$ELSE}
    FCanvas.Free;
    FCanvas:=nil;
  {$ENDIF}

  FromPage:=Min(PageCount, 1);
  ToPage  :=PageCount;
end;

procedure    TARPrinter.Abort;
var i:integer;
begin
  if (PageList.Count > 0) and Assigned(FCanvas) then begin
    {$IFDEF CLX}
    {$ELSE}
      FCanvas.Free;
      FCanvas:=nil;
    {$ENDIF}
  end;

  FCanvas:=nil;
  for i:=0 to PageList.Count - 1 do begin
    TMetafile(PageList[i]).Free;
  end;
  PageList.Clear;
  FCurPage  :=0;

  FromPage :=0;
  ToPage   :=0;
end;

procedure    TARPrinter.SetCurPage(ACurPage:integer);
begin
  ACurPage:=Min(Max(ACurPage, 1), PageCount);

  if FCurPage <> ACurPage then begin
    FCurPage:=ACurPage;
  end;
end;

function    TARPrinter.GetMetafile:TMetafile;
begin
  if (FCurPage >= 1) and (FCurPage <= PageList.Count) then begin
    Result:=PageList[FCurPage - 1];
  end else begin
    Result:=nil;
  end;
end;

function    TARPrinter.GetPageCount:integer;
begin
  Result:=PageList.Count;
end;

function    TARPrinter.GetTitle:string;
begin
  if PrinterInstalled then begin
    Result:=Printer.Title;
    FTitle:=Result;
  end else begin
    Result:=FTitle;
  end;
end;

procedure    TARPrinter.SetTitle(ATitle:string);
begin
  FTitle:=ATitle;
  
  if PrinterInstalled then begin
    Printer.Title:=ATitle;
  end;
end;

function    TARPrinter.GetCopies:integer;
begin
  if PrinterInstalled then begin
    FCopies :=Printer.Copies;
  end;
  Result  :=FCopies;
end;

procedure    TARPrinter.SetCopies(ACopies:integer);
begin
  FCopies:=ACopies;
  if PrinterInstalled then begin
    Printer.Copies:=FCopies;
  end;
end;

function    TARPrinter.GetCollation:boolean;
begin
  if PrinterInstalled then begin
    {$IFDEF CLX}
      Result:=false;
    {$ELSE}
      Result:=pcCollation in Printer.Capabilities;
    {$ENDIF}
  end else begin
    Result:=false;
  end;
end;

function    TARPrinter.GetOrientation:TPrinterOrientation;
begin
  if PrinterInstalled then begin
    Result:=Printer.Orientation;
    FOrientation:=Result;
  end else begin
    Result:=FOrientation;
  end;
end;

procedure    TARPrinter.SetOrientation(AOrientation:TPrinterOrientation);
begin
  FOrientation:=AOrientation;
  if PrinterInstalled then begin
    Printer.Orientation:=AOrientation;
  end;
end;

procedure    TARPrinter.PrinterSetupDialog;
{$IFDEF CLX}
{$ELSE}
  var dlg:TPrinterSetupDialog;
{$ENDIF}
begin
  {$IFDEF CLX}
    SavePrinterSetting;
    if Printer.ExecuteSetup then begin
      LoadPrinterSetting;
      if RefreshPrinter and Assigned(OnPrinterChange) then OnPrinterChange(Self);
    end;
  {$ELSE}
    dlg:=TPrinterSetupDialog.Create(Application.MainForm);
    try
      SavePrinterSetting;
      if dlg.Execute then begin
        LoadPrinterSetting;
        if RefreshPrinter and Assigned(OnPrinterChange) then OnPrinterChange(Self);
      end;
    finally
      dlg.Free;
    end;
  {$ENDIF}
end;

procedure    TARPrinter.PrintDialog;
{$IFDEF CLX}
{$ELSE}
  var dlg: TPrintDialog;
{$ENDIF}
begin
  {$IFDEF CLX}
    SavePrinterSetting;
    if Printer.ExecuteSetup then begin
      LoadPrinterSetting;
      if RefreshPrinter and Assigned(OnPrinterChange) then OnPrinterChange(Self);
      Print;
    end;
  {$ELSE}
    dlg:=TPrintDialog.Create(Application.MainForm);
    try
      SavePrinterSetting;
      dlg.MinPage :=1;
      dlg.MaxPage :=PageCount;
      dlg.FromPage:=FromPage;
      dlg.ToPage  :=ToPage;
      dlg.Copies  :=Copies;
      dlg.Collate :=true;
      dlg.Options :=[poPageNums,poWarning,poHelp,poDisablePrintToFile];

      if dlg.Execute then begin
        LoadPrinterSetting;
        FromPage:=dlg.FromPage;
        ToPage  :=dlg.ToPage;
        Copies  :=dlg.Copies;
        if RefreshPrinter and Assigned(OnPrinterChange) then OnPrinterChange(Self);
        Print;
      end;
    finally
      dlg.Free;
    end;
  {$ENDIF}
end;

procedure    TARPrinter.Print;
var i:integer;
    {$IFDEF CLX}
    {$ELSE}
      SavedDC:integer;
    {$ENDIF}
begin
  if PageCount > 0 then begin

    Printer.BeginDoc;
    Printer.Title:=Title;

    try
      for i:=FromPage to ToPage do begin
        CurrPage:=i;

        {$IFDEF CLX}
          Printer.Canvas.Draw(Printer.XDPI, Printer.YDPI, Metafile);
        {$ELSE}
          SavedDC := SaveDC(Printer.Canvas.Handle);
          try
            SetMapMode      (Printer.Canvas.Handle, MM_ANISOTROPIC);
            SetWindowExtEx  (Printer.Canvas.Handle, Screen.PixelsPerInch,  Screen.PixelsPerInch,  nil);
            SetViewportExtEx(Printer.Canvas.Handle, PrinterPPIX , PrinterPPIY, nil);
            Printer.Canvas.Draw(0, 0, Metafile);
          finally
            RestoreDC(Printer.Canvas.Handle, SavedDC);
          end;
        {$ENDIF}

        if i < ToPage then Printer.NewPage;
      end;

      Printer.EndDoc;
    except
      Printer.Abort;
    end;
  end;
end;

procedure   TARPrinter.Preview;
var ARPreviewDialog:TARPreviewDialog;
begin
//  ARPreviewDialog:=TARPreviewDialog.Create(Application.MainForm);
//  ARPreviewDialog.ARPrinter:=Self;
//  ARPreviewDialog.ShowModal;
//  fixed by Ben Hay (ben_hay@bigfoot.com) 08.15.2000

  ARPreviewDialog:=TARPreviewDialog.Create(Application.MainForm);
  try
    ARPreviewDialog.ARPrinter:=Self;
    ARPreviewDialog.ShowModal;
  finally
    ARPreviewDialog.Free;
  end;
end;

procedure TARPrinter.LoadPrinterSetting;
{$IFDEF CLX}
{$ELSE}
  var Driver, Device, Port:PChar;
      DeviceMode:THandle;
      DevMode: PDeviceMode;
{$ENDIF}
begin
//    Orientation         :=Printer.Orientation;
  {$IFDEF CLX}
    PageSize.CLXPageSize:=Printer.PrintAdapter.PageSize;
    PageSize.Width      :=Printer.PageWidth  / Printer.XDPI * 25.4;
    PageSize.Height     :=Printer.PageHeight / Printer.YDPI * 25.4;
  {$ELSE}
    if Printer.Printers.Count > 0 then begin
      GetMem(Driver,  256);
      GetMem(Device,  256);
      GetMem(Port,    256);
      try
        Printer.GetPrinter(Driver, Device, Port, DeviceMode);
        DevMode:=GlobalLock(DeviceMode);
        if (DevMode^.dmPaperSize > 0) and (DevMode^.dmPaperSize < High(cARPageSizeDefs)) then begin
          PageSize.WinPageSize:=DevMode^.dmPaperSize;
        end else begin
          PageSize.WinPageSize:=0;
          PageSize.Width      :=DevMode^.dmPaperWidth;
          PageSize.Height     :=DevMode^.dmPaperLength;
        end;
        GlobalUnlock(DeviceMode);
      finally
        FreeMem(Port);
        FreeMem(Device);
        FreeMem(Driver);
      end;
    end;
  {$ENDIF}
end;

procedure TARPrinter.SavePrinterSetting;
  {$IFDEF CLX}
  {$ELSE}
var
    Driver, Device, Port:PChar;
    DeviceMode:THandle;
    DevMode: PDeviceMode;
  {$ENDIF}
begin
  {$IFDEF CLX}
    Printer.PrintAdapter.PageSize :=PageSize.CLXPageSize;

    QPrinter_setMinMax    (qph, 1, PageCount);
    QPrinter_setFromTo    (qph, FromPage, ToPage);
    QPrinter_setNumCopies (qph, Copies);
  {$ELSE}
    if Printer.Printers.Count > 0 then begin
      GetMem(Driver,  256);
      GetMem(Device,  256);
      GetMem(Port,    256);
      try
        Printer.GetPrinter(Driver, Device, Port, DeviceMode);
          DevMode:=GlobalLock(DeviceMode);
            if FPageSize.WinPageSize > 0 then begin
              DevMode^.dmPaperSize:=FPageSize.WinPageSize;
            end else begin
              DevMode^.dmPaperSize  :=32767;//0;
              DevMode^.dmPaperWidth :=Round(FPageSize.Width);
              DevMode^.dmPaperLength:=Round(FPageSize.Height);
            end;
          GlobalUnlock(DeviceMode);
        Printer.SetPrinter(Driver, Device, Port, DeviceMode);
      finally
        FreeMem(Port);
        FreeMem(Device);
        FreeMem(Driver);
      end;
    end;
  {$ENDIF}
end;

procedure TARPrinter.SetPageSize(const Value: TARPageSize);
{$IFDEF CLX}
{$ELSE}
  var Driver, Device, Port:PChar;
      DeviceMode:THandle;
      DevMode: PDeviceMode;
{$ENDIF}
begin
  FPageSize.Assign(Value);

  {$IFDEF CLX}
    Printer.PrintAdapter.PageSize:=FPageSize.CLXPageSize;
  {$ELSE}
    if Printer.Printers.Count > 0 then begin
      GetMem(Driver,  256);
      GetMem(Device,  256);
      GetMem(Port,    256);
      try
        Printer.GetPrinter(Driver, Device, Port, DeviceMode);
          DevMode:=GlobalLock(DeviceMode);
            if FPageSize.WinPageSize > 0 then begin
              DevMode^.dmPaperSize:=FPageSize.WinPageSize;
            end else begin
              DevMode^.dmPaperSize  :=32767;//0;
              DevMode^.dmPaperWidth :=Round(FPageSize.Width);
              DevMode^.dmPaperLength:=Round(FPageSize.Height);
            end;
          GlobalUnlock(DeviceMode);
        Printer.SetPrinter(Driver, Device, Port, DeviceMode);
      finally
        FreeMem(Port);
        FreeMem(Device);
        FreeMem(Driver);
      end;
    end;
  {$ENDIF}
end;

function TARPrinter.GetFromPage: integer;
begin
  Result:=FFromPage;
end;

function TARPrinter.GetToPage: integer;
begin
  Result:=FToPage
end;

procedure TARPrinter.SetFromPage(const Value: integer);
begin
  FFromPage:=Value;
end;

procedure TARPrinter.SetToPage(const Value: integer);
begin
  FToPage:=Value;
end;

////////////////////////////////////////////////////////////////////////////////
{ TARPreview }

{$IFDEF CLX}
{$ELSE}
function    TARPreview.GetVSTrackPos:integer;
var si: TScrollInfo;
begin
  si.cbSize:=SizeOf(si);
  si.fMask:=SIF_ALL;
  GetScrollInfo(Handle, SB_VERT, si);
  Result:=si.nTrackPos;
end;

function    TARPreview.GetVSPos:integer;
var si: TScrollInfo;
begin
  si.cbSize:=SizeOf(si);
  si.fMask:=SIF_ALL;
  GetScrollInfo(Handle, SB_VERT, si);
  Result:=si.nPos;
end;

procedure   TARPreview.SetVSPos(AVSPos:integer);
var si: TScrollInfo;
begin
  si.cbSize:=SizeOf(si);
  si.fMask:=SIF_POS;
  si.nPos:=AVSPos;
  SetScrollInfo(Handle, SB_VERT, si, true);

  if Assigned(FOnChangeView) then FOnChangeView(Self);
end;

function    TARPreview.GetVSMax:integer;
var si: TScrollInfo;
begin
  si.cbSize:=SizeOf(si);
  si.fMask:=SIF_ALL;
  GetScrollInfo(Handle, SB_VERT, si);
  Result:=si.nMax;
end;

procedure   TARPreview.SetVSMax(AVSMax:integer);
var si: TScrollInfo;
begin
  si.cbSize:=SizeOf(si);
  si.fMask:=SIF_RANGE;
  si.nMin:=0;
  si.nMax:=AVSMax;
  SetScrollInfo(Handle, SB_VERT, si, true);

  if Assigned(FOnChangeView) then FOnChangeView(Self);
end;

function    TARPreview.GetVSPage:integer;
var si: TScrollInfo;
begin
  si.cbSize:=SizeOf(si);
  si.fMask:=SIF_ALL;
  GetScrollInfo(Handle, SB_VERT, si);
  Result:=si.nPage;
end;

procedure   TARPreview.SetVSPage(AVSPage:integer);
var si: TScrollInfo;
begin
  si.cbSize:=SizeOf(si);
  si.fMask:=SIF_PAGE;
  si.nPage:=AVSPage;
  SetScrollInfo(Handle, SB_VERT, si, true);
  if Assigned(FOnChangeView) then FOnChangeView(Self);
end;

function    TARPreview.GetHSTrackPos:integer;
var si: TScrollInfo;
begin
  si.cbSize:=SizeOf(si);
  si.fMask:=SIF_ALL;
  GetScrollInfo(Handle, SB_HORZ, si);
  Result:=si.nTrackPos;
end;

function    TARPreview.GetHSPos:integer;
var si: TScrollInfo;
begin
  si.cbSize:=SizeOf(si);
  si.fMask:=SIF_ALL;
  GetScrollInfo(Handle, SB_HORZ, si);
  Result:=si.nPos;
end;

procedure   TARPreview.SetHSPos(AHSPos:integer);
var si: TScrollInfo;
begin
  si.cbSize:=SizeOf(si);
  si.fMask:=SIF_POS;
  si.nPos:=AHSPos;
  SetScrollInfo(Handle, SB_HORZ, si, true);

  if Assigned(FOnChangeView) then FOnChangeView(Self);
end;

function    TARPreview.GetHSMax:integer;
var si: TScrollInfo;
begin
  si.cbSize:=SizeOf(si);
  si.fMask:=SIF_ALL;
  GetScrollInfo(Handle, SB_HORZ, si);
  Result:=si.nMax;
end;

procedure   TARPreview.SetHSMax(AHSMax:integer);
var si: TScrollInfo;
begin
  si.cbSize:=SizeOf(si);
  si.fMask:=SIF_RANGE;
  si.nMin:=0;
  si.nMax:=AHSMax;
  SetScrollInfo(Handle, SB_HORZ, si, true);

  if Assigned(FOnChangeView) then FOnChangeView(Self);
end;

function    TARPreview.GetHSPage:integer;
var si: TScrollInfo;
begin
  si.cbSize:=SizeOf(si);
  si.fMask:=SIF_ALL;
  GetScrollInfo(Handle, SB_HORZ, si);
  Result:=si.nPage;
end;

procedure   TARPreview.SetHSPage(AHSPage:integer);
var si: TScrollInfo;
begin
  si.cbSize:=SizeOf(si);
  si.fMask:=SIF_PAGE;
  si.nPage:=AHSPage;
  SetScrollInfo(Handle, SB_HORZ, si, true);
  if Assigned(FOnChangeView) then FOnChangeView(Self);
end;
{$ENDIF}
////////////////////////////////////////////////////////////////////////////////

constructor TARPreview.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);

  {$IFNDEF D3}
    OnMouseWheelUp:=MouseWheelUp;
    OnMouseWheelDown:=MouseWheelDown;
  {$ENDIF}

  ControlStyle:=[csClickEvents, csCaptureMouse, csDoubleClicks];

  ParentColor :=False;
  Color       :=clBtnFace;
  TabStop     :=true;

  FZoom       :=100;
  FZoomMode   :=zmFixed;

  {$IFDEF CLX}
  {$ELSE}
    {$IFNDEF D3}
      DoubleBuffered:=false;
    {$ENDIF}
  {$ENDIF}

  ARPrinter   :=TARPrinter.Create(Self);
end;

destructor  TARPreview.Destroy;
begin
  inherited Destroy;
end;

{$IFDEF CLX}
{$ELSE}
procedure   TARPreview.CreateParams(var p: TCreateParams);
begin
  inherited CreateParams(p);
  with p do begin
    Style   :=(Style or WS_VSCROLL or WS_HSCROLL) or WS_TABSTOP  ;

    ExStyle :=ExStyle or WS_EX_CLIENTEDGE;
  end;
end;

procedure   TARPreview.WMVSCROLL(var Message: TWMScroll );
begin
  if FScrollUpdate then exit;
  FScrollUpdate:=true;

  case Message.ScrollCode of
    SB_LINEDOWN:begin
                  TopY:=TopY + 10;
                end;
    SB_LINEUP  :begin
                  TopY:=TopY - 10;
                end;
    SB_PAGEUP  :begin
                  TopY:=TopY - ClientHeight;
                end;
    SB_PAGEDOWN:begin
                  TopY:=TopY + ClientHeight;
                end;
    SB_THUMBPOSITION:begin
                  TopY:=VSPos;
                end;
    SB_THUMBTRACK:begin
                  TopY:=GetVSTrackPos;
                end;
  end;

  FScrollUpdate:=false;

  if Assigned(FOnChangeView) then FOnChangeView(Self);
end;

procedure   TARPreview.WMHSCROLL(var Message: TWMScroll );
begin
  if FScrollUpdate then exit;
  FScrollUpdate:=true;

  case Message.ScrollCode of
    SB_LINEDOWN:begin
                  LeftX:=LeftX + 10;
                end;
    SB_LINEUP  :begin
                  LeftX:=LeftX - 10;
                end;
    SB_PAGEUP  :begin
                  LeftX:=LeftX - ClientWidth;
                end;
    SB_PAGEDOWN :begin
                  LeftX:=LeftX + ClientWidth;
                end;
    SB_THUMBPOSITION:begin
                  LeftX:=HSPos;
                end;
    SB_THUMBTRACK:begin
                  LeftX:=GetHSTrackPos;
                end;
  end;

  FScrollUpdate:=false;

  if Assigned(FOnChangeView) then FOnChangeView(Self);
end;
{$ENDIF}

procedure   TARPreview.SetTopY(ATopY:integer);
begin
  FTopY:=Min(Max(ATopY, 0), ViewPageDY - ClientHeight);
  VSPos:=FTopY;
  Paint;

  if Assigned(FOnChangeView) then FOnChangeView(Self);
end;

procedure   TARPreview.SetLeftX(ALeftX:integer);
begin
  FLeftX:=Min(Max(ALeftX, 0), ViewPageDX - ClientWidth);
  HSPos:=FLeftX;
  Paint;

  if Assigned(FOnChangeView) then FOnChangeView(Self);
end;

procedure    TARPreview.SetPageDX(APageDX:integer);
begin
  FPageDX :=APageDX;
  HSMax   :=ViewPageDX;
  HSPage  :=ClientWidth;

  if Assigned(FOnChangeView) then FOnChangeView(Self);
end;

procedure    TARPreview.SetPageDY(APageDY:integer);
begin
  FPageDY :=APageDY;
  VSMax   :=ViewPageDY;
  VSPage  :=ClientHeight;

  if Assigned(FOnChangeView) then FOnChangeView(Self);
end;

function    TARPreview.GetViewPageDX:integer;
begin
  Result:=Round(PageDX * Zoom / 100.0);
end;

function    TARPreview.GetViewPageDY:integer;
begin
  Result:=Round(PageDY * Zoom / 100.0);
end;

function    TARPreview.GetCurPage:integer;
begin
  Result:=ARPrinter.CurrPage;
end;

procedure    TARPreview.SetCurPage(ACurPage:integer);
begin
  ARPrinter.CurrPage:=ACurPage;
  Paint;

  if Assigned(FOnChangeView) then FOnChangeView(Self);
end;

function    TARPreview.GetPageCount:integer;
begin
  Result:=ARPrinter.PageCount;
end;

procedure   TARPreview.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited KeyDown(Key, Shift);

  case key of
    VK_HOME:  begin
                if ssCtrl in Shift then begin
                  CurrPage:=0;
                end else begin
                  TopY:=0;
                end;
              end;
    VK_END :  begin
                if ssCtrl in Shift then begin
                  CurrPage:=PageCount;
                end else begin
                TopY:=100000;
                end;
              end;
    VK_PRIOR: begin
                TopY:=TopY - ClientHeight;
              end;
    VK_NEXT:  begin
                TopY:=TopY + ClientHeight;
              end;
    VK_UP:    begin
                TopY:=TopY - 20;
              end;
    VK_DOWN:  begin
                TopY:=TopY + 20;
              end;
    VK_LEFT:  begin
                CurrPage:=CurrPage - 1;
              end;
    VK_RIGHT: begin
                CurrPage:=CurrPage + 1;
              end;
    VK_ADD:   begin
                Zoom:=Zoom + 10;
              end;
    VK_SUBTRACT:begin
                Zoom:=Zoom - 10;
              end;
  end;

  if Assigned(FOnChangeView) then FOnChangeView(Self);
end;

{$IFDEF CLX}
{$ELSE}
procedure   TARPreview.WMGetDlgCode(var Message: TWMGetDlgCode);
begin
  inherited;
  Message.Result := DLGC_WANTARROWS or DLGC_WANTCHARS;
end;
{$ENDIF}

procedure   TARPreview.UpdateScrolls;
begin
  if FScrollUpdate then exit;
  FScrollUpdate:=true;

    HSPos:=LeftX;
    VSPos:=TopY;

    HSMax   :=ViewPageDX;
    HSPage  :=ClientWidth;

    VSMax   :=ViewPageDY;
    VSPage  :=ClientHeight;

  FScrollUpdate:=false;
end;

procedure    TARPreview.SetARPrinter(AARPrinter:TARPrinter);
begin
  if Assigned(FARPrinter) then begin
    FARPrinter.Free;
  end;

  FARPrinter:=AARPrinter;
end;

procedure   TARPreview.SetShowNonPrintingArea(AShowNonPrintingArea:boolean);
begin
  if FShowNonPrintingArea <> AShowNonPrintingArea then begin
    FShowNonPrintingArea:=AShowNonPrintingArea;
    Paint;

    if Assigned(FOnChangeView) then FOnChangeView(Self);
  end;
end;

procedure   TARPreview.SetZoom(AZoom:integer);
begin
  AZoom:=Min(ARPreviewZoomMax, Max(ARPreviewZoomMin, AZoom));
  if FZoom <> AZoom then begin
    FZoom:=AZoom;
    FZoomMode:=zmFixed;
    Paint;

    if Assigned(FOnChangeView) then FOnChangeView(Self);
  end;
end;

procedure   TARPreview.ZoomToWidth;
begin
  Zoom:=Round(100 * ClientWidth / ARPrinter.PaperWidth) - 3;
  FZoomMode:=zmToWidth;
end;

procedure   TARPreview.ZoomToFit;
begin
  Zoom:=Min(Round(100 * ClientWidth / ARPrinter.PaperWidth), Round(100 * ClientHeight / ARPrinter.PaperHeight)) - 2;
  FZoomMode:=zmToFit;
end;

{$IFDEF CLX}
procedure   TARPreview.Resize;
begin
  inherited;
  Realign;

  VSPage:=ClientHeight;
  HSPage:=ClientWidth;

  case FZoomMode of
    zmToFit:  ZoomToFit;
    zmToWidth:ZoomToWidth;
  end;

  if Assigned(FOnChangeView) then FOnChangeView(Self);
end;

{$ELSE}
procedure   TARPreview.WMSize(var Message: TWMSize);
begin
  inherited;
  Realign;

  VSPage:=ClientHeight;
  HSPage:=ClientWidth;

  case FZoomMode of
    zmToFit:  ZoomToFit;
    zmToWidth:ZoomToWidth;
  end;

  if Assigned(FOnChangeView) then FOnChangeView(Self);
end;

procedure   TARPreview.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
  Message.Result:=1;
end;
{$ENDIF}

{$IFNDEF D3}
procedure   TARPreview.MouseWheelUp(Sender:TObject; Shift:TShiftState; MousePos:TPoint; var Handled:boolean);
begin
  TopY:=TopY - 20;
end;

procedure   TARPreview.MouseWheelDown(Sender:TObject; Shift:TShiftState; MousePos:TPoint; var Handled:boolean);
begin
  TopY:=TopY + 20;
end;
{$ENDIF}

var OldY:integer = -1000;
    OldX:integer = -1000;

procedure   TARPreview.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited;

  OldX:=X;
  OldY:=Y;
end;

procedure   TARPreview.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
  inherited;

  if ssLeft in Shift then begin
    if (OldX <> X) or (OldY <> Y) then begin
      LeftX :=LeftX + (OldX - X);
      TopY  :=TopY  + (OldY - Y);

      OldX:=X;
      OldY:=Y;
    end;
  end;
end;

procedure    TARPreview.Paint;
var OrgX, OrgY:integer;
    MemCanvas:TCanvas;
    {$IFDEF CLX}
      MemBitmap:TBitmap;
      m:QWMatrixH;
    {$ELSE}
      MemDC:HDC;
      MemBitmap, OldBitmap: HBITMAP;
    {$ENDIF}
begin
  {$IFDEF CLX}
//    UpdateScrolls;

    FLeftX :=HSPos;
    FTopY  :=VSPos;

    PageDX:=ARPrinter.PaperWidth;
    PageDY:=ARPrinter.PaperHeight;

    if ClientWidth >  ViewPageDX then begin
      OrgX:=(ClientWidth -  ViewPageDX) div 2
    end else begin
      OrgX:=Max(Min(-LeftX, 0),  ClientWidth - ViewPageDX);
    end;

    if ClientHeight > ViewPageDY then begin
      OrgY:=(ClientHeight - ViewPageDY) div 2
    end else begin
      OrgY:=Max(Min(-TopY, 0),  ClientHeight - ViewPageDY);
    end;

    MemBitmap:=TBitmap.Create;
    MemBitmap.Width :=ClientWidth;
    MemBitmap.Height:=ClientHeight;
    MemCanvas:=MemBitmap.Canvas;
    MemCanvas.Start;
    try
      MemCanvas.Brush.Style:=bsSolid;
      MemCanvas.Brush.Color:=clBtnFace;
      MemCanvas.FillRect(ClientRect);

      //paint white page
      MemCanvas.Brush.Color:=clGray;
      MemCanvas.FillRect (Rect(OrgX + ViewPageDX,          OrgY + 5, OrgX + ViewPageDX + 5, OrgY + ViewPageDY + 5));
      MemCanvas.FillRect (Rect(OrgX + 5,          OrgY + ViewPageDY, OrgX + ViewPageDX + 5, OrgY + ViewPageDY + 5));

      MemCanvas.Brush.Color:=clWIndow;
      MemCanvas.Rectangle(OrgX, OrgY, OrgX + ViewPageDX, OrgY + ViewPageDY);

      m:=QWMatrix_create(Zoom / 100.0, 0.0, 0.0, Zoom / 100.0, OrgX, OrgY);
      QPainter_setWorldMatrix(MemCanvas.Handle, m, false);
      QWMatrix_destroy(m);

      if PrinterInstalled then begin
        if ShowNonPrintingArea then begin
          MemCanvas.Brush.Color:=clLtGray;
          MemCanvas.Rectangle(                   0,                   0,  ARPrinter.PaperWidth,  ARPrinter.PaperHeight);
          MemCanvas.Brush.Color:=clWhite;
          MemCanvas.Rectangle(ARPrinter.LeftMargin, ARPrinter.TopMargin,  ARPrinter.RightMargin, ARPrinter.BottomMargin);
        end;
        MemCanvas.Draw(ARPrinter.LeftMargin, ARPrinter.TopMargin, ARPrinter.Metafile);
      end else begin
        MemCanvas.Draw(0, 0, ARPrinter.Metafile);
      end;

      Canvas.SetClipRect(ClientRect);
      try
        Canvas.Draw(0, 0, MemBitmap);
      finally
        Canvas.ResetClipRegion;
      end;
    finally
      MemCanvas.Stop;
      MemBitmap.Free;
    end;
  {$ELSE}
    UpdateScrolls;

    PageDX:=ARPrinter.PaperWidth;
    PageDY:=ARPrinter.PaperHeight;

    if ClientWidth >  ViewPageDX then begin
      OrgX:=(ClientWidth -  ViewPageDX) div 2
    end else begin
      OrgX:=Max(Min(-LeftX, 0),  ClientWidth - ViewPageDX);
    end;

    if ClientHeight > ViewPageDY then begin
      OrgY:=(ClientHeight - ViewPageDY) div 2
    end else begin
      OrgY:=Max(Min(-TopY, 0),  ClientHeight - ViewPageDY);
    end;

    MemBitmap       :=CreateCompatibleBitmap(Canvas.Handle, ClientRect.Right, ClientRect.Bottom);
    MemDC           :=CreateCompatibleDC(Canvas.Handle);
    MemCanvas       :=TCanvas.Create;
    MemCanvas.Handle:=MemDC;

    OldBitmap := SelectObject(MemDC, MemBitmap);
    try
      MemCanvas.Brush.Style:=bsSolid;
      MemCanvas.Brush.Color:=clBtnFace;
      MemCanvas.FillRect(ClientRect);

      //paint white page
      MemCanvas.Brush.Color:=clGray;
      MemCanvas.FillRect (Rect(OrgX + ViewPageDX,          OrgY + 5, OrgX + ViewPageDX + 5, OrgY + ViewPageDY + 5));
      MemCanvas.FillRect (Rect(OrgX + 5,          OrgY + ViewPageDY, OrgX + ViewPageDX + 5, OrgY + ViewPageDY + 5));

      MemCanvas.Brush.Color:=clWIndow;
      {$IFDEF D3}
        MemCanvas.Rectangle(OrgX, OrgY, OrgX + ViewPageDX, OrgY + ViewPageDY);
      {$ELSE}
        MemCanvas.Rectangle(Rect(OrgX, OrgY, OrgX + ViewPageDX, OrgY + ViewPageDY));
      {$ENDIF}

      SetMapMode(MemDC, MM_ISOTROPIC);

      SetViewportOrgEx(MemDC, OrgX, OrgY, nil);
  //    SetWindowExtEx  (MemDC,        100,        100,  nil);
  //    SetViewportExtEx(MemDC, Zoom div 1, Zoom div 1,  nil);

      SetWindowExtEx  (MemDC,      1000,      1000, nil);
      SetViewportExtEx(MemDC, Zoom * 10, Zoom * 10, nil);

      if PrinterInstalled then begin
        if ShowNonPrintingArea then begin
          MemCanvas.Brush.Color:=clLtGray;
          MemCanvas.Rectangle(                   0,                   0,  ARPrinter.PaperWidth,  ARPrinter.PaperHeight);
          MemCanvas.Brush.Color:=clWhite;
          MemCanvas.Rectangle(ARPrinter.LeftMargin, ARPrinter.TopMargin,  ARPrinter.RightMargin, ARPrinter.BottomMargin);
        end;

        MemCanvas.Draw(ARPrinter.LeftMargin, ARPrinter.TopMargin, ARPrinter.Metafile);

      end else begin
        MemCanvas.Draw(0, 0, ARPrinter.Metafile);
      end;

      SetViewportOrgEx(MemDC, 0, 0, nil);
      SetWindowExtEx  (MemDC, 1, 1, nil);
      SetViewportExtEx(MemDC, 1, 1, nil);

      BitBlt(Canvas.Handle, 0, 0, ClientRect.Right, ClientRect.Bottom, MemDC, 0, 0, SRCCOPY);
    finally
      SelectObject(MemDC, OldBitmap);
      MemCanvas.Free;
      DeleteDC(MemDC);
      DeleteObject(MemBitmap);
    end;
  {$ENDIF}
end;

{ TARPageSize }

constructor TARPageSize.Create;
begin
  inherited Create;
end;

destructor TARPageSize.Destroy;
begin
  if Assigned(FARTemplate)  then  FARTemplate.Free;

//sa: 11.06.2001
  if Assigned(FPageNames)   then  FPageNames.Free;
//--

  inherited;
end;

procedure TARPageSize.ARTemplateChange(Sender: TObject);
var p, i:integer;
    sname, sparam:string;
    iparam, code:integer;
    L:TARTemplate;
    sl:TAIStringList;
begin
  sl:=TAIStringList.Create('TARPageSize.ARTemplateChange sl');
  try
    L:=FARTemplate;

    sname :=TrimLeft(L[0]);
    if sname = 'PAGESIZE' then begin
      Name:='';
      Width:=0.0;
      Height:=0.0;

      for i:=1 to L.Count - 1 do begin
        p:=System.Pos('=', L[i]);
        if p <> 0 then begin
          sname   :=TrimLeft(System.Copy(L[i], 1, p - 1));
          sparam  :=System.Copy(L[i], p + 1, 2000000000);
          System.Val(sparam, iparam, code);
        end else begin
          sname :=TrimLeft(L[i]);
          sparam:='';
          iparam:=-1;
        end;

        if sname = 'NAME'     then Name   :=sparam  else
        if sname = 'WIDTH'    then Width  :=iparam  else
        if sname = 'HEIGHT'   then Height :=iparam  else
        if sname = 'ENDPAGESIZE' then break;
      end;
    end;
  finally
    sl.Free;
  end;
end;

function TARPageSize.GetARTemplate:TARTemplate;
var sl:TAIStringList;
begin
  sl:=TAIStringList.Create('TARPageSize.GetARTemplate sl');
  try
    if not Assigned(FARTemplate) then begin
      FARTemplate:=TARTemplate.Create('TARPageSize.FARTemplate');
      FARTemplate.OnChange:=ARTemplateChange;
    end;
    FARTemplate.OnChange:=nil;
      FARTemplate.Clear;
      Result:=FARTemplate;

      FARTemplate.Add('PAGESIZE');
      FARTemplate.Add(Format('  NAME=%s',   [Name]));
      FARTemplate.Add(Format('  WIDTH=%d',  [Round(Width)]));
      FARTemplate.Add(Format('  HEIGHT=%d', [Round(Height)]));
      FARTemplate.Add('ENDPAGESIZE');
    FARTemplate.OnChange:=ARTemplateChange;
  finally
    sl.Free;
  end;
end;

procedure TARPageSize.SetARTemplate(const AARTemplate: TARTemplate);
begin
  if not Assigned(FARTemplate) then begin
    FARTemplate:=TARTemplate.Create('TARPageSize.FARTemplate');
    FARTemplate.OnChange:=ARTemplateChange;
  end;
  FARTemplate.Assign(AARTemplate);
end;

procedure TARPageSize.SetCLXPageSize(const Value: TARPaperSizeCLX);
var i:integer;
begin
  for i:=Low(cARPageSizeDefs) to High(cARPageSizeDefs) do begin
    if Ord(Value) = cARPageSizeDefs[i].CLXType then begin
      Name:=cARPageSizeDefs[i].Name;
      exit;
    end;
  end;
end;

procedure TARPageSize.SetWinPageSize(const Value: TARPaperSizeWin);
var i:integer;
begin
  for i:=Low(cARPageSizeDefs) to High(cARPageSizeDefs) do begin
    if Value = cARPageSizeDefs[i].WinType then begin
      Name:=cARPageSizeDefs[i].Name;
      exit;
    end;
  end;
end;

procedure TARPageSize.SetOrientation(const Value: TPrinterOrientation);
begin
  FOrientation := Value;
end;

function TARPageSize.GetPageNames: TStrings;
var i:integer;
begin
  if not Assigned(FPageNames) then begin
    FPageNames:=TStringList.Create;
    FPageNames.Sorted:=true;
    FPageNames.Duplicates:=dupIgnore;

    for i:=Low(cARPageSizeDefs) to High(cARPageSizeDefs) do begin
      FPageNames.AddObject(cARPageSizeDefs[i].Name, pointer(i));
    end;
  end;

  Result:=FPageNames;
end;

procedure TARPageSize.Assign(Source: TPersistent);
begin
  if Source is TARPageSize then begin
    Name:=TARPageSize(Source).Name;
    if FCurPageSizeIdx = 0 then begin
      Width :=TARPageSize(Source).Width;
      Height:=TARPageSize(Source).Height;
    end;
  end else begin
    inherited Assign(Source);
  end;
end;

procedure TARPageSize.SetHeight(const Value: extended);
var i:integer;
begin
  FHeight :=Value;

  for i:=Low(cARPageSizeDefs) to High(cARPageSizeDefs) do begin
    if (FWidth = (cARPageSizeDefs[i].Width * 10.0)) and (FHeight = (cARPageSizeDefs[i].Height * 10.0)) then begin
      FName           :=cARPageSizeDefs[i].Name;
      FWinPageSize    :=cARPageSizeDefs[i].WinType;
      FCLXPageSize    :=TARPaperSizeCLX(cARPageSizeDefs[i].CLXType);
      FCurPageSizeIdx :=i;
      exit;
    end;
  end;

  FName           :=cARPageSizeDefs[0].Name;
  FWinPageSize    :=cARPageSizeDefs[0].WinType;
  FCLXPageSize    :=TARPaperSizeCLX(cARPageSizeDefs[0].CLXType);
  FCurPageSizeIdx :=0;
end;

procedure TARPageSize.SetWidth(const Value: extended);
var i:integer;
begin
  FWidth := Value;

  for i:=Low(cARPageSizeDefs) to High(cARPageSizeDefs) do begin
    if (FWidth = (cARPageSizeDefs[i].Width * 10.0)) and (FHeight = (cARPageSizeDefs[i].Height * 10.0)) then begin
      FName           :=cARPageSizeDefs[i].Name;
      FWinPageSize    :=cARPageSizeDefs[i].WinType;
      FCLXPageSize    :=TARPaperSizeCLX(cARPageSizeDefs[i].CLXType);
      FCurPageSizeIdx :=i;
      exit;
    end;
  end;

  FName           :=cARPageSizeDefs[0].Name;
  FWinPageSize    :=cARPageSizeDefs[0].WinType;
  FCLXPageSize    :=TARPaperSizeCLX(cARPageSizeDefs[0].CLXType);
  FCurPageSizeIdx :=0;
end;

procedure TARPageSize.SetName(const Value: string);
var idx:integer;
begin
  FName := Value;

  idx:=PageNames.IndexOf(FName);
  if idx = -1 then begin
    FCurPageSizeIdx :=0;
    FHeight         :=0.0;
    FWidth          :=0.0;
    FName           :=cARPageSizeDefs[0].Name;
  end else begin
    FCurPageSizeIdx:=integer(PageNames.Objects[idx]);
    FWidth      :=cARPageSizeDefs[FCurPageSizeIdx].Width * 10.0;
    FHeight     :=cARPageSizeDefs[FCurPageSizeIdx].Height * 10.0;
    FWinPageSize:=cARPageSizeDefs[FCurPageSizeIdx].WinType;
    if cARPageSizeDefs[FCurPageSizeIdx].CLXType = -1 then begin
      FCLXPageSize:=psNPageSize;
    end else begin
      FCLXPageSize:=TARPaperSizeCLX(cARPageSizeDefs[FCurPageSizeIdx].CLXType);
    end;
  end;
end;

end.

