unit ARPropEditor;
//************************************************************//
//                         AND Report                        //
//  Eugene Hrulev (mrHru) Laxian Key, Omsk, Russia 1999,2001 //
//  Oleg Melnikov (muler) Laxian Key, Omsk, Russia 1999,2001 //
//************************************************************//

{$INCLUDE DEFINES.INC}

interface

uses
  Classes, SysUtils,
  {$IFDEF CLX}
    QGraphics, QForms, QControls, QButtons, QStdCtrls, QComCtrls,
    DesignEditors, DesignIntf,
  {$ELSE}
    Windows, Graphics, Forms, Controls, Buttons, StdCtrls, ComCtrls,
    {$IFDEF D6}
      DesignIntf, DesignEditors,
    {$ELSE}
      DsgnIntf, DsgnWnds,
    {$ENDIF}
  {$ENDIF}

  ARPrinter, ARRep, ARGridRep, AREditorMain;

type
  TARTemplateProperty = class(TClassProperty)
  public
    procedure Edit; override;
    function  GetAttributes: TPropertyAttributes; override;
  end;

  TARTemplateEditor = class(TDefaultEditor)
  protected
    {$IFDEF CLX}
      procedure EditProperty(const PropertyEditor: IProperty; var Continue: Boolean); override;
    {$ELSE}
      {$IFDEF D6}
        procedure EditProperty(const PropertyEditor: IProperty; var Continue: Boolean); override;
      {$ELSE}
        procedure EditProperty(PropertyEditor: TPropertyEditor; var Continue, FreeEditor: Boolean); override;
      {$ENDIF}
    {$ENDIF}
  public
    procedure ExecuteVerb(Index: Integer); override;
    function  GetVerb(Index: Integer): string; override;
    function  GetVerbCount: Integer; override;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponentEditor(TARGridRep, TARTemplateEditor);
  RegisterPropertyEditor(TypeInfo(TARTemplate), nil, 'ARTemplate', TARTemplateProperty);
end;

procedure TARTemplateProperty.Edit;
var AREditor: TAREditorDlg;
begin
  AREditor := TAREditorDlg.Create(Application);
  try
    AREditor.SetEditingRep(TARGridRep(GetComponent(0)));
    AREditor.ShowModal;
  finally
    AREditor.Free;
  end;
end;

function TARTemplateProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog];
end;

{TARTemplateEditor}

{$IFDEF CLX}
  procedure TARTemplateEditor.EditProperty(const PropertyEditor: IProperty; var Continue: Boolean);
{$ELSE}
  {$IFDEF D6}
    procedure TARTemplateEditor.EditProperty(const PropertyEditor: IProperty; var Continue: Boolean);
  {$ELSE}
    procedure TARTemplateEditor.EditProperty(PropertyEditor: TPropertyEditor; var Continue, FreeEditor: Boolean);
  {$ENDIF}
{$ENDIF}
var
  PropName: string;
begin
  PropName := PropertyEditor.GetName;
  if (CompareText(PropName, 'ARTemplate') = 0) then begin
    PropertyEditor.Edit;
    Continue  :=False;

    {$IFNDEF D6}
      {$IFNDEF CLX}
        FreeEditor:=true;
      {$ENDIF}
    {$ENDIF}
  end;
end;

function TARTemplateEditor.GetVerbCount: Integer;
begin
  Result := 1;
end;

function TARTemplateEditor.GetVerb(Index: Integer): string;
begin
  if Index = 0 then begin
    Result := 'Edit AND Report &Template...'
  end else begin
    Result := '';
  end;
end;

procedure TARTemplateEditor.ExecuteVerb(Index: Integer);
begin
  if Index = 0 then Edit;
end;

end.
