unit ARRepCommon;
//************************************************************//
//                         AND Report                        //
//  Eugene Hrulev (mrHru) Laxian Key, Omsk, Russia 1999,2001 //
//  Oleg Melnikov (muler) Laxian Key, Omsk, Russia 1999,2001 //
//************************************************************//

{$INCLUDE DEFINES.INC}

// changes
// CD: Claudio Driussi  cldriussi@swit.net

interface

uses
  Classes, SysUtils, Math,
  {$IFDEF CLX}
    QGraphics, QPrinters, QComCtrls, QForms, Types, QControls,
  {$ELSE}
    Graphics,  Printers,  ComCtrls,  Forms, Controls, Windows, Messages,
    RichEdit,
  {$ENDIF}

  ARPrinter, ARResStrings, ARUtils;

type
  {$IFDEF CLX}
    TParControl         = class(TWidgetControl);
  {$ELSE}
    TParControl         = class(TWinControl);
  {$ENDIF}

  THAlign       = (ahLeft, ahRight,  ahCenter);
  TVAlign       = (avTop,  avBottom, avCenter);
  TFrameStyles  = (frLeft, frRight,  frTop, frBottom);
  TFrameStyle   = set of TFrameStyles;
  TLineStyle    = (cbsSolid, cbsDash, cbsDot, cbsDashDot, cbsDashDotDot, cbsDouble11, cbsDouble21, cbsDouble12);

const
  CellBorderWidths:array[cbsSolid .. cbsDouble12] of integer = (1, 1, 1, 1, 1, 3, 4, 4);

  CellBorderDots:array[cbsSolid .. cbsDouble12, 0..7] of integer = (
    ( 10000000,  10000000,  0,  0,  0,  0,  0,  0), //solid
    (       18,         6,  0,  0,  0,  0,  0,  0), //dash
    (        3,         3,  3,  3,  3,  3,  3,  3), //dot
    (        9,         6,  3,  6,  0,  0,  0,  0), //dashdot
    (        9,         3,  3,  3,  3,  3,  0,  0), //dashdotdot
    ( 10000000,  10000000,  0,  0,  0,  0,  0,  0), //Double11
    ( 10000000,  10000000,  0,  0,  0,  0,  0,  0), //Double21
    ( 10000000,  10000000,  0,  0,  0,  0,  0,  0)  //Double12
  );

  CellBorderNames :array[cbsSolid .. cbsDouble12] of string  = (
    'Solid',
    'Dash',
    'Dot',
    'DashDot',
    'DashDotDot',
    'Double11',
    'Double21',
    'Double12'
  );

function  GetBorderStyleByName(const Name:string):TLineStyle;

type
  TSelectedCellStyle  = (scsNone, scsInvert, scsGray, scsSelected);
  TFocusedCellStyle   = (fcsNone, fcsRect, fcsRoundRect, fcsBlackAnt, fcsRedAnt);

  TARUnits    = (uPix, uMM, uCm, uIn);

  TARBandState  = set of (arbsBeginPara, arbsEndPara, arbsPageBreak, arbsResetPageNumber);

  TARRepOption = (roEditing);
  TARRepOptions = set of TARRepOption;

  TARSelectCellEvent      = procedure(Sender:TObject; IdxBand:integer; IdxCell:integer) of object;
  TARSelectBandEvent      = procedure(Sender:TObject; IdxBand:integer) of object;
  TARResizeBandEvent      = procedure(Sender:TObject; IdxBand:integer) of object;
  TARResizeCellEvent      = procedure(Sender:TObject; IdxBand:integer; IdxCell:integer) of object;
  TARCanEditEvent         = procedure(Sender:TObject; IdxBand:integer; IdxCell:integer; var CanEdit:boolean) of object;
//  TARItemChangeEvent      = procedure(Sender:TObject; IdxItem:integer) of object;
  TARShowEditorEvent      = procedure(Sender:TObject; ch:char) of object;
  TARDrawBandTitleEvent   = procedure(Sender:TObject; IdxBand:integer; Canvas:TCanvas; r:TRect) of object;
  TARExpandBandEvent      = procedure(Sender:TObject; IdxBand:integer) of object;
  TARCollapseBandEvent    = procedure(Sender:TObject; IdxBand:integer) of object;

  TARBandChangeEvent      = procedure(band:TObject) of object;
  TARCellChangeEvent      = procedure(band:TObject; cell:TObject) of object;

var
  ARMoneySeparator:char = '.';

const
  DefFontName     = 'ARIAL';
  DefFontSize     = 10;
  DefFontColor    = clBlack;
  DefFontStyle    = [];
  DefFontCharset  = DEFAULT_CHARSET;

  DefFrameColor   = clBlack;
  DefFrameWidth   = 1;
  DefFrameStyle   = [frLeft, frRight,  frTop, frBottom];

  DefBkColor      = clWhite;
  DefHAlign       = ahCenter;
  DefVAlign       = avTop;
  DefHeight       = 25;
  DefWidth        = 100;
  DefWordWrap     = true;
  DefAutoStretch  = true;
  DefTile         = false;

  DefCellValue    = 'Cell';
  DefBandName     = 'Band';
  DefRepName      = 'Report';

  DefOrientation  = poPortrait;
{$IFDEF CLX}
  DefPaperSize    = psA4;
{$ELSE}
  DefPaperSize    = DMPAPER_A4;
{$ENDIF}
  DefPageSizeName = 'A4';

  DefLeftMargin   = 0;
  DefTopMargin    = 0;
  DefRightMargin  = 0;
  DefBottomMargin = 0;

const
  BeginVarToken = '[';
  EndVarToken   = ']';

procedure DrawBorder(Canvas:TCanvas; r:TRect; ls:TLineStyle; fs:TFrameStyles; Color:TColor; BgColor:TColor; Width:integer; WidthLeftTop:integer; WidthRightBottom:integer);

procedure DrawAngleText(Canvas:TCanvas; r:TRect; text:string; HA:THAlign; VA:TVAlign; WordWrap:boolean; angle:integer);

{$IFDEF D3}
procedure DrawRectText(Canvas:TCanvas; s:string; r:TRect; ClipRect:TRect; HA:THAlign; VA:TVAlign; WordWrap:boolean; IsRTF:boolean; TextAngle:integer);
{$ELSE}
procedure DrawRectText(Canvas:TCanvas; s:string; r:TRect; ClipRect:TRect; HA:THAlign; VA:TVAlign; WordWrap:boolean; IsRTF:boolean; TextAngle:integer = 0);
{$ENDIF}

function  TextHeight(Canvas:TCanvas; s:string; width:integer; HA:THAlign; VA:TVAlign; WordWrap:boolean; IsRTF:boolean; FirstChar:integer):integer;

{$IFDEF D3}
procedure PrintRectText(Canvas:TCanvas; s:string; r:TRect; ClipRect:TRect; HA:THAlign; VA:TVAlign; WordWrap:boolean; IsRTF:boolean; var PrintChar:integer; TextAngle:integer);
{$ELSE}
procedure PrintRectText(Canvas:TCanvas; s:string; r:TRect; ClipRect:TRect; HA:THAlign; VA:TVAlign; WordWrap:boolean; IsRTF:boolean; var PrintChar:integer; TextAngle:integer = 0);
{$ENDIF}

function  Between(a:integer; amin:integer; adelta:integer):boolean;

function  UnitRatio(Units:TARUnits):extended;
function  ConvertUnits(Val:extended; UnitsFrom:TARUnits; UnitsTo:TARUnits):extended;
function  ConvertTo(Val:extended; UnitsTo:TARUnits):extended;
function  ConvertFrom(Val:extended; UnitsFrom:TARUnits):extended;

function  UnitsName(Units:TARUnits):string;
function  UnitsShortName(Units:TARUnits):string;

function  ToPix(Val:extended):integer;
function  ToInch(Val:extended):extended;
function  ToCm(Val:extended):extended;
function  ToMm(Val:extended):extended;

function  FromPix(Val:extended):extended;
function  FromInch(Val:extended):extended;
function  FromCm(Val:extended):extended;
function  FromMm(Val:extended):extended;

function  GraphicToString(Graphic:TGraphic):string;
procedure StringToGraphic(s:string; Graphic:TGraphic);

//procedure _AddRefRTF;
//procedure _ReleaseRTF;

type
{$IFDEF CLX}
  TARRTF = class(TComponent)
{$ELSE}
  TARRTF = class(TRichEdit)
{$ENDIF}
  public
    constructor Create(AOwner:TComponent);override;
    function    TextHeight(ACanvas:TCanvas; s:string; AWidth:integer; AWordWrap:boolean; FirstChar:integer):integer;
    procedure   DrawText(ACanvas:TCanvas; s:string; r:TRect; ClipRect:TRect; HA:THAlign; AWordWrap:boolean);
    procedure   PrintText(ACanvas:TCanvas; s:string; r:TRect; ClipRect:TRect; HA:THAlign; AWordWrap:boolean; var PrintChar:integer);
  end;

var RTF:TARRTF;

implementation

function  GetBorderStyleByName(const Name:string):TLineStyle;

begin
  for Result:=Low(TLineStyle) to High(TLineStyle) do begin
    if CT(Name, CellBorderNames[Result]) then exit;
  end;

  Result:=cbsSolid;
end;

// Returns the ratio of units to the tenth of a millimeter
function  UnitRatio(Units:TARUnits):extended;
begin
  case Units of
    uPix: Result:=254 / Screen.PixelsPerInch;
    uMM : Result:=10;
    uCm : Result:=100;
    uIn : Result:=254;
  else    Result:=100;
  end;
end;

function  ConvertUnits(Val:extended; UnitsFrom:TARUnits; UnitsTo:TARUnits):extended;
begin
  if UnitsFrom = UnitsTo then begin
    Result:=Val;
  end else begin
//    Result:=Val / UnitRatio(UnitsFrom) * UnitRatio(UnitsTo);
    Result:=Val * UnitRatio(UnitsFrom) / UnitRatio(UnitsTo);
  end;
end;

function  ConvertTo(Val:extended; UnitsTo:TARUnits):extended;
begin
  Result:=Val / UnitRatio(UnitsTo);
end;

function  ConvertFrom(Val:extended; UnitsFrom:TARUnits):extended;
begin
  Result:=Val * UnitRatio(UnitsFrom);
end;

function  UnitsName(Units:TARUnits):string;
begin
  case Units of
    uPix: Result:=rsAREditor_UnitPixels;
    uMM : Result:=rsAREditor_UnitMillimeters;
    uCm : Result:=rsAREditor_UnitCentimeters;
    uIn : Result:=rsAREditor_UnitInches;
  end;
end;

function  UnitsShortName(Units:TARUnits):string;
begin
  case Units of
    uPix: Result:=rsAREditor_UnitShortPixels;
    uMM : Result:=rsAREditor_UnitShortMillimeters;
    uCm : Result:=rsAREditor_UnitShortCentimeters;
    uIn : Result:=rsAREditor_UnitShortInches;
  end;
end;

function  ToPix(Val:extended):integer;
begin
  Result:=Round(Val / UnitRatio(uPix));
end;
function  ToInch(Val:extended):extended;
begin
  Result:=Round(Val / UnitRatio(uIn));
end;
function  ToCm(Val:extended):extended;
begin
  Result:=Round(Val / UnitRatio(uCm));
end;
function  ToMm(Val:extended):extended;
begin
  Result:=Round(Val / UnitRatio(uMm));
end;

function  FromPix(Val:extended):extended;
begin
  Result:=Round(UnitRatio(uPix) * Val);
end;
function  FromInch(Val:extended):extended;
begin
  Result:=UnitRatio(uIn) * Val;
end;
function  FromCm(Val:extended):extended;
begin
  Result:=UnitRatio(uCM) * Val;
end;
function  FromMm(Val:extended):extended;
begin
  Result:=UnitRatio(uMM) * Val;
end;


////////////////////////////////////////////////
function  Between(a:integer; amin:integer; adelta:integer):boolean;
begin
  if (a >= amin) and (a <= (amin + adelta)) then Result:=true else Result:=false;
end;

procedure SplitStringSpace(s:string; sl:TAIStringList);
var n, len:integer;
    ss:string;
begin
  sl.Clear;

  len:=Length(s);
  if len > 0 then begin
    n:=1;
    ss:='';
    repeat
      if s[n] <> ' ' then begin
        ss:=ss + s[n];
        inc(n);
        if (n <= len) and (s[n] = ' ') then begin
          sl.Add(ss);
          ss:='';
        end;
      end else begin
        sl.Add(' ');
        ss:='';
        inc(n);
      end;
    until n > len;
    if Length(ss) > 0 then sl.Add(ss);
  end;
end;

procedure SplitString(s:string; del:string; sl:TAIStringList);
begin
  sl.Text:=s;
end;

procedure WrapString(sl:TAIStringList; Canvas:TCanvas; width:integer);
var i:integer;
    c,c1:string;
    cl:TAIStringList;
begin
  if sl.Count = 0 then exit;

  cl:=TAIStringList.Create('WrapString cl');
  try
    i:=0;
    while i < sl.Count do begin
      if Canvas.TextWidth(sl[i]) > width then begin

        SplitStringSpace(sl[i], cl);
        sl.Delete(i);
        while cl.Count > 0 do begin
          c :=cl[0];
          c1:=cl[0];
          cl.Delete(0);
          while cl.Count > 0 do begin
            c:=c+cl[0];
            if (Canvas.TextWidth(c) > width) then break;
            c1:=c1+cl[0];
            cl.Delete(0);
          end;
          sl.Insert(i, c1);
          inc(i);
        end;
      end else begin
        inc(i);
      end;
    end;
  finally
    cl.Free;
  end;
end;

function  GraphicToString(Graphic:TGraphic):string;
var ms:TMemoryStream;
    b:byte;
begin
  ms:=TMemoryStream.Create;
  try
    Graphic.SaveToStream(ms);
    ms.Seek(0, soFromBeginning);

    Result:='';
    while ms.Read(b, 1) > 0 do begin
      Result:=Result + Format('%.2x', [b]);
    end;
  finally
    ms.Free;
  end;
end;

procedure StringToGraphic(s:string; Graphic:TGraphic);
var ms:TMemoryStream;
    b:byte;
    i:integer;
begin
  ms:=TMemoryStream.Create;
  try
    i:=1;
    while i <= Length(s) do begin
      if s[i] <= '9' then
        b:=(Ord(s[i]) - Ord('0')) and $0f
      else
        b:=(Ord(s[i]) - Ord('A') + 10) and $0f;
      inc(i);

      if s[i] <= '9' then
        b:=(b shl 4) or (Ord(s[i]) - Ord('0')) and $0f
      else
        b:=(b shl 4) or (Ord(s[i]) - Ord('A') + 10) and $0f;
      inc(i);

      ms.Write(b, 1);
    end;

    ms.Seek(0, soFromBeginning);
    Graphic.LoadFromStream(ms);
  finally
    ms.Free;
  end;
end;

{$IFDEF D3}
procedure PrintRectText(Canvas:TCanvas; s:string; r:TRect; ClipRect:TRect; HA:THAlign; VA:TVAlign; WordWrap:boolean; IsRTF:boolean; var PrintChar:integer; TextAngle:integer);
{$ELSE}
procedure PrintRectText(Canvas:TCanvas; s:string; r:TRect; ClipRect:TRect; HA:THAlign; VA:TVAlign; WordWrap:boolean; IsRTF:boolean; var PrintChar:integer; TextAngle:integer = 0);
{$ENDIF}
begin
  if IsRTF then begin
    RTF.PrintText(Canvas, s, r, ClipRect, HA, WordWrap, PrintChar);
  end else begin
    DrawRectText(Canvas, s, r, ClipRect, HA, VA, WordWrap, IsRTF, 0);
  end;
end;

function  TextHeight(Canvas:TCanvas; s:string; width:integer; HA:THAlign; VA:TVAlign; WordWrap:boolean; IsRTF:boolean; FirstChar:integer):integer;
var sl:TAIStringList;
begin
  if IsRTF then begin
    Result:=RTF.TextHeight(Canvas, s, width, WordWrap, FirstChar);
  end else begin
    sl:=TAIStringList.Create('TextHeight sl');
    try
      SplitString(s, LF, sl);

      if WordWrap then begin
        WrapString(sl, Canvas, width);
      end;
      Result:=0;
      if sl.Count > 0 then begin
        Result:=sl.Count * (Canvas.TextHeight(sl[0]) + 1);
      end;
    finally
      sl.Free;
    end;
  end;
end;

/////////////////////////////////////// TARRTF ///////////////////////////////////
constructor TARRTF.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);

{$IFDEF CLX}
{$ELSE}
  Visible:=false;
  Parent:=TWinControl(AOwner);
{$ENDIF}
end;

function    TARRTF.TextHeight(ACanvas:TCanvas; s:string; AWidth:integer; AWordWrap:boolean; FirstChar:integer):integer;
{$IFDEF CLX}
{$ELSE}
var Range: TFormatRange;
    SaveRect:TRect;
    LogX, LogY, OldMap:integer;
{$ENDIF}
begin
{$IFDEF CLX}
  Result:=ARRepCommon.TextHeight(ACanvas, s, AWidth, ahLeft, avTop, AWordWrap, false, FirstChar);
{$ELSE}
//  if not Assigned(Parent) then Parent:=Application.MainForm;

  WordWrap:=AWordWrap;
  Clear;
  Font.Assign(ACanvas.Font);
  SelText:=s;

  FillChar(Range, SizeOf(TFormatRange), 0);

  Range.hdc       :=ACanvas.Handle;
  Range.hdcTarget :=ACanvas.Handle;

  LogX := GetDeviceCaps(Range.hdc, LOGPIXELSX);
  LogY := GetDeviceCaps(Range.hdc, LOGPIXELSY);

  SaveRect.Left   := 0;
  SaveRect.Right  := AWidth * 1440 div LogX;
  SaveRect.Top    := 0;
  SaveRect.Bottom := High(integer) div 4096; // !!! not work less then 128  :-((((

  OldMap := SetMapMode(Range.hdc, MM_TEXT);
  Perform(EM_FORMATRANGE, 0, 0);    // flush buffer
  try
    Range.rc        :=SaveRect;
    Range.rcPage    :=SaveRect;
    Range.chrg.cpMin:=FirstChar;
    Range.chrg.cpMax:=-1;
    Result:=Perform(EM_FORMATRANGE, 0, Longint(@Range));
    Result:=(Range.rc.Bottom * LogY  div 1440) + 0;
  finally
    Perform(EM_FORMATRANGE, 0, 0);  // flush buffer
    SetMapMode(Range.hdc, OldMap);  // restore previous map mode
    Clear;
  end;
{$ENDIF}
end;

procedure   TARRTF.DrawText(ACanvas:TCanvas; s:string; r:TRect; ClipRect:TRect; HA:THAlign; AWordWrap:boolean);
{$IFDEF CLX}
{$ELSE}
var Range: TFormatRange;
    LogX, LogY, OldMap:integer;
    SaveRect:TRect;
{$ENDIF}
begin
{$IFDEF CLX}
  ARRepCommon.DrawRectText(ACanvas, s, r, ClipRect, HA, avTop, AWordWrap, false, 0);
{$ELSE}
//  if not Assigned(Parent) then Parent:=Application.MainForm;

  WordWrap:=AWordWrap;
  Alignment:=TAlignment(HA);
  Clear;
  Font.Assign(ACanvas.Font);
  SelText:=s;

  FillChar(Range, SizeOf(TFormatRange), 0);
  Range.hdc       := ACanvas.Handle;
  Range.hdcTarget := ACanvas.Handle;

  LogX := GetDeviceCaps(Range.hdc, LOGPIXELSX);
  LogY := GetDeviceCaps(Range.hdc, LOGPIXELSY);

  SaveRect.Left   := r.Left   * 1440 div LogX;
  SaveRect.Right  := r.Right  * 1440 div LogX;
  SaveRect.Top    := r.Top    * 1440 div LogY;
  SaveRect.Bottom := r.Bottom * 1440 div LogY;

  Range.rc    := SaveRect;
  Range.rcPage:= SaveRect;

  Range.chrg.cpMin := 0;
  Range.chrg.cpMax := -1;

  OldMap := SetMapMode(Range.hdc, MM_TEXT);

  Perform(EM_FORMATRANGE, 0, 0);    // flush buffer
  try
    Perform(EM_FORMATRANGE, 1, Longint(@Range));
  finally
    Perform(EM_FORMATRANGE, 0, 0);  // flush buffer
    SetMapMode(Range.hdc, OldMap);  // restore previous map mode
    Clear;
  end;
{$ENDIF}
end;
procedure   TARRTF.PrintText(ACanvas:TCanvas; s:string; r:TRect; ClipRect:TRect; HA:THAlign; AWordWrap:boolean; var PrintChar:integer);
{$IFDEF CLX}
{$ELSE}
var Range: TFormatRange;
    LogX, LogY, OldMap, MaxChar, LastPrintChar:integer;
    SaveRect:TRect;
    Bottom, Res:integer;

  function  RX(Render:integer):integer;
  begin
    SaveRect.Bottom:=Bottom * 1440 div LogY;
    Range.rc        :=SaveRect;
    Range.rcPage    :=SaveRect;
    Range.chrg.cpMin:=PrintChar;
    Range.chrg.cpMax:=MaxChar;
    LastPrintChar   :=Perform(EM_FORMATRANGE, Render, Longint(@Range));
    Result:=Range.rc.Bottom * LogY div 1440;
  end;
{$ENDIF}
begin
{$IFDEF CLX}
  ARRepCommon.PrintRectText(ACanvas, s, r, ClipRect, HA, avTop, AWordWrap, false, PrintChar, 0);
{$ELSE}
//  if not Assigned(Parent) then Parent:=Application.MainForm;

  WordWrap:=AWordWrap;
  Alignment:=TAlignment(HA);
  Clear;
  Font.Assign(ACanvas.Font);
  SelText:=s;

  FillChar(Range, SizeOf(TFormatRange), 0);
  Range.hdc       := ACanvas.Handle;
  Range.hdcTarget := ACanvas.Handle;

  LogX := GetDeviceCaps(Range.hdc, LOGPIXELSX); LogY := GetDeviceCaps(Range.hdc, LOGPIXELSY);

  SaveRect.Left  :=r.Left   * 1440 div LogX;    SaveRect.Right :=r.Right  * 1440 div LogX;

  SaveRect.Top   :=ClipRect.Top    * 1440 div LogY;

  OldMap:=SetMapMode(Range.hdc, MM_TEXT);
  Perform(EM_FORMATRANGE, 0, 0);    // flush buffer
  try
    Bottom:=ClipRect.Bottom;
    MaxChar:=PrintChar + 1;
    Res:=RX(0);
    if Res <= ClipRect.Bottom then begin  // we can print at least one line
      MaxChar:=-1;
      for Bottom:=ClipRect.Bottom downto ClipRect.Top do begin
        Res:=RX(0);
        if Res <= ClipRect.Bottom then begin
          RX(1);
          break;
        end;
      end;
      PrintChar:=LastPrintChar;
    end;
  finally
    Perform(EM_FORMATRANGE, 0, 0);  // flush buffer
    SetMapMode(Range.hdc, OldMap);  // restore previous map mode
    Clear;
  end;
{$ENDIF}
end;

{$IFDEF D3}
procedure DrawRectText(Canvas:TCanvas; s:string; r:TRect; ClipRect:TRect; HA:THAlign; VA:TVAlign; WordWrap:boolean; IsRTF:boolean; TextAngle:integer);
{$ELSE}
procedure DrawRectText(Canvas:TCanvas; s:string; r:TRect; ClipRect:TRect; HA:THAlign; VA:TVAlign; WordWrap:boolean; IsRTF:boolean; TextAngle:integer = 0);
{$ENDIF}
var i,maxh,h,x,y:integer;
    sl:TAIStringList;
    {$IFDEF CLX}
      intr:TRect;
    {$ENDIF}
begin
  if IsRTF then begin
    RTF.DrawText(Canvas, s, r, ClipRect, HA, WordWrap);
  end else begin
    if TextAngle <> 0 then begin
      DrawAngleText(Canvas, r, s, HA, VA, WordWrap, TextAngle);
    end else begin
      sl:=TAIStringList.Create('DrawRectText sl');
      try
        r.Left  :=r.Left + 1;
        r.Top   :=r.Top + 1;
        r.Right :=r.Right - 0;
        r.Bottom:=r.Bottom - 0;

        SplitString(s, LF, sl);

        if WordWrap then begin
          WrapString(sl, Canvas, r.Right - r.Left);
        end;

        maxh:=0;
        if sl.Count > 0 then begin
          for i:=0 to sl.Count - 1 do begin
            h:=Canvas.TextHeight(sl[i]);
            if h > maxh then maxh:=h;
          end;

          h:=maxh * sl.Count;

          Canvas.Brush.Style:=bsClear;

          for i:=0 to sl.Count - 1 do begin
            case VA of
              avTop   :  y:=r.Top + maxh * i;
              avCenter:  y:=r.Top + (r.Bottom - r.Top - h) div 2 + maxh * i;
              avBottom:  y:=r.Bottom - h + maxh * i;
            else         y:=r.Top + maxh * i;
            end;

            case HA of
              ahLeft  :  x:=r.Left;
              ahCenter:  x:=r.Left + (r.Right - r.Left - Canvas.TextWidth(sl[i])) div 2;
              ahRight :  x:=r.Right - Canvas.TextWidth(sl[i]) - 2; //CD avoid to cut last char
//              ahRight :  x:=r.Right - Canvas.TextWidth(sl[i]);
            else         x:=r.Left;
            end;

            {$IFDEF CLX}
              if IntersectRect(intr, r, ClipRect) then begin
                Canvas.TextRect(intr, x, y, sl[i]);
              end;
            {$ELSE}
              Canvas.TextRect(r, x, y, sl[i]);
            {$ENDIF}
          end;
        end;
      finally
        sl.Free;
      end;
    end;
  end;
end;

{
procedure DrawAngleText(Canvas:TCanvas; r:TRect; angle:integer; text:string; ha:THAlign; va:TVAlign);
var hfnt, hfntPrev:HFONT;
    plf:TLogFont;
    wsrc, hsrc, w, h, rx, ry, tx, ty:integer;
    sl:TAIStringList;
begin
  while angle < 0 do angle:=angle + 360;
  angle:=angle mod 360;

  wsrc:=Canvas.TextWidth(text);
  hsrc:=Canvas.TextHeight(text);

  with plf do begin
    lfHeight        :=Canvas.Font.Height;
    lfWidth         :=0;
    lfEscapement    :=angle * 10;
    lfOrientation   :=angle * 10;
    if fsBold in Canvas.Font.Style then begin
      lfWeight        :=FW_BOLD;
    end else begin
      lfWeight        :=FW_NORMAL;
    end;

    lfItalic        :=byte(fsItalic in Canvas.Font.Style);
    lfUnderline     :=byte(fsUnderline in Canvas.Font.Style);
    lfStrikeOut     :=byte(fsStrikeOut in Canvas.Font.Style);
    lfCharSet       :=byte(Canvas.Font.Charset);
    lfQuality       :=DEFAULT_QUALITY;
    lfOutPrecision  :=OUT_DEFAULT_PRECIS;
    lfClipPrecision :=CLIP_DEFAULT_PRECIS;
    case Canvas.Font.Pitch of
      fpVariable: lfPitchAndFamily := VARIABLE_PITCH;
      fpFixed:    lfPitchAndFamily := FIXED_PITCH;
      else        lfPitchAndFamily := DEFAULT_PITCH;
    end;
    StrPCopy(lfFaceName, Canvas.Font.Name)
  end;

  SetBkMode(Canvas.Handle, TRANSPARENT);

  tx:=0; ty:=0; w:=0; h:=0; rx:=0; ry:=0;

  case angle of
    0:    begin tx:=0;    ty:=0;    w:=wsrc; h:=hsrc; end;
    90:   begin tx:=0;    ty:=wsrc; w:=hsrc; h:=wsrc; end;
    180:  begin tx:=wsrc; ty:=hsrc; w:=wsrc; h:=hsrc; end;
    270:  begin tx:=hsrc; ty:=0;    w:=hsrc; h:=wsrc; end;
  end;

  case ha of
    ahLeft  : rx:=r.Left + tx;
    ahRight : rx:=r.Right - w + tx;
    ahCenter: rx:=r.Left + (r.Right - r.Left - w) div 2 + tx;
  end;

  case va of
    avTop   : ry:=r.Top + ty;
    avBottom: ry:=r.Bottom - h + ty;
    avCenter: ry:=r.Top + (r.Bottom - r.Top - h) div 2 + ty;
  end;

  hfnt    :=CreateFontIndirect(plf);
  hfntPrev:=SelectObject(Canvas.Handle, hfnt);
  try
    TextOut(Canvas.Handle, rx, ry, PChar(text), Length(text));
  finally
    SelectObject(Canvas.Handle, hfntPrev);
    DeleteObject(hfnt);
    SetBkMode(Canvas.Handle, OPAQUE);
  end;
end;
}

procedure DrawAngleText(Canvas:TCanvas; r:TRect; text:string; HA:THAlign; VA:TVAlign; WordWrap:boolean; angle:integer);
{$IFDEF CLX}
{$ELSE}
var hfnt, hfntPrev:HFONT;
    plf:TLogFont;
    i, wsrc, hsrc, w, h, rx, ry, tx, ty, dirx, diry, maxh:integer;
    sl:TAIStringList;
{$ENDIF}
begin
{$IFDEF CLX}
  ARRepCommon.DrawRectText(Canvas, text, r, r, HA, VA, WordWrap, false, 0);
{$ELSE}
  while angle < 0 do angle:=angle + 360;
  angle:=angle mod 360;

  r.Left  :=r.Left + 2;
  r.Top   :=r.Top + 2;
  r.Right :=r.Right - 1;
  r.Bottom:=r.Bottom - 1;

  sl:=TAIStringList.Create('DrawRectText sl');
  try
    SplitString(text, LF, sl);

    if WordWrap then begin
      if (angle mod 180) = 0 then begin
        WrapString(sl, Canvas, r.Right - r.Left);
      end else begin
        WrapString(sl, Canvas, r.Bottom - r.Top);
      end;
    end;

    maxh:=0;
    if sl.Count > 0 then begin
      for i:=0 to sl.Count - 1 do begin
        sl.Objects[i]:=pointer(Canvas.TextWidth(sl[i]));
      end;

      for i:=0 to sl.Count - 1 do begin
        maxh:=Max(maxh, Canvas.TextHeight(sl[i]));
      end;
      hsrc:=maxh * sl.Count;

      with plf do begin
        lfHeight        :=Canvas.Font.Height;
        lfWidth         :=0;
        lfEscapement    :=angle * 10;
        lfOrientation   :=angle * 10;
        if fsBold in Canvas.Font.Style then begin
          lfWeight        :=FW_BOLD;
        end else begin
          lfWeight        :=FW_NORMAL;
        end;

        lfItalic        :=byte(fsItalic in Canvas.Font.Style);
        lfUnderline     :=byte(fsUnderline in Canvas.Font.Style);
        lfStrikeOut     :=byte(fsStrikeOut in Canvas.Font.Style);
        lfCharSet       :=byte(Canvas.Font.Charset);
        lfQuality       :=DEFAULT_QUALITY;
        lfOutPrecision  :=OUT_DEFAULT_PRECIS;
        lfClipPrecision :=CLIP_DEFAULT_PRECIS;
        case Canvas.Font.Pitch of
          fpVariable: lfPitchAndFamily := VARIABLE_PITCH;
          fpFixed:    lfPitchAndFamily := FIXED_PITCH;
          else        lfPitchAndFamily := DEFAULT_PITCH;
        end;
        StrPCopy(lfFaceName, Canvas.Font.Name)
      end;

      SetBkMode(Canvas.Handle, TRANSPARENT);

      tx:=0; ty:=0; w:=0; h:=0; rx:=0; ry:=0; dirx:=0; diry:=0;

      Canvas.Brush.Style:=bsClear;

      hfnt    :=CreateFontIndirect(plf);
      hfntPrev:=SelectObject(Canvas.Handle, hfnt);
      try
        for i:=0 to sl.Count - 1 do begin
          wsrc:=integer(sl.Objects[i]);
          case angle of
            0:    begin tx:=0;    ty:=0;    w:=wsrc; h:=hsrc; dirx:=    0; diry:= maxh; end;
            90:   begin tx:=0;    ty:=wsrc; w:=hsrc; h:=wsrc; dirx:= maxh; diry:=    0; end;
            180:  begin tx:=wsrc; ty:=hsrc; w:=wsrc; h:=hsrc; dirx:=    0; diry:=-maxh; end;
            270:  begin tx:=hsrc; ty:=0;    w:=hsrc; h:=wsrc; dirx:=-maxh; diry:=    0; end;
          end;

          case ha of
            ahLeft  : rx:=r.Left;
            ahRight : rx:=r.Right - w;
            ahCenter: rx:=r.Left + (r.Right - r.Left - w) div 2;
          end;
          rx:=rx + tx + dirx * i;

          case va of
            avTop   : ry:=r.Top;
            avBottom: ry:=r.Bottom - h;
            avCenter: ry:=r.Top + (r.Bottom - r.Top - h) div 2;
          end;
          ry:=ry + ty + diry * i;

          TextOut(Canvas.Handle, rx, ry, PChar(sl[i]), Length(sl[i]));
        end;
      finally
        SelectObject(Canvas.Handle, hfntPrev);
        DeleteObject(hfnt);
        SetBkMode(Canvas.Handle, OPAQUE);
      end;
    end;
  finally
    sl.Free();
  end;
{$ENDIF}
end;

procedure DrawBorder(Canvas:TCanvas; r:TRect; ls:TLineStyle; fs:TFrameStyles; Color:TColor; BgColor:TColor; Width:integer; WidthLeftTop:integer; WidthRightBottom:integer);
  procedure HorLine(x1, x2, y:integer);
  var n, cnt, x, xend:integer;
  begin
//  Canvas.FillRect(Rect(x1, y, x2 + 1, y + 1));

    n   :=0;
    cnt :=0;
    x   :=x1;
    while x < x2 do begin
      xend:=Min(x + CellBorderDots[ls, n], x2 + 1);
      Canvas.FillRect(Rect(x, y, xend, y + 1));
      cnt:=cnt + CellBorderDots[ls, n] + CellBorderDots[ls, n + 1];
      x:=x + CellBorderDots[ls, n] + CellBorderDots[ls, n + 1];
      n:=n + 2;
      if cnt >= 24 then begin
        n   :=0;
        cnt :=0;
      end;
    end;
  end;

  procedure VerLine(x, y1, y2:integer);
  var n, cnt, y, yend:integer;
  begin
//    Canvas.FillRect(Rect(x, y1, x + 1, y2 + 1));

    n   :=0;
    cnt :=0;
    y   :=y1;
    while y < y2 do begin
      yend:=Min(y + CellBorderDots[ls, n], y2 + 1);
      Canvas.FillRect(Rect(x, y, x + 1, yend));
      cnt:=cnt + CellBorderDots[ls, n] + CellBorderDots[ls, n + 1];
      y:=y + CellBorderDots[ls, n] + CellBorderDots[ls, n + 1];
      n:=n + 2;
      if cnt >= 24 then begin
        n   :=0;
        cnt :=0;
      end;
    end;
  end;

var wlt1, wlt2, wrb1, wrb2:integer;
begin
  if WidthLeftTop <= 0 then begin
    wlt1:=0;
    wlt2:=0;
  end else begin
    wlt1:=WidthLeftTop - 1;
    wlt2:=WidthLeftTop - 2;
  end;
  if WidthRightBottom <= 0 then begin
    wrb1:=0;
    wrb2:=0;
  end else begin
    wrb1:=WidthRightBottom - 1;
    wrb2:=WidthRightBottom - 2;
  end;

  Canvas.Brush.Style:=bsSolid;
  Canvas.Brush.Color:=Color;

  begin
    case ls of
      cbsSolid:
        begin
          case fs of
            frLeft    : Canvas.FillRect(Rect(r.Left,              r.Top,                r.Left + Width, r.Bottom + 1));
            frRight   : Canvas.FillRect(Rect(r.Right + 1 - Width, r.Top,                r.Right + 1,    r.Bottom + 1));
            frTop     : Canvas.FillRect(Rect(r.Left,              r.Top,                r.Right + 1,    r.Top + Width));
            frBottom  : Canvas.FillRect(Rect(r.Left,              r.Bottom + 1 - Width, r.Right + 1,    r.Bottom + 1));
          end;
        end;
      cbsDash,
      cbsDot,
      cbsDashDot,
      cbsDashDotDot :
        begin
          case fs of
            frLeft    : VerLine(r.Left,   r.Top,    r.Bottom);
            frRight   : VerLine(r.Right,  r.Top,    r.Bottom);
            frTop     : HorLine(r.Left,   r.Right,  r.Top);
            frBottom  : HorLine(r.Left,   r.Right,  r.Bottom);
          end;
        end;
      cbsDouble11:
        begin
          case fs of
            frLeft    :
              begin
                VerLine(r.Left,         r.Top,          r.Bottom);
                VerLine(r.Left + 2,     r.Top + wlt1,   r.Bottom - wrb1);
              end;
            frRight   :
              begin
                VerLine(r.Right,        r.Top,          r.Bottom);
                VerLine(r.Right - 2,    r.Top + wlt1,   r.Bottom - wrb1);
              end;
            frTop     :
              begin
                HorLine(r.Left,         r.Right,        r.Top);
                HorLine(r.Left + wlt1,  r.Right - wrb1, r.Top + 2);
              end;
            frBottom  :
              begin
                HorLine(r.Left,         r.Right,        r.Bottom);
                HorLine(r.Left + wlt1,  r.Right - wrb1, r.Bottom - 2);
              end;
          end;
        end;
      cbsDouble21:
        begin
          case fs of
            frLeft    :
              begin
                VerLine(r.Left,         r.Top,          r.Bottom);
                VerLine(r.Left + 1,     r.Top,          r.Bottom);
                VerLine(r.Left + 3,     r.Top + wlt1,   r.Bottom - wrb1);
              end;
            frRight   :
              begin
                VerLine(r.Right,        r.Top,          r.Bottom);
                VerLine(r.Right - 1,    r.Top,          r.Bottom);
                VerLine(r.Right - 3,    r.Top + wlt1,   r.Bottom - wrb1);
              end;
            frTop     :
              begin
                HorLine(r.Left,         r.Right,        r.Top);
                HorLine(r.Left,         r.Right,        r.Top + 1);
                HorLine(r.Left + wlt1,  r.Right - wrb1, r.Top + 3);
              end;
            frBottom  :
              begin
                HorLine(r.Left,         r.Right,        r.Bottom);
                HorLine(r.Left,         r.Right,        r.Bottom - 1);
                HorLine(r.Left + wlt1,  r.Right - wrb1, r.Bottom - 3);
              end;
          end;
        end;
      cbsDouble12:
        begin
          case fs of
            frLeft    :
              begin
                VerLine(r.Left,         r.Top,          r.Bottom);
                VerLine(r.Left + 2,     r.Top + wlt2,   r.Bottom - wrb2);
                VerLine(r.Left + 3,     r.Top + wlt1,   r.Bottom - wrb1);
              end;
            frRight   :
              begin
                VerLine(r.Right,        r.Top,          r.Bottom);
                VerLine(r.Right - 2,    r.Top + wlt2,   r.Bottom - wrb2);
                VerLine(r.Right - 3,    r.Top + wlt1,   r.Bottom - wrb1);
              end;
            frTop     :
              begin
                HorLine(r.Left,         r.Right,        r.Top);
                HorLine(r.Left + wlt2,  r.Right - wrb2, r.Top + 2);
                HorLine(r.Left + wlt1,  r.Right - wrb1, r.Top + 3);
              end;
            frBottom  :
              begin
                HorLine(r.Left,         r.Right,        r.Bottom);
                HorLine(r.Left + wlt2,  r.Right - wrb2, r.Bottom - 2);
                HorLine(r.Left + wlt1,  r.Right - wrb1, r.Bottom - 3);
              end;
          end;
        end;
    end;
  end;
end;

/////////////////////////////////////// TARRTF ///////////////////////////////////
initialization
  ARMoneySeparator:=DecimalSeparator;
end.


