unit ARRulerBar;
//************************************************************//
//                         AND Report                        //
//  Eugene Hrulev (mrHru) Laxian Key, Omsk, Russia 1999,2001 //
//  Oleg Melnikov (muler) Laxian Key, Omsk, Russia 1999,2001 //
//************************************************************//

{$INCLUDE DEFINES.INC}

interface

uses
  Classes, SysUtils, Math,
  {$IFDEF CLX}
    QExtCtrls, QGraphics, QControls, Types,
  {$ELSE}
    ExtCtrls, Graphics, Controls,

    Windows, Forms,
  {$ENDIF}

  ARRepCommon, ARUtils;

type

  TARRulerBarOrientation = (rbHorizontal, rbVertical);

  TARRulerBar = class(TCustomControl)
//  TARRulerBar = class(TCustomPanel)
  private
    FOrientation:TARRulerBarOrientation;

    FGutter     :integer;

    FPageWidth  :extended;
    FLeftMargin :extended;
    FRightMargin:extended;
    FUnits      :TARUnits;
    FSelBegin   :extended;
    FSelEnd     :extended;
    FCur        :integer;

    FOnChange   :TNotifyEvent;
    FInUpdate   :integer;
    FModified   :boolean;

    FSelColor   :TColor;
    FRulerColor :TColor;

    procedure   SetSelColor(ASelColor:TColor);
    procedure   SetRulerColor(ARulerColor:TColor);

    procedure   SetOrientation(AOrientation:TARRulerBarOrientation);

    procedure   SetCur(ACur:integer);
    procedure   SetGutter(AGutter:integer);

    procedure   SetPageWidth(APageWidth:extended);
    procedure   SetLeftMargin(ALeftMargin:extended);
    procedure   SetRightMargin(ARightMargin:extended);
    procedure   SetUnits(AUnits:TARUnits);
    procedure   SetSelBegin(ASelBegin:extended);
    procedure   SetSelEnd(ASelEnd:extended);
  protected
    procedure   MouseMove(Shift: TShiftState; X, Y: Integer);override;
    procedure   MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);override;
    procedure   Loaded;override;
    procedure   Paint;override;
    procedure   DoUpdate;
    function    InUpdate:boolean;

    function    UnitsToText(Val:extended):string;
  public
    constructor Create(AOwner:TComponent);override;
    destructor  Destroy;override;

    procedure   BeginUpdate;
    procedure   EndUpdate;
  published
    property    Orientation:TARRulerBarOrientation  read  FOrientation  write SetOrientation;

    property    Gutter:integer                      read  FGutter       write SetGutter;

    property    PageWidth:extended                  read  FPageWidth    write SetPageWidth;
    property    LeftMargin:extended                 read  FLeftMargin   write SetLeftMargin;
    property    RightMargin:extended                read  FRightMargin  write SetRightMargin;
    property    Units:TARUnits                      read  FUnits        write SetUnits;
    property    SelBegin:extended                   read  FSelBegin     write SetSelBegin;
    property    SelEnd:extended                     read  FSelEnd       write SetSelEnd;
    property    Cur:integer                         read  FCur          write SetCur;

    property    SelColor:TColor                     read  FSelColor     write SetSelColor;
    property    RulerColor:TColor                   read  FRulerColor   write SetRulerColor;
    property    Color;
    property    Font;

    property    OnChange:TNotifyEvent               read  FOnChange     write FOnChange;
  end;

implementation
{$R *.res}

destructor  TARRulerBar.Destroy;
begin
  inherited;
end;

procedure   TARRulerBar.Loaded;
begin
  inherited;

  Caption:='';
end;

procedure   TARRulerBar.SetOrientation(AOrientation:TARRulerBarOrientation);
begin
  if FOrientation <> AOrientation then begin
    FOrientation:=AOrientation;
    case FOrientation of
      rbHorizontal: begin
                      Align :=alTop;
                      Height:=35;
                    end;
      rbVertical:   begin
                      Align:=alLeft;
                      Width:=35;
                    end;
    end;
    Invalidate;
  end;
end;

procedure   TARRulerBar.SetGutter(AGutter:integer);
begin
  if FGutter <> AGutter then begin
    FGutter:=AGutter;
    FModified:=true;
    DoUpdate;
  end;
end;

procedure   TARRulerBar.SetCur(ACur:integer);
begin
  if FCur <> ACur then begin
    FCur:=ACur;
    FModified:=true;
    DoUpdate;
  end;
end;

procedure   TARRulerBar.SetSelColor(ASelColor:TColor);
begin
  if FSelColor <> ASelColor then begin
    FSelColor:=ASelColor;
    FModified:=true;
    DoUpdate;
  end;
end;

procedure   TARRulerBar.SetRulerColor(ARulerColor:TColor);
begin
  if FRulerColor <> ARulerColor then begin
    FRulerColor:=ARulerColor;
    FModified:=true;
    DoUpdate;
  end;
end;

procedure   TARRulerBar.SetPageWidth(APageWidth:extended);
begin
  if FPageWidth <> APageWidth then begin
    FPageWidth:=APageWidth;
    FModified:=true;
    DoUpdate;
  end;
end;

procedure   TARRulerBar.SetLeftMargin(ALeftMargin:extended);
begin
  if FLeftMargin <> ALeftMargin then begin
    FLeftMargin:=ALeftMargin;
    FModified:=true;
    DoUpdate;
  end;
end;

procedure   TARRulerBar.SetRightMargin(ARightMargin:extended);
begin
  if FRightMargin <> ARightMargin then begin
    FRightMargin:=ARightMargin;
    FModified:=true;
    DoUpdate;
  end;
end;

procedure   TARRulerBar.SetUnits(AUnits:TARUnits);
begin
  if FUnits <> AUnits then begin
    FUnits:=AUnits;
    FModified:=true;
    DoUpdate;
  end;
end;

procedure   TARRulerBar.SetSelBegin(ASelBegin:extended);
begin
  if FSelBegin <> ASelBegin then begin
    FSelBegin:=ASelBegin;
    FModified:=true;
    DoUpdate;
  end;
end;

procedure   TARRulerBar.SetSelEnd(ASelEnd:extended);
begin
  if FSelEnd <> ASelEnd then begin
    FSelEnd:=ASelEnd;
    FModified:=true;
    DoUpdate;
  end;
end;

function    TARRulerBar.UnitsToText(Val:extended):string;
begin
  if (Units = uPix) or (Round(Val) = Val) then begin
    Result:=IntToStr(Round(Val));
  end else begin
    Result:=Format('%.2f', [Val]);
  end;
end;

type TMovedMargin = (mmNone, mmLeft, mmRight);

var DownX:integer;
    DownMargin:extended;
    MovedMargin:TMovedMargin;

procedure   TARRulerBar.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Orientation = rbHorizontal then begin
    if ssLeft in Shift then begin
      DownX:=X;
    end;
  end;
end;

procedure   TARRulerBar.MouseMove(Shift: TShiftState; X, Y: Integer);
var d:integer;
begin
  if Orientation = rbHorizontal then begin
    BeginUpdate;

      if not (ssLeft in Shift) then begin
        if Abs(X - (Gutter - ToPix(LeftMargin))) < 5 then begin
          Cursor      :=crHSplit;
          MovedMargin :=mmLeft;
          DownMargin  :=LeftMargin;
        end else begin
          if Abs(X - (Gutter - ToPix(LeftMargin) + ToPix(PageWidth) - ToPix(RightMargin))) < 5 then begin
            Cursor      :=crHSplit;
            MovedMargin :=mmRight;
            DownMargin  :=RightMargin;
          end else begin
            Cursor:= crDefault;
            MovedMargin :=mmNone;
          end;
        end;
      end else begin
        if ssLeft in Shift then begin
          d:= X - DownX;
          case MovedMargin of
            mmNone  : ;
            mmLeft  : begin
                        {$IFDEF D3}
                          LeftMargin  :=MaxEx(DownMargin - FromPix(d), 0);
                        {$ELSE}
                          LeftMargin  :=Max(DownMargin - FromPix(d), 0);
                        {$ENDIF}
                        if Assigned(FOnChange) then FOnChange(Self);
                      end;
            mmRight : begin
                        {$IFDEF D3}
                          RightMargin :=MaxEx(DownMargin - FromPix(d), 0);
                        {$ELSE}
                          RightMargin :=Max(DownMargin - FromPix(d), 0);
                        {$ENDIF}
                        if Assigned(FOnChange) then FOnChange(Self);
                      end;
          end;
        end;
      end;
      Cur:=X;

    EndUpdate;
  end else begin
    Cur:=Y;
  end;
end;

procedure   TARRulerBar.DoUpdate;
begin
  if (not InUpdate) and FModified then begin
    FModified:=false;
    {$IFDEF CLX}
      InvalidateRect(ClientRect, false);
    {$ELSE}
      Invalidate;
//      InvalidateRect( ClientRect, false);
    {$ENDIF}
  end;
end;

function    TARRulerBar.InUpdate:boolean;
begin
  Result:=FInUpdate <> 0;
end;

procedure   TARRulerBar.BeginUpdate;
begin
  inc(FInUpdate);
end;

procedure   TARRulerBar.EndUpdate;
begin
  dec(FInUpdate);
  DoUpdate;
end;

procedure ShowText(Canvas:TCanvas; r:TRect; angle:integer; text:string);
{$IFDEF CLX}
{$ELSE}
  var hfnt, hfntPrev:HFONT;
      plf:TLogFont;
      th:integer;
{$ENDIF}
begin
  {$IFDEF CLX}
  {$ELSE}
    th:=Canvas.TextWidth(text);

    with plf do begin
      lfHeight        :=Canvas.Font.Height;
      lfWidth         :=0;
      lfEscapement    :=angle * 10;
      lfOrientation   :=angle * 10;
      lfWeight        :=FW_NORMAL;

      lfItalic        :=byte(fsItalic in Canvas.Font.Style);
      lfUnderline     :=byte(fsUnderline in Canvas.Font.Style);
      lfStrikeOut     :=byte(fsStrikeOut in Canvas.Font.Style);
      lfCharSet       :=byte(Canvas.Font.Charset);
      lfQuality       :=DEFAULT_QUALITY;
      lfOutPrecision  :=OUT_DEFAULT_PRECIS;
      lfClipPrecision :=CLIP_DEFAULT_PRECIS;
      case Canvas.Font.Pitch of
        fpVariable: lfPitchAndFamily := VARIABLE_PITCH;
        fpFixed:    lfPitchAndFamily := FIXED_PITCH;
        else        lfPitchAndFamily := DEFAULT_PITCH;
      end;
      StrPCopy(lfFaceName, Canvas.Font.Name)
    end;

    SetBkMode(Canvas.Handle, TRANSPARENT);

    hfnt    :=CreateFontIndirect(plf);
    hfntPrev:=SelectObject(Canvas.Handle, hfnt);
    try
      TextOut(Canvas.Handle,
              r.Left,
              r.Bottom - (r.Bottom - r.Top - th) div 2,
              PChar(text),
              Length(text)
             );
    finally
      SelectObject(Canvas.Handle, hfntPrev);
      DeleteObject(hfnt);
      SetBkMode(Canvas.Handle, OPAQUE);
    end;
  {$ENDIF}
end;

procedure   TARRulerBar.Paint;
var u:extended;
    s:string;
    i:integer;
    cnv:TCanvas;

  procedure HShowPixGrid;
  var k:integer;
  begin
    for k:=0 to ToPix(PageWidth) - ToPix(LeftMargin) - ToPix(RightMargin) do begin
      if (k mod 100) = 0 then begin
        cnv.MoveTo(Gutter + k, 8); cnv.LineTo(Gutter + k, 22);
        s:=UnitsToText(k);
        cnv.TextOut(Gutter + k + 2, 3, s);
      end else begin
        if (k mod 50) = 0 then begin
          cnv.MoveTo(Gutter + k, 15); cnv.LineTo(Gutter + k, 22);
        end else begin
          if (k mod 10) = 0 then begin
            cnv.MoveTo(Gutter + k, 19); cnv.LineTo(Gutter + k, 22);
          end;
        end;
      end;
    end;
  end;
  procedure HShowMMGrid;
  begin
    u:=0;
    repeat
      i:=ToPix(u);
      cnv.MoveTo(Gutter + i, 8);
      cnv.LineTo(Gutter + i, 22);
      s:=UnitsToText(u / UnitRatio(uMM));
      cnv.TextOut(Gutter + i +2, 3, s);

      u:=u + 50; i:=ToPix(u);
      if u > (PageWidth - LeftMargin - RightMargin) then break;
      cnv.MoveTo(Gutter + i, 15);
      cnv.LineTo(Gutter + i, 22);

      u:=u + 50;
    until u > (PageWidth - LeftMargin - RightMargin);
  end;
  procedure HShowCMGrid;
  begin
    u:=0;
    repeat
      i:=ToPix(u);
      cnv.MoveTo(Gutter + i, 8); cnv.LineTo(Gutter + i, 22);
      s:=UnitsToText(u / UnitRatio(uCM));
      cnv.TextOut(Gutter + i + 2, 3, s);

      u:=u + 50; i:=ToPix(u);
      if u > (PageWidth - LeftMargin - RightMargin) then break;
      cnv.MoveTo(Gutter + i, 15); cnv.LineTo(Gutter + i, 22);

      u:=u + 50;
    until u > (PageWidth - LeftMargin - RightMargin);
  end;
  procedure HShowInGrid;
  var k:integer;
  begin
    u:=0;
    repeat
      i:=ToPix(u);
      cnv.MoveTo(Gutter + i, 8); cnv.LineTo(Gutter + i, 22);
      s:=UnitsToText(u / UnitRatio(uIn));
      cnv.TextOut(Gutter + i + 2, 3, s);

      for k:=1 to 4 do begin
        u:=u + 254 / 10; i:=ToPix(u);
        if u > (PageWidth - LeftMargin - RightMargin) then break;
        cnv.MoveTo(Gutter + i, 19); cnv.LineTo(Gutter + i, 22);
      end;

      u:=u + 254 / 10; i:=ToPix(u);
      if u > (PageWidth - LeftMargin - RightMargin) then break;
      cnv.MoveTo(Gutter + i, 15); cnv.LineTo(Gutter + i, 22);

      for k:=1 to 4 do begin
        u:=u + 254 / 10;
        if u > (PageWidth - LeftMargin - RightMargin) then break;
        i:=ToPix(u);
        cnv.MoveTo(Gutter + i, 19); cnv.LineTo(Gutter + i, 22);
      end;

      u:=u + 254 / 10;
    until u > (PageWidth - LeftMargin - RightMargin);
  end;
  ///////////////////////
  procedure VShowPixGrid;
  var k:integer;
  begin
    for k:=0 to ClientHeight - Gutter do begin
      if (k mod 100) = 0 then begin
        cnv.MoveTo(8, Gutter + k); cnv.LineTo(22, Gutter + k);
        s:=UnitsToText(k);
        ShowText(Canvas, Rect(3, Gutter + k + 2, 34, Gutter + k + cnv.TextWidth(s)), 90, s);
      end else begin
        if (k mod 50) = 0 then begin
          cnv.MoveTo(15, Gutter + k); cnv.LineTo(22, Gutter + k);
        end else begin
          if (k mod 10) = 0 then begin
            cnv.MoveTo(19, Gutter + k); cnv.LineTo(22, Gutter + k);
          end;
        end;
      end;
    end;
  end;
  procedure VShowMMGrid;
  begin
    u:=0; i:=0;
    repeat
      cnv.MoveTo(8, Gutter + i); cnv.LineTo(22, Gutter + i);
      s:=UnitsToText(u / UnitRatio(uMM));
      ShowText(Canvas, Rect(3, Gutter + i + 2, 34, Gutter + i + cnv.TextWidth(s)), 90, s);

      u:=u + 50; i:=ToPix(u);
      if i > (ClientHeight - Gutter) then break;
      cnv.MoveTo(15, Gutter + i); cnv.LineTo(22, Gutter + i);

      u:=u + 50; i:=ToPix(u);
    until i > (ClientHeight - Gutter);
  end;
  procedure VShowCMGrid;
  begin
    u:=0; i:=0;
    repeat
      cnv.MoveTo(8, Gutter + i); cnv.LineTo(22, Gutter + i);
      s:=UnitsToText(u / UnitRatio(uCM));
      ShowText(Canvas, Rect(3, Gutter + i + 2, 34, Gutter + i + cnv.TextWidth(s)), 90, s);

      u:=u + 50; i:=ToPix(u);
      if i > (ClientHeight - Gutter) then break;
      cnv.MoveTo(15, Gutter + i); cnv.LineTo(22, Gutter + i);

      u:=u + 50; i:=ToPix(u);
    until i > (ClientHeight - Gutter);
  end;
  procedure VShowInGrid;
  var k:integer;
  begin
    u:=0; i:=0;
    repeat
      cnv.MoveTo(8, Gutter + i); cnv.LineTo(22, Gutter + i);
      s:=UnitsToText(u / UnitRatio(uIn));
      ShowText(Canvas, Rect(3, Gutter + i + 2, 34, Gutter + i + cnv.TextWidth(s)), 90, s);

      for k:=1 to 4 do begin
        u:=u + 254 / 10; i:=ToPix(u);
        if i > (ClientHeight - Gutter) then break;
        cnv.MoveTo(19, Gutter + i); cnv.LineTo(22, Gutter + i);
      end;

      u:=u + 254 / 10; i:=ToPix(u);
      if i > (ClientHeight - Gutter) then break;
      cnv.MoveTo(15, Gutter + i); cnv.LineTo(22, Gutter + i);

      for k:=1 to 4 do begin
        u:=u + 254 / 10; i:=ToPix(u);
        if i > (ClientHeight - Gutter) then break;
        cnv.MoveTo(19, Gutter + i); cnv.LineTo(22, Gutter + i);
      end;

      u:=u + 254 / 10; i:=ToPix(u);
    until i > (ClientHeight - Gutter);
  end;
  ///////////////////////
  procedure ShowGrid;
  begin
    {$IFDEF CLX}
    {$ELSE}
      SetBkMode(cnv.Handle, TRANSPARENT);
    {$ENDIF}
    case Orientation of
      rbHorizontal: begin
                      case Units of
                        uPix: HShowPixGrid;
                        uMM:  HShowMMGrid;
                        uCm:  HShowCMGrid;
                        uIn:  HShowInGrid;
                      end;
                    end;
      rbVertical:   begin
                      case Units of
                        uPix: VShowPixGrid;
                        uMM:  VShowMMGrid;
                        uCm:  VShowCMGrid;
                        uIn:  VShowInGrid;
                      end;
                    end;
    end;
    {$IFDEF CLX}
    {$ELSE}
      SetBkMode(cnv.Handle, OPAQUE);
    {$ENDIF}
  end;
{$IFDEF CLX}
  var bmp:TBitmap;
{$ELSE}
{$ENDIF}
  var a, b, c, d:integer;
begin
  {$IFDEF CLX}
    bmp:=TBitmap.Create;
    bmp.Width :=ClientWidth;
    bmp.Height:=ClientHeight;
    cnv:=bmp.Canvas;
    cnv.Start();
  {$ELSE}
    cnv:=Canvas;
  {$ENDIF}
  try
    cnv.Font.Assign(Font);
    cnv.Brush.Color:=Color;
    cnv.FillRect(ClientRect);

    case Orientation of
      rbHorizontal:begin
        a:=Gutter - ToPix(LeftMargin);
        b:=Gutter;
        c:=Gutter - ToPix(LeftMargin) + ToPix(PageWidth) - ToPix(RightMargin);
        d:=Gutter - ToPix(LeftMargin) + ToPix(PageWidth);

        cnv.Pen.Color  :=Color;
        cnv.Brush.Color:=RulerColor;//$00C0FFFF;
        {$IFDEF D3}
          cnv.Rectangle(a, 2, d, 22);
        {$ELSE}
          cnv.Rectangle(Rect(a, 2, d, 22));
        {$ENDIF}

        cnv.Brush.Color:=clWhite;
        cnv.Pen.Color:=clBlack;
        ShowGrid;

        cnv.Pen.Color  :=Color;
        cnv.Brush.Color:=Color;
        cnv.Rectangle(d, 2, ClientWidth, 34);

        cnv.Pen.Color:=clRed;
        cnv.MoveTo(a, 0);
        cnv.LineTo(a, ClientHeight);

        cnv.Pen.Color:=clBlack;
        cnv.MoveTo(b, 0);
        cnv.LineTo(b, ClientHeight);

        cnv.Pen.Color:=clRed;
        cnv.MoveTo(c, 0);
        cnv.LineTo(c, ClientHeight);

        cnv.Pen.Color:=clBlack;
        cnv.MoveTo(d, 0);
        cnv.LineTo(d, ClientHeight);

        cnv.Pen.Color  :=clBlack;
        cnv.Brush.Color:=SelColor;//$00FFFFC0;
        cnv.Rectangle(a + 1, 3, b, 21);
        cnv.Rectangle(c + 1, 3, d, 21);
        cnv.Rectangle(Gutter + ToPix(SelBegin), 22, Gutter + ToPix(SelEnd), 34);

        s:=Format('%f', [LeftMargin  / UnitRatio(Units)]);
        cnv.TextRect(Rect(a + 2, 4, b - 1, 20), (a + b - cnv.TextWidth(s)) div 2, 5, s);

        s:=Format('%f', [RightMargin  / UnitRatio(Units)]);
        cnv.TextRect(Rect(c + 2, 4, d - 1, 20), (c + d - cnv.TextWidth(s)) div 2, 5, s);

        s:=UnitsToText((SelEnd - SelBegin)  / UnitRatio(Units));
        cnv.TextRect(Rect(Gutter + ToPix(SelBegin) + 1, 23, Gutter + ToPix(SelEnd) -1 , 33),
                        Gutter + (ToPix(SelBegin) + ToPix(SelEnd) - cnv.TextWidth(s)) div 2,
                        21, s);

        {$IFDEF CLX}
        {$ELSE}
          SetBkMode(cnv.Handle, TRANSPARENT);
        {$ENDIF}
        s:=UnitsToText(SelBegin / UnitRatio(Units));
        cnv.TextOut(Gutter + ToPix(SelBegin) - cnv.TextWidth(s) - 2, 21, s);
        {$IFDEF CLX}
        {$ELSE}
          SetBkMode(cnv.Handle, OPAQUE);
        {$ENDIF}

        cnv.Pen.Color:=clGreen;
        cnv.MoveTo(FCur, 0);
        cnv.LineTo(FCur, ClientHeight);
      end;
      rbVertical:begin
        cnv.Pen.Color  :=Color;
        cnv.Brush.Color:=RulerColor;//$00C0FFFF;
        {$IFDEF D3}
          cnv.Rectangle(2, 1, 22, ClientHeight);
        {$ELSE}
          cnv.Rectangle(Rect(2, 1, 22, ClientHeight));
        {$ENDIF}

        cnv.Pen.Color  :=clBlack;
        cnv.Brush.Color:=SelColor;//$00FFFFC0;
        cnv.Rectangle(22, Gutter + ToPix(SelBegin), 34, Gutter + ToPix(SelEnd) + 1);

        cnv.Brush.Color:=clWhite;
        cnv.Pen.Color:=clBlack;
        ShowGrid;

        s:=UnitsToText((SelEnd - SelBegin)  / UnitRatio(Units));
        ShowText(Canvas, Rect(21, Gutter + ToPix(SelBegin), 34, Gutter + ToPix(SelEnd)), 90, s);

        {$IFDEF CLX}
        {$ELSE}
          SetBkMode(cnv.Handle, TRANSPARENT);
        {$ENDIF}
        s:=UnitsToText(SelBegin / UnitRatio(Units));
        ShowText(Canvas, Rect(21, Gutter + ToPix(SelBegin) - cnv.TextWidth(s) - 2, 34, Gutter + ToPix(SelBegin)), 90, s);
        {$IFDEF CLX}
        {$ELSE}
          SetBkMode(cnv.Handle, OPAQUE);
        {$ENDIF}

        cnv.Pen.Color:=clGreen;
        cnv.MoveTo(0, FCur);
        cnv.LineTo(ClientWidth, FCur);
      end;
    end;
    {$IFDEF CLX}
      cnv.Stop();
      Canvas.Draw(0, 0, bmp);
    {$ENDIF}
  finally
    {$IFDEF CLX}
      bmp.Free();
    {$ENDIF}
  end;
end;

constructor TARRulerBar.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);

  Align:=alTop;

  {$IFDEF CLX}
  {$ELSE}
    {$IFNDEF D3}
      DoubleBuffered:=true;
    {$ENDIF}
  {$ENDIF}
end;

end.


