unit ARSQL;
//************************************************************//
//                         AND Report                        //
//  Eugene Hrulev (mrHru) Laxian Key, Omsk, Russia 1999,2001 //
//  Oleg Melnikov (muler) Laxian Key, Omsk, Russia 1999,2001 //
//************************************************************//

interface

uses
  Windows, Classes, DB, DBTables, Graphics, SysUtils,

  ARUtils;

const
  DefaultDatabaseName:string  = '_CurrentDB';
  MaxParamCount               = 64;

type
  TSQL = class
  private
    Tag         :integer;
    Picture     :TPicture;

    POwner      :TComponent;

    function    GetI (const FieldName:string):integer;
    procedure   SetI (const FieldName:string;v:integer);

    function    GetS (const FieldName:string):string;
    procedure   SetS (const FieldName:string;v:string);

    function    GetBLOB (const FieldName:string):string;
    procedure   SetBLOB (const FieldName:string;v:string);

    function    GetT (const FieldName:string):string;
    procedure   SetT (const FieldName:string;v:string);

    function    GetF (const FieldName:string):extended;
    procedure   SetF (const FieldName:string;v:extended);

    function    GetC (const FieldName:string):currency;
    procedure   SetC (const FieldName:string;v:currency);

    function    GetM (const FieldName:string):extended;
    procedure   SetM (const FieldName:string;v:extended);

    function    GetD (const FieldName:string):TDateTime;
    procedure   SetD (const FieldName:string;v:TDateTime);

    function    GetB (const FieldName:string):boolean;
    procedure   SetB (const FieldName:string;v:boolean);

    function    GetG (const FieldName:string):TGraphic;

    function    GetDataSet:TDataSet;

    function    GetSQL:TStrings;

    function    GetEOF  :boolean;
    function    GetBOF  :boolean;
    function    GetEmpty:boolean;

    function    GetPrepared:boolean;
    procedure   SetPrepared(APrepared:boolean);

    function    GetRequestLive:boolean;
    procedure   SetRequestLive(ARequestLive:boolean);

    function    GetDatabase:TDatabase;
  protected
    Q           :TQuery;
  public
    ss:string;

    constructor Create;
    constructor CreateDB(DataBase:TDatabase);
    destructor  Destroy;override;

    procedure   Prepare;
    procedure   Close;
    procedure   Open;
    procedure   Exec;

    procedure   Edit;
    procedure   Post;

    procedure   New;
    procedure   Add(s:string);
//    function    P(PName:string;PValue:Variant):string;

    procedure   First;
    procedure   Next;
    procedure   Last;
    procedure   Prior;

    procedure   Refresh;

    function    RecordCount:integer;

    property    I[const FieldName: string]:integer    read GetI write SetI;
    property    S[const FieldName: string]:string     read GetS write SetS;
    property    BLOB[const FieldName: string]:string  read GetBLOB write SetBLOB;
    property    F[const FieldName: string]:extended   read GetF write SetF;
    property    C[const FieldName: string]:currency   read GetC write SetC;
    property    M[const FieldName: string]:extended   read GetM write SetM;
    property    D[const FieldName: string]:TDateTime  read GetD write SetD;
    property    B[const FieldName: string]:boolean    read GetB write SetB;
    property    G[const FieldName: string]:TGraphic   read GetG;

    property    T[const FieldName: string]:string     read GetT write SetT;  //read from BLOB

    property    EOF:boolean   read GetEOF;
    property    BOF:boolean   read GetBOF;
    property    Empty:boolean read GetEmpty;

    property    DataSet:TDataSet    read GetDataSet;

    property    SQL:TStrings        read GetSQL;

    property    Owner:TComponent    read POwner           write  POwner;

    property    RequestLive:boolean read GetRequestLive   write  SetRequestLive;
    property    Prepared:boolean    read GetPrepared      write  SetPrepared;

    property    Database:TDatabase  read GetDatabase;
  end;

const
  MoneyPrecision:integer=100;

function  MR(money:extended):extended;  //money rounder

var
  SQLCnt:integer;

function GetUniqueNumber: integer;

implementation

{$O-}
function  MR(money:extended):extended;  // money rounder
begin
  Result:=(Int(Abs(money)*MoneyPrecision + 0.5)/MoneyPrecision);
  if money < 0 then Result:=-Result;
end;
{$O+}

constructor TSQL.Create;
begin
  inc(SQLCnt);
  Q:=TQuery.Create(nil);
  Q.Name:='TSQLQuery' + IntToStr(GetUniqueNumber);
  Q.DatabaseName:=DefaultDatabaseName;

  Tag:=GetUniqueNumber;
end;

constructor TSQL.CreateDB(Database:TDatabase);
begin
  Q:=TQuery.Create(nil);
  Q.Name:='TSQLQuery' + IntToStr(GetUniqueNumber);
  Q.DatabaseName:=Database.DatabaseName;

  Tag:=GetUniqueNumber;
end;

destructor  TSQL.Destroy;
begin
  if Assigned(Picture) then Picture.Free;

  Q.Close;
  Q.Free;
  dec(SQLCnt);
  inherited;
end;

//paste from Delphi BkQuery demo
var
  GuardUniqueNumber :integer;
  UniqueNumber      :integer;

function GetUniqueNumber: integer;
asm
@@1:  MOV     EDX,1
      XCHG    GuardUniqueNumber,EDX
      OR      EDX,EDX
      JZ      @@2

      PUSH    0
      CALL    Sleep
      JMP     @@1

@@2:  MOV     EAX,UniqueNumber
      INC     EAX
      MOV     UniqueNumber,EAX
      MOV     GuardUniqueNumber,EDX
      RET
end;

function  TSQL.GetT (const FieldName:string):string;
var bs:TBlobStream;
    p:PChar;
begin
  if not Q.Active then Q.Open;

  bs:=TBlobStream.Create(Q.FieldByName(FieldName) as TMemoField, bmRead);
  GetMem(p, bs.Size + 1);
  try
    bs.Read(p^, bs.Size);
    p[bs.Size]:=#0;
    Result:=p;
  finally
    FreeMem(p);
    bs.Free;
  end;
end;

procedure TSQL.SetT (const FieldName:string;v:string);
var p:PChar;
    len:integer;
begin
  len:=Length(v);
  GetMem(p, len + 1);
  try
    StrPLCopy(p, v, len);
    len:=Length(v);
    p[len]:=#0;
    Q.ParamByName(FieldName).SetBlobData(p, len);
  finally
    FreeMem(p);
  end;
end;

function  TSQL.GetEOF  :boolean;
begin
  if Q.Active then begin
    Result:=Q.EOF;
  end else begin
    Result:=true;
  end;
end;
function  TSQL.GetBOF  :boolean;
begin
  if Q.Active then begin
    Result:=Q.BOF;
  end else begin
    Result:=true;
  end;
end;
function  TSQL.GetEmpty:boolean;
begin
  Result:=EOF and BOF;
end;

function  TSQL.GetSQL:TStrings;
begin
  Result:=Q.SQL;
end;

procedure TSQL.New;
begin
  Q.Close;
  Q.SQL.Clear;
  Q.RequestLive:=false;
//  PCount:=0;
  ss:='';
  if Assigned(Picture) then Picture.Free;
  Picture:=nil;
end;

procedure TSQL.Prepare;
begin
  if not Q.Prepared then Q.Prepare;
end;

procedure TSQL.Close;
begin
  Q.Close;
  if Assigned(Picture) then Picture.Free;
  Picture:=nil;
end;

procedure TSQL.Open;
begin
  Q.Open;
end;

procedure TSQL.Exec;
begin
  Q.ExecSQL;
end;

procedure TSQL.Add(s:string);
begin
  Q.SQL.Add(s);
  ss:=ss+s;
end;

procedure TSQL.Edit;
begin
  Q.Edit;
end;

procedure TSQL.Post;
begin
  Q.Post;
end;

procedure TSQL.First;
begin
  Q.First;
end;

procedure TSQL.Next;
begin
  Q.Next;
end;

procedure TSQL.Last;
begin
  Q.Last;
end;

procedure TSQL.Prior;
begin
  Q.Prior;
end;

procedure TSQL.Refresh;
begin
  Q.Refresh;
end;

function  TSQL.RecordCount:integer;
begin
  if Q.Active then begin
    Result:=Q.RecordCount;
  end else begin
    Result:=0;
  end;
end;

function  TSQL.GetI (const FieldName:string):integer;
begin
  if not Q.Active then Q.Open;
  Result:=Q.FieldByName(FieldName).AsInteger;
end;

procedure TSQL.SetI (const FieldName:string;v:integer);
begin
  Q.ParamByName(FieldName).AsInteger:=v;
end;

function  TSQL.GetS (const FieldName:string):string;
begin
  if not Q.Active then Q.Open;
  Result:=Q.FieldByName(FieldName).AsString;
end;

procedure TSQL.SetS (const FieldName:string;v:string);
begin
  Q.ParamByName(FieldName).AsString:=v;
end;

function  TSQL.GetBLOB (const FieldName:string):string;
begin
  if not Q.Active then Q.Open;
  Result:=(Q.FieldByName(FieldName) as TBlobField).AsString;
end;

procedure TSQL.SetBLOB (const FieldName:string;v:string);
var bs:TBlobStream;
    sl:TAIStringList;
begin
  Q.Edit;
  bs:=TBlobStream.Create(Q.FieldByName(FieldName) as TBlobField, bmWrite);
  sl:=TAIStringList.Create('TSQL.SetBLOB sl');
  try
    sl.Text:=v;
    sl.SaveToStream(bs);
  finally
    bs.Free;
    sl.Free;
  end;
  Q.Post;
end;

function  TSQL.GetF (const FieldName:string):extended;
begin
  if not Q.Active then Q.Open;
  Result:=Q.FieldByName(FieldName).AsFloat;
end;
procedure TSQL.SetF (const FieldName:string;v:extended);
begin
  Q.ParamByName(FieldName).AsFloat:=v;
end;

function  TSQL.GetC (const FieldName:string):currency;
begin
  if not Q.Active then Q.Open;
  try
    Result:=Q.FieldByName(FieldName).AsCurrency;
  except
    Result:=0.0;
  end;
end;
procedure TSQL.SetC (const FieldName:string;v:currency);
begin
  Q.ParamByName(FieldName).AsCurrency:=v;
end;

function  TSQL.GetM (const FieldName:string):extended;
begin
  if not Q.Active then Q.Open;
  Result:=MR(Q.FieldByName(FieldName).AsFloat);
end;

procedure TSQL.SetM (const FieldName:string;v:extended);
begin
  Q.ParamByName(FieldName).AsFloat:=MR(v);
end;

function  TSQL.GetD (const FieldName:string):TDateTime;
begin
  if not Q.Active then Q.Open;
  Result:=Q.FieldByName(FieldName).AsDateTime;
end;

procedure TSQL.SetD (const FieldName:string;v:TDateTime);
begin
  Q.ParamByName(FieldName).AsDateTime:=v;
end;

function  TSQL.GetB (const FieldName:string):boolean;
begin
  if not Q.Active then Q.Open;
  Result:=boolean(Q.FieldByName(FieldName).AsInteger);
end;

procedure TSQL.SetB (const FieldName:string;v:boolean);
begin
  Q.ParamByName(FieldName).AsInteger:=integer(v);
end;

function  TSQL.GetG (const FieldName:string):TGraphic;
begin
  if not Q.Active then Q.Open;

  if not Assigned(Picture) then Picture:=TPicture.Create;

  Picture.Assign(Q.FieldByName(FieldName) as TBlobField);
  Result:=Picture.Graphic;
end;

function  TSQL.GetDataSet:TDataSet;
begin
  Result:=Q;
end;

function  TSQL.GetRequestLive:boolean;
begin
  Result:=Q.RequestLive;
end;

procedure  TSQL.SetRequestLive(ARequestLive:boolean);
begin
  Q.RequestLive:=ARequestLive;
end;

function    TSQL.GetDatabase:TDatabase;
begin
  Result:=Q.Database;
end;

function    TSQL.GetPrepared:boolean;
begin
  Result:=Q.Prepared;
end;

procedure   TSQL.SetPrepared(APrepared:boolean);
begin
  Q.Prepared:=APrepared;
end;

end.

