unit ARScrollingWidget;
//************************************************************//
//                         AND Report                        //
//  Eugene Hrulev (mrHru) Laxian Key, Omsk, Russia 1999,2001 //
//  Oleg Melnikov (muler) Laxian Key, Omsk, Russia 1999,2001 //
//************************************************************//

{$INCLUDE DEFINES.INC}

interface

uses
  SysUtils, Classes,
  {$IFDEF CLX}
    QControls, QStdCtrls, QForms, QExtCtrls, Types,
  {$ELSE}
    {$IFDEF D6}
      Types,
    {$ELSE}
      Windows,
    {$ENDIF}
    Controls, StdCtrls, ExtCtrls, Forms,
  {$ENDIF}
  Math;

type
  {$IFDEF D3}
    TARScrollingWidget = class(TCustomPanel)
  {$ELSE}
    TARScrollingWidget = class(TCustomControl)
  {$ENDIF}
  private
    FVertSB     :TScrollBar;
    FHorzSB     :TScrollBar;
    FPanel      :TPanel;
    FScrUpdCnt  :integer;
    FOnScroll   :TNotifyEvent;

    FHSMax      :integer;
    FVSMax      :integer;

    function    GetHSMax: integer;
    function    GetHSPage: integer;
    function    GetHSPos: integer;
    function    GetVSMax: integer;
    function    GetVSPage: integer;
    function    GetVSPos: integer;
    procedure   SetHSMax(const Value: integer);
    procedure   SetHSPage(const Value: integer);
    procedure   SetHSPos(const Value: integer);
    procedure   SetVSMax(const Value: integer);
    procedure   SetVSPage(const Value: integer);
    procedure   SetVSPos(const Value: integer);

    function    IsHSB:boolean;
    function    IsVSB:boolean;
  protected
    procedure   SetHSParams(APos:integer; APage:integer; AMax:integer);
    procedure   SetVSParams(APos:integer; APage:integer; AMax:integer);

    procedure   Resize;override;

    procedure   BeginScrollUpdate;
    procedure   EndScrollUpdate;
    procedure   RealignScrolls;
    procedure   SBScroll(Sender: TObject; ScrollCode: TScrollCode; var ScrollPos: Integer);
    function    GetClientRect:TRect;override;
  public
    constructor Create(AOwner:TComponent);override;
    destructor  Destroy;override;
  published
    property    VSPos :integer    read  GetVSPos      write SetVSPos;
    property    VSMax :integer    read  GetVSMax      write SetVSMax;
    property    VSPage:integer    read  GetVSPage     write SetVSPage;
    property    HSPos :integer    read  GetHSPos      write SetHSPos;
    property    HSMax :integer    read  GetHSMax      write SetHSMax;
    property    HSPage:integer    read  GetHSPage     write SetHSPage;

    property    Color;
    property    OnScroll:TNotifyEvent read FOnScroll  write FOnScroll;
  end;

implementation

{ TARScrollingWidget }

function  Max(a:integer; b:integer):integer;
begin
  if a > b then begin
    Result:=a;
  end else begin
    Result:=b;
  end;
end;

constructor TARScrollingWidget.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  Self.Width    :=100;
  Self.Height   :=100;

  FHorzSB             :=TScrollBar.Create(Self);
  FHorzSB.Parent      :=Self;
  FHorzSB.TabStop     :=false;
  FHorzSB.Kind        :=sbHorizontal;
  FHorzSB.Align       :=alNone;
  FHorzSB.SetBounds(0, Height - FHorzSB.Height, Width, FHorzSB.Height);
//  FHorzSB.Anchors     :=[akLeft,akRight,akBottom];
  FHorzSB.Visible     :=false;
  FHorzSB.Position    :=0;
  FHorzSB.SmallChange :=10;
  FHorzSB.LargeChange :=1;
  FHorzSB.Max         :=0;

  FVertSB             :=TScrollBar.Create(Self);
  FVertSB.Parent      :=Self;
  FVertSB.TabStop     :=false;
  FVertSB.Kind        :=sbVertical;
  FVertSB.Align       :=alNone;
  FVertSB.SetBounds(Width - FVertSB.Width, 0, FVertSB.Width, Height);
//  FVertSB.Anchors     :=[akTop,akRight,akBottom];
  FVertSB.Visible     :=false;
  FVertSB.Position    :=0;
  FVertSB.SmallChange :=10;
  FVertSB.LargeChange :=1;
  FVertSB.Max         :=0;

  FPanel              :=TPanel.Create(Self);
  FPanel.Parent       :=Self;
  FPanel.Visible      :=false;
  FPanel.BorderStyle  :=bsNone;
  FPanel.BevelInner   :=bvNone;
  FPanel.BevelOuter   :=bvNone;
  FPanel.BorderWidth  :=0;
//  FPanel.Anchors      :=[akRight,akBottom];
  FPanel.SetBounds( Width  - FVertSB.Width, Height - FHorzSB.Height, FVertSB.Width, FHorzSB.Height);

  FHorzSB.OnScroll    :=SBScroll;
  FVertSB.OnScroll    :=SBScroll;
end;

destructor TARScrollingWidget.Destroy;
begin
  FHorzSB.Free();
  FVertSB.Free();

  inherited Destroy();
end;
////////////////////////////////////////////////////////////////////////////////
function TARScrollingWidget.GetHSMax(): integer;
begin
  Result:=FHSMax;
end;
procedure TARScrollingWidget.SetHSMax(const Value: integer);
begin
  BeginScrollUpdate();
  try
    FHSMax:=Value;
    if Assigned(FHorzSB) then begin
      FHorzSB.Max:=Max(0, FHSMax - FHorzSB.LargeChange);
    end;
  finally
    EndScrollUpdate();
  end;
end;


function TARScrollingWidget.GetHSPage(): integer;
begin
  Result:=FHorzSB.LargeChange;
end;
procedure TARScrollingWidget.SetHSPage(const Value: integer);
begin
  BeginScrollUpdate();
  try
    if Assigned(FHorzSB) then begin
      FHorzSB.Max:=Max(0, FHSMax - Value);
      FHorzSB.LargeChange:=Value;
    end;
  finally
    EndScrollUpdate();
  end;
end;

function TARScrollingWidget.GetHSPos(): integer;
begin
  Result:=FHorzSB.Position;
end;

procedure TARScrollingWidget.SetHSPos(const Value: integer);
begin
  BeginScrollUpdate();
  try
    if Assigned(FHorzSB) then begin
      FHorzSB.Position:=Value;
    end;
  finally
    EndScrollUpdate();
  end;
end;

function TARScrollingWidget.GetVSMax(): integer;
begin
  Result:=FVSMax;
end;
procedure TARScrollingWidget.SetVSMax(const Value: integer);
begin
  BeginScrollUpdate();
  try
    FVSMax:=Value;
    if Assigned(FVertSB) then begin
      FVertSB.Max:=Max(0, FVSMax - FVertSB.LargeChange);
    end;
  finally
    EndScrollUpdate();
  end;
end;


function TARScrollingWidget.GetVSPage(): integer;
begin
  Result:=FVertSB.LargeChange;
end;
procedure TARScrollingWidget.SetVSPage(const Value: integer);
begin
  BeginScrollUpdate();
  try
    if Assigned(FVertSB) and (Value > 0) then begin

      FVertSB.Max:=Max(0, FVSMax - Value);
      FVertSB.LargeChange:=Value;
    end;
  finally
    EndScrollUpdate();
  end;
end;


function TARScrollingWidget.GetVSPos(): integer;
begin
  Result:=FVertSB.Position;
end;
procedure TARScrollingWidget.SetVSPos(const Value: integer);
begin
  BeginScrollUpdate();
  try
    if Assigned(FVertSB) then begin
      FVertSB.Position:=Value;
    end;
  finally
    EndScrollUpdate();
  end;
end;

procedure TARScrollingWidget.SBScroll(Sender: TObject; ScrollCode: TScrollCode; var ScrollPos: Integer);
begin
  Paint();

  if Assigned(FOnScroll) then begin
    FOnScroll(Self);
  end;
end;

procedure TARScrollingWidget.BeginScrollUpdate();
begin
  inc(FScrUpdCnt);
end;

procedure TARScrollingWidget.EndScrollUpdate();
begin
  dec(FScrUpdCnt);
  if FScrUpdCnt = 0 then begin
    RealignScrolls();
  end;
end;

procedure TARScrollingWidget.RealignScrolls();
var hv, vv, procmsg:boolean;
begin
  hv:=false;
  vv:=false;
  procmsg:=false;

  if Assigned(FHorzSB) then begin
    hv:=IsHSB();
  end;
  if Assigned(FVertSB) then begin
    vv:=IsVSB();
  end;

  if Assigned(FHorzSB) then begin
    if hv then begin
      if vv then begin
        FHorzSB.Width:=Width - FVertSB.Width;
      end else begin
        FHorzSB.Width:=Width;
      end;
    end;
    if FHorzSB.Visible <> hv then begin
      FHorzSB.Visible:=hv;
      procmsg:=true;
    end;
  end;

  if Assigned(FVertSB) then begin
    if vv then begin
      if hv then begin
        FVertSB.Height:=Height - FHorzSB.Height;
      end else begin
        FVertSB.Height:=Height;
      end;
    end;
    if FVertSB.Visible <> vv then begin
      FVertSB.Visible:=vv;
      procmsg:=true;
    end;
  end;

  if Assigned(FPanel) then begin
    if FPanel.Visible <> (hv and vv) then begin
      FPanel.Visible:=hv and vv;
      procmsg:=true;
    end;
  end;

  if procmsg then begin
    Application.ProcessMessages();
  end;
end;

function    TARScrollingWidget.IsHSB():boolean;
begin
  Result:=Assigned(FHorzSB) and (HSMax > 0) and (HSPage > 0) and (HSMax > HSPage);
end;

function    TARScrollingWidget.IsVSB():boolean;
begin
  Result:=Assigned(FVertSB) and (VSMax > 0) and (VSPage > 0) and (VSMax > VSPage);
end;

function    TARScrollingWidget.GetClientRect():TRect;
begin
  Result:=Rect(0, 0, Width, Height);
  if IsHSB() then begin
    Result.Bottom:=Result.Bottom - FHorzSB.Height;
  end;
  if IsVSB() then begin
    Result.Right:=Result.Right - FVertSB.Width;
  end;
end;

procedure TARScrollingWidget.Resize;
begin
  inherited Resize;

  BeginScrollUpdate();
    HSPage:=ClientWidth;
    VSPage:=ClientHeight;

    if Assigned(FHorzSB) then FHorzSB.Top :=Height - FHorzSB.Height;
    if Assigned(FVertSB) then FVertSB.Left:=Width  - FVertSB.Width;

    if Assigned(FPanel) then begin
      FPanel.Top :=Height - FPanel.Height;
      FPanel.Left:=Width  - FPanel.Width;
    end;
  EndScrollUpdate();
end;

{
procedure TARScrollingWidget.Paint();
var x, y:integer;
begin
  Canvas.Start();
  try
    Canvas.FillRect(Rect(0, 0, Width, Height));
    for x:=0 to 9 do begin
      for y:=0 to 9 do begin
        Canvas.Rectangle(x * 40 - HSPos, y * 40 - VSPos, x * 40 + 39 - HSPos, y * 40 + 39 - VSPos);
        Canvas.TextOut  (x * 40 - HSPos + 5, y * 40 - VSPos + 5, IntToStr(x) + ':' + IntToStr(y));
      end;
    end;
  finally
    Canvas.Stop();
  end;
end;
}

procedure TARScrollingWidget.SetHSParams(APos, APage, AMax: integer);
begin
  if Assigned(FHorzSB) then begin
    HSPos :=APos;
    HSPage:=APage;
    HSMax :=AMax;
  end;
end;

procedure TARScrollingWidget.SetVSParams(APos, APage, AMax: integer);
begin
  if Assigned(FVertSB) then begin
    VSPos :=APos;
    VSPage:=APage;
    VSMax :=AMax;
  end;
end;

end.


