unit ARToolsCustomize;
//************************************************************//
//                         AND Report                        //
//  Eugene Hrulev (mrHru) Laxian Key, Omsk, Russia 1999,2001 //
//  Oleg Melnikov (muler) Laxian Key, Omsk, Russia 1999,2001 //
//************************************************************//

{$INCLUDE DEFINES.INC}

interface

uses
  SysUtils, Classes,

  {$IFDEF CLX}
    QForms, QComCtrls, QStdCtrls, QButtons, QCheckLst, QControls,
  {$ELSE}
    Forms, Controls, ComCtrls, StdCtrls, Buttons, CheckLst,

    Windows, Messages, Graphics, Dialogs,
    ToolWin, ExtCtrls,
  {$ENDIF}

  ARResStrings, ARRepCommon, ARActions, ARButtons;

type
  TARToolsCustomizeDlg = class(TForm)
    PageControl1: TPageControl;
    CategoryLB: TListBox;
    CloseBtn: TBitBtn;
    Memo: TMemo;
    TBL: TCheckListBox;
    TBD3: TScrollBox;
    TBD: TToolBar;
    CommandsTabSheet: TTabSheet;
    ToolBarsTabSheet: TTabSheet;
    RestoreDefaultsButton: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure CloseBtnClick(Sender: TObject);
    procedure CategoryLBClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure TBLClickCheck(Sender: TObject);
    procedure RestoreDefaultsButtonClick(Sender: TObject);
  private
    OldOnHint :TNotifyEvent;

    procedure ShowHint(Sender: TObject);
    {$IFNDEF D3}
      procedure ControlEndDoc(Sender, Target: TObject; X, Y: Integer);
    {$ENDIF}
  public
  end;

implementation

uses AREditorMain;

{$R *.dfm}

procedure TARToolsCustomizeDlg.CloseBtnClick(Sender: TObject);
begin
  Close;
end;

procedure TARToolsCustomizeDlg.CategoryLBClick(Sender: TObject);
var cat:string;
    i:integer;
    ctrl:TDummyControl;
begin
  while TBD.ControlCount > 0 do begin
    TBD.Controls[0].Free;
  end;

  cat:=CategoryLB.Items[CategoryLB.ItemIndex];

  for i:=0 to AREditorDlg.ActList.Count - 1 do begin
    if ANSICompareText(cat, Flat(AREditorDlg.ActList.Item[i].Category)) = 0 then begin
      ctrl:=TDummyControl(AREditorDlg.ActList.Item[i].CreateControl(Self, TParControl(TBD)));
      if Assigned(ctrl) then begin
        {$IFDEF CLX}
        {$ELSE}
          ctrl.DragMode :=dmAutomatic;
          {$IFNDEF D3}
            ctrl.DragKind :=dkDock;
            ctrl.OnEndDock:=ControlEndDoc;
          {$ENDIF}
        {$ENDIF}
        ctrl.Enabled  :=true;
      end;
    end;
  end;
end;

procedure TARToolsCustomizeDlg.ShowHint(Sender: TObject);
begin
  Memo.Text:=Application.Hint;
end;

procedure TARToolsCustomizeDlg.FormShow(Sender: TObject);
var i, j:integer;
    tbr:TToolBar;
begin
  OldOnHint:=Application.OnHint;
  Application.OnHint:=ShowHint;

  for i:=0 to AREditorDlg.ComponentCount - 1 do begin
    if AREditorDlg.Components[i] is TToolBar then begin
      tbr:=TToolBar(AREditorDlg.Components[i]);
      {$IFDEF CLX}
      {$ELSE}
        {$IFNDEF D3}
          tbr.DockSite:=true;
          tbr.DragKind:=dkDock;
        {$ENDIF}
        tbr.DragMode:=dmAutomatic;
      {$ENDIF}
      for j:=0 to tbr.ControlCount - 1 do begin
        with TDummyControl(tbr.Controls[j]) do begin
          {$IFDEF CLX}
          {$ELSE}
            DragMode  :=dmAutomatic;
            {$IFNDEF D3}
              DragKind  :=dkDock;
              OnEndDock :=ControlEndDoc;
            {$ENDIF}
          {$ENDIF}
        end;
      end;
    end;
  end;

  TBL.Items.Clear;

  for i:=0 to AREditorDlg.ActList.CategoryList.Count - 1 do begin
    tbr:=TToolBar(AREditorDlg.FindComponent(AREditorDlg.ActList.CategoryList[i] + 'ToolBar'));
    if Assigned(tbr) then begin
      TBL.Items.Add(AREditorDlg.ActList.CategoryList[i]);
      TBL.Checked[i]:=tbr.Visible;
      TBL.Items.Objects[i]:=tbr;
    end;
  end;
end;

procedure TARToolsCustomizeDlg.TBLClickCheck(Sender: TObject);
var tbr:TToolBar;
begin
  tbr:=TToolBar(TBL.Items.Objects[TBL.ItemIndex]);

  if Assigned(tbr) then begin
    if tbr.Visible then begin
      tbr.Visible:=false;
      TBL.Checked[TBL.ItemIndex]:=false;
    end else begin
      tbr.Visible:=true;
      TBL.Checked[TBL.ItemIndex]:=true;
    end;
  end;
end;

procedure TARToolsCustomizeDlg.FormClose(Sender: TObject; var Action: TCloseAction);
var i, j:integer;
    tbr:TToolBar;
begin
  Application.OnHint:=OldOnHint;

  for i:=0 to ComponentCount - 1 do begin
    if (Components[i] is TControl) and (TControl(Components[i]).Parent = TBD) then begin
      TControl(Components[i]).Free;
    end;
  end;

  for i:=0 to AREditorDlg.ComponentCount - 1 do begin
    if AREditorDlg.Components[i] is TToolBar then begin
      tbr:=TToolBar(AREditorDlg.Components[i]);
      {$IFDEF CLX}
      {$ELSE}
        {$IFNDEF D3}
          tbr.DockSite:=false;
        {$ENDIF}
      {$ENDIF}
      for j:=0 to tbr.ControlCount - 1 do begin
        with TDummyControl(tbr.Controls[j]) do begin
          {$IFDEF CLX}
          {$ELSE}
            DragMode:=dmManual;
            {$IFNDEF D3}
              DragKind:=dkDrag;
              OnEndDock:=nil;
            {$ENDIF}
          {$ENDIF}
        end;
      end;
    end;
  end;

  Action:=caFree;
end;

{$IFNDEF D3}
procedure TARToolsCustomizeDlg.ControlEndDoc(Sender, Target: TObject; X, Y: Integer);
begin
  if (Sender is TControl) and (not (TControl(Target) is TBD.ClassType{TToolBar})) then begin
    {$IFDEF CLX}
    {$ELSE}
      TControl(Sender).ManualDock(TBD);
    {$ENDIF}
  end else begin
    AREditorMain.ResizeToolBar(TARToolBar(TControl(Sender).Parent));
  end;
end;
{$ENDIF}

procedure TARToolsCustomizeDlg.RestoreDefaultsButtonClick(Sender: TObject);
begin
  AREditorDlg.RestoreDefaultTools;
end;

procedure TARToolsCustomizeDlg.FormCreate(Sender: TObject);
begin
  Caption                       :=rsARToolsCustomize_ARToolsCustomizeDlgCaption;
  ToolBarsTabSheet.Caption      :=rsARToolsCustomize_ToolBarsTabSheetCaption;
  CommandsTabSheet.Caption      :=rsARToolsCustomize_CommandsTabSheetCaption;
  RestoreDefaultsButton.Caption :=rsARToolsCustomize_RestoreDefaultsButtonCaption;
  CloseBtn.Caption              :=rsAR_CloseCaption;

  CategoryLB.Items.Assign(AREditorDlg.ActList.CategoryList);
end;

end.
