unit ARUserRep;
//************************************************************//
//                         AND Report                        //
//  Eugene Hrulev (mrHru) Laxian Key, Omsk, Russia 1999,2001 //
//  Oleg Melnikov (muler) Laxian Key, Omsk, Russia 1999,2001 //
//************************************************************//

{$INCLUDE DEFINES.INC}

interface

uses
  Classes, SysUtils, Math,
  {$IFDEF CLX}
    QControls, QGraphics, Variants,
  {$ELSE}
    Controls, Graphics,
  {$ENDIF}

  {$IFDEF D6}
    Variants,
  {$ENDIF}

  {$IFDEF MSWINDOWS}
    DB,
  {$ENDIF}

  ARRepCommon, ARUtils, ARRep, ARGridRep, ARPrinter;

type

  TARUserRep = class;

  TARUserRepPlugIn = class(TComponent)
  private
    FARUserRep  :TARUserRep;
    FVarList    :TStringList;

    procedure   SetARUserRep(const Value: TARUserRep);
    function    GetVarByName(const VarName: string): TObject;
    function    GetVarByIndex(VarIndex:integer): TObject;
    function    GetVarCount:integer;
    function    GetVarNames(VarIndex: integer): string;
  protected
    function    CreateVar:TObject;virtual;abstract;
    property    VarByName [const VarName:string]:TObject  read GetVarByName;
    property    VarByIndex[VarIndex:integer]:TObject      read GetVarByIndex;
    property    VarNames  [VarIndex:integer]:string       read GetVarNames;
    property    VarCount:integer                          read GetVarCount;
    procedure   ApplyValues;virtual;
  public
    procedure   ClearValues;virtual;

    constructor Create(AOwner:TComponent);override;
    destructor  Destroy;override;

  published
    property    ARUserRep         :TARUserRep       read FARUserRep write SetARUserRep;
  end;

  TARUserRep = class(TARGridRep)
  private
    PictureList :TAIStringList;
    GroupBands  :TAIStringList;

    oldSrcRep   :TARRep;

    NextTopBandState:TARBandState;

    FPath       :string;
    FOrgRepList :TStringList;
    FPlugInList :TList;

//    function    GetCell(BandName:string; IdxBand:integer; ValName:string):TARCell;
    function    GetBandCount(BandName:string):integer;
  protected
    procedure   MouseMove(Shift: TShiftState; X, Y: Integer);override;
    procedure   MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);override;
    procedure   Loaded;override;
    {$IFNDEF D3}
      procedure   Resize;override;
    {$ENDIF}
    procedure   InternalAddBands(BandName:string; bandlist:TAIList);

    function    GetTemplate:TARTemplate;override;
    procedure   SetTemplate(ATemplate:TARTemplate);override;
    procedure   ApplyValues(src:TARCell; dst:TARCell);override;

    procedure   SetPath(APath:string);

    function    GetS(VarName:string):string;
    procedure   SetS(VarName:string; VarValue:string);

    function    GetRTF(VarName:string):string;
    procedure   SetRTF(VarName:string; VarValue:string);

    function    GetF(VarName:string):extended;
    procedure   SetF(VarName:string; VarValue:extended);

    function    GetI(VarName:string):integer;
    procedure   SetI(VarName:string; VarValue:integer);override;

    function    GetD(VarName:string):TDateTime;
    procedure   SetD(VarName:string; VarValue:TDateTime);

    function    GetT(VarName:string):TDateTime;
    procedure   SetT(VarName:string; VarValue:TDateTime);

    function    GetC(VarName:string):currency;
    procedure   SetC(VarName:string; VarValue:currency);

    function    GetM(VarName:string):currency;
    procedure   SetM(VarName:string; VarValue:currency);

    function    GetG(VarName:string):TGraphic;
    procedure   SetG(VarName:string; VarValue:TGraphic);

    function    GetU(VarName:string):integer;
    procedure   SetU(VarName:string; UserVarValue:integer);

    function    GetTitle:string;
    procedure   SetTitle(ATitle:string);
  public
    Vars        :TAIStringList;

    {$IFDEF MSWINDOWS}
      procedure   SetDSValues(DS:TDataSet);
    {$ENDIF}

    procedure   Load(FileName:string);
    procedure   PrepareTemplate;

    {$IFDEF D3}
      function  AddBands(BandName:string):TARBand;
    {$ELSE}
      function  AddBands(BandName:string; ToBand:TARBand = nil; Expanded:boolean = true):TARBand;
    {$ENDIF}
    function    AddSubBands(BandName:string; Expanded:boolean):TARBand;
    function    AddSubBandsToBand(BandName:string; ToBand:TARBand; Expanded:boolean):TARBand;
    function    MergeBands(BandName: string; ToBand: TARBand): TARBand;

    {$IFDEF D3}
      function  AddGroupBands(const BandName:string; Value:variant):TARBand;
      function  AddGroupBandsToBand(const BandName:string; Value:variant; ToBand:TARBand; Expanded:boolean):TARBand;
    {$ELSE}
      function  AddGroupBands(const BandName:string; Value:variant; ToBand:TARBand = nil; Expanded:boolean = true):TARBand;
    {$ENDIF}
    function    NewGroup(const BandName:string; Value:variant):boolean;

    function    InsertBands(InsertIdxBand:integer; BandName:string):TARBand;
    procedure   DeleteBand(IdxBand:integer);
    procedure   DeleteBands;
    procedure   ClearValues;
    procedure   ClearPictures;  //CD clear only pictures and plugins, not vars

    procedure   MoveToPriorBand;override;
    procedure   MoveToNextBand;override;

    constructor Create(AOwner:TComponent);override;
    destructor  Destroy;override;

//    property    Cells[BandName:string; IdxBand:integer; ValName:string]:TARCell read GetCell;
{$IFDEF D3}
    function    CellsByName(ValName:string):TARCell;
    function    Cells(IdxBand:integer; IdxCell:integer):TARCell;
{$ELSE}
    function    Cells(ValName:string):TARCell;overload;
    function    Cells(IdxBand:integer; IdxCell:integer):TARCell;overload;
{$ENDIF}
    procedure   SetFocus;override;

    procedure   SaveToFile(FFile :string);
    procedure   LoadFromFile(FFile :string);

    procedure   PageBreak;
    procedure   BeginPara;
    procedure   EndPara;
    procedure   ResetPageNumber;

    property    S[VarName:string]:string            read  GetS      write SetS;
    property    RTF[VarName:string]:string          read  GetRTF    write SetRTF;
    property    F[VarName:string]:extended          read  GetF      write SetF;
    property    I[VarName:string]:integer           read  GetI      write SetI;
    property    D[VarName:string]:TDateTime         read  GetD      write SetD;
    property    T[VarName:string]:TDateTime         read  GetT      write SetT;
    property    G[VarName:string]:TGraphic          read  GetG      write SetG;
    property    C[VarName:string]:currency          read  GetC      write SetC;
    property    M[VarName:string]:currency          read  GetM      write SetM;

    property    U[VarName:string]:integer           read  GetU      write SetU;
    property    BandCount[BandName:string]:integer  read  GetBandCount;

    procedure   UnRegisterPlugIn(const PlugIn:TARUserRepPlugIn);
    procedure   RegisterPlugIn  (const PlugIn:TARUserRepPlugIn);
  published
    property    Path:string                         read  FPath     write SetPath;
    property    Title:string                        read  GetTitle  write SetTitle;
  end;

procedure Register;

implementation
{$R *.res}

procedure Register;
begin
  RegisterComponents('ARComponents', [TARUserRep]);
end;

type  TValueFlag = (vfNone, vfMoney, vfRTF);

constructor TARUserRep.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);

  NextTopBandState:=[];

  Gutter        :=0;

//  ShowBandTitle :=false;
  ShowBandTitle :=true;

  Vars          :=TAIStringList.Create('TARUserRep.Create Vars');
  PictureList   :=TAIStringList.Create('TARUserRep.Create PictureList');
  PictureList.Sorted:=true;
  GroupBands    :=TAIStringList.Create('TARUserRep.Create GroupBands');

  OrgRep        :=TARRep.Create();
  FOrgRepList   :=TStringList.Create();
  FOrgRepList.AddObject('', OrgRep);
  FPlugInList   :=TList.Create;

  oldSrcRep     :=nil;

  DeleteBands;
end;

destructor  TARUserRep.Destroy;
var i:integer;
begin
  ClearValues;

  GroupBands.Free;

  for i:=0 to PictureList.Count - 1 do begin
    TPicture(PictureList.Objects[i]).Free;
  end;
  PictureList.Free;

  Vars.Free;

  if Assigned(oldSrcRep) then begin
    SrcRep:=oldSrcRep;
  end;

//  OrgRep.Free;
  for i:=0 to FOrgRepList.Count - 1 do begin
    TARRep(FOrgRepList.Objects[i]).Free();
  end;
  FOrgRepList.Free();

  for i:=0 to FPlugInList.Count - 1 do begin
    TARUserRepPlugIn(FPlugInList[i]).FARUserRep:=nil;
  end;
  FPlugInList.Free;

  inherited;
end;

procedure   TARUserRep.MouseMove(Shift: TShiftState; X, Y: Integer);
var cell:TARCell;
    idxband,idxcell:integer;
begin
  if ssLeft in Shift then begin
    if Y < 0 then begin
      TopY:=TopY - 10;
    end else begin
      if Y > Height then begin
        TopY:=TopY + 10;
      end else begin
        cell:=MouseToCell(X, Y, idxband, idxcell);
        if Assigned(cell) then begin
          if CurrCell <> cell then begin
            CurrBandIdx:=idxband;
            CurrCellIdx:=idxcell;
            SelAdd(CurrBandIdx, CurrCellIdx);
            if Assigned(OnSelectCell) then OnSelectCell(Self, CurrBandIdx, CurrCellIdx);
          end;
        end;
      end;
    end;
  end;
end;

procedure   TARUserRep.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var idxband,idxcell:integer;
begin
  inherited;

  if Assigned(MouseToCell(X, Y, idxband, idxcell)) then begin
    CurrBandIdx :=idxband;
    CurrCellIdx :=idxcell;
    if not (ssCtrl in Shift) then ClearSel;
    SelAdd(CurrBandIdx, CurrCellIdx);
    if Assigned(OnSelectCell) then OnSelectCell(Self, CurrBandIdx, CurrCellIdx);
  end else begin
    if (idxband <> -1) and (X < Gutter) then begin

      if SrcRep[idxband].Expandable then begin
        SrcRep[idxband].Expanded:= not SrcRep[idxband].Expanded;
        Invalidate;
        if SrcRep[idxband].Expanded then begin
          if Assigned(OnExpandBand) then OnExpandBand(Self, idxband);
        end else begin
          if Assigned(OnCollapseBand) then OnCollapseBand(Self, idxband);
        end;
      end else begin
        if Assigned(OnSelectBand) then OnSelectBand(Self, idxband);
      end;
    end;
  end;
end;

procedure   TARUserRep.DeleteBands;
begin
  inherited DeleteBands;
  Invalidate;
end;

procedure   TARUserRep.ClearValues;
//var i:integer;
begin
  Vars.Clear;
  ClearPictures; //CD 
{
  for i:=0 to PictureList.Count - 1 do begin
    TPicture(PictureList.Objects[i]).Free;
  end;
  PictureList.Clear;

  for i:=0 to FPlugInList.Count - 1 do begin
    TARUserRepPlugIn(FPlugInList[i]).ClearValues;
  end;
}
end;

//CD clear only pictures and plugins, not vars
procedure   TARUserRep.ClearPictures;
var i:integer;
begin
  for i:=0 to PictureList.Count - 1 do begin
    TPicture(PictureList.Objects[i]).Free;
  end;
  PictureList.Clear;
  for i:=0 to FPlugInList.Count - 1 do begin
    TARUserRepPlugIn(FPlugInList[i]).ClearValues;
  end;
end;

//////////////////////////////////////////
function    TARUserRep.GetS(VarName:string):string;
begin
  Result:=Vars.Values[ANSIUpperCase(VarName)];
end;

function    TARUserRep.GetRTF(VarName:string):string;
begin
  Result:=Vars.Values[ANSIUpperCase(VarName)];
end;

function    TARUserRep.GetF(VarName:string):extended;
begin
  try
    Result:=StrToFloat(Vars.Values[ANSIUpperCase(VarName)]);
  except
    Result:=0.0;
  end;
end;

function    TARUserRep.GetC(VarName:string):currency;
begin
  try
    Result:=StrToCurr(Vars.Values[ANSIUpperCase(VarName)]);
  except
    Result:=0.0;
  end;
end;

function    TARUserRep.GetM(VarName:string):currency;
var p:integer;
    s:string;
begin
  try
    s:=Vars.Values[ANSIUpperCase(VarName)];
    p:=System.Pos(ARMoneySeparator, s);
    if p > 0 then begin
      s[p]:=DecimalSeparator;
    end;
    Result:=StrToCurr(s);
  except
    Result:=0.0;
  end;
end;

function    TARUserRep.GetI(VarName:string):integer;
begin
  try
    Result:=StrToInt(Vars.Values[ANSIUpperCase(VarName)]);
  except
    Result:=0;
  end;
end;

function    TARUserRep.GetD(VarName:string):TDateTime;
begin
  try
    Result:=StrToDate(Vars.Values[ANSIUpperCase(VarName)]);
  except
    Result:=Now;
  end;
end;

function    TARUserRep.GetT(VarName:string):TDateTime;
begin
  try
    Result:=StrToDateTime(Vars.Values[ANSIUpperCase(VarName)]);
  except
    Result:=Now;
  end;
end;

procedure   TARUserRep.SetS(VarName:string; VarValue:string);
var idx:integer;
begin
  Vars.Values[ANSIUpperCase(VarName)]:=VarValue;

  idx:=Vars.IndexOfName(ANSIUpperCase(VarName));
  if idx >= 0 then begin
    Vars.Objects[idx]:=pointer(Ord(vfNone));
  end;
end;

procedure   TARUserRep.SetRTF(VarName:string; VarValue:string);
var idx:integer;
begin
  Vars.Values[ANSIUpperCase(VarName)]:=VarValue;

  idx:=Vars.IndexOfName(ANSIUpperCase(VarName));
  if idx >= 0 then begin
    Vars.Objects[idx]:=pointer(Ord(vfRTF));
  end;
end;

procedure   TARUserRep.SetF(VarName:string; VarValue:extended);
begin
  Vars.Values[ANSIUpperCase(VarName)]:=Format('%f', [VarValue]);
  Vars.Objects[Vars.IndexOfName(ANSIUpperCase(VarName))]:=pointer(Ord(vfNone));
end;

procedure   TARUserRep.SetI(VarName:string; VarValue:integer);
begin
  Vars.Values[ANSIUpperCase(VarName)]:=IntToStr(VarValue);
  Vars.Objects[Vars.IndexOfName(ANSIUpperCase(VarName))]:=pointer(Ord(vfNone));
end;

procedure   TARUserRep.SetD(VarName:string; VarValue:TDateTime);
begin
  Vars.Values[ANSIUpperCase(VarName)]:=DateToStr(VarValue);
  Vars.Objects[Vars.IndexOfName(ANSIUpperCase(VarName))]:=pointer(Ord(vfNone));
end;

procedure   TARUserRep.SetT(VarName:string; VarValue:TDateTime);
begin
  Vars.Values[ANSIUpperCase(VarName)]:=DateTimeToStr(VarValue);
  Vars.Objects[Vars.IndexOfName(ANSIUpperCase(VarName))]:=pointer(Ord(vfNone));
end;

procedure   TARUserRep.SetC(VarName:string; VarValue:currency);
begin
  Vars.Values[ANSIUpperCase(VarName)]:=Format('%m', [VarValue]);
  Vars.Objects[Vars.IndexOfName(ANSIUpperCase(VarName))]:=pointer(Ord(vfNone));
end;

procedure   TARUserRep.SetM(VarName:string; VarValue:currency);
var OldDecSep:char;
begin
  OldDecSep:=DecimalSeparator;
  DecimalSeparator:=ARMoneySeparator;
  try
    Vars.Values[ANSIUpperCase(VarName)]:=Format('%.2f', [VarValue]);
    Vars.Objects[Vars.IndexOfName(ANSIUpperCase(VarName))]:=pointer(Ord(vfMoney));
  finally
    DecimalSeparator:=OldDecSep;
  end;
end;

function    TARUserRep.GetU(VarName:string):integer;
var idx:integer;
begin
  VarName:=ANSIUpperCase(VarName);
  idx:=SrcRep.UserVarList.IndexOf(VarName);
  if idx = -1 then begin
    Result:=0;
  end else begin
    Result:=integer(SrcRep.UserVarList.Objects[idx]);
  end;
end;

procedure   TARUserRep.SetU(VarName:string; UserVarValue:integer);
var idx:integer;
begin
  VarName:=ANSIUpperCase(VarName);
  idx:=SrcRep.UserVarList.IndexOf(VarName);
  if idx = -1 then begin
    SrcRep.UserVarList.AddObject(VarName, pointer(UserVarValue));
  end else begin
    SrcRep.UserVarList.Objects[idx]:=pointer(UserVarValue);
  end;
end;

function    TARUserRep.GetG(VarName:string):TGraphic;
var idx:integer;
begin
  Result:=nil;
  idx:=PictureList.IndexOf(ANSIUpperCase(VarName));
  if idx <> -1 then begin
    Result:=TPicture(PictureList.Objects[idx]).Graphic;
  end;
end;

procedure   TARUserRep.SetG(VarName:string; VarValue:TGraphic);
var idx:integer;
begin
  VarName:=ANSIUpperCase(VarName);
  idx:=PictureList.IndexOf(VarName);

  if idx = -1 then begin
    PictureList.AddObject(VarName, TPicture.Create);
    idx:=PictureList.IndexOf(VarName);
  end;
  TPicture(PictureList.Objects[idx]).Assign(VarValue);
end;

function    TARUserRep.GetTitle:string;
begin
  Result:=prn.Title;
end;

procedure   TARUserRep.SetTitle(ATitle:string);
begin
  prn.Title:=ATitle;
end;

procedure   TARUserRep.DeleteBand(IdxBand:integer);
begin
  SrcRep.DeleteBand(IdxBand);
  Invalidate;
end;

////////////////
procedure TARUserRep.InternalAddBands(BandName:string; bandlist:TAIList);
var idxband, idxcell, idx:integer;
    b:TARBand;
    H:integer;
begin
  for idx:=0 to FPlugInList.Count - 1 do begin
    TARUserRepPlugIn(FPlugInList[idx]).ApplyValues;
  end;

  SetARRTF;
  SrcRep.Font.Assign(OrgRep.Font);
  for idxband:=0 to OrgRep.BandCount - 1 do begin
    if CT(OrgRep.Bands[idxband].Name, BandName) then begin
      b:=TARBand.Create(OrgRep);
      bandlist.Add(b);
{$IFDEF D3}
      b.AssignBand(OrgRep.Bands[idxband]);
{$ELSE}
      b.Assign(OrgRep.Bands[idxband]);
{$ENDIF}
      for idxcell:=0 to b.CellCount - 1 do begin
        ApplyValues(OrgRep.Bands[idxband].Cells[idxcell], b.Cells[idxcell]);
        b.Cells[idxcell].ProcessValue;

        if b.Cells[idxcell].AutoStretch then begin
          Canvas.Font.Size    :=b.Cells[idxcell].FontSize;
          Canvas.Font.Style   :=b.Cells[idxcell].FontStyle;
          Canvas.Font.Name    :=b.Cells[idxcell].FontName;
          Canvas.Font.Charset :=Self.SrcRep.Font.Charset;

          H:=b.Cells[idxcell].Height(Canvas);
          if H > b.Height then begin
            b.Height:=H;
          end;
        end;
      end;
    end;
  end;
end;

function    TARUserRep.AddSubBands(BandName:string; Expanded:boolean):TARBand;
var b:TARBand;
begin
  Result:=nil;
  if SrcRep.BandCount > 0 then begin
    b:=SrcRep.Bands[SrcRep.BandCount - 1];

    Result:=AddSubBandsToBand(BandName, b, Expanded);
  end;
end;

{
function    TARUserRep.InsertBands(InsertIdxBand:integer; BandName:string):integer;
var  idxband,idxcell:integer;
    b:TARBand;
    H:integer;
begin
  Result:=0;
  SrcRep.Font.Assign(OrgRep.Font);
  for idxband:=0 to OrgRep.BandCount - 1 do begin
    if CT(OrgRep.Bands[idxband].Name, BandName) then begin
			inc(Result);
      b:=NewBand;
      if InsertIdxBand < SrcRep.BandCount - 1 then begin
        SrcRep.Move(SrcRep.BandCount - 1, InsertIdxBand + 1);
      end;
      b.Assign(OrgRep.Bands[idxband]);
      for idxcell:=0 to b.CellCount - 1 do begin
        ApplyValues(OrgRep.Bands[idxband].Cells[idxcell], b.Cells[idxcell]);
        b.Cells[idxcell].ProcessValue;

        if b.Cells[idxcell].AutoStretch then begin
          Canvas.Font.Size    :=b.Cells[idxcell].FontSize;
          Canvas.Font.Style   :=b.Cells[idxcell].FontStyle;
          Canvas.Font.Name    :=b.Cells[idxcell].FontName;
          Canvas.Font.Charset :=Self.SrcRep.Font.Charset;

          H:=b.Cells[idxcell].Height(Canvas);
          if H > b.Height then begin
            b.Height:=H;
          end;
        end;
      end;
    end;
  end;
  Invalidate;
end;
}
function    TARUserRep.InsertBands(InsertIdxBand:integer; BandName:string):TARBand;
var i:integer;
    bl:TAIList;
begin
  Result:=nil;
  bl:=TAIList.Create('TARUserRep.InsertBands bl');
  try
    InternalAddBands(BandName, bl);

    if bl.Count > 0 then begin
      if InsertIdxBand < SrcRep.BandCount - 1 then begin
        for i:=0 to bl.Count - 1 do begin
          SrcRep.Insert(InsertIdxBand, bl[i]);
        end;
      end else begin
        for i:=0 to bl.Count - 1 do begin
          SrcRep.Add(bl[i]);
        end;
        TARBand(bl[bl.Count - 1]).BandState:=NextTopBandState;
        NextTopBandState:=[];
      end;
      Result:=bl[bl.Count - 1];
    end;
  finally
    bl.Free;
  end;
  Invalidate;
end;


{$IFDEF D3}
function  TARUserRep.AddBands(BandName:string):TARBand;
var i:integer;
    bl:TAIList;
begin
  Result:=nil;
  bl:=TAIList.Create('TARUserRep.AddBands bl');
  try
    InternalAddBands(BandName, bl);

    for i:=0 to bl.Count - 1 do SrcRep.Add(bl[i]);
    if bl.Count > 0 then begin
      Result:=bl[bl.Count - 1];

      // set band state to first added band
      TARBand(bl[0]).BandState:=NextTopBandState;
      NextTopBandState:=[];
    end;
  finally
    bl.Free;
  end;
  Invalidate;
end;
{$ELSE}
function  TARUserRep.AddBands(BandName:string; ToBand:TARBand = nil; Expanded:boolean = true):TARBand;
var i:integer;
    bl:TAIList;
begin
  Result:=nil;
  bl:=TAIList.Create('TARUserRep.AddBands bl');
  try
    InternalAddBands(BandName, bl);

    if Assigned(ToBand) then begin
      ToBand.Expanded:=Expanded;
      for i:=0 to bl.Count - 1 do ToBand.Add(bl[i]);
    end else begin
      for i:=0 to bl.Count - 1 do SrcRep.Add(bl[i]);
    end;
    if bl.Count > 0 then begin
      Result:=bl[bl.Count - 1];

      if not Assigned(ToBand) then begin
        // set band state to first added band
        TARBand(bl[0]).BandState:=NextTopBandState;
        NextTopBandState:=[];
      end;
    end;
  finally
    bl.Free;
  end;
  Invalidate;
end;
{$ENDIF}

function    TARUserRep.AddSubBandsToBand(BandName:string; ToBand:TARBand; Expanded:boolean):TARBand;
var i:integer;
    bl:TAIList;
begin
  Result:=nil;
  bl:=TAIList.Create('TARUserRep.AddSubBands bl');
  try
    InternalAddBands(BandName, bl);

    if Assigned(ToBand) then begin
      ToBand.Expanded:=Expanded;
      for i:=0 to bl.Count - 1 do ToBand.Add(bl[i]);
    end else begin
      for i:=0 to bl.Count - 1 do SrcRep.Add(bl[i]);
    end;

    if bl.Count > 0 then begin
      Result:=bl[bl.Count - 1];
    end;
  finally
    bl.Free;
  end;
  Invalidate;
end;

procedure TARUserRep.ApplyValues(src:TARCell; dst:TARCell);
var p:integer;
    i,idx:integer;
begin
  inherited;
  
  dst.Value       :='';
  dst.MinWidth    :=src.Width;
  dst.IsMoneyValue:=false;

  if Assigned(src.VarNameList) then begin
    for i:=0 to src.VarNameList.Count - 1 do begin
      if integer(src.VarNameList.Objects[i]) = 0 then begin
        dst.Value:=dst.Value + src.VarNameList[i];
      end else begin
        dst.Value:=dst.Value + S[src.VarNameList[i]];
        idx:=Vars.IndexOfName(src.VarNameList[i]);
        if idx >= 0 then begin
          dst.IsMoneyValue:=Vars.Objects[idx] = pointer(Ord(vfMoney));
          dst.IsRTF:=Vars.Objects[idx] = pointer(Ord(vfRTF));
        end;
        if Assigned(G[src.VarNameList[i]]) then begin
          dst.Picture.Assign(G[src.VarNameList[i]]);
        end;
      end;
    end;
  end;

  dst.UserVarValue:=-1;
  p:=SrcRep.UserVarList.IndexOf(dst.UserVar);
  if p <> -1 then begin
    dst.UserVarValue:=integer(SrcRep.UserVarList.Objects[p]);
  end;
end;

function    TARUserRep.GetTemplate:TARTemplate;
begin
  Result:=OrgRep.ARTemplate;
end;

procedure   TARUserRep.SetTemplate(ATemplate:TARTemplate);
begin
  OrgRep.ARTemplate:=ATemplate;
  PrepareTemplate();
//  Invalidate;
end;

function    TARUserRep.GetBandCount(BandName:string):integer;
var IdxBand:integer;
begin
  Result:=0;
  for IdxBand:=0 to SrcRep.BandCount - 1 do begin
    if CT(BandName, SrcRep.Bands[IdxBand].Name) then inc(Result);
  end;
end;

procedure   TARUserRep.Loaded;
var IdxBand, IdxCell:integer;
begin
  inherited Loaded;

  if csDesigning in ComponentState then begin
    oldSrcRep:=SrcRep;
    SrcRep:=OrgRep;
  end else begin
    for IdxBand:=0 to OrgRep.BandCount - 1 do begin
      for IdxCell:=0 to OrgRep.Bands[IdxBand].CellCount - 1 do begin
        OrgRep.Bands[IdxBand].Cells[IdxCell].CreateVarNameList;
      end;
    end;
  end;
end;

procedure   TARUserRep.SaveToFile(FFile :string);
begin
  SrcRep.Save(FFile);
end;

procedure   TARUserRep.LoadFromFile(FFile :string);
begin
  SrcRep.Load(FFile);
  NextTopBandState:=[];
end;

function    TARUserRep.NewGroup(const BandName:string; Value:variant):boolean;
begin
  Result:=not CT(GroupBands.Values[BandName], VarAsType(Value, varString));
end;

{$IFDEF D3}
function    TARUserRep.AddGroupBands(const BandName:string; Value:variant):TARBand;
begin
  Result:=AddGroupBandsToBand(BandName, Value, nil, false);
end;
function    TARUserRep.AddGroupBandsToBand(const BandName:string; Value:variant; ToBand:TARBand; Expanded:boolean):TARBand;
var idx, j:integer;
begin
  Result:=nil;
  if not CT(GroupBands.Values[BandName], VarAsType(Value, varString)) then begin

    S[BandName]:=VarAsType(Value, varString);
    Result:=AddSubBandsToBand(BandName, ToBand, Expanded);

    GroupBands.Values[BandName]:=VarAsType(Value, varString);
    idx:=GroupBands.IndexOfName(BandName);
    for j:=idx + 1 to GroupBands.Count - 1 do begin
      GroupBands[j]:='';
    end;
  end;
end;
{$ELSE}
function    TARUserRep.AddGroupBands(const BandName:string; Value:variant; ToBand:TARBand = nil; Expanded:boolean = true):TARBand;
var idx, j:integer;
begin
  Result:=nil;
  if not CT(GroupBands.Values[BandName], VarAsType(Value, varString)) then begin

    S[BandName]:=VarAsType(Value, varString);
    Result:=AddBands(BandName, ToBand, Expanded);

    GroupBands.Values[BandName]:=VarAsType(Value, varString);
    idx:=GroupBands.IndexOfName(BandName);
    for j:=idx + 1 to GroupBands.Count - 1 do begin
      GroupBands[j]:='';
    end;
  end;
end;
{$ENDIF}

{$IFDEF MSWINDOWS}
procedure   TARUserRep.SetDSValues(DS:TDataSet);
var i:integer;
    Picture:TPicture;
begin
  for i:=0 to DS.FieldCount - 1 do begin
    case DS.Fields[i].DataType of
      ftSmallint,
      ftInteger,
      ftWord,
      ftAutoInc,
{$IFNDEF D3}
      ftLargeInt,
      ftGuid,
{$ENDIF}
      ftBCD:        Self.I[DS.Fields[i].FieldName]:=DS.Fields[i].AsInteger;

      ftBoolean:	  if DS.Fields[i].AsBoolean then begin
                      Self.S[DS.Fields[i].FieldName]:='True';
                    end else begin
                      Self.S[DS.Fields[i].FieldName]:='False';
                    end;

      ftString,
      ftMemo,
{$IFNDEF D3}
      ftFixedChar,
      ftWideString,
{$ENDIF}
      ftFmtMemo:    Self.S[DS.Fields[i].FieldName]:=DS.Fields[i].AsString;

      ftFloat:      Self.F[DS.Fields[i].FieldName]:=DS.Fields[i].AsFloat;

      ftCurrency:	  Self.C[DS.Fields[i].FieldName]:=DS.Fields[i].AsCurrency;

      ftDate:       Self.D[DS.Fields[i].FieldName]:=DS.Fields[i].AsDateTime;

      ftTime,
      ftDateTime:   Self.T[DS.Fields[i].FieldName]:=DS.Fields[i].AsDateTime;

      ftBlob,
      ftTypedBinary,
      ftGraphic:    begin
                      Picture:=TPicture.Create;
                      try
                        Picture.Assign(DS.Fields[i] as TBlobField);
                        Self.G[DS.Fields[i].FieldName]:=Picture.Graphic;
                      finally
                        Picture.Free;
                      end;
                    end;

      else          begin
                      Self.S[DS.Fields[i].FieldName]:='Unsupported format';
                    end;
//      ftUnknown	Unknown or undetermined
//      ftBytes	Fixed number of bytes (binary storage)
//      ftVarBytes	Variable number of bytes (binary storage)
//      ftParadoxOle	Paradox OLE field
//      ftDBaseOle	dBASE OLE field
//      ftADT	Abstract Data Type field
//      ftArray	Array field
//      ftReference	REF field
//      ftDataSet	DataSet field
//      ftOraBlob	BLOB fields in Oracle 8 tables
//      ftOraClob	CLOB fields in Oracle 8 tables
//      ftVariant	Data of unknown or undetermined type
//      ftInterface	References to interfaces (IUnknown)
//      ftIDispatch	References to IDispatch interfaces
    end;
  end;
end;
{$ENDIF}

{
function    TARUserRep.GetCell(BandName:string; IdxBand:integer; ValName:string):TARCell;
var AIdxBand, IdxBandOrg, IdxCell, IdxVar, Idx:integer;
begin
  Result:=nil;
  for AIdxBand:=0 to SrcRep.BandCount - 1 do begin
    if CT(BandName, SrcRep.Bands[AIdxBand].Name) then begin
      if IdxBand = 0 then begin //band found
        for IdxBandOrg:=0 to OrgRep.BandCount - 1 do begin  //look for template
          if CT(BandName, OrgRep.Bands[IdxBandOrg].Name) then begin //band found
            for IdxCell:=0 to OrgRep.Bands[IdxBandOrg].CellCount - 1 do begin //look for cell
              for IdxVar:=0 to OrgRep.Bands[IdxBandOrg].Cells[IdxCell].VarNameList.Count - 1 do begin //look for vars
                Idx:=OrgRep.Bands[IdxBandOrg].Cells[IdxCell].VarNameList.IndexOf(ValName);
                if Idx >= 0 then begin //var found
                  Result:=SrcRep.Bands[AIdxBand].Cells[IdxCell];
                  exit;
                end;
              end;
            end;
            exit; //nothing found
          end;
        end;
      end;
      dec(IdxBand);
    end;
  end;
end;
}

{$IFDEF D3}
function    TARUserRep.CellsByName(ValName:string):TARCell;
{$ELSE}
function    TARUserRep.Cells(ValName:string):TARCell;
{$ENDIF}
var IdxBandOrg, IdxCell, IdxVar, Idx:integer;
begin
  Result:=nil;

  for IdxBandOrg:=0 to OrgRep.BandCount - 1 do begin  //look for template
    if CT(CurrBand.Name, OrgRep.Bands[IdxBandOrg].Name) then begin //band found
      for IdxCell:=0 to OrgRep.Bands[IdxBandOrg].CellCount - 1 do begin //look for cell
        for IdxVar:=0 to OrgRep.Bands[IdxBandOrg].Cells[IdxCell].VarNameList.Count - 1 do begin //look for vars
          Idx:=OrgRep.Bands[IdxBandOrg].Cells[IdxCell].VarNameList.IndexOf(ValName);
          if Idx >= 0 then begin //var found
            Result:=CurrBand.Cells[IdxCell];
            exit;
          end;
        end;
      end;
      exit; //nothing found
    end;
  end;
end;

procedure   TARUserRep.SetFocus;
begin
  inherited;
  
  ClearSel;
  SelAdd(CurrBandIdx, CurrCellIdx);
end;

procedure   TARUserRep.PrepareTemplate();
var IdxBand, IdxCell:integer;
begin
  prn.PageSize    :=OrgRep.PageSize;
  prn.Orientation :=OrgRep.Orientation;

  for IdxBand:=0 to OrgRep.BandCount - 1 do begin
    for IdxCell:=0 to OrgRep.Bands[IdxBand].CellCount - 1 do begin
      OrgRep.Bands[IdxBand].Cells[IdxCell].CreateVarNameList;
    end;
  end;

  SrcRep.Font.Assign(OrgRep.Font);
  Repaint;
end;

procedure   TARUserRep.SetPath(APath:string);
begin
  Load(APath);
end;

function    TARUserRep.Cells(IdxBand:integer; IdxCell:integer):TARCell;
begin
  Result:=SrcRep.Cells[IdxBand, IdxCell];
end;

//////////////////////////////////////////////////////////////////////////////////////
procedure   TARUserRep.MoveToPriorBand;
begin
  CurrBandIdx:=Max(SrcRep.IndexOfBand(CurrCell.GetTopBand) - 1, 0);
end;

procedure   TARUserRep.MoveToNextBand;
begin
  CurrBandIdx:=Min(SrcRep.IndexOfBand(CurrCell.GetBottomBand) + 1, SrcRep.BandCount - 1);
end;

procedure   TARUserRep.PageBreak;
begin
  NextTopBandState:=NextTopBandState + [arbsPageBreak];
end;

procedure   TARUserRep.BeginPara;
begin
  EndPara;

  NextTopBandState:=NextTopBandState + [arbsBeginPara];
end;

procedure   TARUserRep.EndPara;
begin
  if SrcRep.BandCount > 0 then begin
    SrcRep.Bands[SrcRep.BandCount - 1].BandState:=SrcRep.Bands[SrcRep.BandCount - 1].BandState + [arbsEndPara];
  end;
  NextTopBandState:=NextTopBandState - [arbsEndPara];
end;

procedure TARUserRep.ResetPageNumber;
begin
  NextTopBandState:=NextTopBandState + [arbsResetPageNumber];
end;

procedure   TARUserRep.Load(FileName:string);
var idx:integer;
begin
  FPath:=FileName;

  if (FOrgRepList.Count = 1) and (FOrgRepList[0] = '') then begin
    OrgRep:=TARRep(FOrgRepList.Objects[0]);
    FOrgRepList[0]:=FileName;
  end else begin
    idx:=FOrgRepList.IndexOf(FileName);
    if idx <> -1 then begin
      OrgRep:=TARRep(FOrgRepList.Objects[idx]);
    end else begin
      OrgRep:=TARRep.Create();
      FOrgRepList.AddObject(FileName, OrgRep);
    end;
  end;

  OrgRep.FileSystem:=Self.FileSystem;
  OrgRep.Load(FileName);
  SrcRep.PageSize     :=OrgRep.PageSize;
  SrcRep.Orientation  :=OrgRep.Orientation;
  SrcRep.LeftMargin   :=OrgRep.LeftMargin;
  SrcRep.TopMargin    :=OrgRep.TopMargin;
  SrcRep.RightMargin  :=OrgRep.RightMargin;
  SrcRep.BottomMargin :=OrgRep.BottomMargin;

  PrepareTemplate();
end;

{$IFNDEF D3}
procedure TARUserRep.Resize;
var idxband:integer;
begin
  for idxband:=0 to SrcRep.BandCount - 1 do begin
    SrcRep.Bands[idxband].SetWidth(Canvas, ClientWidth - Gutter - 5);
  end;

  inherited;
end;
{$ENDIF}

function  TARUserRep.MergeBands(BandName:string; ToBand:TARBand):TARBand;
var i:integer;
    bl:TAIList;
begin
  bl:=TAIList.Create('TARUserRep.MergeBands bl');
  try
    Result:=nil;

    InternalAddBands(BandName, bl);

    if bl.Count > 0 then begin
      ToBand.AddCells(TARBand(bl[0]));
    end;

    for i:=0 to bl.Count - 1 do begin
      TARBand(bl[i]).Free();
    end;

    Result:=ToBand;
  finally
    bl.Free;
  end;
  Invalidate;
end;

procedure   TARUserRep.RegisterPlugIn  (const PlugIn:TARUserRepPlugIn);
begin
  if FPlugInList.IndexOf(PlugIn) = -1 then begin
    FPlugInList.Add(PlugIn);
  end;
end;

procedure   TARUserRep.UnRegisterPlugIn(const PlugIn:TARUserRepPlugIn);
var idx:integer;
begin
  idx:=FPlugInList.IndexOf(PlugIn);
  if idx <> -1 then begin
    FPlugInList.Delete(idx);
  end;
end;

{ TARUserRepPlugIn }

procedure   TARUserRepPlugIn.ClearValues;
var i:integer;
begin
  for i:=0 to VarCount - 1 do begin
    FVarList.Objects[i].Free;
  end;

  FVarList.Clear;
end;

constructor TARUserRepPlugIn.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FVarList:=TStringList.Create;
  FVarList.Sorted:=true;
end;

destructor  TARUserRepPlugIn.Destroy;
begin
  ClearValues;
  FVarList.Free;

  inherited;
end;

procedure   TARUserRepPlugIn.SetARUserRep(const Value: TARUserRep);
begin
  if FARUserRep <> Value then begin
    if Assigned(FARUserRep) then begin
      FARUserRep.UnRegisterPlugIn(Self);
    end;

    FARUserRep:=Value;

    if Assigned(FARUserRep) then begin
      FARUserRep.RegisterPlugIn(Self)
    end;
  end;
end;

function    TARUserRepPlugIn.GetVarCount:integer;
begin
  Result:=FVarList.Count;
end;

procedure   TARUserRepPlugIn.ApplyValues;
begin
end;

function TARUserRepPlugIn.GetVarByName(const VarName: string): TObject;
var idx:integer;
    tmp:TObject;
begin
  idx:=FVarList.IndexOf(VarName);
  if idx = -1 then begin
    tmp:=CreateVar;
    FVarList.AddObject(VarName, tmp);
    Result:=tmp;
  end else begin
    Result:=FVarList.Objects[idx];
  end;
end;

function TARUserRepPlugIn.GetVarByIndex(VarIndex: integer): TObject;
begin
  Result:=FVarList.Objects[VarIndex];
end;

function TARUserRepPlugIn.GetVarNames(VarIndex: integer): string;
begin
  Result:=FVarList[VarIndex];
end;

end.

