unit ARUtils;
//************************************************************//
//                         AND Report                        //
//  Eugene Hrulev (mrHru) Laxian Key, Omsk, Russia 1999,2001 //
//  Oleg Melnikov (muler) Laxian Key, Omsk, Russia 1999,2001 //
//************************************************************//

{$INCLUDE DEFINES.INC}

interface

uses
  Classes, SysUtils, IniFiles,

  {$IFDEF CLX}
    Variants, QForms, QGraphics, QDialogs, QControls, QImgList, Qt;
  {$ELSE}
    {$IFDEF D6}
      Variants,
    {$ENDIF}
    Forms, Graphics, Dialogs, Controls;
  {$ENDIF}


type

  TAIList = class(TList)
  public
    Desc        :string;
    constructor Create(ADesc:string);
    destructor  Destroy;override;
  end;

  TAIStringList = class(TStringList)
  public
    Desc        :string;
    constructor Create(ADesc:string);
    destructor  Destroy;override;
  end;

const
  CR   = #$0d;
  LF   = #$0a;
  CRLF = #$0d + #$0a;

{$IFDEF CLX}
  VK_HOME     = Key_HOME;
  VK_END      = Key_END;
  VK_PRIOR    = Key_PRIOR;
  VK_NEXT     = Key_NEXT;
  VK_UP       = Key_UP;
  VK_DOWN     = Key_DOWN;
  VK_LEFT     = Key_LEFT;
  VK_RIGHT    = Key_RIGHT;
  VK_ADD      = Key_Plus;
  VK_SUBTRACT = Key_Minus;
  VK_ESCAPE   = Key_ESCAPE;
  VK_RETURN   = Key_RETURN;
  VK_F2       = Key_F2;
{$ENDIF}

procedure   SavePlace(Sender:TForm; FormClassName:string);
procedure   RestorePlace(Sender:TForm;FormClassName:string);
function    CT(const s1:string; const s2:string):boolean;// compare strings with case unsensitivy
function    GetProp(var term:string):string;
function    Upper(s:string):string;
function    PropListToStr(PropList:TAIStringList):string;
procedure   StrToPropList(s:string; APropList:TAIStringList);
function    ExtractFileNameOnly(const FileName: string):string; //without .ext
function    ExtractFilePathAndNameOnly(const FileName: string): string;//without .ext
function    SafeVarToStr(v:variant):string;
{$IFDEF D3}
  function    Max(a:integer; b:integer):integer;
  function    Min(a:integer; b:integer):integer;
  function    MaxEx(a:extended; b:extended):extended;
  function    MinEx(a:extended; b:extended):extended;
{$ENDIF}

// loads bmp from resource or returns already loaded
function  GetBitmap(ImageName:string):TBitmap;

procedure ShowAIListCounts;
function  GetAIListCounts:string;
var
  ListList        :TList;
  ListStringList  :TList;

implementation
{$R ImagesGear.res}

function  Upper(s:string):string;
begin
  Result:=ANSIUpperCase(s);
end;

procedure   SavePlace(Sender:TForm;FormClassName:string);
var inf:TIniFile;
begin
  inf:=TIniFile.Create('Interpretator.ini');
    inf.WriteInteger(FormClassName, 'Left',  Sender.Left);
    inf.WriteInteger(FormClassName, 'Top',   Sender.Top);
    inf.WriteInteger(FormClassName, 'Width', Sender.Width);
    inf.WriteInteger(FormClassName, 'Height',Sender.Height);
  inf.Free;
end;

procedure   RestorePlace(Sender:TForm;  FormClassName:string);
var inf:TIniFile;
begin
  inf:=TIniFile.Create('Interpretator.ini');
    Sender.SetBounds( inf.ReadInteger(FormClassName, 'Left', 0),
                      inf.ReadInteger(FormClassName, 'Top', 0),
                      inf.ReadInteger(FormClassName, 'Width', 300),
                      inf.ReadInteger(FormClassName, 'Height', 200)
                    );
  inf.Free;
end;

function    CT(const s1:string; const s2:string):boolean;// compare strings with case unsensitivy
begin
  Result:=ANSIUpperCase(s1) = ANSIUpperCase(s2);
end;

function  GetProp(var term:string):string;
var p:integer;
begin
  Result:=term;

  p:=System.Pos('.', term);
  if p = 1 then begin
    Result:=term;
    term:='';
  end else
  if p > 1 then begin
    Result:=term;
    term:=System.Copy(Result, 1, p-1);
    System.Delete(Result, 1, p);
  end else begin
    Result:='';
  end;
end;

function    ExtractFileNameOnly(const FileName: string):string;
var p:integer;
    s:string;
begin
  Result:='';
  s:=ExtractFileName(Filename);
  p:=System.Pos('.', s);
  if p > 0 then begin
    while p <> 0 do begin
      Result:=Result + System.Copy(s, 1, p - 1);
      System.Delete(s, 1, p);
      p:=System.Pos('.', s);
    end;
  end else begin
    Result:=s;
  end;
end;

function    ExtractFilePathAndNameOnly(const FileName: string):string;
var p:integer;
    s:string;
begin
  Result:='';
  s:=Filename;
  p:=System.Pos('.', s);
  if p > 0 then begin
    while p <> 0 do begin
      Result:=Result + System.Copy(s, 1, p - 1);
      System.Delete(s, 1, p);
      p:=System.Pos('.', s);
    end;
  end else begin
    Result:=s;
  end;
end;

constructor TAIList.Create(ADesc:string);
begin
  inherited Create;
  Desc:=ADesc;
  ListList.Add(Self);
end;
destructor  TAIList.Destroy;
begin
  ListList.Delete(ListList.IndexOf(Self));
  inherited;
end;
constructor TAIStringList.Create(ADesc:string);
begin
  inherited Create;
  Desc:=ADesc;
  ListStringList.Add(Self);
end;
destructor  TAIStringList.Destroy;
begin
  ListStringList.Delete(ListStringList.IndexOf(Self));
  inherited;
end;

function  PropListToStr(PropList:TAIStringList):string;
var i:integer;
begin
  if PropList.Count >0 then begin
    Result:=PropList[0];
    for i:=1 to PropList.Count - 1 do Result:=Result + '.' + PropList[i];
  end else begin
    Result:='';
  end;
end;

procedure   StrToPropList(s:string; APropList:TAIStringList);
var p:integer;
begin
  APropList.Clear;
  if Length(s) > 0 then begin
    s:=Upper(s);
    p:=System.Pos('.', s);
    while p > 0 do begin
      if p > 1 then APropList.Add(System.Copy(s, 1, p-1));
      System.Delete(s, 1, p);
      p:=System.Pos('.', s);
    end;
    APropList.Add(s);
  end;
end;

function    SafeVarToStr(v:variant):string;
begin
  if VarType(v) in [varEmpty, varNull] then begin
    Result:='';
  end else begin
    Result:=v;
  end;
end;


{$IFDEF D3}
function    Max(a:integer; b:integer):integer;
begin
  if a > b then Result:=a else Result:=b;
end;

function    Min(a:integer; b:integer):integer;
begin
  if a < b then Result:=a else Result:=b;
end;

function    MaxEx(a:extended; b:extended):extended;
begin
  if a > b then Result:=a else Result:=b;
end;

function    MinEx(a:extended; b:extended):extended;
begin
  if a < b then Result:=a else Result:=b;
end;

{$ENDIF}

function  GetAIListCounts:string;
var i,j:integer;
    L:TStringList;
    SL:TStringList;
    s:string;
begin
  L:=TStringList.Create;
  SL:=TStringList.Create;
  try
    for i:=0 to ListList.Count - 1 do begin
      j:=L.IndexOf(TAIList(ListList[i]).Desc);
      if j <> -1 then begin
        L.Objects[j]:=pointer(integer(L.Objects[j]) + 1);
      end else begin
        L.AddObject(TAIList(ListList[i]).Desc, pointer(1));
      end;
    end;

    for i:=0 to ListStringList.Count - 1 do begin
      j:=SL.IndexOf(TAIStringList(ListStringList[i]).Desc);
      if j <> -1 then begin
        SL.Objects[j]:=pointer(integer(SL.Objects[j]) + 1);
      end else begin
        SL.AddObject(TAIStringList(ListStringList[i]).Desc, pointer(1));
      end;
    end;

    s:='Lists' + CRLF;
    for i:=0 to L.Count - 1 do begin
      s:=s + Format('%s = %d', [L[i], integer(L.Objects[i])]) + CRLF;
    end;

    s:=s + 'StringLists' + CRLF;
    for i:=0 to SL.Count - 1 do begin
      s:=s + Format('%s = %d', [SL[i], integer(SL.Objects[i])]) + CRLF;
    end;
    Result:=s;

  finally
    L.Free;
    SL.Free;
  end;
end;

procedure ShowAIListCounts;
begin
  MessageDlg(GetAIListCounts, mtInformation, [mbOK], 0);
end;

var ImageList :TStringList;
    {$IFDEF CLX}
    {$ELSE}
//      ImList    :TImageList;
    {$ENDIF}

{//$IFDEF CLX}
function  GetBitmap(ImageName:string):TBitmap;
var idx:integer;
  procedure LoadBitmap;
  var bmp:TBitmap;
  begin
    bmp:=TBitmap.Create();
    bmp.LoadFromResourceName(HInstance, ImageName);
    bmp.TransparentMode:=tmAuto;
    bmp.Transparent:=true;
    ImageList.AddObject(ImageName, bmp);
  end;
begin
  idx:=ImageList.IndexOf(ImageName);
  if idx = -1 then begin
    LoadBitmap;
    idx:=ImageList.IndexOf(ImageName);
  end;

  Result:=TBitmap(ImageList.Objects[idx]);
end;
{//$ELSE}
(*
function  GetBitmap(ImageName:string):TBitmap;
var idx:integer;
  procedure LoadBitmap;
  var bmp:TBitmap;
  begin
    bmp:=TBitmap.Create();
    try
      bmp.LoadFromResourceName(HInstance, ImageName);
      {$IFNDEF D3}
        bmp.TransparentMode:=tmAuto;
      {$ENDIF}
      bmp.Transparent:=true;
      ImList.Add(bmp, nil);
      ImageList.AddObject(ImageName, pointer(ImList.Count - 1));
    finally
      bmp.Free();
    end;
  end;
begin
  idx:=ImageList.IndexOf(ImageName);
  if idx = -1 then begin
    LoadBitmap;
    idx:=ImageList.IndexOf(ImageName);
  end;

  Result:=TBitmap.Create();
  {$IFNDEF D3}
    Result.TransparentMode:=tmAuto;
  {$ENDIF}
  Result.Transparent:=true;
  ImList.GetBitmap(integer(ImageList.Objects[idx]), Result);
end;
*)
{//$ENDIF}

initialization
  ListList        :=TList.Create;
  ListStringList  :=TList.Create;

  {$IFDEF CLX}
  {$ELSE}
//    ImList          :=TImageList.CreateSize(16, 16);
//    ImList.Masked   :=false;
  {$ENDIF}
  ImageList       :=TStringList.Create;
  ImageList.Sorted:=true;

finalization
  ListList.Free();
  ListStringList.Free();

  {//$IFDEF CLX}
    while ImageList.Count > 0 do begin
      TBitmap(ImageList.Objects[0]).Free;
      ImageList.Delete(0);
    end;
  {//$ELSE}
//    ImList.Free();
  {///$ENDIF}
  ImageList.Free();
end.

