unit RepData;

interface

uses
  Classes, ARUtils, ARGridRep, ARUserRep, ARRep, Barcode, ARBarcode, jpeg, SysUtils, graphics, clipbrd;

type
  TRepData = class
    private
      BC: TARBarcode;
      rep: TARUserRep;
      F: Text;
      slBand: TStringList;
      NextBand: String;
    public
      constructor Create(ARep: TARUserRep; FileName: String);
      destructor Destroy; override;
      procedure Load;
      function GetBand: String;
  end;

implementation

//uses dialogs;

// find varNames of a group of band with same name
procedure RepGetVars(Rep: TARUserRep; bandName: String; sl: TStringList);
var i,idxband,idxcell:integer;
    b:TARBand;
begin
  sl.Clear;
  for idxband:=0 to Rep.OrgRep.BandCount - 1 do begin
    if CT(Rep.OrgRep.Bands[idxband].Name, BandName) then begin
      b := Rep.OrgRep.Bands[idxband];
      for idxcell:=0 to b.CellCount - 1 do begin
        b.cells[idxcell].CreateVarNameList;
        if Assigned(b.cells[idxcell].VarNameList) then begin
          for i:=0 to b.cells[idxcell].VarNameList.Count - 1 do begin
            if integer(b.cells[idxcell].VarNameList.Objects[i]) <> 0 then begin
              sl.Add(b.cells[idxcell].VarNameList[i]);
              sl.Objects[sl.Count-1] := b.cells[idxcell] ;
            end;
          end;
        end;
      end;
    end;
  end;
end;


// setup barcode from a string passed
procedure setBarcode(BC: TAsBarcode; s: String);
var st: String;

  function getWord: String;
  var i: Integer;
  begin
    i := pos(',',s);
    if i>0 then begin
      result := copy(s,1,i-1);
      s := copy(s,i+1,maxint);
    end else begin
      result := s;
      s := '';
    end;  
  end;

begin
  // default values
  with BC do begin
    Height := 50;
    Modul := 1;
    Ratio := 2;
    ShowText := bcoCode;
  end;

  // extract first word
  st := UpperCase(GetWord);
  // barcode type
  st := UpperCase(GetWord);
  if      st = '25INTERLEAVED' then BC.Typ := bcCode_2_5_interleaved
  else if st = '25INDUSTRIAL'  then BC.Typ := bcCode_2_5_industrial
  else if st = '25MATRIX'      then BC.Typ := bcCode_2_5_matrix
  else if st = 'CODE39'        then BC.Typ := bcCode39
  else if st = 'CODE39X'       then BC.Typ := bcCode39Extended
  else if st = 'CODE128A'      then BC.Typ := bcCode128A
  else if st = 'CODE128B'      then BC.Typ := bcCode128B
  else if st = 'CODE128C'      then BC.Typ := bcCode128C
  else if st = 'CODE93'        then BC.Typ := bcCode93
  else if st = 'CODE93X'       then BC.Typ := bcCode93Extended
  else if st = 'MSI'           then BC.Typ := bcCodeMSI
  else if st = 'POSTNET'       then BC.Typ := bcCodePostNet
  else if st = 'CODABAR'       then BC.Typ := bcCodeCodabar
  else if st = 'EAN8'          then BC.Typ := bcCodeEAN8
  else if st = 'EAN13'         then BC.Typ := bcCodeEAN13
  else if st = 'UPCA'          then BC.Typ := bcCodeUPC_A
  else if st = 'UPCE0'         then BC.Typ := bcCodeUPC_E0
  else if st = 'UPCE1'         then BC.Typ := bcCodeUPC_E1
  else if st = 'UPCSUPP2'      then BC.Typ := bcCodeUPC_Supp2
  else if st = 'UPCSUPP5'      then BC.Typ := bcCodeUPC_Supp5
  else if st = 'EAN128A'       then BC.Typ := bcCodeEAN128A
  else if st = 'EAN128B'       then BC.Typ := bcCodeEAN128B
  else if st = 'EAN128C'       then BC.Typ := bcCodeEAN128C;

  // the code
  BC.Text := GetWord;

//  if BC.Text = 'PPD' then
//    MessageDlg('ecco', mtInformation, [mbOK], 0);

  // other options
  repeat
    st := UpperCase(GetWord);
    if      pos('HEIGHT',st) = 1 then BC.Height := StrToInt(copy(st,8,maxint))
    else if pos('WIDTH',st) = 1  then BC.Width  := StrToInt(copy(st,7,maxint))
    else if pos('MODUL',st) = 1  then BC.Modul  := StrToInt(copy(st,7,maxint))
    else if pos('RATIO',st) = 1  then BC.Ratio  := StrToInt(copy(st,7,maxint))
    else if pos('TEXT',st) = 1   then begin
                                        if copy(st,6,maxint) = 'NO' then BC.ShowText := bcoNone;
                                      end;
  until st = ''

end;

// assign values to variables of a band
procedure RepAssignVars(Rep: TARUserRep; bandName: String; sl: TStringList; BC: TARBarcode);
var
  s : string;
  JPG : TJPEGImage;
  BMP : TBitmap;
  c : TARCell;
  slVars: TStringList;
  i,t: Integer;
begin
  slVars := TStringList.Create;
  try
    RepGetVars(Rep,bandName,slVars);
    for i := 0 to slVars.Count -1 do begin
      if i < sl.Count then begin
        if pos('@BARCODE',UpperCase(sl[i])) = 1 then begin // --------------- barcode
          setBarcode(BC[slVars[i]],sl[i]) ;
        end else if pos('@GRAPHIC',UpperCase(sl[i])) = 1 then begin //- graphic image
          t := pos(',',sl[i]);
          s := UpperCase(copy(sl[i],t+1,maxint));
          c := TARCell(slVars.Objects[i]);
          c.PictureFileName := s;
          if (ExtractFileExt(s) = '.JPG') or (ExtractFileExt(s) = '.JPEG') then begin
            JPG := TJPEGImage.Create;
            try
              JPG.LoadFromFile(s);
              Clipboard.Assign(jpg)
            except;
            end;
            c.Picture.Bitmap.Assign(Clipboard);
            JPG.Free;
          end else
          if (ExtractFileExt(s) = '.BMP') then begin
            BMP := TBitmap.Create;
            try
              BMP.LoadFromFile(s);
              Clipboard.Assign(BMP)
            except;
            end;
            c.Picture.Bitmap.Assign(Clipboard);
            BMP.Free;
          end;
        end else  // ---------------------------------------------------- normal text
          rep.S[slVars[i]] := sl[i]
      end else
        rep.S[slVars[i]] := '';
    end;
  finally
    slVars.Free;
  end;
end;


constructor TRepData.Create(ARep: TARUserRep; FileName: String);
begin
  inherited create;
  AssignFile(F, FileName);
  Reset(F);
  slBand:= TStringList.Create;
  rep := ARep;
  BC := TARBarcode.Create(nil);
  BC.ARUserRep := ARep;
end;

destructor TRepData.Destroy;
begin
  CloseFile(F);
  slBand.Free;
  BC.ARUserRep := nil;
  BC.Free;
  inherited Destroy;
end;

function TRepData.GetBand: String;
var s,st: String;
    start: Boolean;
begin
  start := false;
  slBand.Clear;

  if eof(F) then begin
    result := '';
    exit;
  end;

  if NextBand = '' then begin
    while pos('=',NextBand) <> 1 do
      ReadLn(F,NextBand);
    NextBand := copy(NextBand,3,MaxInt);
  end;
  result := NextBand;

  while true and (not eof(F)) do begin
    ReadLn(F,s);
    if pos('=',s) = 1 then begin
      if start then slBand.Add(st);
      NextBand := copy(s,3,MaxInt);
      exit;
    end;

    if pos('>',s) = 1 then begin
      if start then slBand.Add(st);
      st := copy(s,5,MaxInt);
      start := true;
    end else begin
      if length(s) > 5 then
        st := st + #13#10 + copy(s,5,MaxInt);
    end;

  end;
  slBand.Add(st);
end;


procedure TRepData.Load;
var
  s: String;
  isTitle: Boolean;
begin
  isTitle := false;
  s := GetBand;
  while s <> '' do begin
    if CT(s,'Title') then begin                    // Tilte
      RepAssignVars(rep,s,slBand,BC);
      Rep.AddBands(s);
      isTitle := true;
    end else if CT(s,'Header') then begin          // Header
      RepAssignVars(rep,s,slBand,BC);
      if not isTitle then Rep.AddBands(s);
    end else if CT(s,'Footer') then begin          // Footer
      RepAssignVars(rep,s,slBand,BC);
    end else if CT(s,'LastFooter') then begin      // LastFooter
      RepAssignVars(rep,s,slBand,BC);
    end else if CT(s,'PageBreak') then begin       // PageBreak
      Rep.PageBreak;
    end else begin                                 // bands
      rep.ClearPictures;
      RepAssignVars(rep,s,slBand,BC);
      Rep.AddBands(s);
    end;
    s := GetBand;
  end;

end;

end.

