unit Main;
//************************************************************//
//                         AND Report                        //
//  Eugene Hrulev (mrHru) Laxian Key, Omsk, Russia 1999,2001 //
//  Oleg Melnikov (muler) Laxian Key, Omsk, Russia 1999,2001 //
//************************************************************//

{$INCLUDE DEFINES.INC}

interface

uses
  SysUtils, Classes,
  {$IFDEF CLX}
    QForms, QExtCtrls, QButtons, QStdCtrls, QComCtrls, QControls, QGraphics, Qt,
  {$ELSE}
  	Windows, Messages, Graphics, Controls, Forms, Dialogs, DBTables,
    StdCtrls, Buttons, ExtCtrls, ComCtrls, Db,
    ARSQL,
  {$ENDIF}

  Biolife, Clients,  Simple,   Group, MasterDetail, SeveralGroup,
  ExpandOnDemand, Calendar, TrickTrak, BarcodeRep,

	ARUtils, ARVFS, ARGridRep, ARUserRep,	AREditRep, ARRepCommon,
  ARExcelExport, ARJPEGExport, ARCustomExport, ARHTMLExport, ARBarcode,
  ARScrollingWidget;

type
  TReportDblClickEventHandler   = procedure(Rep: TARUserRep);
  TReportCanEditEventHandler    = procedure(Rep: TARUserRep; IdxBand:integer; IdxCell:integer; var CanEdit:boolean);
  TReportCellChangeEventHandler = procedure(Rep: TARUserRep; IdxBand:integer; IdxCell:integer);

	TDemoDlg = class(TForm)
    Panel: TPanel;
		CloseBtn: TBitBtn;
		PreviewBtn: TBitBtn;
		Image1: TImage;
		Label1: TLabel;
		RepSelectRG: TRadioGroup;
		Bevel1: TBevel;
		ReportBtn: TBitBtn;
    StatusBar: TStatusBar;
    FocusedCellStyleRG: TRadioGroup;
    SelectedCellStyleRG: TRadioGroup;
    HTMLExportBtn: TBitBtn;
    JPGExportBtn: TBitBtn;
    ExcelExportBtn: TBitBtn;
    ProgressBar: TProgressBar;
    AbortBtn: TBitBtn;
    UserRep: TARUserRep;
    ARJPEGExport1: TARJPEGExport;
    ARExcelExport1: TARExcelExport;
    ARHTMLExport1: TARHTMLExport;
    BC1: TARBarcode;
		procedure CloseBtnClick(Sender: TObject);
		procedure PreviewBtnClick(Sender: TObject);
		procedure ReportBtnClick(Sender: TObject);
    procedure UserRepSelectBand(Sender: TObject; IdxBand: Integer);
    procedure FormShow(Sender: TObject);
    procedure FocusedCellStyleRGClick(Sender: TObject);
    procedure SelectedCellStyleRGClick(Sender: TObject);
    procedure HTMLExportBtnClick(Sender: TObject);
    procedure JPGExportBtnClick(Sender: TObject);
    procedure ExcelExportBtnClick(Sender: TObject);
    procedure ExportProgress(Sender: TObject;
      Stage: TARExportProgressStage; PercentDone: Integer;
      var Abort: Boolean);
    procedure AbortBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
	private
    Terminate :boolean;
	public
    {$IFDEF CLX}
      ClientDataSet2:TClientDataSet;
    {$ELSE}
      Database1     :TDatabase;
    {$ENDIF}

		procedure   CompositeReport(Rep: TARUserRep);
	end;

var
	DemoDlg: TDemoDlg;

var SelectBand:procedure (Rep:TARUserRep; IdxBand: Integer);

implementation

{$R *.dfm}

procedure TDemoDlg.CompositeReport(Rep: TARUserRep);
begin
	BiolifeReport(UserRep);
    UserRep.PageBreak();
    UserRep.ResetPageNumber();

  ClientsReport(UserRep);
    UserRep.PageBreak();
    UserRep.ResetPageNumber();

	SimpleReport(UserRep);
    UserRep.PageBreak();
    UserRep.ResetPageNumber();

	GroupReport(UserRep);
    UserRep.PageBreak();
    UserRep.ResetPageNumber();

	MasterDetailReport(UserRep);
    UserRep.PageBreak();
    UserRep.ResetPageNumber();

	SeveralGroupReport(UserRep);
    UserRep.PageBreak();
    UserRep.ResetPageNumber();

  ExpandOnDemandReport(UserRep);

	BarcodeReport(UserRep, BC1);
    UserRep.PageBreak();
    UserRep.ResetPageNumber();

  UserRep.FixedBands:=0;
  UserRep.FixedCells:=0;
end;

procedure TDemoDlg.CloseBtnClick(Sender: TObject);
begin
	Close;
end;

procedure TDemoDlg.PreviewBtnClick(Sender: TObject);
begin
	UserRep.Preview;
end;

procedure TDemoDlg.ReportBtnClick(Sender: TObject);
begin
  UserRep.FixedCells:=0;
  UserRep.Columns:=0;
  UserRep.Hint:='';
  SelectBand:=nil;

	Panel.Enabled:=false;
	try
		UserRep.DeleteBands;

		case RepSelectRG.ItemIndex of
			0:	BiolifeReport(UserRep);
      1:  ClientsReport(UserRep);
			2:	SimpleReport(UserRep);
			3:	GroupReport(UserRep);
			4:	MasterDetailReport(UserRep);
			5:	SeveralGroupReport(UserRep);
			6:	ExpandOnDemandReport(UserRep);
			7:	CompositeReport(UserRep);
			8:	CalendarReport(UserRep);
			9:	ShowGame;
     10:  BarcodeReport(UserRep, BC1);
		end;

		UserRep.SetFocus;
	finally
		Panel.Enabled:=true;
	end;
end;

procedure TDemoDlg.UserRepSelectBand(Sender: TObject; IdxBand: Integer);
begin
  if Assigned(SelectBand) then begin
    SelectBand(Sender as TARUserRep, IdxBand);
  end;
end;

procedure TDemoDlg.FormShow(Sender: TObject);
begin
  ReportBtnClick(Self);
//  ExcelExportBtnClick(Self);
end;

procedure TDemoDlg.FocusedCellStyleRGClick(Sender: TObject);
begin
  UserRep.FocusedCellStyle:=TFocusedCellStyle(FocusedCellStyleRG.ItemIndex);
end;

procedure TDemoDlg.SelectedCellStyleRGClick(Sender: TObject);
begin
  UserRep.SelectedCellStyle:=TSelectedCellStyle(SelectedCellStyleRG.ItemIndex);
end;

procedure TDemoDlg.ExportProgress(Sender: TObject;
  Stage: TARExportProgressStage; PercentDone: Integer; var Abort: Boolean);
begin
  case Stage of
    epsStarting:
      begin
        Terminate:=false;
        AbortBtn.Visible:=true;
        ProgressBar.Min:=0;
        ProgressBar.Max:=100;
        ProgressBar.Position:=0;
      end;
    epsRunning:
      begin
        ProgressBar.Position:=PercentDone;
        Abort:=Terminate;
      end;
    epsEnding:
      begin
        AbortBtn.Visible:=false;
        ProgressBar.Position:=0;
      end;
  end;
end;

procedure TDemoDlg.AbortBtnClick(Sender: TObject);
begin
  Terminate:=true;
end;

procedure TDemoDlg.HTMLExportBtnClick(Sender: TObject);
{$IFDEF CLX}
{$ELSE}
  var cmd:string;
{$ENDIF}
begin
  ARHTMLExport1.Execute;

  {$IFDEF CLX}
  {$ELSE}
    cmd:='explorer ' + ARHTMLExport1.FileName;

    WinExec(PChar(cmd), SW_SHOW);
  {$ENDIF}
end;

procedure TDemoDlg.JPGExportBtnClick(Sender: TObject);
begin
  ARJPEGExport1.Execute;
end;

procedure TDemoDlg.ExcelExportBtnClick(Sender: TObject);
begin
  ARExcelExport1.Execute;
end;

procedure TDemoDlg.FormCreate(Sender: TObject);
begin
  {$IFDEF CLX}
  {$ELSE}
    Database1             :=TDatabase.Create(Self);
    Database1.AliasName   :='DBDEMOS';
    Database1.DatabaseName:='DBDemos';
    Database1.SessionName :='Default';
    Database1.Connected   :=true;
  {$ENDIF}
end;

initialization
  {$IFDEF CLX}
  {$ELSE}
    Classes.RegisterClass(TDatabase);
  {$ENDIF}

end.


