unit TrickTrak;
//************************************************************//
//                         AND Report                        //
//  Eugene Hrulev (mrHru) Laxian Key, Omsk, Russia 1999,2001 //
//  Oleg Melnikov (muler) Laxian Key, Omsk, Russia 1999,2001 //
//************************************************************//

{$INCLUDE DEFINES.INC}

interface

uses
  SysUtils, Classes,
  {$IFDEF CLX}
    QForms, QButtons, QGraphics, QStdCtrls, QControls,
  {$ELSE}
    Windows, Messages, Graphics, Controls, Forms, Dialogs, StdCtrls, Buttons,
  {$ENDIF}
  ARUtils, ARGridRep, ARUserRep, ARScrollingWidget;

type
  TGameDlg = class(TForm)
    R: TARUserRep;
    BitBtn1: TBitBtn;
    procedure FormShow(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
    procedure RSelectCell(Sender: TObject; IdxBand, IdxCell: Integer);
  private
    InReset   :boolean;
    InWin     :boolean;
    function  GetM(x:integer; y:integer):integer;
    procedure SetM(x:integer; y:integer; Val:integer);
  public
    property  M[x:integer; y:integer]:integer read GetM write SetM;
    procedure Reset;
    procedure Step(x:integer; y:integer);
    procedure CheckWin;
  end;

procedure ShowGame;

var
  GameDlg: TGameDlg;

implementation

{$R *.dfm}

uses Main;

procedure ShowGame;
begin
  GameDlg:=TGameDlg.Create(DemoDlg);
  try
    GameDlg.ShowModal;
  finally
    GameDlg.Free;
  end;
end;

procedure TGameDlg.BitBtn1Click(Sender: TObject);
begin
  Close;
end;

procedure TGameDlg.FormShow(Sender: TObject);
begin
  R.AddBands('Band');

  Reset;
end;

function  TGameDlg.GetM(x:integer; y:integer):integer;
begin
  if (x < 1) or (x > 4) or (y < 1) or (y > 4) then begin
    Result:=-1;
  end else begin
    if R.SrcRep.Cells[y, x].S = '' then begin
      Result:=0;
    end else begin
      Result:=R.SrcRep.Cells[y, x].I;
    end;
  end;
end;

procedure TGameDlg.SetM(x:integer; y:integer; Val:integer);
begin
  if (x >= 1) and (x <= 4) and (y >= 1) and (y <= 4) then begin
    if (Val = 0) or (Val = 16) then begin
      R.SrcRep.Cells[y, x].S:='';
    end else begin
      R.SrcRep.Cells[y, x].I:=Val;
    end;
  end;
end;

procedure TGameDlg.Reset;
var x, y, i:integer;
begin
  InReset:=true;
  for x:=1 to 4 do begin
    for y:=1 to 4 do begin
      M[x, y]:=(x - 1) + (y - 1) * 4 + 1;
    end;
  end;

  for i:=1 to 100 do begin
    x:=Random(4) + 1;
    y:=Random(4) + 1;
    Step(x, y);
  end;
  InReset:=false;
  InWin:=false;
end;

procedure TGameDlg.Step(x:integer; y:integer);
begin
  if (x >= 1) and (x <= 4) and (y >= 1) and (y <= 4) then begin

    if M[x, y - 1] = 0 then begin
      M[x, y - 1]:=M[x, y];
      M[x, y]:=0;
      exit;
    end;

    if M[x, y + 1] = 0 then begin
      M[x, y + 1]:=M[x, y];
      M[x, y]:=0;
      exit;
    end;

    if M[x - 1, y] = 0 then begin
      M[x - 1, y]:=M[x, y];
      M[x, y]:=0;
      exit;
    end;

    if M[x + 1, y] = 0 then begin
      M[x + 1, y]:=M[x, y];
      M[x, y]:=0;
      exit;
    end;

  end;
end;

procedure TGameDlg.RSelectCell(Sender: TObject; IdxBand, IdxCell: Integer);
begin
  if CT(R.SrcRep.Cells[IdxBand, IdxCell].S, 'Reset') then begin
    Reset;
  end else begin
    if InWin then exit;
    Step(IdxCell, IdxBand);
    CheckWin;
  end;
end;

procedure TGameDlg.CheckWin;
var x, y, i:integer;
begin
  if InReset then exit;

  for x:=1 to 4 do begin
    for y:=1 to 4 do begin
      if M[x, y] <> (((x - 1) + (y - 1) * 4 + 1) mod 16) then exit;
    end;
  end;

  for x:=1 to 4 do begin
    for y:=1 to 4 do begin
      M[x, y]:=0;
    end;
  end;

  for i:=1 to 100 do begin
    x:=Random(4) + 1;
    y:=Random(4) + 1;
    R.SrcRep.Cells[x, y].Color:=Random(256) * 256 * 256 + Random(256) * 256 + Random(256);
    Application.ProcessMessages;
  end;

  for x:=1 to 4 do begin
    for y:=1 to 4 do begin
      R.SrcRep.Cells[x, y].Color:=clWhite;
    end;
  end;
  R.SrcRep.Cells[1, 1].S:='Y'; R.SrcRep.Cells[1, 2].S:='o'; R.SrcRep.Cells[1, 3].S:='u';
  R.SrcRep.Cells[2, 1].S:='A'; R.SrcRep.Cells[2, 2].S:='r'; R.SrcRep.Cells[2, 3].S:='e';
  R.SrcRep.Cells[3, 1].S:='W'; R.SrcRep.Cells[3, 2].S:='i'; R.SrcRep.Cells[3, 3].S:='n';
  InWin:=true;
end;

end.
