unit xSQL;
//************************************************************//
//                         AND Interpreter                   //
//  Eugene Hrulev (mrHru) Laxian Key, Omsk, Russia 1999,2001 //
//  Oleg Melnikov (muler) Laxian Key, Omsk, Russia 1999,2001 //
//************************************************************//

{$INCLUDE DEFINES.INC}

interface

uses
  Windows, Classes, Forms, Controls, IBSQL, IBDatabase, SysUtils, Variants,
  Graphics;

var       DefaultIBDB:TIBDatabase;
         	ISQLCnt:integer;
const   	MoneyPrecision:integer=100;

type
  ITrans = interface(IInterface)
    function    GetDB():TIBDatabase;
    procedure   SetDB(ADB:TIBDatabase);

    function    GetInTrans():boolean;
    function    GetTrans():TIBTransaction;

		procedure   StartTransaction();
		procedure   Commit();
    procedure   CommitRetaining();
		procedure   Rollback();
    procedure   RollbackRetaining();

    property    DB:TIBDatabase        read GetDB        write SetDB;
    property    InTrans:boolean       read GetInTrans;
    property    Trans:TIBTransaction  read GetTrans;
  end;

  TTrans = class(TInterfacedObject, ITrans)
  private
    FTr         :TIBTransaction;

    function    GetDB():TIBDatabase;
    procedure   SetDB(ADB:TIBDatabase);

    function    GetInTrans():boolean;
    function    GetTrans():TIBTransaction;
  public
    constructor Create();
    destructor  Destroy();override;

		procedure   StartTransaction();
		procedure   Commit();
    procedure   CommitRetaining();
		procedure   Rollback();
    procedure   RollbackRetaining();
  end;

	ISQL = interface(IInterface)
		function    GetI (const FieldName:string):integer;
		procedure   SetI (const FieldName:string;v:integer);

		function    GetS (const FieldName:string):string;
		procedure   SetS (const FieldName:string;v:string);

    function    GetTS(const FieldName: string): string;
    procedure   SetTS(const FieldName: string; v: string);

		function    GetF (const FieldName:string):extended;
		procedure   SetF (const FieldName:string;v:extended);

		function    GetC (const FieldName:string):currency;
		procedure   SetC (const FieldName:string;v:currency);

		function    GetMS(const FieldName:string):string;

		function    GetM (const FieldName:string):currency;
		procedure   SetM (const FieldName:string;v:currency);

		function    GetD (const FieldName:string):TDateTime;
		procedure   SetD (const FieldName:string;v:TDateTime);

		function    GetB (const FieldName:string):boolean;
		procedure   SetB (const FieldName:string;v:boolean);

		function    GetV (const FieldName:string):variant;
		procedure   SetV (const FieldName:string;v:variant);

		function    GetG (const FieldName:string):TGraphic;
		function    GetSS():string;

    function    GetAutoTrans():boolean;
    procedure   SetAutoTrans(AAutoTrans:boolean);

    function	  GetQ():TIBSQL;
		function	  GetSQL:TStrings;

		function	  GetEOF	:boolean;
		function	  GetBOF	:boolean;
		function	  GetEmpty:boolean;

    function	  GetTr:ITrans;
    procedure   SetTr(ATr:ITrans);

		function	  GetOwner():TObject;
    procedure   SetOwner(AOwner:TObject);

    function	  GetFields(const Idx: Integer):TIBXSQLVAR;
		function	  GetActive:boolean;
    function    GetRecordCount: integer;
    function    GetCurrent: TIBXSQLDA;

    property    Fields[const Idx: Integer]: TIBXSQLVAR  read GetFields;

		procedure   Prepare;
		procedure   Close;
		procedure   Open;
		procedure   Exec;

		procedure   New;
		procedure   Add(s:string);

		procedure   Next;

		property    I[const FieldName: string]:integer    read  GetI  write SetI;
		property    S[const FieldName: string]:string     read  GetS  write SetS;
		property    TS[const FieldName: string]:string    read  GetTS write SetTS;
		property    F[const FieldName: string]:extended   read  GetF  write SetF;
		property    C[const FieldName: string]:currency   read  GetC  write SetC;
		property    M[const FieldName: string]:currency   read  GetM  write SetM;
    property    MS[const FieldName:string]:string     read  GetMS;
		property    D[const FieldName: string]:TDateTime  read  GetD  write SetD;
		property    B[const FieldName: string]:boolean		read  GetB  write SetB;
		property    V[const FieldName: string]:variant    read  GetV  write SetV;
		property    G[const FieldName: string]:TGraphic 	read  GetG;

    procedure   BatchOutput(OutputObject: TIBBatchOutput);
    procedure   BatchInput (InputObject:  TIBBatchInput);

		procedure   StartTransaction;
		procedure   Commit;
    procedure   CommitRetaining;
		procedure   Rollback;
    procedure   RollbackRetaining;

    property    AutoTrans:boolean   read  GetAutoTrans    write SetAutoTrans;
    property    Tr:ITrans           read  GetTr           write SetTr;
		property    RecordCount:integer read  GetRecordCount;

		property	  Active:boolean      read  GetActive;
		property	  EOF:boolean 	      read  GetEOF;
		property	  BOF:boolean 	      read  GetBOF;
		property	  Empty:boolean       read  GetEmpty;

		property	  Owner:TObject       read  GetOwner	      write	SetOwner;
		property    SQL:TStrings        read  GetSQL;
		property	  ss:string           read  GetSS;
    property	  Current:TIBXSQLDA   read  GetCurrent;
    property	  Query:TIBSQL        read  GetQ;
	end;

function  NewSQL(AIBDB:TIBDatabase; ATr:ITrans; AAutoTrans:boolean = false):ISQL;overload;
function  NewSQL(AIBDB:TIBDatabase):ISQL;overload;
function  NewSQL(ATr:ITrans):ISQL;overload;
function  NewSQL(ASQL:ISQL):ISQL;overload;
function  NewSQL():ISQL;overload;

function  NewTr(AIBDB:TIBDatabase = nil):ITrans;
function  MR(money:currency):currency;  //money rounder
function  GetUniqueNumber: integer;forward;

implementation

{ TSQL }
type
	TSQL = class(TInterfacedObject, ISQL)
	private
		Tag         :integer;
		FQ          :TIBSQL;
    Picture     :TPicture;

		FOwner			:TObject;

		FSS         :string;
    FAutoTrans: boolean;

    FTr         :ITrans;

		function    GetI (const FieldName:string):integer;
		procedure   SetI (const FieldName:string;v:integer);

		function    GetS (const FieldName:string):string;
		procedure   SetS (const FieldName:string;v:string);

    function    GetTS(const FieldName: string): string;
    procedure   SetTS(const FieldName: string; v: string);

//		function    GetT (const FieldName:string):string;
//		procedure   SetT (const FieldName:string;v:string);

		function    GetF (const FieldName:string):extended;
		procedure   SetF (const FieldName:string;v:extended);

		function    GetC (const FieldName:string):currency;
		procedure   SetC (const FieldName:string;v:currency);

		function    GetMS(const FieldName:string):string;

		function    GetM (const FieldName:string):currency;
		procedure   SetM (const FieldName:string;v:currency);

		function    GetD (const FieldName:string):TDateTime;
		procedure   SetD (const FieldName:string;v:TDateTime);

		function    GetB (const FieldName:string):boolean;
		procedure   SetB (const FieldName:string;v:boolean);

		function    GetV (const FieldName:string):variant;
		procedure   SetV (const FieldName:string;v:variant);

		function    GetG (const FieldName:string):TGraphic;
		function    GetSS():string;

    function    GetAutoTrans():boolean;
    procedure   SetAutoTrans(AAutoTrans:boolean);

		function	  GetSQL:TStrings;

		function	  GetEOF	:boolean;
		function	  GetBOF	:boolean;
		function	  GetEmpty:boolean;

    function	  GetTr:ITrans;
    procedure   SetTr(ATr:ITrans);

		function	  GetOwner():TObject;
    procedure   SetOwner(AOwner:TObject);

    function	  GetFields(const Idx: Integer):TIBXSQLVAR;
		function	  GetActive:boolean;
    function    GetRecordCount: integer;
    function    GetCurrent: TIBXSQLDA;
    function    GetQ():TIBSQL;
	public
    property    Fields[const Idx: Integer]: TIBXSQLVAR  read GetFields;

		procedure   Prepare;
		procedure   Close;
		procedure   Open;
		procedure   Exec;

		procedure   New;
		procedure   Add(s:string);

		procedure   Next;

		property    I[const FieldName: string]:integer    read  GetI  write SetI;
		property    S[const FieldName: string]:string     read  GetS  write SetS;
		property    TS[const FieldName: string]:string    read  GetTS write SetTS;
		property    F[const FieldName: string]:extended   read  GetF  write SetF;
		property    C[const FieldName: string]:currency   read  GetC  write SetC;
		property    M[const FieldName: string]:currency   read  GetM  write SetM;
    property    MS[const FieldName:string]:string     read  GetMS;
		property    D[const FieldName: string]:TDateTime  read  GetD  write SetD;
		property    B[const FieldName: string]:boolean		read  GetB  write SetB;
		property    V[const FieldName: string]:variant    read  GetV  write SetV;
		property    G[const FieldName: string]:TGraphic 	read  GetG;

//		property  T[const FieldName: string]:string     read GetT write SetT;	//read from BLOB

    procedure   BatchOutput(OutputObject: TIBBatchOutput);
    procedure   BatchInput (InputObject:  TIBBatchInput);

		procedure   StartTransaction;
		procedure   Commit;
    procedure   CommitRetaining;
		procedure   Rollback;
    procedure   RollbackRetaining;

		constructor Create(AIBDB:TIBDatabase; ATr:ITrans; AAutoTrans:boolean = false);overload;
		constructor Create(AIBDB:TIBDatabase);overload;
		constructor Create(ATr:ITrans);overload;
		constructor Create(ASQL:ISQL);overload;
		constructor Create;overload;
		destructor  Destroy;override;

  published
    property    AutoTrans:boolean   read  FAutoTrans      write FAutoTrans;
    property    Tr:ITrans           read  GetTr           write SetTr;
		property    RecordCount:integer read  GetRecordCount;

		property	  Active:boolean      read  GetActive;
		property	  EOF:boolean 	      read  GetEOF;
		property	  BOF:boolean 	      read  GetBOF;
		property	  Empty:boolean       read  GetEmpty;

		property	  Owner:TObject       read  FOwner 		      write	FOwner;
		property    SQL:TStrings        read  GetSQL;
		property	  ss:string           read  FSS;
    property	  Current:TIBXSQLDA   read  GetCurrent;
    property	  Query:TIBSQL        read  GetQ;
	end;

constructor TSQL.Create(AIBDB:TIBDatabase; ATr:ITrans; AAutoTrans:boolean = false);
begin
	inc(ISQLCnt);

  Ftr           :=ATr;

	FQ            :=TIBSQL.Create(nil);
  FQ.Name       :='TIBSQL_' + IntToStr(GetUniqueNumber);
	Tag           :=GetUniqueNumber;
  AutoTrans     :=AAutoTrans;

	FQ.Database   :=AIBDB;
  FQ.Transaction:=ATr.Trans;
end;

constructor TSQL.Create(AIBDB:TIBDatabase);
begin
  Self.Create(AIBDB, NewTr(AIBDB), true);
end;

constructor TSQL.Create(ATr:ITrans);
begin
  Self.Create(DefaultIBDB, Atr);
end;

constructor TSQL.Create(ASQL:ISQL);
begin
  Self.Create(ASQL.Query.Database, ASQL.Tr);
end;

constructor TSQL.Create;
begin
  Self.Create(DefaultIBDB);
end;

destructor  TSQL.Destroy;
begin
  if Assigned(Picture) then Picture.Free;

	FQ.Close;

  if AutoTrans then begin
    if Tr.InTrans then begin
      Tr.Commit();
    end;
  end;

	FQ.Free;
	dec(ISQLCnt);

	inherited;
end;

procedure TSQL.Prepare;
begin
  if FQ.Open then FQ.Close;

  if not Tr.InTrans then begin
    Tr.StartTransaction();
    AutoTrans:=true;
  end;

	FQ.Prepare();
end;

procedure TSQL.Close;
begin
	FQ.Close();

  if AutoTrans then begin
    if Tr.InTrans then Tr.Commit();
    AutoTrans:=false;
  end;
end;

procedure TSQL.Open;
begin
  if not Tr.InTrans then begin
    Tr.StartTransaction;
    AutoTrans:=true;
  end;

  FQ.ExecQuery;
end;

procedure TSQL.Exec;
begin
  if not Tr.InTrans then begin
    Tr.StartTransaction;
    AutoTrans:=true;
  end;

  FQ.ExecQuery();

  if AutoTrans then begin
    if Tr.InTrans then Tr.Commit();
    AutoTrans:=false;
  end;
end;

procedure TSQL.StartTransaction;
begin
  Tr.StartTransaction;
  AutoTrans:=false;
end;

procedure TSQL.CommitRetaining;
begin
  Tr.CommitRetaining;
  AutoTrans:=false;
end;

procedure TSQL.Commit;
begin
  Tr.Commit;
  AutoTrans:=false;
end;

procedure TSQL.RollbackRetaining;
begin
  Tr.RollbackRetaining;
  AutoTrans:=false;
end;

procedure TSQL.Rollback;
begin
  Tr.Rollback;
  AutoTrans:=false;
end;

{
function  TSQL.GetT (const FieldName:string):string;
var	bs:TBlobStream;
		p:PChar;
begin
	bs:=TBlobStream.Create(FQ.FieldByName(FieldName). as TMemoField, bmRead);
	GetMem(p, bs.Size + 1);
	try
		bs.Read(p^, bs.Size);
		p[bs.Size]:=#0;
		Result:=p;
	finally
		FreeMem(p);
		bs.Free;
	end;
end;

procedure TSQL.SetT (const FieldName:string;v:string);
var p:PChar;
		len:integer;
begin
	len:=Length(v);
	GetMem(p, len + 1);
	try
		StrPLCopy(p, v, len);
		len:=Length(v);
    p[len]:=#0;
		FQ.ParamByName(FieldName).SetBlobData(p, len);
	finally
		FreeMem(p);
	end;
end;
}
function	TSQL.GetActive:boolean;
begin
	Result:=FQ.Open;
end;

function	TSQL.GetEOF	:boolean;
begin
	if FQ.Open then begin
		Result:=FQ.EOF;
	end else begin
		Result:=true;
	end;
end;
function	TSQL.GetBOF	:boolean;
begin
	if FQ.Open then begin
		Result:=FQ.BOF;
	end else begin
		Result:=true;
	end;
end;
function	TSQL.GetEmpty:boolean;
begin
	Result:=EOF and BOF;
end;

function	TSQL.GetSQL:TStrings;
begin
	Result:=FQ.SQL;
end;

procedure TSQL.New;
begin
  if AutoTrans then begin
    if Tr.InTrans then Tr.Commit;
  end;

	FQ.Close;
	FQ.SQL.Clear;
	FSS:='';
end;

procedure TSQL.Add(s:string);
begin
	FQ.SQL.Add(s);
	FSS:=FSS + s + ' ';
end;

procedure TSQL.Next;
begin
	FQ.Next;
end;

function  TSQL.GetRecordCount:integer;
begin
	Result:=FQ.RecordCount;
end;

function  TSQL.GetI (const FieldName:string):integer;
begin
	Result:=FQ.FieldByName(FieldName).AsInteger;
end;

procedure TSQL.SetI (const FieldName:string;v:integer);
begin
  if not Tr.InTrans then begin
    Tr.StartTransaction;
    AutoTrans:=true;
  end;
  FQ.Params.ByName(FieldName).AsInteger:=v;
end;

function  TSQL.GetS (const FieldName:string):string;
begin
	Result:=FQ.FieldByName(FieldName).AsString;
end;

procedure TSQL.SetS (const FieldName:string;v:string);
begin
  if not Tr.InTrans then begin
    Tr.StartTransaction;
    AutoTrans:=true;
  end;
	FQ.Params.ByName(FieldName).AsString:=v;
end;

function  TSQL.GetTS (const FieldName:string):string;
begin
	Result:=FQ.FieldByName(FieldName).AsTrimString;
end;

procedure TSQL.SetTS (const FieldName:string;v:string);
begin
  if not Tr.InTrans then begin
    Tr.StartTransaction;
    AutoTrans:=true;
  end;
	FQ.Params.ByName(FieldName).AsTrimString:=v;
end;

function  TSQL.GetF (const FieldName:string):extended;
begin
	if VarType(FQ.FieldByName(FieldName).Value) = varDate then begin
  	Result:=FQ.FieldByName(FieldName).AsDateTime;
  end else begin
  	Result:=FQ.FieldByName(FieldName).AsDouble;
  end;
end;
procedure TSQL.SetF (const FieldName:string;v:extended);
begin
  if not Tr.InTrans then begin
    Tr.StartTransaction;
    AutoTrans:=true;
  end;
	FQ.Params.ByName(FieldName).AsDouble:=v;
end;

function  TSQL.GetC (const FieldName:string):currency;
begin
	Result:=FQ.FieldByName(FieldName).AsCurrency;
end;
procedure TSQL.SetC (const FieldName:string;v:currency);
begin
  if not Tr.InTrans then begin
    Tr.StartTransaction;
    AutoTrans:=true;
  end;
	FQ.Params.ByName(FieldName).AsCurrency:=v;
end;

function  TSQL.GetM (const FieldName:string):currency;
begin
	Result:=MR(FQ.FieldByName(FieldName).AsCurrency);
end;

procedure TSQL.SetM (const FieldName:string;v:currency);
begin
  if not Tr.InTrans then begin
    Tr.StartTransaction;
    AutoTrans:=true;
  end;
	FQ.Params.ByName(FieldName).AsCurrency:=v;
end;

function  TSQL.GetD (const FieldName:string):TDateTime;
begin
	Result:=FQ.FieldByName(FieldName).AsDateTime;
end;

procedure TSQL.SetD (const FieldName:string;v:TDateTime);
begin
  if not Tr.InTrans then begin
    Tr.StartTransaction;
    AutoTrans:=true;
  end;
	FQ.Params.ByName(FieldName).AsDateTime:=v;
end;

function  TSQL.GetB (const FieldName:string):boolean;
begin
	Result:=boolean(FQ.FieldByName(FieldName).AsInteger);
end;

procedure TSQL.SetB (const FieldName:string;v:boolean);
begin
  if not Tr.InTrans then begin
    Tr.StartTransaction;
    AutoTrans:=true;
  end;
	FQ.Params.ByName(FieldName).AsInteger:=integer(v);
end;

function  TSQL.GetV (const FieldName:string):variant;
begin
	Result:=FQ.FieldByName(FieldName).AsVariant;
end;

procedure TSQL.SetV (const FieldName:string;v:variant);
begin
  if not Tr.InTrans then begin
    Tr.StartTransaction;
    AutoTrans:=true;
  end;
	FQ.Params.ByName(FieldName).AsVariant:=v;
end;

function  TSQL.GetG (const FieldName:string):TGraphic;
var stream:TMemoryStream;
begin
  if not Assigned(Picture) then Picture:=TPicture.Create;

  stream:=TMemoryStream.Create();
  try
    FQ.FieldByName(FieldName).SaveToStream(stream);
    Picture.Graphic.LoadFromStream(stream);
    Result:=Picture.Graphic;
  finally
    stream.Free();
  end;
end;

{
function  TSQL.GetDataSet:TDataSet;
begin
	Result:=FQ;
end;
}
function	  TSQL.GetTr:ITrans;
begin
	Result:=FTr;
end;

procedure   TSQL.SetTr(ATr:ITrans);
begin
  AutoTrans:=false;
	FTr:=Atr;
end;

function    TSQL.GetFields(const Idx: Integer):TIBXSQLVAR;
begin
  Result:=FQ.Fields[Idx];
end;

function    TSQL.GetCurrent: TIBXSQLDA;
begin
  Result := FQ.Current;
end;

procedure   TSQL.BatchOutput(OutputObject: TIBBatchOutput);
begin
  FQ.BatchOutput(OutputObject);
end;

procedure   TSQL.BatchInput (InputObject:  TIBBatchInput);
begin
  FQ.BatchInput (InputObject);
end;

function TSQL.GetMS(const FieldName: string): string;
var DS:char;
    c:currency;
    p, i, j:integer;
    s:string;
    negsign:boolean;
begin
  c:=M[FieldName];
  if c <> 0 then begin
    DS:=DecimalSeparator;
    try
      negsign:=c < 0.0;
      c:=Abs(c);

      DecimalSeparator:='-';
      s:=Format('%.2f', [c]);

      p:=System.Pos('-', s);
      Result:='';
      j:=0;
      for i:=p - 1 downto 1 do begin
        Result:=s[i] + Result;
        inc(j);
        if (j mod 3) = 0 then begin
          Result:=' ' + Result;
        end;
      end;
      Result:=Result + '-' + s[p + 1] + s[p + 2];

      if negsign then Result:='-' + Result;
    finally
      DecimalSeparator:=DS;
    end;
  end else begin
    Result:='';
  end;
end;

function TSQL.GetAutoTrans: boolean;
begin
  Result:=FAutoTrans;
end;

function TSQL.GetOwner: TObject;
begin
  Result:=FOwner;
end;

function TSQL.GetSS: string;
begin
  Result:=FSS;
end;

procedure TSQL.SetAutoTrans(AAutoTrans: boolean);
begin
  FAutoTrans:=AAutoTrans;
end;

procedure TSQL.SetOwner(AOwner: TObject);
begin
  FOwner:=AOwner;
end;

function TSQL.GetQ: TIBSQL;
begin
  Result:=FQ;
end;

{ TTrans }

constructor TTrans.Create;
begin
  FTr:=TIBTransaction.Create(nil);
end;

procedure TTrans.Commit;
begin
  FTr.Commit();
end;

procedure TTrans.CommitRetaining;
begin
  FTr.CommitRetaining();
end;

destructor TTrans.Destroy;
begin
  FTr.Free();

  inherited;
end;

function TTrans.GetDB: TIBDatabase;
begin
  Result:=FTr.DefaultDatabase;
end;

function TTrans.GetInTrans: boolean;
begin
  Result:=FTr.InTransaction;
end;

function TTrans.GetTrans: TIBTransaction;
begin
  Result:=FTr;
end;

procedure TTrans.Rollback;
begin
  FTr.Rollback();
end;

procedure TTrans.RollbackRetaining;
begin
  FTr.RollbackRetaining();
end;

procedure TTrans.SetDB(ADB: TIBDatabase);
begin
  FTr.DefaultDatabase:=ADB;
end;

procedure TTrans.StartTransaction;
begin
  FTr.StartTransaction();
end;

{ Routines }

function  NewTr(AIBDB:TIBDatabase = nil):ITrans;
begin
  Result:=TTrans.Create();
  if Assigned(AIBDB) then begin
    Result.DB:=AIBDB;
  end else begin
    Result.DB:=DefaultIBDB;
  end;

//  Result.Params.Add('nowait');
//  Result.IdleTimer:=30;
//  Result.DefaultAction:=taRollback;
end;

{$O-}
function  MR(money:currency):currency;  // money rounder
begin
	Result:=(Int(Abs(money)*MoneyPrecision + 0.5)/MoneyPrecision);
	if money < 0 then Result:=-Result;
end;
{$O+}

var
	GuardUniqueNumber :integer;
	UniqueNumber      :integer;

function GetUniqueNumber: integer;
asm
@@1:  MOV     EDX,1
			XCHG    GuardUniqueNumber,EDX
			OR      EDX,EDX
			JZ      @@2

			PUSH    0
			CALL    Sleep
			JMP     @@1

@@2:  MOV     EAX,UniqueNumber
			INC     EAX
			MOV     UniqueNumber,EAX
			MOV     GuardUniqueNumber,EDX
			RET
end;

function  NewSQL(AIBDB:TIBDatabase; ATr:ITrans; AAutoTrans:boolean = false):ISQL;
begin
  Result:=TSQL.Create(AIBDB, ATr, AAutoTrans);
end;

function  NewSQL(AIBDB:TIBDatabase):ISQL;
begin
  Result:=TSQL.Create(AIBDB);
end;

function  NewSQL(ATr:ITrans):ISQL;
begin
  Result:=TSQL.Create(ATr);
end;

function  NewSQL(ASQL:ISQL):ISQL;
begin
  Result:=TSQL.Create(ASQL);
end;

function  NewSQL():ISQL;
begin
  Result:=TSQL.Create();
end;

end.


